#ifndef MAJACAMMODES__H
#define MAJACAMMODES__H

#include "majatypes.h"

namespace maja
{

/*!
	type of the different mode that the camera can stand.
	Each type may could freely be added with another one, 
	in order to make a more specific camera.

	status: please refer to each one for its availability
*/
typedef muint _MajaCameraMode;

//! never use it
static const muint	camera_mode_none=	0x0;

//! free camera, almost for user-camera movements
//! (as in quake-like).
//! status: useable
static const muint	camera_mode_free=	0x1;
	
//! camera that doesn't move.
//! status: not useable
static const muint	camera_mode_static=	0x2;
	
//! target following camera.
//! statud not useable
static const muint	camera_mode_follow=	0x4;
	
//! camera that is at the eyes of the player.
//! status: not useable
static const muint	camera_mode_pilot=	0x8;
	
//! revolution moves camera: as a planet has a 
//! revolution around its star.
//! status: useable
static const muint	camera_mode_revolution=	0x10;

//! own camera (as quake-like).
//! status: not useable
static const muint	camera_mode_own=	0x20;
	
//! undefined camera (could be a user defined camera)
//! status not useable
static const muint	camera_mode_user=	0x40;
}
#endif
