/*
	majacom.h
	ce fichier est une partie de Maja - this is a part of Maja
	essai basique sur une com
	basic try on a com

	$Id: $

	$Log: $

	begin:			Mon Feb 10 2003
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef MAJACOM__H
#define MAJACOM__H

#include "majatypes.h"
#include "majafunctors.h"
#include "majatemp.h"

namespace maja
{
	/*!
	 * this is the new way maja's components will communicate to each other internally
	 */

	typedef muint 		mcom;

	static const mcom	mcom_nothing		= 0x0;
	
	//! normal behaves repercutions to the caller's state
	static const mcom	mcom_no_change		= 0x1;

	static const mcom	mcom_goto_next		= 0x2;

	static const mcom	mcom_goto_prev		= 0x4;



	//! normal behaves answers
	static const mcom	mcom_accept		= 0x10;

	static const mcom	mcom_refuse		= 0x20;

	static const mcom	mcom_do_not_care	= 0x40;

	
	//! communication mode (please see MajaCom bellow)
	static const mcom	mcom_begin_com		= 0x100;

	static const mcom	mcom_end_com		= 0x200;


	// errors: everything superior or equal is an error value.
	static const mcom	mcom_error		= 0x100000;

	//! exit level com
	static const mcom	mcom_exit_here		= 0x1000000;

	static const mcom	mcom_exit_engine	= 0x2000000;

	static const mcom	mcom_exit_module	= 0x4000000;

	static const mcom	mcom_exit_program	= 0x8000000;


	//! internal error
	static const mcom	mcom_abnormal_error	= 0x10000000;
	

	/*!
	 * first try on a common maja communication object.
	 * a comobject should have input/outputs read/write capabilities,
	 * and should communicate with almost anything that are functions like,
	 * as a function, a member function and so on.
	 * I is the input class that we derive from.
	 * O is the output class.
	 * COMOBJECT is an eventual extra-object of communication. Please note
	 * 	that COMOBJECT need to follow some particular way of declaration.
	 * 	The class need to overload its () operator in order with this
	 * 	prototype: mcom COMOBJECT::operator () ( mint);
	 * 	we do not check this type at any instanciation.
	 * TM is the threading model of an instance of this type. TM is not
	 * 	checked by default.
	 */
	template
	<
		class I,
		class O,
		class COMOBJECT= MajaNoChecking<mint>,
		class TM= MajaNoChecking<mint>//MajaEmptyType
	>
	class MajaCom: public I, public TM
	{	
		O	*output;
		
		protected:

		mcom	o_com;

		MajaFunctor<mcom,MajaTL_1( mcom)> *cmd;
	
		public:

		MajaCom( O *o): I(){
			output= o;
			cmd= mnull;
		}

		virtual ~MajaCom(){
			if( cmd)
				delete cmd;
		}

		// read a mesg from output
		virtual mcom	operator >> ( mcom& com){
			
			*output >> com;
			
			return o_com;
		}
	
		// write a msg to output
		virtual mvoid	operator << ( mcom msg){
			
			o << msg;

		}

		mvoid SetComObject( COMOBJECT *o){
			cmd= new MajaFunctor<mcom,MajaTL_1( mint)>( *o);
			if( !cmd)
				;
		}
	};
}

#endif
