/*
	majawingr.h
	ce fichier est une partie de Maja - this is a part of Maja
	gestionnaire de fentre temps-rel et limit
	limited & real-time window manager
	
	$Id: $

	$Log: $

	begin:			Dec 2002
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef MAJA_FUNCTORS__H
#define MAJA_FUNCTORS__H

#include <memory>
#include "majatypelists.h"

namespace maja
{

// maja functor implements
// 
template <typename R, class TL> class MajaFunctorImpl;

template <typename R>
class MajaFunctorImpl<R, MajaNullType>
{
	public:
	
	virtual R operator()()= 0;
	
	virtual MajaFunctorImpl* Clone() const= 0;
	
	virtual ~MajaFunctorImpl(){}
};

template <typename R, typename P1>
class MajaFunctorImpl<R, MajaTL_1( P1)>
{
	public:

	virtual R operator()( P1)= 0;
	
	virtual MajaFunctorImpl* Clone() const= 0;
	
	virtual ~MajaFunctorImpl(){}
};

template <typename R, typename P1, typename P2>
class MajaFunctorImpl<R, MajaTL_2( P1, P2)>
{
	public:

	virtual R operator()( P1, P2)= 0;
	
	virtual MajaFunctorImpl* Clone() const= 0;

	virtual ~MajaFunctorImpl(){}
};


// handler of a functor
template <class ParentFunctor, typename Func>
class MajaFunctorHandler: public MajaFunctorImpl<	typename ParentFunctor::_result,
							typename ParentFunctor::_param_list>
{
	private:

	Func	_func;

	public:

	typedef typename ParentFunctor::_result	_result;

	MajaFunctorHandler( const Func& func): _func( func){
		;
	}

	MajaFunctorHandler* Clone() const{
		return new MajaFunctorHandler( *this);
	}

	_result operator()(){
		return _func();
	}

	_result operator()( typename ParentFunctor::_param1 p1){
		return _func( p1);
	}

	_result operator()( typename ParentFunctor::_param1 p1, typename ParentFunctor::_param2 p2){
		return _func( p1, p2);
	}
};


// the functor could make communication threw
// objects, and could be time-late execution.
template <typename R, class TL>
class MajaFunctor
{
	private:

	typedef R							_result;

	typedef MajaFunctorImpl<R,TL>					_MajaImpl;
	
	typedef TL							_param_list;
	
	typedef typename MajaTypeAt<TL,0,MajaEmptyType>::_result	_param1;
	typedef typename MajaTypeAt<TL,1,MajaEmptyType>::_result	_param2;
	typedef typename MajaTypeAt<TL,2,MajaEmptyType>::_result	_param3;
	typedef typename MajaTypeAt<TL,3,MajaEmptyType>::_result	_param4;
	typedef typename MajaTypeAt<TL,4,MajaEmptyType>::_result	_param5;

	std::auto_ptr<_MajaImpl>					_sp_impl;
	
	public:

	MajaFunctor();

	MajaFunctor( const MajaFunctor&);

	// in order to pass functions, we need to pass the address of it, not it as is...
	template <typename Func> 
	MajaFunctor( const Func& fun): _sp_impl( new MajaFunctorHandler<MajaFunctor, Func>( fun)){
		;
	}

	MajaFunctor& operator=( const MajaFunctor&);

	explicit MajaFunctor( std::auto_ptr<_MajaImpl> sp_impl);
		
	R operator()(){
		return (*_sp_impl)();
	}

	R operator()( _param1 p1){
		return (*_sp_impl)( p1);
	}

	R operator()( _param1 p1, _param2 p2){
		return (*_sp_impl)( p1, p2);
	}
};

}// namespace maja

#endif
