#ifndef MAJA_TXT__H
#define MAJA_TXT__H

#ifndef WIN32
#include <GL/glx.h>
#else
#include <windows.h>
#endif
#include <string>

namespace maja
{
//
// outway classes:
// T is generally string type, but could be
// any type compliant with its rules.
// Their should be many other ways to text output...
// 

//
// simple stdio cout text output
// 
template <class T>
struct MajaStdOut
{
	inline void Print( T& _t=0){
		std::cout << _t;
	}

	protected:

	~MajaStdOut(){}
};

//
// simple stderr cerr text output
// 
template <class T>
struct MajaStdErr
{
	inline void Print( T& _t=0){
		std::cerr << _t;
	}

	protected:

	~MajaStdErr(){}
};

#ifndef WIN32

//
// glx print way.
// You must be in an adequation context under OpenGL
// renderizing flow in order to see what you want to write.
// (You could use it in MajaWindow::HudDraw() by example).
//
// T is the type of a font (Font under X11)
// _f is the font used (fixed by default)
// _d is a pointer to the display (needeed under X11)
// _i is the started generated display list id
// 
template
<
	class T, 
	class F=Font
>
struct MajaGlxOut
{
	F		_f;	// may not be here...
	Display		*_d;
	GLuint 		_i;
	int		_x;
	int		_y;
	int		_w;
	int		_h;

	MajaGlxOut(){
		_d= NULL;
		_i= 0;
		_x= 0;
		_y= 0;
		_w= 10;
		_h= 10;
	}

	protected:

	~MajaGlxOut(){
		;
	}

	public:
	
	inline void Print( T& _t){
		glListBase( _i);
		glCallLists( _t.length(), GL_UNSIGNED_BYTE, (GLubyte*)_t.c_str());
	}

	inline void Locate( int x, int y, int z=0){
		_x=x;
		_y=y;
		glRasterPos2i( _x, _y);
	}

	inline void SetFont( std::string font_name="fixed"){
		int 		first=0;
		int 		last=0;
		XFontStruct 	*fontInfo;
		static std::string tmp_str= font_name;

//		if( tmp_str==font_name)//			return;
		
		fontInfo= XLoadQueryFont( _d, font_name.c_str());
		if( !fontInfo){
	        	printf("Error: font %s not found\n", font_name.c_str());
			exit(0);
		}
		_f= fontInfo->fid;
		first= fontInfo->min_char_or_byte2;
		last= fontInfo->max_char_or_byte2;
		_i = glGenLists((GLuint) last + 1);
		if( !_i){
			printf("Error: unable to allocate font lists\n");
			exit(0);
		}
		glXUseXFont( _f, first, last-first+1, _i+first);
	}
};
#endif
}// namespace maja
#endif

