#ifndef MAJA_TYPELISTS__H
#define MAJA_TYPELISTS__H

#include "majatypes.h"

namespace maja
{
	// the type list, _head and _tail are types related
	template <class T, class U>
	struct MajaTypeList
	{
		typedef T _head;
		typedef U _tail;
	};

	// this is simple typedefinition for common type lists :one up to five arguments
	// could be given to them with arguments is a type.
	#define MajaTL_1(T1)			MajaTypeList<T1, MajaNullType>
	#define MajaTL_2(T1, T2)		MajaTypeList<T1, MajaTL_1( T2)>
	#define MajaTL_3(T1, T2, T3)		MajaTypeList<T1, MajaTL_2( T2, T3)>
	#define MajaTL_4(T1, T2, T3, T4)	MajaTypeList<T1, MajaTL_3( T2, T3, T4)>
	#define MajaTL_5(T1, T2, T3, T4, T5)	MajaTypeList<T1, MajaTL_4( T2, T3, T4, T5)>

	// length of a type list
	template <class TList> struct MajaTLLength;

	// typelists length specialization for null type
	template <> struct MajaTLLength<MajaNullType>
	{
		enum{ 
			_value= 0
		};
	};

	// specialization for T & U
	template <class T, class U>
	struct MajaTLLength<MajaTypeList<T,U> >
	{
		enum{ 
			_value=1+ MajaTLLength<U>::_value
		};
	};


	// the type that is at the specified index
	template <class TL, unsigned int index, class O> struct MajaTypeAt;

	// the type at when at exists and all types are concrete (not null types)
	template <class H, class T, class O>
	struct MajaTypeAt<MajaTypeList<H,T>,0,O>
	{
		typedef H _result;
	};

	// O is needed when H is MajaNullType
	// when there is an out of bound in the type list,
	// we force _result to be the alternative O type
	// O must be to a default type as not only functors could use it
	template <unsigned int i, class O>
	struct MajaTypeAt<MajaNullType,i,O>
	{
		typedef O _result;
	};

	// default type at the index specified
	template <class H, class T, unsigned int i, class O>
	struct MajaTypeAt<MajaTypeList<H,T>,i,O>
	{
		typedef typename MajaTypeAt<T,i-1,O>::_result _result;
	};
}
#endif
