/* $Id: $
 * majaclock.h , this is a part of Maja
 * timer, clock, chrono
 * Copyright (C) september 2000-2002 Jean-Dominique Frattini
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Cette license est valable en terme de droit franais. Cela signifie que c'est la
 * lgislation franaise qui est en vigueur pour l'ensemble des sources contenues
 * dans la librairie. Sauf image.cpp et image.h, se reposant sur aucune license explicite,
 * mais qui sont fournies aimablement dans les librairies Mesa sous license LGPL.
 *
 * Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou
 * le modifier sous les termes de la license publique gnrale
 * publie par la Free Software Foundation; soit la version 2 de la
 * license, ou (si vous le voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir d'tre utile, mais
 * SANS AUCUNE GARANTIE; et mme une garantie implicite,
 * y compris les garanties de COMMERCIALISATION ou D'ADAPTATION
 * DANS UN BUT SPCIFIQUE. Prenez connaissance de la License Publique
 * Gnrale GNU pour plus de dtail.
 *
 * Vous devez avoir reu une copie de la License Publique Gnrale
 * GNU avec ce programme; si ce n'est pas le cas, vrivez  la
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: $
 */

#ifndef __MAJA_CLOCK_H
#define __MAJA_CLOCK_H

#include <stdlib.h>
#ifdef WIN32
#include <sys/timeb.h>
#else
#include <sys/time.h>
#endif
#include "../common/majatypes.h"

namespace maja
{
	
	#ifndef WIN32
	#define __CLOCK_STEP 1000000
	#define __CLOCK_FREQUENCY 0.000001
	#else
	#define __CLOCK_STEP 1000
	#define __CLOCK_FREQUENCY 0.001
	#endif

	/*!
		Clock class for chronometer, time, any info about time, precised at 10 pow-6 or 10 pow-3 second
		seems not to be optimized
		i don't know if latency time due to class (calls to methods, use of member) is null enough
		to be forgotten
		Windows timing function is not well. must use another (what's said in the opengl forum)
	*/
	class MajaClock
	{
		static real system_clock;		//! system cpu time
		real start_clock;					//! time of the start of chronometer	
		real clock;							//! time of the clock
		real stop_clock;					//! time of stop of chronometer
		real last_clock;					//! time at last call of clok
		public:
		real step_clock;					//! laps between 2 calls of clock
	
		MajaClock( void){system_clock=Chrono();}

		//! start of timing (chrono at 0'00"000000)
		inline real Start( void){ return( start_clock=Chrono());}
	
		//! stop timing
		inline real Stop ( void){ return( stop_clock=Chrono()-start_clock);}
	
		//! time at call
		inline real Clock( void){ return( clock=Chrono()-start_clock);}
	
		//! time laps between 2 call of this method from a same instance
		inline real Step ( void){
			Clock();
		 	step_clock=clock-last_clock;
			last_clock=clock;
			return step_clock;
		}
	
		private:
		//! private method that give the cpu time
		inline static real Chrono( void){
			#ifndef _WIN32
				struct timeval tv;
				gettimeofday( &tv, null);
				system_clock = (real)tv.tv_sec;
				system_clock+= (real)tv.tv_usec*__CLOCK_FREQUENCY;
			#else
				struct timeb tm;
				ftime( &tm);
				system_clock= (real)tm.time;
				system_clock+= (real)tm.millitm*__CLOCK_FREQUENCY;
			#endif
			return system_clock;
		}
	};
}
#endif
