/*
	majacore.h
	ce fichier est une partie de Maja - this is a part of Maja
	le coeur de la gestion de Maja.
	The Maja core management.

	$Id: $

	$Log: $

	begin:		Thu Sep 05 2002
	copyright:	(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:		jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef __MAJA_CORE_H
#define __MAJA_CORE_H

#include <iostream>
#include "majaclock.h"
#include "majascheduler.h"
#include "../tools/containers/majastack.h"
#include "majaevent.h"
#include "majaservice.h"

namespace maja
{
/*!
	NOTES:
	* THIS COULD BE USEABLE FOR ADVANCED USERS ONLY. THEY SHOULD
	KNOW HOW WORKS THE CORE TOO. THIS IS EXPERIMENTAL WAY.
	
	* THIS WILL DETERMINATE THE FIRST MINOR VERSION OF MAJA (0.1)
	WHEN IT WILL BE FULL IMPLEMENTED AND TESTED. SO, DON'T USE
	IT UNTIL THE VERSION is 0.1.
*/

namespace MAJACORE
{

/*!
	This is the system core of Maja.
	Anything will be due to this.
	
	YOU MAY COULD USE IT SURELY, BUT THIS IS STILL EXPERIMENT.
*/
class MajaCore: public	MajaScheduler
{
	static const mchar 					internal_name[]= 	"MajaCore ";
	
	protected:

	//! The clock of Maja
	MajaClock						&main_clock;

	//! if the clock was allocated dynamically, users should not use it
	mbool							clock_allocated;
	
	//! a stack of modules loaded and running
	//! the mode is LIFO, so the one who goes out first is the module
	//! that has the most chance to run in the scheduling rounding mode.
	MAJACONTAINER::MajaStack<MajaService<MajaSModule> >	main_service_stack;
	
	MAJACONTAINER::MajaList<MajaService<MajaSModule> >	main_service_list;

	mbool							gui_is_present;

	public:

	MajaCore( mvoid);

	~MajaCore( mvoid);

	mbool TestGUI( mvoid);
	
	//! when a new core is instanciated, call Query() in order to know
	//! if the core is ready to run. return mtrue in this case, otherwise mfalse.
	//! note: if under Linux, and using graphic module, check for X started
	mbool Query( mvoid);
	
	//
	//! any inheritence of this class have to define these member functions:
	//
	
	//! [ begin ] , round the whole
	mvoid* Go( mvoid *arg= mnull);
	
	//! pause the rounding, calling Go() after should continue
	mvoid* Pause( mvoid *arg= mnull);
	
	//! stop the whole, calling Go() after should restart all
	mvoid* Stop( mvoid *arg= mnull);
	
	//! reset to the default parameters ( as when first run )
	mvoid* Reset( mvoid *arg= mnull);
	
	//! exit from the whole and the core
	//! means the end of the program
	mvoid* Exit( mvoid *arg= mnull);
	
	//! Add a new service to the core/scheduler
	//! you must specify a referency to the service first, and the type of module (cf MajaScheduler.h)
	//! return mtrue if all what's OK
	//! TODO: new archi tells services are different than modules, which are services modules & core services
	mbool AddNewService( MajaService<MajaSModule>& service, __MajaModuleType type);
	
	protected:
	
	//! take care of the type of module and link it to the appropriate pointer
	//! arguments are the same as AddNewService
	//! return mtrue if could set the service correctly
	//! TODO: care of all the modules (sound...)
	mbool Care( MajaService<MajaSModule>& service, __MajaModuleType type);

	public:
	
	inline mvoid SetWindow( MajaWindow *win){ window= win;}

	inline MajaWindow& GetWindow( mvoid){ return *window;}
	
	inline MajaWinMgr* GetWindowManager( mvoid){ return th_winmgr;}
};
}
}//namespace maja
#endif
