/*
	majascheduler.h
	ce fichier est une partie de Maja - this is a part of Maja
	scheduleur temps-rel
	real-time scheduler
	
	$Id: $

	$Log: $

	begin:			Fri Jul 5 2002
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef __MAJA_SCHEDULER_H
#define __MAJA_SCHEDULER_H

#include <typeinfo>
#include <iostream>
#include "../layers/wm/majamainwindow.h"
#include "majaclock.h"
#include "majastates.h"
#include "majasmodule.h"
#include "majaevent.h"
#include "majawinmgr.h"
#include "majacoremutex.h"

namespace maja
{
/*! \namespace
	ordonnancement de la librairie Maja avec les pthread
	Maja library scheduling with pthread
*/
namespace MAJACORE
{	
	#ifndef SHOW_SCHED_INFO
	#define SHOW_SCHED_INFO
	#endif
	
	//! Which module we are using in ?
	#define __MajaModuleUse		muint
	#define __MajaModuleType	muint

	// TODO: these 2 defines above must be renamed in one: __MajaModuleCore,
	// 	 so, MMU_use... will be renamed MMU_module...
	
	//! none
	#define MMU_use_none		0x0
	
	//! graphics (rendering, drawing & minimum interactivity)
	#define MMU_use_graphics	0x1
	
	//! interactivity (not necessarly with player)
	#define MMU_use_interactivity	0x2
	
	//! sound & music
	#define MMU_use_sound		0x4
	
	//! networking
	#define MMU_use_network		0x8
	
	//! physics (for the moves, collisions, accel, gravity...)
	#define MMU_use_physics		0x10
	
	//! game management: rules, scenarios
	#define MMU_use_game		0x20
	
	//! free stuff for user (permit to add many others by this way)
	#define MMU_use_user		0x40
	
	//! the scheduler will manage all these stuffs who are available
	#define MMU_use_all_scheduling	0xFF
	
	/*!
	 * the maja scheduler
	 * schedule all the library, using modules/
	 * this class is not intended to users who should prefer
	 * use MajaCore, which is derived from it.
	 *
	 * status: safe
	 *
	 * info: may add templates for all its modules so it could behave
	 * 	so more generic as possible.
	 */
	class MajaScheduler: public MajaSModule
	{
		//! internal name
		mchar			*maja_internal_name;
		
		protected:

		MajaWindow		*window;
		
		//! the thread rendering module
		MajaSModule		*th_graphic;
		
		//! the thread interactivity module
		MajaSModule		*th_interact;

		MajaSModule		*th_game;
		
		//! anyways to others
		
		//! any user module
		MajaSModule		*th_user;
		
		//! knows which types of modules are used
		__MajaModuleUse		use_module;
		
		//! general state
		__MajaGeneralState	general_state;
		
		//! scheduling state
		__MajaSchedulingState	scheduling_state;
		
		//! type of scheduling (generally maja)
		__MajaScheduleType	scheduling_type;
		
		//! number of modules working now
		muint			modules_looping;
		
		//! the pid of this processus
		mint			pid;

		//! set it to true if that's the end of all
		mbool			maja_end_loop;

		//! the priority of the scheduler: 
		//! YOU MUST SET ALL OTHER PRIORITIES LESSER THAN THIS ONE
		//! OTHERWISE, YOU'LL PROBABLY ENCOUNTER SOME NO-RETURN HANGS !
		mint			__MAJA_SCHEDULER_PRIORITY;

		//! an idle-time module
		//! users never use it.
		MajaWinMgr		*th_winmgr;

		//! a stack of events that could occur while running
		MAJACONTAINER::MajaStack<MajaEvent>	main_event_stack;

		public:
		
		MajaScheduler( mvoid);
		
		~MajaScheduler( mvoid);	
		
		//! start scheduling with the modules and the properties set
		mint Start( mvoid *arg);

		protected:

		//! Set the rendering module
		mvoid SetRenderModule( MajaSModule *module);
		
		//! set the interactivity module
		mvoid SetInteractivityModule( MajaSModule *module);

		mvoid SetGameModule( MajaSModule *module);
		
		//! set the user module
		mvoid SetUserModule( MajaSModule *module);
		
		//! start the graphic engine module
		mint StartGraphicModule( mvoid);
		
		//! start the interactivity engine module
		mint StartInteractivityModule( mvoid);

		//! start the game module
		mint StartGameModule( mvoid);

		//! start the winmgr service thread
		mint StartWinMgrModule( mvoid);

		//! start the gamemgr service thread
		mint StartGameMgrModule( mvoid);

		public:

		//! start all the modules loaded
		mint StartModules( mvoid);
		
		//! get enter the default mode: how
		mint EnterDefaultMode( mvoid *arg= mnull);
		
		//! schedule all
		mint Schedule( mvoid *arg);
		
		protected:
		
		//! get the state of the scheduling
		//! users should don't use it
		inline __MajaSchedulingState GetSchedulingState( mvoid){ 
			return scheduling_state;
		}
		
		//! set the state of the scheduling
		//! users should don't use it
		inline mvoid SetSchedulingState( __MajaSchedulingState s){ 
			scheduling_state= s;
		}
	};
}

}//namespace maja
#endif
