/*
	majasmodule.h
	ce fichier est une partie de Maja - this is a part of Maja
	modules d'ordonnanceur
	scheduler modules

	$Id: $

	$Log: $

	begin:			Fri Jul 5 2002
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef __MAJA_SMODULE_H
#define __MAJA_SMODULE_H

#include <typeinfo>
#include "../common/majatypes.h"
#include "../layers/th/majathread.h"
#include "majaclock.h"
#include "majastates.h"
#include "../layers/th/mutex.h"
#include "../common/majatemp.h"

namespace maja
{
	/*!
	 * how we schedule Maja
	 * default value = type maja, it's the normal way we cover
	 * all the modules you created by inheritance should be of type user
	 * all the modules not linked with maja should have none set
	 */
	enum __MajaScheduleType
	{
		//! none: means all default mode with non real-time scheduling
		MST_type_none= 0,
		
		//! maja: means using majs defaults sets in real-time with RR main thread
		MST_type_maja= 1,
		
		//! user: means using what's the user want
		MST_type_user= 2
	};

	namespace MAJACORE
	{
	
	/*! 
	 * the default value of the maximum time that a module could stay in busy mode
	 * before the scheduler 'unbusies' it. this is 2 seconds default, and the value
	 * is given in seconds
	 *
	 * infos: this may not be used and won't be anymore
	 */
	#define __MAJA_DEFAULT_MAX_SEC_BUSY 	2;
	#define __MAJA_MAX_MODULE_INNER_LOOP	5;
	
	/*!
	 * A virtual class for making scheduler modules and the scheduler.
	 * a module must comply with one of the types hanged by MajaScheduler.
	 * add: any module is intended to run an engine, so it may have its member (cf below class)
	 *
	 * status: useable
	 */
	class MajaSModule:	public MAJATHREAD::MajaThread, 
				public MAJATHREAD::MajaMutex
	{
		protected:
			
		//! internal clock of the module (this is not used automatically)
		MajaClock		int_clock;
		
		//! internal name
		mchar			*maja_internal_name;
		
		//! current state (in real-time)
		__MajaModuleState	state;
		
		//! current type (may be defined once for all)
		__MajaScheduleType	type;
		
		//! any sub-module. Almost modules don't need any, so let it NULL
		MajaSModule		*sub_module;

		mdouble			max_sec_busy;

		mint			max_inner_loop;

		static mbool		first_module;

		public:

		MajaSModule( muint mutex_nb= 1);
		
		~MajaSModule( mvoid);
		
		//! set its state
		inline mvoid SetState( __MajaModuleState s){ state= s;}
		
		//! get its state
		inline __MajaModuleState GetState( mvoid){ return state;}
		
		//! check its state
		inline mint CheckState( mvoid){ 
			return state;
		}
		
		//! define its name as its class name, but you could overpass this
		mvoid SetName( mchar* name= mnull);
		
		inline mchar* GetName( mvoid){ 
			return maja_internal_name;
		}
		
		//! Set a sub-module to this module (some modules could need it to
		//! get a correct rounding. ie: the renderer.
		virtual mbool SetSubModule( MajaSModule *sm);
		
		//! get a pointer to any sub-module, if not return NULL
		inline MajaSModule* GetSubModule( mvoid){ 
			return sub_module;
		}
		
		//! get enter the default mode we have wanted, work with maja type only
		virtual mint EnterDefaultMode( mvoid *params= mnull)= 0;

		virtual mvoid* Go( mvoid *args)= 0;

		inline mdouble GetMaxBusyTime( mvoid){ 
			return max_sec_busy;
		}
	};

	/*!
	 * this is a more 'spcific' module, but not less generic as we
	 * could give it anything on it (almost its engine).
	 */
	template <class MAJALAYER>
	class MajaLayeredModule: public MajaSModule
	{
		protected:
			
		MAJALAYER	*layer;

		public:

		MajaLayeredModule( muint mutex_nb= 1): MajaSModule( mutex_nb){
			;
		}

		inline mvoid SetLayer( MAJALAYER *l){ layer= l;}

		inline MAJALAYER& GetLayer(){ return layer;}
	};
	
	}//namespace MAJACORE
}//namespace maja
#endif
