/*
	majasys.h
	ce fichier est une partie de Maja - this is a part of Maja
	gestion principale du systme
	main system management

	$Id: $

	$Log: $

	begin:		Sep 2000
	copyright:	(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:		jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef __MAJA_SYS_H
#define __MAJA_SYS_H

#include <string>
#include <iostream>
#include <typeinfo>
#include "majaclock.h"
#include "majacore.h"

#include "modules/gamemodule.h"
#include "modules/graphicmodule.h"
#include "modules/intermodule.h"
#include "../engines/scenery/majasengine.h"
#include "../engines/game/majagameengine.h"
#include "../layers/wm/majamainwindow.h"

#ifndef WIN32
#include <unistd.h>
#include <X11/Xlib.h>
#include "../layers/wm/majaxwindow.h"
#else
#include <windows.h>
#include "../layers/wm/majawwindow.h"
#endif

#include "../engines/graphic/majatextures.h" // tools/tex 

#include "../engines/scenery/majaobject.h" // oups...
#include "../engines/scenery/majacamera.h" // ouille...
#include "../engines/graphic/graphext.h"

#include "majamemmgr.h"

#include "../engines/menu/majamenu.h"

#define __MAJA_VERSION			"0.0.29-rev3.0"
#define __MAJA_COPYRIGHT		"(c) 2000,2001,2002,2003 by Jean-Dominique Frattini."
#define __MAJA_WARNINFO_1		"This is a free software, however all the text"
#define __MAJA_WARNINFO_2		"above and until here must remain readable."

//! THIS IS JUST TRUE FOR IMPLEMENTION AND IS NOT REALLY AT THIS POINT !!!
#define MAJA_SYS_VERSION_0_1		1

namespace maja
{
	/*!
	 * this is the default game engine maja will use
	 * if user doesn't modify it. When using all
	 * default maja's component, you will use it hidden.
	 */
	typedef MAJAGAME::MajaGameEngine
	<
		MajaObject,
		MajaIEngine,
		MAJASCENERY::MajaWorldEngine,
		MAJAGRAPHIC::MajaDisplay,
		MajaMenu<MajaDrawable,MajaMenuItem,MajaItemPolicy>,
		MajaTextures
	>
	MajaDefaultGameEngine;

	/*!
		Warning & Errors management
		
		cf majadef.h for the values that level_pb can take
	*/
	mint __maja_exit( const mchar *exit_pb, const muint level_pb);

	namespace MAJACORE
	{
	
	/*!
	 * system management: this is generally the first
	 * user class, and may prefer its interface (see
	 * class below).
	 *
	 * infos: changes will almost depend on its interface's
	 * 	new needs, and other core component needing an
	 * 	eveolution.
	 *
	 * state: safe.
	 */
	template <class GE=MajaDefaultGameEngine>
	class MajaSystem
	{
		protected:

		/*!
		 * process management
		 */
		struct _MajaProcess
		{
			//! link to the argv main argument
			mchar				**_argv;
	
			//! count _argv
			mint				_argc;
	
			//! id of the process: if 0, it's root
			mint				_id;
	
			//! id of the effective process: if 0, it's CAP_SYS_NICE ?
			mint				_eid;
	
			//! id of the group
			mint				_gid;

			_MajaProcess( mvoid){
				_argv= mnull;
				_argc= 0;
				_id=   -1;
				_eid=  -1;
				_gid=  -1;
			}
			
		} 					process_info;

		/*!
		 * computer infos
		 */
		struct _MajaComputer
		{
			//! speed in MHz of the CPU
			mdouble				cpu_speed;

			//! total ram size in MB
			mdouble				ram_size;

			//! free ram size in MB
			mdouble				free_ram_size;

			//! total swap size in MB
			mdouble				swap_size;

			//! free swap size in MB
			mdouble				free_swap_size;

			//! total AGP aperture size in MB
			mdouble				agp_aperture_size;

			_MajaComputer( mvoid){
				cpu_speed=		0;
				ram_size=		0;
				free_ram_size=		0;
				swap_size=		0;
				free_swap_size=		0;
				agp_aperture_size=	0;
			}
		} 							computer_info;
		
		#ifndef WIN32
		Display							*sys_display;
		#endif

		MajaWindow						*window;

		MAJACORE::MajaCore&					sys_core;

		MAJAGRAPHIC::MajaRendererModule				*maja_s_render;
		
		/*MAJAINTERACTIVE::*/MajaInteractive			*maja_s_inter;

		MajaSGamer<GE,MajaIEngine,MAJAGRAPHIC::MajaDisplay>	*maja_s_game;

		MAJAGRAPHIC::MajaDisplay 				*maja_disp;

		MajaTextures						maja_tex;

		GE							*game_engine;

		MajaGExt						*gl_ext;
	
		MajaMemoryManager<mfloat>				mem_mgr;

		std::string						maja_copyright;

		mbool							use_console;

		MajaSystem( mvoid);

		MajaSystem( mint argc, mchar **argv);

		MajaSystem( const MajaSystem& sys);

		~MajaSystem( mvoid);

		mbool ProcessInit( mint argc, mchar **argv);

		mbool ProcessExit( mvoid);

		mbool StartSubSystems( mvoid);

		mbool StopSubSystems( mvoid);

		mbool Check( mvoid);
		
		mvoid Go( mvoid);

		mvoid SysInit( mvoid);

		mvoid SysExit( mvoid);

		mbool ComputerInfoInit( mvoid);

		mvoid CheckArguments( mint argc, mchar **argv);

		mbool StartGameEngine( mvoid);

		//! will stop the game engine
		//!
		//! status: not implemented
		mbool StopGameEngine( mvoid);
	};

	}// namespace MAJACORE

	/*!
	 * MajaSystemInterface is the interface to the system with the user.
	 * This is the main class a user has to use before the game and graphic
	 * components. With it, you'll get then the game engine, and for now, the
	 * rendering engine (cf below).
	 * 
	 * status: in development, quiete useable.
	 *
	 * todo:* need to get the core (for advanced uses)
	 * 	* need to start the scheduler on a defered
	 * 	thread if needed.
	 * 	* need to start a console if needed.
	 *	* need more talk with users.
	 */
	template
	<
		class GE= MajaDefaultGameEngine
	>
	class MajaSystemInterface: protected MAJACORE::MajaSystem<GE>
	{
		public:

		//! create and init the system and the game engine.
		//! after this call the game engine is available to be gotten.
		MajaSystemInterface( mint argc, mchar **argv): MAJACORE::MajaSystem<GE>( argc, argv)
		{
			if( !StartGameEngine())
				throw std::exception();
		}

		~MajaSystemInterface( mvoid)
		{
			std::cerr << std::endl << "MajaSystem(Interface): ended.";
		}

		//! after creating the system interface, you could get the game
		//! engine in order to make some things with it (as adding an
		//! object, a camera ...).
		GE& GetGameEngine( mvoid)
		{
			return *game_engine;
		}

		//! when calling to this function, you call Maja to start its core, and
		//! then whatever needed by she. After that, you will see what you put
		//! in the game engine (and for now, the rendering engine too).
		mbool Start( mvoid);

		mbool Stop( mvoid);
	};
}//namespace maja
#endif
