/*
	majawingr.h
	ce fichier est une partie de Maja - this is a part of Maja
	gestionnaire de fentre temps-rel et limit
	limited & real-time window manager
	
	$Id: $

	$Log: $

	begin:			Fri Jul 5 2002
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef MAJA_WINMGR__H
#define MAJA_WINMGR__H

#include <typeinfo>
#include <iostream>
#include "majaclock.h"
#include "majastates.h"
#include "majasmodule.h"
#include "majaevent.h"
#include "../layers/th/mutex.h"
#include "majacoremutex.h"
#include "../layers/wm/majamainwindow.h"
#include "../tools/containers/majastack.h"

namespace maja
{
	
namespace MAJACORE
{

#ifdef SHOW_WINMGR_INFO
#undef SHOW_WINMGR_INFO
#endif

/*!
 * window manager module.
 * this is managed internally by the scheduler, so don't use it.
 * this is a window events manager that could hangs graphic and
 * interactivity modules (all what's related to a window).
 *
 * infos: need a more study.
 *
 * status: safe for internal use.
 */
class MajaWinMgr: public MajaSModule
{
	protected:
	
	//! the window to be managed
	MajaWindow					*window;

	//! a memorized stack of events
	//! user should not modify its content
	MAJACONTAINER::MajaStack<_MajaWidgetEvent> 	event_stack;
	
	MAJACONTAINER::MajaStack<_MajaWidgetEvent> 	key_event_stack;

	_MajaWidgetEvent				*w_event;

	muint						events_nb;

	//! know if the event was treated or not by its matching module
	//! only internal use. But if you create your own module using
	//! window event such as resize, redraw, keyboard entries, mouse
	//! moves, buttons..., you'll have to set this in accordance.
	//! refers to the matching module files.
	mbool						event_treated;

	//! this is false all this thread's life
	mbool						close_thread;

	//! the thread rendering module
	MajaSModule					*th_graphic;
		
	//! the thread interactivity module
	MajaSModule					*th_interact;

	public:

	//! the only way to create a window manager is to spcify
	//! a window by referency. But, users should not use it
	//! directly as it is internally created by the core-scheduler.
	MajaWinMgr( mvoid):	MajaSModule()
	{
		close_thread= 		mfalse;
		maja_schedule_algo= 	SCHED_FIFO;
		event_treated= 		mfalse;
		w_event= 		mnull;
		events_nb=		0;
		window= 		mnull;
	}

	//! rounding this thread
	mvoid* Go( mvoid* arg);

	//! is called in order to create the thread in the scheduling
	mint EnterDefaultMode( mvoid *params= mnull);	

	inline MAJACONTAINER::MajaStack<_MajaWidgetEvent>& GetEventStack( mvoid){
		return event_stack;
	}

	inline MAJACONTAINER::MajaStack<_MajaWidgetEvent>& GetKeyEventStack( mvoid){ 
		return key_event_stack;
	}

	inline mvoid PushCurrentEventInStack( mvoid){
		_MajaWidgetEvent *tmp= new _MajaWidgetEvent( window->GetEvent(), "no_name");
		if( !tmp){
			std::cerr << std::endl << "could not push event !";
			throw std::exception();
		}
		event_stack.Push( tmp);
	}

	inline mvoid PopCurrentEventInStack( mvoid){
		_MajaWidgetEvent *tmp= event_stack.Peek();
		if( tmp){
			event_stack.Pop( tmp);
			delete tmp;
		}
		else
			std::cerr << std::endl << "Error, can't pop empty data !";
	}

	inline mvoid PushCurrentEventInKeyStack( mvoid){
		_MajaWidgetEvent *tmp= new _MajaWidgetEvent( window->GetEvent(), "no_name");
		if( !tmp)
			return; // error
		key_event_stack.Push( tmp);
	}

	inline mvoid PopCurrentEventInKeyStack( mvoid){
		_MajaWidgetEvent *tmp= key_event_stack.Peek();
		if( tmp){
			key_event_stack.Pop( tmp);
			delete tmp;
		}
		else
			std::cerr << std::endl << "Error, can't pop empty data !";
	}

	//! set if the event was treated or not,
	//! if it was ( ev= true), we decrement the events counter: events_nb
	inline mvoid SetEventTreated( mbool ev){
		if( ev)
			if( events_nb>=0)
				events_nb--;
			else
				std::cerr << std::endl << "MajaWinMgr: error: has more treated events than events.";
		else
			;
	
		event_treated= ev;		
	}

	inline muint GetEventsNumber( mvoid){ 
		return events_nb;
	}

	protected:

	inline mvoid SetEventsNumber( muint i){ 
		events_nb= i;
	}

	public:

	inline mbool GetEventTreated( mvoid){ 
		return event_treated;
	}

	inline mvoid SetGraphicModule( MajaSModule *module){ 
		th_graphic= module;
	}

	inline mvoid SetInteractivityModule( MajaSModule *module){ 
		th_interact= module;
	}

	inline mvoid SetWindow( MajaWindow *w){
		window= w;
		window->SetEventStack( &GetEventStack());
		window->SetKeyEventStack( &GetKeyEventStack());
	}

};
}
}
#endif

