/*
	graphicmodule.h
	ce fichier est une partie de Maja - this is a part of Maja
	le module de jeux
	the game module
	
	$Id: $

	$Log: $

	begin:			Fri Jul 5 2002
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef MAJA_GAMEMODULE__H
#define MAJA_GAMEMODULE__H

#include "../majascheduler.h"
#include "../majasmodule.h"
#include "../majastates.h"

#include "../../common/majacom.h"

namespace maja
{
namespace MAJACORE
{

template
<
	class GameEngine,
	class InterEngine,
	class RenderingEngine // which is MajaDisplay
>
class MajaSGamer: public MajaSModule
{
	protected:

	InterEngine		*i_engine;
	GameEngine		*g_engine;
	RenderingEngine		*r_engine;
	
	public:
		
	MajaSGamer():	MajaSModule()
	{
		maja_schedule_algo= SCHED_FIFO;
		g_engine= new GameEngine();
		if( !g_engine)
			;
		i_engine= mnull;
		r_engine= mnull;
	}

	~MajaSGamer( mvoid){
		delete g_engine;
	}

	mvoid SetIEngine( InterEngine *e){
		i_engine= e;
		g_engine->SetInteractiveEngine( *e);
	}

	mvoid SetREngine( RenderingEngine *r){
		r_engine= r;
	}

	// well, bad, but necessary since no other way...
	GameEngine& GetGameEngine( mvoid){ 
		return *g_engine;
	}
	
	/*!
	 * get enter the thread execution
	 * you could give here parameters to give to the thread
	 */
	mint EnterDefaultMode( mvoid *params= mnull){

		sched_param sp;

		GetPriority();
		
		pthread_attr_init( &attributes);
		pthread_attr_setschedpolicy( &attributes, maja_schedule_algo);
		pthread_attr_getschedparam( &attributes, &sp);		
		sp.sched_priority= priority;
		pthread_attr_setschedparam( &attributes, &sp);

		if( !Create( mnull)){
			std::cerr << std::endl << "could not create MajaSGamer module thread";
			if( errno==EPERM)
				std::cerr << std::endl << "you may not be seteuid=0";
			return 0;
		}

		return 1;
	}

	mvoid* Go( mvoid* args)
	{
		mint		last_cancel_state=	0;
		mcom		ans= mcom_nothing;

		pthread_setcancelstate( PTHREAD_CANCEL_DISABLE, &last_cancel_state);

		Lock();
		if( GetState()!=MMS_state_start){
			maja_core_mutex.stderr_mutex.Lock();
			std::cerr << std::endl << "MajaSGamer: wanted to go, but not started ! exiting...";
			maja_core_mutex.stderr_mutex.Unlock();
			SetState( MMS_state_off);
			Unlock();

			return mnull;
		}
		if( !g_engine || !i_engine || !r_engine){
			std::cerr << std::endl << "MajaSGamer: one engine or more is not set ! exiting...";
			Unlock();
			
			return mnull;
		}
		Unlock();

		maja_core_mutex.stderr_mutex.Lock();
		std::cerr << std::endl << "MajaSGamer: waiting for scheduler order...";
		maja_core_mutex.stderr_mutex.Unlock();
		
		Lock();
		while( state!=MMS_state_on)
			CondWait();
		Unlock();

		maja_core_mutex.stderr_mutex.Lock();
		std::cerr << std::endl << "MajaSGamer: rounding...";
		maja_core_mutex.stderr_mutex.Unlock();
		while( 1){
			Lock();
			if( GetState()==MMS_state_off){
				Unlock();
				break;
			}
			SetState( MMS_state_busy);
//			i_engine->Lock();
			r_engine->Lock();
			ans= g_engine->Run();
			r_engine->Unlock();
//			i_engine->Unlock();
			
			if( ans&mcom_exit_program){
				maja_core_mutex.stderr_mutex.Lock();
				std::cerr << std::endl << "MajaSGamer: exiting program...";
				maja_core_mutex.stderr_mutex.Unlock();
				SetState( MMS_state_off);
				Unlock();
				return (void*)"exit\0";
			}
			
			SetState( MMS_state_on);
			Unlock();
		}
		
		return mnull;
	}
	
};

}
}
#endif
