/*
	intermodule.h
	ce fichier est une partie de Maja - this is a part of Maja
	module service pour l'interaction
	service module for interactivity
	
	$Id: $

	$Log: $

	begin:			Fri Jul 5 2002
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

/*! \file
	Ordonnancement des diffrents modules que peut comporter Maja
	Scheduling of the different modules that can stand Maja
*/

#ifndef __INTERMODULE_H
#define __INTERMODULE_H

#include <typeinfo>
#include "../../core/majascheduler.h"
#include "../../core/majasmodule.h"
#include "../../layers/th/mutex.h"
//#include "../../engines/interactivity/maja.h"
#include "../../layers/wm/majamainwindow.h"
#include "../../engines/interactivity/majaiengine.h"

namespace maja
{
	
#ifdef SHOW_INTER_INFO
#undef SHOW_INTER_INFO
#endif

using namespace MAJACORE;

template <class MAJALAYER>
class MajaSInteractivity: public MajaLayeredModule<MAJALAYER>
{
	protected:
	
	MajaIEngine		*i_engine;
	MajaWindow		*window;

	public:
	
	MajaSInteractivity( mvoid): MajaLayeredModule<MAJALAYER>(){
		window= mnull;
		i_engine= new MajaIEngine();
		if( !i_engine){
			std::cerr << std::endl << "memory full";
			throw std::exception();
		}
	}

	~MajaSInteractivity( mvoid){
		if( i_engine)
			delete i_engine;
	}
	
	mvoid SetWindow( MajaWindow *win){ 
		if( !win){
			std::cerr << std::endl << "MajaSInteractivity: setting window with null pointer...";
		}
		window= win;
	}

	MajaIEngine* GetEngine( mvoid){ 
		return i_engine;
	}

	mint EnterDefaultMode( mvoid *params)
	{
		sched_param sp;

		if( !window){
			std::cerr << std::endl << "MajaSInteractivity: no window was specified !";
			return -1;
		}

		GetPriority();
		
		pthread_attr_init( &attributes);
		pthread_attr_setschedpolicy( &attributes, maja_schedule_algo);
		pthread_attr_getschedparam( &attributes, &sp);		
		sp.sched_priority= priority;
		pthread_attr_setschedparam( &attributes, &sp);

		if( !Create( mnull)){
			std::cerr << std::endl << "could not create MajaSInteractivity module thread";
			if( errno==EPERM)
				std::cerr << std::endl << "you may not be seteuid=0";
			return 0;
		}

		return 1;
	}

	mvoid* Go( mvoid* args)
	{
		mbool			done=			mfalse;
		mint			last_cancel_state=	0;
		__MajaWindowState	win_state=		MWS_state_dead;
		_MajaWidgetEvent	*widget_event=		mnull;

		pthread_setcancelstate( PTHREAD_CANCEL_DISABLE, &last_cancel_state);

		Lock();
		if( GetState()!=MMS_state_start){
			maja_core_mutex.stderr_mutex.Lock();
			std::cerr << std::endl << "MajaSInteractivity: wanted to go, but not started ! exiting...";
			maja_core_mutex.stderr_mutex.Unlock();
			SetState( MMS_state_off);
			Unlock();

			return mnull;
		}
		Unlock();

		maja_core_mutex.stderr_mutex.Lock();	
		std::cerr << std::endl << "MajaSInteractivity: waiting for scheduler order...";
		maja_core_mutex.stderr_mutex.Unlock();
		
		Lock();
		while( state!=MMS_state_on)
			CondWait();
		Unlock();

		maja_core_mutex.stderr_mutex.Lock();
		std::cerr << std::endl << "MajaSInteractivity: rounding...";
		maja_core_mutex.stderr_mutex.Unlock();

		while( !done){		
			Lock();
			switch( GetState()){
				case MMS_state_pause:
					#ifdef SHOW_INTER_INFO
					maja_core_mutex.stderr_mutex.Lock();
					std::cerr << std::endl << "MajaSInteractivity: paused.";
					maja_core_mutex.stderr_mutex.Unlock();
					#endif
					Unlock();
					usleep( 10000);
					continue;
				case MMS_state_cancel:
				case MMS_state_off:
					done= true;
					#ifdef SHOW_INTER_INFO
					maja_core_mutex.stderr_mutex.Lock();
					std::cerr << std::endl << "MajaSInteractivity: state is dead.";
					maja_core_mutex.stderr_mutex.Unlock();
					#endif
					Unlock();
					pthread_exit( mnull);
				default:
					;
			}
			SetState( MMS_state_busy);
			Unlock();

			window->event_mutex.Lock();
			if( window->GetKeyEventStack()->GetSize()>=1){
				#ifdef SHOW_INTER_INFO
				maja_core_mutex.stderr_mutex.Lock();
				std::cerr << std::endl << "MajaSInteractivity: checking event";
				maja_core_mutex.stderr_mutex.Unlock();
				#endif
				widget_event= window->GetKeyEventStack()->Peek();
				
				// make the engine working with the event:
				// il faudrait que l'on retourne un autre type qui connait si
				// on doit quitter, faire du plein-cran, ou si une touche n'est
				// plus appuy (pour la poper de la liste par exemple).
				i_engine->Lock();
				win_state= i_engine->CheckEvent( widget_event->_e);
				i_engine->Unlock();
				
				if( win_state!= MWS_state_need_close)
					window->GetKeyEventStack()->Pop( widget_event);
				else{
					maja_core_mutex.stderr_mutex.Lock();
					std::cerr << std::endl << "MajaSInteractivity: user wants to close the app...";
					maja_core_mutex.stderr_mutex.Unlock();
					
					window->Lock();
					window->SetState( MWS_state_need_close);
					window->Unlock();
					
					Lock();
					SetState( MMS_state_off);
					Unlock();
					window->event_mutex.Unlock();
					break;
				}
			}
			window->event_mutex.Unlock();
	
			Lock();
			SetState( MMS_state_on);		
			Unlock();
		} // while

		maja_core_mutex.stderr_mutex.Lock();
		std::cerr << std::endl << "exiting from MajaSInteractivity::Go()";
		maja_core_mutex.stderr_mutex.Unlock();

		// TODO: emmit a X event here to not block the window manager, so that this last could
		// continue running and then ends correctly. The problem is that this thread is the first
		// to end when user wants to exit, and window manager doesn't ends as it only waits for
		// new window events as it's to do. This wait is a call to a blocking function.
	
		pthread_exit( mnull);
	}
};

typedef MajaSInteractivity<MajaWinMgr> MajaInteractive;
}
#endif
