/*
	majagameloader.h
	ce fichier est une partie de Maja - this is a part of Maja
	le chargement moteur de jeux
	the loading game engine

	$Id: $

	$Log: $

	begin:			nov 2003
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef MAJA_GAMELOADER__H
#define MAJA_GAMELOADER__H

#include "majagamemain.h"
#include <list>

namespace maja
{
namespace MAJAGAME
{	
	/*!
	 * game loader: is intended to get all the to-be-loading
	 * objects we want to load together.
	 * O is any object that has Load() and Unload() member functions.
	 * R is the rendering engine (MajaDisplay)
	 * T is the texture loader (MajaTextures). T is no more used until
	 * we could properly load textures from one thread and use them in
	 * other threads freely.
	 * 
	 * info: you could put any MajaObject (MO), but later, these
	 * 	won't have any Loading capability as it will be defered to
	 * 	another enitity which will link it with the MO and/or the
	 * 	renderer.
	 * 	As MajaTextures is not a MajaObject, we need a third type...
	 */
	template
	<
		class O,
		class R,
		class T
	>
	class MajaGameLoader: virtual public MajaGameMain<O>
	{
		protected:

		MAJACONTAINER::MajaList<O>	loading_list;
		
		std::list<MajaObjectType>	type_list;

		R				*r_engine;

		T				*texture_loader;
		
		public:

		MajaGameLoader( mvoid):	MajaGameMain<O>()
		{
			r_engine= mnull;
			texture_loader= mnull;
		}
	
		inline mvoid Load( mvoid){
			Run();
		}

		inline mvoid Unload(){
			Unrun();
		}
		
		//! here running the loading, but we don't load all as there is a
		//! problem between threads and opengl textures scopes. So textures
		//! must be loaded before, or later in the rendering thread.
		mcom Run( mvoid){
			MAJACONTAINER::MajaWidget<O> *tmp= loading_list.GetFirst();
			
			if( !tmp)
				return 0;
			
			do{
				tmp->GetObject()->Load();

				// now as the object is loaded, we could put it in the
				// rendering engine (MajaDisplay) in order it to be drawn.
				r_engine->AddObject( *tmp->GetObject());
				
				tmp=tmp->GetNext();
			}while( tmp);

			//!todo: could not load properly here since textures seems
			//! to be single-thread scope.
		/*	r_engine->GetDrawable().Enter( graphic_context);
			texture_loader->LoadAll();
			r_engine->GetDrawable().Leave();
		*/
			return 1;
		}

		mcom Unrun( mvoid){
			return 1;
		}

		inline mvoid AddLoadElement( O& e, MajaObjectType t= object_type_object){
			loading_list.AddObject( &e);
			type_list.push_back( t);
		}

		protected:
		
		inline mvoid SetTextureLoader( T& t){
			texture_loader= &t;
		}
	};
}
}
#endif
