/*
	majagamemain.h
	ce fichier est une partie de Maja - this is a part of Maja
	partie du moteur de jeux
	part of the game engine

	$Id: $

	$Log: $

	begin:			nov 2002
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef MAJA_GAMEMAIN__H
#define MAJA_GAMEMAIN__H

#include "../../tools/containers/majalist.h"
#include "../../common/majatypes.h"
#include "../../common/majacom.h"

#ifdef WIN32
#include <windows.h>
#else
#include <GL/glx.h>
#endif

namespace maja
{
	/*!
	 * the specification type of any MO (maybe in common ?)
	 */
	typedef muint	 	MajaObjectType;

	//! a simple object (MajaObject)
	const MajaObjectType	object_type_object	= 0x1;

	//! a simple terrain (MajaPath)
	const MajaObjectType	object_type_terrain	= 0x2;

	//! a simple obstacle (MajaScene)
	const MajaObjectType	object_type_obstacle	= 0x4;

	//! a simple light (MajaLight)
	const MajaObjectType	object_type_light	= 0x8;

	//! a simple camera (MajaCamera)
	const MajaObjectType	object_type_camera	= 0x10;

	//! a simple actor (MajaActor)
	const MajaObjectType	object_type_actor	= 0x20;

	//! as a weapon, a key, anything useable
	const MajaObjectType	object_type_useable	= 0x40;

	//! as a proxi-door...
	const MajaObjectType	object_type_interactive = 0x80;

	//! a simple player
	const MajaObjectType	object_type_player	= 0x100;		// player has to remain the maximum value
	
	namespace MAJAGAME
	{
		#ifdef WIN32
		typedef HWC MC;
		#else
		typedef GLXContext MC;
		#endif

		/*!
		 * main virtual class for gaming
		 * O is generally a MajaObject
		 * C is the OpenGL context, never redefined it yourself
		 *
		 * infos: not user class
		 * status: safe
		 */
		template
		<	
			class O,
			class C= MC
		>
		class MajaGameMain
		{
			protected:
			
			MAJACONTAINER::MajaList<O>	moment;

			C				graphic_context;
		
			MajaGameMain( mvoid){
				graphic_context= 0;
			}
		
			virtual ~MajaGameMain( mvoid){
				;
			}

			virtual mcom Run( mvoid)= 0;
		
			virtual mcom Unrun( mvoid)= 0;

			virtual mvoid Add( O& m){ 
				moment.AddObject( &m);
			}

			virtual mvoid SetList( MAJACONTAINER::MajaList<O>& list){
				moment= list;
			}
		};
	}
}
#endif
