/*
	majagameengine.h
	ce fichier est une partie de Maja - this is a part of Maja
	le monde du moteur de jeux
	the world of the game engine

	$Id: $

	$Log: $

	begin:			nov 2003
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef MAJA_GAMEWORLD__H
#define MAJA_GAMEWORLD__H

#include "majagamemain.h"
#include "../../tools/maths/majavector.h"

namespace maja
{
namespace MAJAGAME
{
	/*!
	 * game world: is intended to get
	 * all the MO to be in the game
	 * M is any class in scenary (any MajaObject)
	 * I is an interactivity engine (MajaIEngine)
	 * S is the scenery engine (MajaSEngine)
	 */
	template
	<
		class O,
		class I,
		class S
	>
	class MajaGameWorld: virtual 	public MajaGameMain<O>,
					public S
	{
		protected:

		// should I use lists or trees for all containers ?
		// trees seem a best way as they are hierarchical,
		// and thus, we could manage them more precisely.
		
		MAJACONTAINER::MajaList<O>	living_list;
		
		MAJACONTAINER::MajaList<O>	dead_list;
		
		MAJACONTAINER::MajaList<O>	tool_list;

		I				*i_engine;

		MAJACONTAINER::MajaList<O>	controled_object;

		mint				*current_key;
		
		MAJAMATHS::MajaVector		*current_mouse;

		mbool				defined;
		
		public:

		MajaGameWorld( mvoid):	MajaGameMain<O>()
		{
			i_engine= mnull;
			current_key= mnull;
			current_mouse= mnull;
			defined= mfalse;
		}

		~MajaGameWorld( mvoid){
			;
		}

		mcom PreRun( mvoid){
			std::cerr << std::endl << "MajaGameWorld: configuring world...";
			mbool s= S::Configure();
			std::cerr << s?" done.":" failed.";

			return s&mcom_goto_next;
		}

		// here is the running of the world and all it objects in it.
		// we check the mouse here in order to make interactivity goes.
		// later will have here the scenery running.
		mcom Run( mvoid){
			static MAJACONTAINER::MajaWidget<O> *tmp= mnull;
			static mint *i= i_engine->GetCurrentKey();

			if( !defined){
				current_mouse=i_engine->GetMouse();
				if( !i || !current_mouse){
					std::cerr << std::endl << "MajaGameWorld: ERROR: could not link with interactivity.";
					return 0;
				}
				defined= mtrue;
			}
			
			if( tmp= controled_object.GetFirst()){
				do{
					S::Set( i, current_mouse);
					tmp=tmp->GetNext();
				}while( tmp);
			}
			S::Run();

			return 1;
		}

		mcom PosRun( mvoid){
			;
		}

		mcom Unrun( mvoid){
			return 1;
		}

		mvoid AddWorldEntity( O& e, MajaObjectType type){
			mbool added= mfalse;
			
			if( type&object_type_player){
				std::cerr << std::endl << "MajaGameWorld: adding a player...";
				living_list.AddObject( &e);
				S::AddToActorList( e, mtrue);
				controled_object.AddObject( &e);
				added=mtrue;
			}
			else if( type&object_type_actor && !added){
				living_list.AddObject( &e);
				S::AddToActorList( e);
				added=mtrue;
			}
			// this is not else as we could combine here:
			if( type&object_type_camera && !added){
				tool_list.AddObject( &e);
				//S::AddToActorList( e);
				added=mtrue;
			}
			else if( type&object_type_light && !added){
				tool_list.AddObject( &e);
				//S::AddToLightList( e);
				added=mtrue;
			}
			else if( type&object_type_obstacle && !added){
				dead_list.AddObject( &e);
				S::AddToSceneList( e);
				added=mtrue;
			}
			else if( type&object_type_terrain && !added){
				dead_list.AddObject( &e);
				S::AddToPathList( e);
				added=mtrue;
			}			
			else if( type&object_type_object && !added){
				dead_list.AddObject( &e);
				S::AddToObjectList( e);
			}
		}
	};
}
}
#endif
