/*
	gbase.h
	ce fichier est une partie de Maja - this is a part of Maja
	dfinitions de base pour la partie graphique
	basis definitions for the graphic part

	$Id: $

	$Log: $

	begin:			Wed Jun 19 2002
	copyright:		(C) 2002 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/*****************************************************************************************************
 *																			*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.									*
 *	Contactez-nous pour toute tentative d'utilisation commerciale, svp.				*
 *	-																		*
 *	This program is free software; you can redistribute it and/or modify				*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or				*
 *	(at your option) any later version.											*
 *	Contact us for any commercial use attempt, please.							*
 *																			*
 *****************************************************************************************************/

#ifndef __MAJA_G_BASE
#define __MAJA_G_BASE

#ifdef WIN32
#include <windows.h>
#endif
#include <GL/gl.h>
#include <GL/glu.h>
#include <stdlib.h>
#include <iostream>
#include "graphext.h"

namespace maja
{
namespace MAJAGRAPHIC{

/*!
	any graphic part that contains info about how to render as material or vertex data
	will use this type in order to Maja knows how to render it in the best way or in the
	way the user decided.	
*/
enum __maja_graphic_technic
{
	//! if renderer method is not in the current list, choose this one
	//! avoid using it !
	MGT_none= 0,
	
	//! if you want to use simple display list, choose this one
	//! for now, if you want to use display lists, you'd better take a look
	//! at majadisplay.h, and all MajaObjects. It's not useable here for now.
	MGT_dlist,
	
	//! if you want to use arrays inner display lists, it's this one
	//! this is not implemented yet
	MGT_dlist_array,
	
	//! if you want drawing with arrays, it's this one
	//! this is simple array drawing method that could speed up the renderer
	MGT_array,
	
	//! use it if you have OpenGL Compiled Vertex Arrays capabilities
	//! note that all data stored into CVA could not be transformed after.
	//! so, use it for your static scene: road, skyscrapers...
	//! it may be safe using it
	MGT_cva_array,
	
	//! use it if you have OpenGL NVIDIA Vertex Array Range capabilities
	//! it is safe to use it
	MGT_nv_var_array,
	
	//! use it if you have NVIDIA vertex program capabilities
	//! this is not implemented yet, and not planned too...
	MGT_nv_vp_array
};

/*!
	basis class for graphism
	any classes which derive from this one (MajaGData, MajaGraphicEngine...)
	could use the capacities of choosing the rendering technic to use.
	So, any part of the graphic tree, branches or leaves could have different
	technic. Then, beware of not blending upside down !
	You should not use this class.
	
	status: useable
*/
class MajaGEntity
{
	protected:
	
	//! the technic used to render
	__maja_graphic_technic		g_technic;
	
	//! the extension support (neaded for some technics)
	static MajaGExt			*gl_ext;
	
	public:
	MajaGEntity( void);
	
	virtual ~MajaGEntity( void){
		;
	}
	
	//! Set the graphic technic of the object
	inline void SetGraphicTechnic( __maja_graphic_technic technic){ g_technic= technic;}
	
	//! Get the graphic technic
	inline __maja_graphic_technic GetGraphicTechnic( void){ return g_technic;}
};
}// namespace MAJAGRAPHIC
}
#endif
