/*
	gdatatree.h
	ce fichier est une partie de Maja - this is a part of Maja
	donnes graphiques, feuilles de l'arbre graphique
	graphic data, leaves of the graphic tree
	
	$Id: $

	$Log: $

	begin:			Wed Jun 19 2002
	copyright:		(C) 2002 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	Contactez-nous pour toute tentative d'utilisation commerciale, svp.			*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *	Contact us for any commercial use attempt, please.					*
 *												*
 ************************************************************************************************/

#ifndef __GDATA_TREE_H
#define __GDATA_TREE_H

#include "graphext.h"
#include "gbase.h"
#include "majagdata.h"
#include "../../tools/maths/majamaths.h"

namespace maja
{
namespace MAJAGRAPHIC{

using MAJAMATHS::MajaVector;

/*!
	data on a tree: the leaves of this one
	data are the last graphic informations about one thing to render
	you then have vertex, normals, colors, texels here
	
	status: in development
*/
class MajaGDataTree: public MajaGData
{
	public:
	//! this member could be left to NULL if always use within MajaGParam
	MajaGDataTree *parent;
	
	//! position of what's drawn
	MajaVector *position;
	
	//! how it's orientated
	MajaVector *orientation;
	
	//! begin index of vertex array
	GLuint start_index;
	
	//! if the data in (when derived) are static for a while
	//! then set it to true. it includes that you don't apply
	//! OpenGL transformations on them
	bool static_data;
	
	MajaGDataTree( void):MajaGData()
	{
		parent= NULL;
		position= NULL;
		orientation= NULL;
		start_index=0;
		static_data= false;
	}
	
	MajaGDataTree( MajaGDataTree& tree):MajaGData( (MajaGData&)(tree) )
	{
		parent= tree.parent;
		position= tree.position;
		orientation= tree.orientation;
		static_data= tree.static_data;
	}
	
	~MajaGDataTree( void){;}

	//! only one parent !
	//! this is not used for now
	inline void SetParent( MajaGDataTree& p){ parent= &p;}
	
	//! unlink with parent (or set to NULL with SetParent)
	inline void UnsetParent( void){ parent= NULL;}
	
	//! get the parent
	inline MajaGDataTree* GetParent( void){ return parent;}

	//! render the part data
	// info: we could really perform it by templates manipulations...
	// 	but it will need some gymnastic (need a new conception too)
	inline GLuint Render( void)
	{
		if( g_technic>= MGT_cva_array){
			// this may be useful sometimes, when triangles are not set in a follow.
			//ext.DrawRangeElements( GL_TRIANGLES, start_index, faces_nb, 3*faces_nb, GL_UNSIGNED_INT, index);
			gl_ext->DrawArrays( GL_TRIANGLES, start_index, 3*faces_nb);
		}
		else if( g_technic== MGT_array){
			glDrawArrays( GL_TRIANGLES, start_index, 3*faces_nb);
		}
		else if( g_technic>= MGT_dlist)
			;//glCallList( this_display_list);
		else{
			std::cerr << std::endl << "MajaGDataTree: No valid graphic technic is set";
			return 0;
		}
		
		return 1;
	}
	
	//! Check the integrity of the data
	inline bool Check( void)
	{
		if( !parent){ std::cerr << std::endl << "no parent, just a warning";}
		if( !vertices){ std::cerr << std::endl << "no vertices, check for other data too"; return false;}
		
		return true;
	}

	//! here, just link the pointers, but in basis class, it's allocated, and destroyed in destructor.
	//! so, if you use this in place of CreateIndex that allocates, always set the index to NULL
	//! by calling SetIndex( NULL) before any destruction of MajaGDataTree instance.
	inline void SetIndex( GLuint *id)
	{
		index= id;
	}
	
	//! set the start index
	inline void SetStartIndex( GLuint st)
	{
		start_index= st;
	}
	
	//! get if the data in are static
	inline bool GetStaticData( void){ return static_data;}
	
	//! set if the data are static, true is they are
	inline void SetStaticData( bool is_static){ static_data= is_static;}
};
}
}
#endif
