/*
	graphext.h
	ce fichier est une partie de Maja - this is a part of Maja
	support des extensions OpenGL
	OpenGL extensions support

	$Id: $

	$Log: $

	begin:			2001
	copyright:		(C) 2000,2001,2001,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org

	Thanks to Lev, Nutty and others from the forum to have make me more understand
	what OpenGL is. There would surely have no OpenGL extension support without you.
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef __GRAPHEXT_H
#define __GRAPHEXT_H

#ifndef WIN32
#include <GL/glx.h>
#else
#include <windows.h>
#include <GL/gl.h>
#endif
#include <iostream>
#include <string>
#include "../../common/majatypes.h"

namespace maja
{

/*!
	you should not use these functions like that
*/

// this was shown once on an OpenGL.org forum
typedef mvoid (*(*__get_gl_ext_proc_ptr)( const GLubyte *))();

#ifndef WIN32

typedef mvoid*
(*PFNGLXALLOCATEMEMORYNVPROC)( GLsizei size, GLfloat readf, GLfloat writef, GLfloat priority);

typedef mvoid
(*PFNGLXFREEMEMORYNVPROC) ( mvoid *p);

#else

typedef mvoid*
(*PFNWGLALLOCATEMEMORYNVPROC)(GLsizei size, GLfloat readf, GLfloat writef, GLfloat priority);	

typedef mvoid
(*PFNWGLFREEMEMORYNVPROC) ( mvoid *p);

#endif

typedef mvoid
(*PFNGLVERTEXARRAYRANGENV)( GLsizei size, const GLvoid *pointer);

typedef mvoid
(*PFNGLFLUSHVERTEXARRAYRANGENVPROC) (mvoid);

#ifndef WIN32
typedef mvoid
(*PFNGLVERTEXPOINTEREXTPROC)
(GLint size, GLenum type, GLsizei stride, GLsizei count, const GLvoid *pointer);

typedef mvoid
(*PFNGLNORMALPOINTEREXTPROC) (GLenum type, GLsizei stride, GLsizei count, const GLvoid *pointer);

typedef mvoid
(*PFNGLCOLORPOINTEREXTPROC)
(GLint size, GLenum type, GLsizei stride, GLsizei count, const GLvoid *pointer);

typedef mvoid
(*PFNGLTEXCOORDPOINTEREXTPROC)
(GLint size, GLenum type, GLsizei stride, GLsizei count, const GLvoid *pointer);
#endif

typedef mvoid
(*PFNGLDRAWRANGEELEMENTSEXTPROC)
(GLenum mode, GLuint start, GLuint end, GLsizei count, GLenum type, const GLvoid *indices);

#ifndef WIN32
typedef mvoid
(*PFNGLINDEXPOINTEREXTPROC) (GLenum type, GLsizei stride, GLsizei count, const GLvoid *pointer);

typedef mvoid
(*PFNGLDRAWARRAYSEXTPROC) ( GLenum, GLint, GLsizei);
#endif

typedef mvoid
(*PFNGLLOCKARRAYSEXTPROC) (GLint first, GLsizei count);

typedef mvoid
(*PFNGLUNLOCKARRAYSEXTPROC) (mvoid);

typedef mvoid
(*PFNGLBINDTEXTUREEXTPROC) (GLenum target, GLuint texture);

typedef GLboolean
(*PFNGLARETEXTURESRESIDENTEXTPROC) (GLsizei n, const GLuint *textures, GLboolean *residences);

typedef mvoid
(*PFNGLDELETETEXTURESEXTPROC) (GLsizei n, const GLuint *textures);

typedef mvoid
(*PFNGLGENTEXTURESEXTPROC) (GLsizei n, GLuint *textures);

typedef GLboolean
(*PFNGLISTEXTUREEXTPROC) (GLuint texture);

typedef mvoid
(*PFNGLPRIORITIZETEXTURESEXTPROC) (GLsizei n, const GLuint *textures, const GLclampf *priorities);

typedef mvoid
(*PFNGLMULTITEXCOORD2FVARBPROC) (GLenum t, GLfloat *t);

typedef mvoid
(*PFNGLCLIENTACTIVETEXTUREARBPROC) (GLenum t);

/*!
	OpenGL 1.2 and 1.3 extensions support for Linux and Windows
	the first instance must call to Detect() and the other not,
	all have been memorized by the first.
	this class is used internally by many others
	
	status: useable

	infos: always prefer using core/majamemmgr.h for any allocation/deallocation
*/
class MajaGExt
{
	protected:
	
	//! win/linux pointer function, would better be static too
	__get_gl_ext_proc_ptr						GetFunctionPointer;
	
	//! a pointer to the graphic memory
	static GLfloat							*memory_pointer;	
	
	//! decale from the pointer to remember
	static mlong 							heap;
	
	//! count the instances using Allocation
	static muint							count;
	
	//! saving all the memory allocated
	static mlong							*memory_counter;
	
	private:
	//! nvidia card
	static mbool 							use_nv;
	
	//! nvidia vertex array range support
	static mbool 							use_nv_array;
	
	//! extension vertex array (CVA)
	static mbool 							use_cva_array;
	
	//! nvidia allocation support
	static mbool 							use_nv_alloc;
	
	//! gl ext for drawing glDrawRangeElementEXT & glDrawArraysEXT are supported & geforce found
	static mbool 							use_nv_draw;
	
	//! use ext drawing: id before but no NVidia geforce found
	static mbool 							use_ext_draw;
	
	//! opengl extensions
	static mbool 							use_gl_ext;
	
	//! any rendering context ?
	static mbool 							ctx_available;
	
	//! which ext are supported ?
	static GLubyte 							*ext_string;
	
	//! which rendering are we using ?
	static GLubyte 							*render_string;
	
	//! this is mtrue when have called Detect()
	static mbool 							detected;

//! TODO:
//!	could optimize to one type that point to the specific type, this will avoid double in Agp*Memory
	
	public:

	#ifndef WIN32
	
	// will need to have an option to set if memory is set into video memory or only AGP.
	//! allocate memory with AGP if possible, other normal RAM allocation
	static PFNGLXALLOCATEMEMORYNVPROC 				AgpAllocateMemory;						
	
	//! free the memory (only call once for all)
	static PFNGLXFREEMEMORYNVPROC 					AgpFreeMemory;								
	
	#else
	
	//! idem for windows
	static PFNWGLALLOCATEMEMORYNVPROC 				AgpAllocateMemory;
	static PFNWGLFREEMEMORYNVPROC 					AgpFreeMemory;
	
	#endif
	
	public:
	
	//! pointer to vertex array extension function
	static PFNGLVERTEXPOINTEREXTPROC				VertexPointer;
	
	//! pointer to normal array extension function
	static PFNGLNORMALPOINTEREXTPROC				NormalPointer;
	
	//! pointer to color array extension function
	static PFNGLCOLORPOINTEREXTPROC					ColorPointer;
	
	//! pointer to texel array extension function
	static PFNGLTEXCOORDPOINTEREXTPROC				TexelPointer;
	
	//! pointer to index color array extension function (not used by Maja)
	static PFNGLINDEXPOINTEREXTPROC					IndexPointer;
	
	//! NVidia VAR function
	static PFNGLVERTEXARRAYRANGENV 					VertexArrayRange;
	//! flush it
	static PFNGLFLUSHVERTEXARRAYRANGENVPROC				FlushVertexArrayRange;
	
	//! OpenGL extension best drawing function for NVidia
	static PFNGLDRAWRANGEELEMENTSEXTPROC 				DrawRangeElements;
	
	//! OpenGL draw arrays extension
	static PFNGLDRAWARRAYSEXTPROC					DrawArrays;
	
	//! lock/unlock arrays
	static PFNGLLOCKARRAYSEXTPROC					LockArrays;
	static PFNGLUNLOCKARRAYSEXTPROC					UnlockArrays;

	//! textures coordinates of multi-textures
	static PFNGLMULTITEXCOORD2FVARBPROC				MultiTexCoord;

	//! activating a multitexture (only client version)
	static PFNGLCLIENTACTIVETEXTUREARBPROC				ClientActiveTexture;
	
	//! textures extensions
	static PFNGLBINDTEXTUREEXTPROC					BindTexture;
	
	//! return if textures are resident in memory
	static PFNGLARETEXTURESRESIDENTEXTPROC				AreTexturesResident;
	
	//! delete some extension textures
	static PFNGLDELETETEXTURESEXTPROC				DeleteTextures;
	
	//! generate extension textures
	static PFNGLGENTEXTURESEXTPROC					GenTextures;
	
	//! return if texture is extension texture
	static PFNGLISTEXTUREEXTPROC					IsTexture;
	
	//! set the priority of the extension textures specified
	static PFNGLPRIORITIZETEXTURESEXTPROC				PrioritizeTextures;

	static mbool							multitexture_support;
	
	public:
	
	//! create a new instance of this type
	MajaGExt( mvoid);
	
	//! this destructor looks if this is the last instanciated to be
	//! destroyed, and then, if so, free all the memeory all the
	//! instancies of this class had allocated.
	//! So, if you want to keep the memory living, you should have at
	//! least, at all time, one instance of this type.
	~MajaGExt( mvoid);
	
	//!
	//! These are user methods:
	//!
	
	//! Detecting the support. only but call at the first instance !
	mbool Detect( mvoid);

	mbool GetDetected( mvoid){
		return detected;
	}
	
	//! Dumping support information at console screen
	mvoid ShowInfo( mvoid);
	
	//! return if NVidia geforce support was found
//! TODO: take a regard to other NVidia cards, and why not ATI ?
	inline mbool GetNV( mvoid){ return use_nv;}
	
	//! return if NVidia drawing functions are supported
	inline mbool GetNvDraw( mvoid){ return use_nv_draw;}

	inline mbool GetExtDraw( mvoid){ return use_ext_draw;}

	inline mbool GetCVAArray( mvoid){ return use_cva_array;}
	
	//! WARN ! don't keep to use this method for normal allocation as malloc can do !
	//! or do it at your own risk (if AGP not supported).
	//!
	//! Allocate memory into AGP if supported or into RAM
	//! bytes is the size in bytes of the memory to be allocated, it's added internally,
	//! so, the user doesn't need to remember the last sizes wanted at the last calls.
	//! really_allocate is set to mtrue at the last call of this function, so all the memory
	//! is allocated in one block.
	//! this is dependent to Nvidia, or use the normal memory instead
	mlong AllocateMemory( mlong bytes, mbool really_allocate= mfalse);
	
	//! Free the memory allocated by AllocateMemory()
	//! only call once for all the calls at AllocateMemory() !
	mvoid FreeMemory( mvoid);
	
	//! Return a pointer to the allocation done by AllocateMemory
	//! always return mnull until AllocateMemory() really_allocate argument is set to 1
	//! so, only check if the AGP is really support at the last call of AllocateMemory
	inline mvoid* GetMemoryPointer( mvoid){ return (mvoid*)memory_pointer;}
	
	//! get the decale from the pointer to the first element allocated
	//! apparemment pourrait tre dsuette.
	inline GLfloat* GetAddressDecale( mvoid){ return (GLfloat*)memory_pointer+heap;}
	
	//! get the total size allocated or wanted to be:
	inline mlong GetHeap( mvoid){ return heap;}
	
	//! get if extensions renders are possible
	inline mbool GetExtRenderer( mvoid){ return use_ext_draw;}

	inline mbool GetMultitextureSupport( mvoid){ return multitexture_support;}
	
};//nv_extension;
}
#endif
