/*
	majadisplay.h
	ce fichier est une partie de Maja - this is a part of Maja
	gre le rendu des MO
	render management of the MO

	$Id: $

	$Log: $

	begin:			september 2000
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef __MAJA_DISPLAY_H
#define __MAJA_DISPLAY_H

#include <typeinfo>
#include <math.h>
#include "../scenery/majacamera.h"
#include "../../tools/maths/geometry.h"
#include "majagengine.h"
#include "majalight.h"
#include "shadowmatrix.h"
#include "../../layers/wm/majamainwindow.h"
#include "../../layers/wm/majaxwindow.h"
#include "majagconf.h"
#include "../scenery/majaactor.h"
#include "../../layers/th/mutex.h"
#include "../../core/majamemmgr.h"
#include "majahud.h"
#include "majatextures.h"
#include "../menu/majamenu.h"

namespace maja
{
/*!
 * MAJAGRAPHIC is intended for providing graphic
 * objects to be used in graphic casis. Displays,
 * textures, lights, graphic engines, integrator
 * are some of its classes.
 */
namespace MAJAGRAPHIC{

static mbool	need_end= mfalse;

/*!
 * MajaDisplay is the rendering engine.
 *
 * status: useable, will change
 *
 * TODO: modifications: 
 * 	* new class: MajaWorld which contains the MajaObjects
 * 	* new class: "MajaStudio" which contains what makes the reality (cameras, lights...)
 * 	* 	-> surely resulting of this, will aggregate these above into MajaTheaterStudio
 * 
*/
class MajaDisplay: public MAJATHREAD::MajaMutex
{
	MAJACONTAINER::MajaList<MajaObject>	object_list;

	//! in test
	MajaGraphicEngine			*g_engine;
	
	//! true if we want to use the graphic engine (recommanded)
	mbool					use_g_engine;
	
	//! internal name of the class, do not rename
	mchar 					*maja_internal_name;

	MajaCamera				*camera;
	
	//! mode of the used camera (the only one for now)
	_MajaCameraMode				camera_mode;
	
	//! all was computed if true
	mbool					g_computed;	
	
	protected:
	
	MajaShadowMatrix			shadow_matrix;
	
	MajaLight				*light;

	MajaWindow				*window;

	MajaGConf				gl_conf;

	MajaMemoryManager<mfloat>		*mem_mgr;

	std::string				maja_copyright;
	
	std::string				framerate;
	
	MajaTextures				maja_tex;

	//TODO: system_hud for inner outputs and user_hud for customizable hud
	MajaHud<MajaDrawable>			hud;

	MajaMenu<
		MajaDrawable,
		MajaMenuItem,
		MajaItemPolicy
		>				menu;

	mbool					use_menu;

	mbool					use_hud;

	public:
	
	MajaDisplay( mvoid);
	
	~MajaDisplay( mvoid);

	//! this initializes the new MajaGConf member
	//! so users could change the 'GConf' until he
	//! calls this method; and should use it as soon
	//! as possible (before any core programming).
	mint Init( mvoid);
	
	mbool Check( mvoid);

	/*!
		compute all the graphic data to be performed
		for the best used into the display
	*/
	mint Compute( mvoid);

	inline mbool GetComputed( mvoid){
		return g_computed;
	}

	inline mint Draw( mvoid){
		static mbool first= mtrue;
		
		// all hud & menu here since no loading for that... if created in main(), this won't be useable yet...
		if( first){	// surely in ...
			static MAJAMATHS::MajaVector cr_pos( 5,10,0);
			static MAJAMATHS::MajaVector hud_pos( 50,50,0);
			static MAJAMATHS::MajaVector hud_size(	window->GetAttributes().width-100, 
								window->GetAttributes().height-100,
								0);
			static MAJAMATHS::MajaQuaternion hud_color( .3,.3,.3,.7);
			static MAJAMATHS::MajaVector fr_pos( hud_size.x-30, 10, 0);
			static MAJAMATHS::MajaVector ca_pos( 10, hud_size.y-20, 0);

			light= new MajaLight();
			light->DefaultLight();
			
			GLuint l= maja_tex.Create( "images/maja.tga");
			maja_tex.LoadAll();
			window->Resize();
			gl_conf.SetUseBlending( mtrue);
			
			hud.Connect( window->GetDrawable(), "fixed", l);
			hud.SetPosition( hud_pos);
			hud.SetSize( hud_size);
			hud.SetColor( hud_color);
			hud.AddInfo( maja_copyright, cr_pos);
			hud.AddInfo( framerate, fr_pos);
			hud.AddInfo( camera->PrintInfo(), ca_pos);

			MAJAMATHS::MajaVector menu_size( window->GetAttributes().width, window->GetAttributes().height, 0);
			static MajaMenuItem *item_quit= mnull;
			static MajaMenuItem *item_start= mnull;			
			MAJAMATHS::MajaVector is_pos( menu_size.x/2-50,menu_size.y/2-50,0);
			MAJAMATHS::MajaVector iq_pos( is_pos.x,is_pos.y+60,0);
			MAJAMATHS::MajaVector iq_size( 100,40,0);
			MAJAMATHS::MajaVector is_size( 100,40,0);
			MAJAMATHS::MajaVector bg_pos( 0,0,-.5);
			MAJAMATHS::MajaVector bg_size( menu_size);

			item_quit= new MajaMenuItemObject<inner::MajaExit_<mcom,mint> >;
			item_start= new MajaMenuItemObject<inner::MajaStart_<mcom,mint> >;
			item_quit->SetPos( iq_pos);
			item_quit->SetSize( iq_size);
			item_start->SetPos( is_pos);
			item_start->SetSize( is_size);
			
			menu.Connect( window->GetDrawable(), "fixed");
			menu.SetPos( 0, 0, 0);
			menu.SetSize( menu_size);
			menu.SetUseBackground( mtrue);
			menu.SetBackgroundTexture( l);
			menu.GetBackground().SetSize( bg_size);
			menu.GetBackground().SetPos( bg_pos);
			
			menu.AddItem( *item_start);
			menu.AddItem( *item_quit);
		
			first= mfalse;
		}

		if( need_end)
			return -1;

		Render();
		framerate= window->GetFramerate();
		camera->PrintInfo();

		return 1;
	}
	
	/*!
		Render all
		infos: for the HUD display
		it should be the MajaMenu part to manage it, so it could
		have MajaGLString too, and here just a call to them.
		Otherwise, all MO have a HUD inner themselves, so it could
		be done in one call.
	*/
	inline mvoid Render( mvoid){
		window->PreDraw();
		glPushMatrix();
		camera->Refresh();
		g_engine->Render();
		window->PreHudDraw();
		if( use_hud)
			hud.Display();
		if( use_menu)
			menu.Draw();
		window->PostHudDraw();
		glPopMatrix();
		window->PostDraw();
	}
	
	//! Enable or Disable the use of the graphic engine
	//! if it's enabled, you could get more efficiency & quickest rendering
	//! you should not switch between two state (enable & disable) during
	//! the running as the graphic engine needs to perform some compute
	inline mvoid SetGEngineUse( mbool use){ use_g_engine= use;}
	
	//! Get the state of the use of the graphic engine
	inline mbool GetGEngineUse( mvoid){ 
		return use_g_engine;
	}

	inline MajaGraphicEngine& GetGEngine( mvoid){ 
		return *g_engine;
	}
	
	mbool DoShadows( mvoid);

	//! set the window by referency
	inline mvoid SetWindow( MajaWindow *win){ 
		window= win;
	}

	//! get the referency to the window
	inline MajaWindow& GetWindow( mvoid){ 
		return *window;
	}

	//! get if the window was created or not
	inline mbool WindowWasCreated( mvoid){ 
		return window->GetCreated();
	}

	//!todo: object or camera ?
	inline mvoid SetCamera( MajaObject *cam){
		camera= (MajaCamera*)cam;
	}


	inline mvoid AddObject( MajaObject& o){
		object_list.AddObject( &o);
	}
	
	inline MajaDrawable& GetDrawable( mvoid){
		return window->GetDrawable();
	}

	inline MajaCamera& GetCamera( mvoid){
		return *camera;
	}

	inline mvoid SetMemoryManager( MajaMemoryManager<mfloat>& ptr){
		mem_mgr= &ptr;
	}

	inline mvoid SetMajaCopyright( std::string s){
		maja_copyright= s;
	}

	inline MajaMenu<MajaDrawable,MajaMenuItem,MajaItemPolicy>& GetMenu( mvoid){
		return menu;
	}

	mvoid SetUseMenu( mbool v){
		use_menu= v;
	}

	mvoid SetUseHud( mbool v){
		use_hud= v;
	}
};
}
}
#endif
