/*
	majagdata.h
	ce fichier est une partie de Maja - this is a part of Maja
	donnes graphiques  rendre
	graphic data to render

	$Id: $

	$Log: $

	begin:			Wed Jun 19 2002
	copyright:		(C) 2002 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef __MAJA_G_DATA
#define __MAJA_G_DATA

#include "gbase.h"

namespace maja
{
namespace MAJAGRAPHIC{

/*!
	Initial Data class
	You should not use it to render anything but use
	its childhood that is known from the engine.
	
	status: useable, not terminated 'lack some member funcs'
*/
class MajaGData: public MajaGEntity
{
	protected:
	
	//! this is true if all data were allocated, so needed to be deleted
	bool to_be_destroyed;
	
	public:
	
	//! an array of vertices
	//! this is stored as: vert1_x, vert1_y, vert1_z, vert2_x...
	GLfloat *vertices;
	
	//! an array of RGBA colors, so all vertices could be transparent
	//! this is stored as: col1_r, col1_g, col1_b, col1_a, col2_r...
	GLfloat *colors;
	
	//! an array of vertices normals
	//! norm1_x, norm1_y, norm1_z, norm2_x...
	GLfloat *normals;
	
	//! this is an array of texels related to vertices
	//! textures are only 2D in Maja.
	//! so they are stored like: texel1_x, texel1_y, texel2_x...
	GLfloat *texels;
	
	//! this is an array of faces_normals
	//! this is not used yet
	//! this should be stored like this: facen1_x, facen1_y, facen1_z, facen2_x...
	GLfloat *face_normals;
	
	//! number of faces represented by all these values above
	GLuint faces_nb;
	
	//! an identifier to a parameter
	//! this is not used (maybe deprecated)
	GLuint param_id;
	
	//! an array of indicies related to arrays
	//! this could be let to null if and only if the faces are following
	//! each others in all the arrays (this is the default way of Maja)
	GLuint *index;	
	
	//! the data may not be deserved to be drawn, or could no be
	//! drawable for a while. so put it at true, otherwise, false
	//! this is not used by Maja for now
	bool drawable;
	
	MajaGData( void);
	
	MajaGData( MajaGData& data);
	
	~MajaGData( void);
	
	//! set the identifier to the parameter
	inline void SetParamId( GLuint param_id){ this->param_id= param_id;}
	
	//! add one or more new face.
	//! if you have more than one face, any data of each array must follow each others in the way
	bool SetNewFace( GLfloat *ve, GLfloat *no, GLfloat *co, GLfloat *te, GLfloat *fa, GLuint nb=1);
	
// TODO: les autres tests (sommets...)
	bool CheckIntegrity( void){
		if( param_id<=0)
			return false;
		if( faces_nb<=0)
			return false;
		if( index==NULL)
			return false;
			
		return true;
	}
	
	//! Create the indicies array
	//! default starts it from 0 to the number of verticies
	//! you could decide it starts from another value
	void CreateIndex( GLuint start= 0){
		if( !index)
			index= new GLuint[3*faces_nb];
		else{
			std::cerr << std::endl << "index existant !!!";
			return;
		}
		for( GLuint i=0; i<3*faces_nb; i++)
			index[i]=i+start;
	}
	
	//! this render this data (not implemented as it's not deserved to be rendered, but should be an idea)
	GLuint Render( void){ return 0;}
};
}
}
#endif
