/*
	majagintegrator.h
	ce fichier est une partie de Maja - this is a part of Maja
	crer des MajaGMesh  partir des MajaMesh des modles 3D chargs
	create MajaGMesh from the loaded 3D models MajaMesh
	
	$Id: $

	$Log: $

	begin:			Wed Jun 19 2002
	copyright:		(C) 2002 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	Contactez-nous pour toute tentative d'utilisation commerciale, svp.			*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *	Contact us for any commercial use attempt, please.					*
 *												*
 ************************************************************************************************/

#ifndef __MAJA_INTEGRATOR
#define __MAJA_INTEGRATOR

#include <typeinfo>
#include "gbase.h"
#include "../modeler/majamesh.h"
#include "majagmaterial.h"
#include "majagtext.h"
#include "majagdata.h"
#include "majagmesh.h"

#include <iostream>
#include <string>

namespace maja
{
namespace MAJAGRAPHIC{

enum __Maja_Integrity_Level
{
	MIL_none=0,
	MIL_no_texture,
	MIL_no_material,
	MIL_no_vertice,
	MIL_all
};

/*!
	integrate a MajaMesh and create a MajaGMesh,
	specific to Maja inner graphism
	
	status: safe
*/
class MajaGIntegrator
{
	static __Maja_Integrity_Level integrity_level;
	
	protected:
	
	MajaMesh& 	input_mesh;

	MajaMesh_	*i_mesh;
	
	MajaGMesh 	output_mesh;
	
	GLuint 		*mat_refering;
	
	GLuint 		*tex_refering;
	
	GLuint 		*param_refering;
	
	GLuint 		param_count;
	
	mchar		*maja_internal_name;
	
	public:
	
//! TODO: take a look at this reference allocation, so now don't use it
	MajaGIntegrator( void);
	
	MajaGIntegrator( MajaGIntegrator& i);
	
	//! the preference constructor is this one
	MajaGIntegrator( MajaMesh& mesh);
	
	virtual ~MajaGIntegrator( void);
	
	//! avoid using it until the default constructor repaired
	inline void SetMesh( MajaMesh& mesh){ input_mesh= mesh;}
	
	bool Compute( void);
	
	inline MajaGMesh* GetMesh( void){ return &output_mesh;}
	
	GLuint MakeDisplayList( void);
	
	protected:
	
	bool ComputeMaterials( void);
	
	bool ComputeTextures( void);
	
	bool ComputeRefering( void);

	bool ComputeDatas( void);
};

template
<
	class I
>
class MajaGMeshMaker
{
	protected:

	//! this is the output computed mesh. this won't be
	//! destroyed when destroying MajaGIEngine.
	MajaGMesh	*o;

	public:

	MajaGMeshMaker(){
		o= mnull;
	}

	~MajaGMeshMaker(){
		;// not delete o !
	}

	mbool Compute( I& i);
	
	MajaGMesh& GetMesh( mvoid){ 
		return *o;
	}

	protected:
	
	mbool ComputeMaterials( I& i);
	
	mbool ComputeTextures( I& i);
	
	mbool ComputeRefering( I& i);

	mbool ComputeDatas( I& i);
};

/*!
 * will need to compute all right even if living.
 * will need to do its work in the most quickest way.
 */
template
<
	class INPUT,
	class OUTPUT,
	template<class,class> class ENGINE
>
class MajaGIntegrator_: protected ENGINE<INPUT,OUTPUT>
{
	public:

	MajaGIntegrator_():	ENGINE<INPUT,OUTPUT>(){
		i= mnull;
	}

	mbool Compute( INPUT& i){
		return ENGINE<INPUT,OUTPUT>::Compute( i);
	}

	OUTPUT& GetMesh(){
		return ENGINE<INPUT,OUTPUT>::GetMesh();
	}
};


}
}
#endif
