/*
	majagparam.h
	ce fichier est une partie de Maja - this is a part of Maja
	branche/noeud qui supporte les paramtres comme les matriaux et textures
	branch/node that hangs parameters as material and texture

	$Id: $

	$Log: $

	begin:			Tue Jun 25 2002
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef __GPARAM_TREE_H
#define __GPARAM_TREE_H

#include "graphext.h"
#include "gbase.h"
#include "majagdata.h"
#include "gdatatree.h"
#include "majagmaterial.h"
#include "majagtext.h"
#include "../../tools/maths/geometry.h"

namespace maja
{
namespace MAJAGRAPHIC{

/*!
	parameter node of the tree
	parent is a branch, child are other param or leaves
	(this is the way but not reality for now)

	status: safe

	infos: for now, only one child to each MajaGParam (a shame!)
*/
class MajaGParam/*: public MajaGEntity*/
{
	protected:
		
	MajaGDataTree		*tmp;

	//! childhood are only leaves for now
	MajaGDataTree		*children;

	//! extension support
	MajaGExt		*gl_ext;

	//! which technic is used
	__maja_graphic_technic	g_technic;
	
	public:
	
	//! a specific material
	MajaGMaterialTree	*mat;
	
	//! a specific texture
	MajaGTextureTree	*tex;
	
	//! number of children
	GLuint			child_nb;

	MajaGParam( void);
	
	~MajaGParam( void){
		delete gl_ext;
	}

	//! check the integrity of what's in
	bool Check( void){
		if( !mat){ 
			std::cerr << std::endl << "no material"; 
			return false;
		}
		if( !tex){ 
			std::cerr << std::endl << "no texture"; 
			return false;
		}
		if( !children){ 
			std::cerr << std::endl << "warning: no children";
		}
		if( child_nb<=0){ 
			std::cerr << std::endl << "child but number set to 0"; 
			return false;
		}
		
		return children[0].Check();
	}
	
	//! set the children
	inline void SetChildren( MajaGDataTree *c){ 
		children= c;
	}
	
	//! set up the technic of rendering
	inline void SetGraphicTechnic( __maja_graphic_technic t){ 
		g_technic= t;
	}

	//! render this part
	inline GLuint Render( void)
	{
		GLuint i=0;
		tmp= children;

		if( !tmp) 
			return 0;
		
		for( i=1;i<child_nb+1;i++){
			glPushMatrix();
			glPushClientAttrib( GL_COLOR_BUFFER_BIT | GL_TEXTURE_BIT);
			
			//TODO: must speed up this by one matrix !
			MajaVector& pos= *children->position;
			glTranslatef( pos.x, pos.y, pos.z);
			MajaVector& rot= *children->orientation;
			glRotatef( _RAD_TO_DEG( rot.x), 0,1,0);
			glRotatef( _RAD_TO_DEG( rot.y), 1,0,0);
			glRotatef( _RAD_TO_DEG( rot.z), 0,0,1);
			
			glMaterialfv(	GL_FRONT, GL_AMBIENT,	mat->mat);
			glMaterialfv(	GL_FRONT, GL_DIFFUSE,	&(mat->mat[3]));
			glMaterialfv(	GL_FRONT, GL_SPECULAR,	&(mat->mat[6]));
			glMaterialf(	GL_FRONT, GL_SHININESS,	mat->mat[9]);
			
			if( tex->nb>1)
				gl_ext->ClientActiveTexture( GL_TEXTURE0_ARB);
			gl_ext->BindTexture( GL_TEXTURE_2D, tex->tex_id[0]);
			glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
			glMatrixMode( GL_TEXTURE);
			glLoadIdentity();
			glPushMatrix();
			glRotatef( *tex->tex_rot, 0,0,1);
			if( tex->nb==2){
				gl_ext->ClientActiveTexture( GL_TEXTURE1_ARB);
				gl_ext->BindTexture( GL_TEXTURE_2D, tex->tex_id[1]);
			}
			tmp->Render();
			glPopMatrix();			
			glMatrixMode( GL_MODELVIEW);
			glPopClientAttrib();
			glPopMatrix();
			
			tmp= &children[i];
		}
		return 1;
	}
};
}
}
#endif
