/*
	majagtext.h
	this is a part of Maja
	textures management seen for the graphic engine

	$Id: $

	$Log: $

	begin:			Jun 2002
	copyright:		(C) 2002 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/********************************************************************************
 *										*
 *	This program is free software; you can redistribute it and/or modify	*
 *	it under the terms of the GNU General Public License as published by	*
 *	the Free Software Foundation; either version 2 of the License, or	*
 *	(at your option) any later version.					*
 *	Please read the license (COPYING)					*
 *										*
 ********************************************************************************/

#ifndef __MAJAGTEXT_H
#define __MAJAGTEXT_H

#include "gbase.h"

namespace maja
{
namespace MAJAGRAPHIC{
/*!
	Texture in the point of view of the graphic
	These are just identifiers managements
*/
class MajaGTexture: public MajaGEntity
{
	protected:
	
	//! a priority for making the texture resident (range 0->1)
	GLfloat		resident_priority;
	
	//! the texture matrix for this part
	//! all was applied, so it's performed to fast rendering
	//MAJAMATHS::MajaMatrix	texture_matrix;
	
	public:
	
	//! an array of textures
	GLuint	*textures;
	
	GLuint	*textures2;
	
	//! an array of textures orientation
	//! must match the same order than the textures array
	GLfloat	*tex_rot;
	
	//! number of textures (and so tex_rot)
	GLuint	nb;
	
	MajaGTexture( void);
	
	~MajaGTexture( void);
	
	//! Set an new texture to the arrays
	// should be renamed Add*
	GLuint SetNewTexture( GLuint tex, GLfloat rot=0, GLuint tex2=0, GLfloat rot2=0);
	
	//! Remove a texture by its texture's identifier
	bool RemoveTexture( GLuint n);
	
	//! Get an index in the array of a specific texture id
	GLuint GetTextureIndex( GLuint tex_id);
	
	//! Set the priority texture
	inline void SetTexturePriority( GLfloat priority){ resident_priority= priority;}
	
	inline GLfloat GetTexturePriority( void){ return resident_priority;}
	
	//! Return a percent number that could be used to know
	//! how the texture could be resident, so how fast it could be
	//! the value is btw 0 and 1. If 0, it's not a resident texture, if
	//! it's 1, the texture is surely resident
	GLfloat AreResidentTextures( void);
};

/*!
	A type that could enter the graphic engine tree
	TODO: need texture params here...
*/
class MajaGTextureTree
{
	public:
	
	//! pointer to a unic texture id
	GLuint *tex_id;
	
	//! pointer to a unic texture rotation
	GLfloat *tex_rot;

	GLuint nb;

	MajaGTextureTree( GLuint n){
		nb=n;
		tex_id= new GLuint[nb];
		tex_rot= new GLfloat[nb];
	}

	~MajaGTextureTree( void){
		delete[] tex_id;
		delete[] tex_rot;
	}
};
}
}
#endif
