/*
	majahud.h
	ce fichier est une partie de Maja - this is a part of Maja
	affichage tte haute pour l'utilisateur
	user head-up display

	$Id: $

	$Log: $

	begin:			january 22 2003
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef MAJA_HUD__H
#define MAJA_HUD__H

#include <list>
#include <string>
#include "../../common/majatypes.h"
#include "../../common/majatemp.h"
#include "../../common/majatxtmgr.h"
#include "../../tools/maths/majavector.h"
#include "../../tools/maths/majaquaternion.h"

namespace maja
{
	/*!
	 * a head-up display (as in fireplanes) that could be
	 * used in MajaDisplay in order to be shown.
	 * Users should add any pointer to relative std::string
	 * and MajaVector in the appropriate list, using AddInfo().
	 *
	 * status: experimental
	 */
	template <class D>
	class MajaHud
	{
		protected:

		//! any windowing text outputs for MOs.
		//! it needs that text_out._d is defined with the display (under X11)
		MajaGlString				text_out;

		std::list<std::string*>			outputs;
		
		std::list<MAJAMATHS::MajaVector*>	locates;

		muint					out_nb;

		MAJAMATHS::MajaVector			pos;
		MAJAMATHS::MajaVector			size;
		MAJAMATHS::MajaQuaternion		color;

		GLuint					logo_id;

		public:

		MajaHud( mvoid){
			out_nb=  0;
			color.x=.5;
			color.y=.5;
			color.z=.5;
			color.w=.5;
		}

		~MajaHud( mvoid){
			;
		}

		//! connecting the hud to the display (vital for Linux).
		//! call it after creating and before any other functions.
		//! D is a MajaDrawable, font_name is the name of the font
		//! to be used (and so to be allocated), and id matches to
		//! any texture id that could serve to the background paper,
		//! if wanted.
		mvoid Connect( D d, std::string font_name, GLuint id=0){
			text_out._d= d._d;
			text_out.SetFont( font_name.c_str());
			logo_id= id;
		}

		mvoid Display( mvoid){
			std::list<MAJAMATHS::MajaVector*>::iterator i= locates.begin();
			std::list<std::string*>::iterator k= outputs.begin();
			
			glPushMatrix();

			glPushMatrix();
			glColor4f( 1,1,1,1);
			for( ;i!=locates.end();i++,k++){
				text_out.Locate( (mint)((*i)->x), (mint)((*i)->y));
				text_out.Print( **k);
			}
			glPopMatrix();
			
		/*	glBindTexture( GL_TEXTURE_2D, logo_id);
			glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
			glBegin( GL_QUADS);
			glColor4f( color.x, color.y, color.z, color.w);
			glTexCoord2f( 0,0);
			glVertex3f( pos.x,pos.y,pos.z);
			glTexCoord2f( 1,0);
			glVertex3f( pos.x+size.x,pos.y,pos.z);
			glTexCoord2f( 1,1);
			glVertex3f( pos.x+size.x,pos.y+size.y,pos.z);
			glTexCoord2f( 0,1);
			glVertex3f( pos.x,pos.y+size.y,pos.z);
			glEnd();
		*/	
			glPopMatrix();
		}

		mvoid AddInfo( std::string& info, MAJAMATHS::MajaVector& p){
			outputs.push_back( &info);
			p.x+=pos.x;
			p.y+=pos.y;
			locates.push_back( &p);
			++out_nb;
		}

		mvoid SetPosition( MAJAMATHS::MajaVector p){
			pos= p;
		}

		mvoid SetSize( MAJAMATHS::MajaVector s){
			size= s;
		}

		mvoid SetColor( MAJAMATHS::MajaQuaternion c){
			color= c;
		}
	};
}
#endif
