/*
	majalight.h
	ce fichier est une partie de Maja - this is a part of Maja
	gestion de la lumire basique
	basic light management

	$Id: $

	$Log: $

	begin:			Sep 2000
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef __LIGHT_H
#define __LIGHT_H

#ifdef WIN32
#include <windows.h>
#endif
#include <GL/gl.h>
#include "../../tools/maths/majamaths.h"
#include "../../tools/maths/majaquaternion.h"

namespace maja
{

static MAJAMATHS::MajaQuaternion mat_amb_white(0.75,0.75,0.8,1);

/*!
	MajaLight is defined by its position, its diffused color, its speculate
	this class isn't developped. Some methods can't be called anywhere
	as DefineLight()
	
	status: useable, would change
*/
class MajaLight
{
	//! numro OpenGL de la lumire
	static muint light_opened;
	
	muint light_number;
	
	protected:
	
	//! position of the light
	//! x, y & z define the direction of the light if w=0
	//! x, y & z define the position of the light if w!=0
	MAJAMATHS::MajaQuaternion light_pos;
	
	//! direction if light not directional (light_pos.w!=0)
	MAJAMATHS::MajaQuaternion light_dir;
	
	//! Diffuse component of the light
	MAJAMATHS::MajaQuaternion light_diff;
	
	//! Specular component of the light
	MAJAMATHS::MajaQuaternion light_spec;
	
	public:

	//! Create a new OpenGL Light. Give it an opengl light number, and
	//! increment the static variable, so next light creation will have the next
	//! available light number.
	MajaLight(){
		light_number= 0;
	}
	
	MajaLight( MAJAMATHS::MajaQuaternion& pos, MAJAMATHS::MajaQuaternion& diff, MAJAMATHS::MajaQuaternion& spec);
	
	MajaLight( MajaLight& l);
	
	virtual ~MajaLight(){}

	//! Light on or off
	mvoid SwitchOnLight( mvoid);
	
	mvoid SwitchOffLight( mvoid);
	
	//! Set diffuse component
	inline mvoid SetDiffuseColor( MAJAMATHS::MajaQuaternion& color);
	
	//! get the diffuse component
	inline MAJAMATHS::MajaQuaternion& GetDiffuseColor( mvoid){ 
		return light_diff;
	}
	
	//! set the specular component
	inline mvoid SetSpecularColor( MAJAMATHS::MajaQuaternion& color);
	
	//! get the specular component
	inline MAJAMATHS::MajaQuaternion& GetSpecularColor( mvoid){ 
		return light_spec;
	}
	
	//! note that position may be a direction
	inline mvoid SetPosition( MAJAMATHS::MajaQuaternion& pos);
	
	inline MAJAMATHS::MajaQuaternion& GetPosition( mvoid){ 
		return light_pos;
	}
	
	//! light is directional as the sun or as a door light
	inline mvoid LightIsDirectional( mvoid){ 
		light_pos.w=0;
	}
	
	inline mvoid LightIsPositional( mvoid){ 
		light_pos.w=1;
	}
	
	//! set direction
	inline mvoid SetDirection( MAJAMATHS::MajaQuaternion& dir);

	mvoid DefaultLight( mvoid);
};

/*
	An ambiant light: not implemented
*/
class MajaAmbiant: public MajaLight
{
	public:
	
	GLfloat light_amb;

	~MajaAmbiant(){}
};

}
#endif
