/* $Id:$
 * textures for Maja
 * Copyright (C) september 2000,2001,2002 Jean-Dominique Frattini
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Cette license est valable en terme de droit franais. Cela signifie que c'est la
 * lgislation franaise qui est en vigueur pour l'ensemble des sources contenues
 * dans la librairie. Sauf image.cpp et image.h, se reposant sur aucune license explicite,
 * mais qui sont fournies aimablement dans les librairies Mesa sous license LGPL.
 *
 * Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou
 * le modifier sous les termes de la license publique gnrale
 * publie par la Free Software Foundation; soit la version 2 de la
 * license, ou (si vous le voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir d'tre utile, mais
 * SANS AUCUNE GARANTIE; et mme une garantie implicite,
 * y compris les garanties de COMMERCIALISATION ou D'ADAPTATION
 * DANS UN BUT SPCIFIQUE. Prenez connaissance de la License Publique
 * Gnrale GNU pour plus de dtail.
 *
 * Vous devez avoir reu une copie de la License Publique Gnrale
 * GNU avec ce programme; si ce n'est pas le cas, vrivez  la
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log:$
 */

#ifndef __MAJA_TEX_H
#define __MAJA_TEX_H

#include <GL/gl.h>
#include <GL/glu.h>
#include <iostream>
#include "majaimage.h"
#include "../../tools/containers/majalist.h"
#include "graphext.h"

namespace maja
{
#define MAJA_NO_TEXTURE 			"_maja_void_texture"
#define MAJA_TEXTURE_DIRECTORY 			"images/"

/*!
	Manage a texture:
	load, assign
*/
class MajaTex
{
	static int number;					//! total number of different textures
	
	unsigned int current_id;				//! the identifiant (opengl) of the texture
	
	char *name;						//! name of the texture: file with directory
																				//! THIS IS NOT OBLIGATORY WITH DIRECTORY !
																				//! DUE TO 3DS IMPORT
	char *directory;					//! directory of the file (must be specified)

	static MajaGExt	g_ext;
	
	public:	
	
	MajaTex( void){
		name= NULL;
		directory= NULL;
		current_id= 0;
	}
	
	~MajaTex( void){
		if( name)
			delete name; 
		if( directory)
			delete directory;
	}
	
	//! Get the opengl identifiant of the texture
	inline GLuint GetId( void){ 
		return current_id;
	}
	
	//! Get the number of textures loaded
	inline unsigned int GetNumber( void){ 
		return number;
	}
	
	//! Get the name
	inline char* GetName( void){ 
		return name;
	}
	
	//! set name
	inline void SetName( char *n){
		if( name)
			delete name;
		name= new char[strlen( n)+1];
		strcpy( name, n);
	}
	
	//! Get the directory
	inline char* GetDirectory( void){ 
		return directory;
	}
	
	//! Set the directory
	inline void SetDirectory( char *dir){
		if( !directory)
			directory= new char[strlen( dir)+1];
		if( !directory){
			std::cerr << std::endl << "Not enough memory.";
			return;
			//throw std::exception;
		}
		strcpy( directory, dir);
	}
	
	//! initialize the texture, set the identifiant here
	void Init( char *file_name);
	
	//! load the texture
	bool LoadImage();

	inline bool Load(){ 
		return LoadImage();
	}
};
}
#endif
