/*
	shadowmatrix.h
	ce fichier est une partie de Maja - this is a part of Maja
	matrice de projection d'ombre
	shadow projection matrix

	$Id: $

	$Log: $

	begin:			Thu Aug 22 2002
	copyright:		(C) 2002 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	Contactez-nous pour toute tentative d'utilisation commerciale, svp.			*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *	Contact us for any commercial use attempt, please.					*
 *												*
 ************************************************************************************************/

#include "../../tools/maths/majamaths.h"
#include <stdlib.h>

namespace maja
{
	
using MAJAMATHS::MajaMatrix;
using MAJAMATHS::MajaQuaternion;
using MAJAMATHS::_scalar;

/*!
	Shadow-projection matrix
*/
class MajaShadowMatrix
{
	protected:
	
	//! the shadow matrix
	MajaMatrix			*matrix;
	
	//! the light position (x,y,z,w) with w=0 or w=1 only !!
	MajaQuaternion&		light_pos;
	
	//! the plan equation: ax+by+cz+d=0
	MajaQuaternion&		plane_equation;
	
	public:
	
	//! default constructor
	MajaShadowMatrix( void):	light_pos( *new MajaQuaternion),
							plane_equation( *new MajaQuaternion)
	{
		//cerr << endl << "constructeur de shadow matrix";
	}
	
	//! normal constructor
	MajaShadowMatrix( MajaQuaternion&	l_pos, MajaQuaternion&	p_eq):	light_pos( l_pos),
																		plane_equation( p_eq)
	{
		;
	}
	
	//! copy constructor
	MajaShadowMatrix( MajaShadowMatrix& matrix):	light_pos( matrix.light_pos),
							plane_equation( matrix.plane_equation)
	{
		;
	}
	
	//! destructor
	~MajaShadowMatrix( void)
	{
		delete &light_pos;
		delete &plane_equation;
	}
	
	//! set light position
	inline void SetLightPosition( MajaQuaternion pos){ light_pos= pos;}
	
	//! get light position
	inline MajaQuaternion& GetLightPosition( void){ return light_pos;}
	
	//! set plan equation
	inline void SetPlanEquation( MajaQuaternion equ){ plane_equation= equ;}
	
	//! get the plan equation
	inline MajaQuaternion&	GetPlanEquation( void){ return plane_equation;}
	
	//! get the shadow matrix
	inline MajaMatrix&	GetShadowMatrix( void){ return *matrix;}
	
	inline _scalar*	GetMatrixPointer( void){ return matrix->GetPointer();}
	
	//! set the shadow matrix
	inline void	SetShadowMatrix( MajaMatrix& m)
	{
		if( !matrix){
			matrix= new MajaMatrix( m);
			if( !matrix){
				std::cerr << std::endl << "memory full";
			}
			return;
		}
		*matrix=m;
	}
	
	//! generate, create the shadow-projection matrix
	inline bool Generate( void) throw( int)
	{
		_scalar	dot= plane_equation.DotProduct( light_pos);
		_scalar	*scalar_ptr= NULL;
		
		try{
			scalar_ptr= new _scalar[16];
			if( !scalar_ptr)
				throw 1;
				
			int i, j;
			
			for( i=0; i<16; i++)
				scalar_ptr[ i ] = 0;
			
			_scalar	*lp= light_pos.GetPointer();
			_scalar	*pe= plane_equation.GetPointer();
			
			if( !lp || !pe)
				throw 2;
			
			for( i=0; i<4; i++)
				scalar_ptr[ 5*i ] = dot;
				
			for( i=0; i<4; i++)
				for( j=0; j<4; j++)
					*( scalar_ptr + 4 * i + j ) -= (  ( * ( lp + j ) )  *  ( * ( pe + i ) )  ) ;
			
			matrix= new MajaMatrix( &scalar_ptr[0]);
			if( !matrix)
				throw 3;
				
			delete [ ] scalar_ptr;
			return true;
		}
		
		catch( int err_lv){
			switch( err_lv){				
				case 0:
					exit( -1);					
				case 1:
					return false;					
				case 2:
				case 3:
					delete [ ] scalar_ptr;
					return false;					
				default: exit( -1);
			}
		}
	}
};
}
