/*
	keymatch.h
	ce fichier est une partie de Maja - this is a part of Maja
	correspondances du clavier pour glx & windows
	keyboard matches for glx & windows

	$Id: $

	$Log: $

	begin:			Mon Jul 15 2002
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef MAJA_KEYMATCH_H
#define MAJA_KEYMATCH_H

#ifndef WIN32
#include <X11/X.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>

namespace maja
{

#ifdef MAJA_USE_GLUT
#undef MAJA_USE_GLUT
#endif
#else
#warning "Maja: Under Windows, keyboard only supported by glut"
#include <GL/glut.h>
#define MAJA_USE_GLUT
#endif


#ifndef MAJA_KEY
#define MAJA_KEY	int
#endif


typedef MAJA_KEY __MajaKeyMouse;		// deprecated
typedef MAJA_KEY MAJA_KEY_MOUSE;

// they need to be negative until X keys are said unknown values
// (and so on for windows)
const __MajaKeyMouse MAJA_MOUSE_UP=	-111;
const __MajaKeyMouse MAJA_MOUSE_DOWN=	-112;
const __MajaKeyMouse MAJA_MOUSE_LEFT=	-113;
const __MajaKeyMouse MAJA_MOUSE_RIGHT=	-114;
const __MajaKeyMouse MAJA_MOUSE_X=	-115;
const __MajaKeyMouse MAJA_MOUSE_Y=	-116;
const __MajaKeyMouse MAJA_MOUSE_1=	-117;
const __MajaKeyMouse MAJA_MOUSE_2=	-118;
const __MajaKeyMouse MAJA_MOUSE_3=	-119;
const __MajaKeyMouse MAJA_MOUSE_4=	-120;
const __MajaKeyMouse MAJA_MOUSE_5=	-121;


/*
	Maja keys normalization
	All of this is just a replic of what's used in the environnement
	you can find out the matches here in their respective files.
	see pre processor includes for that
	for now, only french AZERTY keyboard is well known, normal alphanum keys
	should stay the same for english (but may be qwerty)
*/
#ifndef MAJA_USE_GLUT

#define MAJA_KEY_NULL					0x0			// no key

// first line of the keyboard
#define MAJA_KEY_ESCAPE					XK_Escape
#define MAJA_KEY_F1					XK_F1
#define MAJA_KEY_F2					XK_F2
#define MAJA_KEY_F3					XK_F3
#define MAJA_KEY_F4					XK_F4
#define MAJA_KEY_F5					XK_F5
#define MAJA_KEY_F6					XK_F6
#define MAJA_KEY_F7					XK_F7
#define MAJA_KEY_F8					XK_F8
#define MAJA_KEY_F9					XK_F9
#define MAJA_KEY_F10					XK_F10
#define MAJA_KEY_F11					XK_F11
#define MAJA_KEY_F12					XK_F12
#define MAJA_KEY_PRINT_SCREEN				XK_Sys_Req
#define MAJA_KEY_STOP_LOOP				XK_Scroll_Lock
#define MAJA_KEY_PAUSE					XK_Pause

// second line
#define MAJA_KEY_SQUARE					0x0			// 	NONE !!!

#define MAJA_KEY_1					XK_KP_1			// 1
#define MAJA_KEY_AND					0x0			// &	NONE !!!
#define MAJA_KEY_2					XK_KP_2			// 2
#define MAJA_KEY_E_AIGU					XK_Eacute		// 
#define MAJA_KEY_e_AIGU					XK_eacute		// 
#define MAJA_KEY_TILDE					XK_asciitilde		// ~ or XK_dead_tilde
#define MAJA_KEY_3					XK_KP_3			// 3
#define MAJA_KEY_DOUBLE_QUOTE				XK_quotedbl		// "
#define MAJA_KEY_DIEZE					XK_ssharp		// #
#define MAJA_KEY_4					XK_KP_4			// 4
#define MAJA_KEY_APOSTROPHE				XK_apostrophe		// '
#define MAJA_KEY_OPEN_BRACE				XK_braceleft		// {
#define MAJA_KEY_5					XK_KP_5			// 5
#define MAJA_KEY_OPEN_PAR				XK_parenleft		// (
#define MAJA_KEY_OPEN_BRACKET				XK_bracketleft		// [
#define MAJA_KEY_6					XK_KP_6			// 6
#define MAJA_KEY_MINUS					XK_minus		// -
#define MAJA_KEY_PIPE					XK_bar			// |
#define MAJA_KEY_7					XK_KP_7			// 7
#define MAJA_KEY_E_GRAVE				XK_Egrave		// 
#define MAJA_KEY_e_GRAVE				XK_egrave		// 
#define MAJA_KEY_GRAVE					XK_grave		// seem it's `
#define MAJA_KEY_8					XK_KP_8			// 8
#define MAJA_KEY_UNDERSCORE				XK_underscore		// _
#define MAJA_KEY_BACKSLASH				XK_backslash		// '\'
#define MAJA_KEY_9					XK_KP_9			// 9
#define MAJA_KEY_C_CEDILLA				XK_XK_Ccedilla		// 
#define MAJA_KEY_c_CEDILLA				XK_ccedilla		// 
#define MAJA_KEY_CIRCUMFLEX				XK_asciicircum		// ^
#define MAJA_KEY_0					XK_KP_0			// 0
#define MAJA_KEY_A_GRAVE				XK_Agrave		// 
#define MAJA_KEY_a_GRAVE				XK_agrave		// 
#define MAJA_KEY_AT					XK_at			// @
#define MAJA_KEY_DEGREE					XK_degree		// 
#define MAJA_KEY_CLOSE_PAR				XK_parenright		// )
#define MAJA_KEY_CLOSE_BRACKET				XK_bracketright		// ]
#define MAJA_KEY_PLUS					XK_plus			// +
#define MAJA_KEY_EQUAL					XK_equal		// =
#define MAJA_KEY_CLOSE_BRACE				XK_braceright		// }
#define MAJA_KEY_BACKSPACE 				XK_BackSpace		// <-
#define MAJA_KEY_INSER					0x0			// INSER	NONE !!!
#define MAJA_KEY_BEGIN					XK_Begin		// begin
#define MAJA_KEY_PAGE_UP				XK_Page_Up		// page up
#define MAJA_KEY_NUMLOCK				XK_Num_Lock		// VERR NUM

// 3rd line
#define MAJA_KEY_TAB					XK_Tab
#define MAJA_KEY_A					XK_A
#define MAJA_KEY_a					XK_a
#define MAJA_KEY_Z					XK_Z
#define MAJA_KEY_z					XK_z
#define MAJA_KEY_E					XK_E
#define MAJA_KEY_e					XK_e
#define MAJA_KEY_R					XK_R
#define MAJA_KEY_r					XK_r
#define MAJA_KEY_T					XK_T
#define MAJA_KEY_t					XK_t
#define MAJA_KEY_Y					XK_Y
#define MAJA_KEY_y					XK_y
#define MAJA_KEY_U					XK_U
#define MAJA_KEY_u					XK_u
#define MAJA_KEY_I					XK_I
#define MAJA_KEY_i					XK_i
#define MAJA_KEY_O					XK_O
#define MAJA_KEY_o					XK_o
#define MAJA_KEY_P					XK_P
#define MAJA_KEY_p					XK_p
#define MAJA_KEY_DEAD_CIRCUMFLEX			XK_dead_circumflex	// ()
#define MAJA_KEY_TRAIMA					0x0			// ()		NONE !!!
#define MAJA_KEY_DOLLAR					XK_dollar		// $
#define MAJA_KEY_STERLING				XK_sterling		// 
#define MAJA_KEY_O_CROSSED				XK_currency		//  ???
#define MAJA_KEY_SUPPR					XK_Delete		// SUPPR
#define MAJA_KEY_END					XK_End			// FIN
#define MAJA_KEY_PAGE_DOWN				XK_Page_Down		// page down

// 3rd & 4th line
#define MAJA_KEY_RETURN					XK_Return		// ENTREE

// 4th line
#define MAJA_KEY_CAPS_LOCK				XK_Caps_Lock		// hope it's not shift lock
#define MAJA_KEY_Q					XK_Q
#define MAJA_KEY_q					XK_q
#define MAJA_KEY_S					XK_S
#define MAJA_KEY_s					XK_s
#define MAJA_KEY_D					XK_D
#define MAJA_KEY_d					XK_d
#define MAJA_KEY_F					XK_F
#define MAJA_KEY_f					XK_f
#define MAJA_KEY_G					XK_G
#define MAJA_KEY_g					XK_g
#define MAJA_KEY_H					XK_H
#define MAJA_KEY_h					XK_h
#define MAJA_KEY_J					XK_J
#define MAJA_KEY_j					XK_j
#define MAJA_KEY_K					XK_K
#define MAJA_KEY_k					XK_k
#define MAJA_KEY_L					XK_L
#define MAJA_KEY_l					XK_l
#define MAJA_KEY_M					XK_M
#define MAJA_KEY_m					XK_m
#define MAJA_KEY_PERCENT				XK_percent		// %
#define MAJA_KEY_U_GRAVE				XK_Ugrave		// 
#define MAJA_KEY_u_GRAVE				XK_ugrave		// 
#define MAJA_KEY_MU					XK_mu			// 
#define MAJA_KEY_ASTERISK				XK_asterisk		// *


// 5th line
#define MAJA_KEY_L_SHIFT				XK_Shift_L
#define MAJA_KEY_GREATER				XK_greater		// >
#define MAJA_KEY_LESS					XK_less			// <
#define MAJA_KEY_W					XK_W
#define MAJA_KEY_w					XK_w
#define MAJA_KEY_X					XK_X
#define MAJA_KEY_x					XK_x
#define MAJA_KEY_C					XK_C
#define MAJA_KEY_c					XK_c
#define MAJA_KEY_V					XK_V
#define MAJA_KEY_v					XK_v
#define MAJA_KEY_B					XK_B
#define MAJA_KEY_b					XK_b
#define MAJA_KEY_N					XK_N
#define MAJA_KEY_n					XK_n
#define MAJA_KEY_QUESTION				XK_question		// ?
#define MAJA_KEY_COMMA					XK_comma		// ,
#define MAJA_KEY_PERIOD					XK_period		// .
#define MAJA_KEY_SEMICOLON				XK_semicolon		// ;
#define MAJA_KEY_SLASH					XK_slash		// '/'
#define MAJA_KEY_DBL_POINT				XK_colon       		// :
#define MAJA_KEY_PARAGRAPH				XK_paragraph		// 
#define MAJA_KEY_EXCLAM					XK_exclam		// ! or XK_exclamdown
#define MAJA_KEY_R_SHIFT				XK_Shift_R		// right shift
#define MAJA_KEY_UP					XK_UP			// up arrow

// 6th line (last one)
#define MAJA_KEY_L_CTRL					XK_Control_L
#define MAJA_KEY_L_ALT					XK_Alt_L
#define MAJA_KEY_SPACE					XK_space
#define MAJA_KEY_R_ALT					XK_Alt_R
#define MAJA_KEY_R_CTRL					XK_Control_R
#define MAJA_KEY_LEFT					XK_LEFT
#define MAJA_KEY_DOWN					XK_DOWN
#define MAJA_KEY_RIGHT					XK_RIGHT

#define MAJA_KEY_SHIFT_LOCK				XK_Shift_Lock

/*#ifndef XK_LATIN1
#ifndef XK_LATIN2
#warning "Seul le clavier franais est reconnu pour le moment"
#endif
#else*/
#define MAJA_KEY_COPYRIGHT				XK_copyright		//  (ALT-GR+Shift+c)
#define MAJA_KEY_HYPHEN					XK_hyphen		// - (???)
#define MAJA_KEY_EURO					0x0			// 		NONE !!!!!!
//#endif

#define MAJA_KEY_DIFF_ASCII				'a'-MAJA_KEY_a

#else // under Win32, define all the same !



#define MAJA_KEY_ESCAPE					27

#define MAJA_KEY_F1					GLUT_KEY_F1
#define MAJA_KEY_F2					GLUT_KEY_F2
#define MAJA_KEY_F3					GLUT_KEY_F3
#define MAJA_KEY_F4					GLUT_KEY_F4
#define MAJA_KEY_F5					GLUT_KEY_F5
#define MAJA_KEY_F6					GLUT_KEY_F6
#define MAJA_KEY_F7					GLUT_KEY_F7
#define MAJA_KEY_F8					GLUT_KEY_F8
#define MAJA_KEY_F9					GLUT_KEY_F9
#define MAJA_KEY_F10					GLUT_KEY_F10
#define MAJA_KEY_F11					GLUT_KEY_F11
#define MAJA_KEY_F12					GLUT_KEY_F12

#define MAJA_KEY_PRINT_SCREEN				0x0
#define MAJA_KEY_STOP_LOOP				0x0
#define MAJA_KEY_PAUSE					0x0
	
#define MAJA_KEY_UP					GLUT_KEY_UP
#define MAJA_KEY_DOWN					GLUT_KEY_DOWN
#define MAJA_KEY_LEFT					GLUT_KEY_LEFT
#define MAJA_KEY_RIGHT					GLUT_KEY_RIGHT

#define MAJA_KEY_INSER					GLUT_KEY_INSERT // 108, cf linux part
#define MAJA_KEY_SUPPR					0x0
#define MAJA_KEY_BEGIN					GLUT_KEY_HOME
#define MAJA_KEY_END					GLUT_KEY_END
#define MAJA_KEY_PAGE_UP				GLUT_KEY_PAGE_UP
#define MAJA_KEY_PAGE_DOWN				GLUT_KEY_PAGE_DOWN
#define MAJA_KEY_NUMLOCK				0x0

#define MAJA_KEY_BACKSPACE 				0x0
#define MAJA_KEY_TAB					0x0
#define MAJA_KEY_RETURN					0x0
#define MAJA_KEY_DELETE					0x0
#define MAJA_KEY_SPACE					XK_space
#define MAJA_KEY_A					'A'
#define MAJA_KEY_Z					'Z'
#define MAJA_KEY_a					'a'
#define MAJA_KEY_z					'z'

#endif // end glut win32 part
}
#endif
