/* mplayer.h, this is a part of Maja
 * Classes declarations
 * Copyright (C) september 2000,2001,2002 Jean-Dominique Frattini
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Cette license est valable en terme de droit franais. Cela signifie que c'est la
 * lgislation franaise qui est en vigueur pour l'ensemble des sources contenues
 * dans la librairie. Sauf image.cpp et image.h, se reposant sur aucune license explicite,
 * mais qui sont fournies aimablement dans les librairies Mesa sous license LGPL.
 *
 * Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou
 * le modifier sous les termes de la license publique gnrale
 * publie par la Free Software Foundation; soit la version 2 de la
 * license, ou (si vous le voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir d'tre utile, mais
 * SANS AUCUNE GARANTIE; et mme une garantie implicite,
 * y compris les garanties de COMMERCIALISATION ou D'ADAPTATION
 * DANS UN BUT SPCIFIQUE. Prenez connaissance de la License Publique
 * Gnrale GNU pour plus de dtail.
 *
 * Vous devez avoir reu une copie de la License Publique Gnrale
 * GNU avec ce programme; si ce n'est pas le cas, vrivez  la
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef MPLAYER_H
#define MPLAYER_H

#include "keymatch.h"

namespace maja
{
/*
	This class controls the interactivity with the keyboard for a player.
	Later, it must manage mouse and joystick.
	Derive from it with a MajaObject polymorph type to another class
	to create a player class as MajaPlayer.
	
	I think this would be better to make a player class (as MajaPlayer)
	derive from a MajaObject class only. MPlayer would only be a friend
	of it. This will keep the entire polymorphism of the MajaObject.

	plus tard, l'utilisateur pourra dfinir le comportement vis  vis des touches:
	si elles doivent tre relches ou pas, par exemple, et a s'applique aussi bien
	 la souris qui est indissocie ici.
	Il y aura aussi plusieurs class 'player' comme MajaSubjectivePlayer, ou
	MajaAngelPlayer ou encore MajaGodPlayer qui dfinissent des faons de jouer, tout
	comme la faon de voir.
*/
class MPlayer
{
	protected:
	
	bool			key_speed;	// appui des touches acclrer ou freiner:
	bool 			key_turn;	// appui sur tourner  gauche ou  droite:
	int  			spd;
	int  			trn;
	int 			mouse;

	MAJA_KEY		forward;
	MAJA_KEY		backward;
	MAJA_KEY		step_right;
	MAJA_KEY		step_left;
	
	public:
	
	int			score;		// an eventual score (if needed) , has to move private
	
	MPlayer( void){
		forward= MAJA_KEY_z;
		backward= MAJA_KEY_s;
		step_right= MAJA_KEY_d;
		step_left= MAJA_KEY_q;
		score=0;
	}

	MPlayer( const MPlayer& m){
		key_speed= m.key_speed;
		key_turn= m.key_turn;
		spd= m.spd;
		trn= m.trn;
		mouse= m.mouse;
		forward= m.forward;
		backward= m.backward;
		step_right= m.step_right;
		step_left= m.step_left;
		score= m.score;
	}
	
	virtual  ~MPlayer(){}
	
	// these 4 methods are to be acceed from a friend class:
	virtual void SetForwardKey( int key)	=0;
	virtual void SetBackwardKey( int key)	=0;
	virtual void SetLeftTurnKey( int key)	=0;
	virtual void SetRightTurnKey( int key)	=0;
	
	virtual void Submit( void *k, void *m)= 0;
};
}
#endif
