/*
	majamenuitem.h
	ce fichier est une partie de Maja - this is a part of Maja
	simple item de menu
	simple menu item

	$Id: $

	$Log: $

	begin:			Fri Jul 5 2002
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef __MAJA_MENUITEM_H
#define __MAJA_MENUITEM_H

#include "majalabel.h"
#include "../../common/majafunctors.h"
#include "../../common/majacom.h"
#include "../../tools/maths/majaquaternion.h"

namespace maja
{
#define MAJA_MENU_INACTIVE 		0x0
#define MAJA_MENU_SLEEP			0X1
#define MAJA_MENU_FOCUS			0x2
#define MAJA_MENU_ACTIVE		0x4

#define MAJA_NO_BUTTON			0x0
#define MAJA_LEFT_BUTTON		0x1
#define MAJA_MIDDLE_BUTTON		0x2
#define MAJA_RIGHT_BUTTON		0x4

/*!
 * some simple menu items.
 * could communicate via a comobject (MajaCom)
 * R is the return value of the com function (here member function)
 * ARG1 is the first (and only) argument to this function.
 * COMOBJECT is any maja compliant communication object (please refer
 * 	to common/majacom.h for more details about her).
 *
 * status: experimental
 */
class MajaMenuItem: public MajaLabel
{
	protected:
	
	GLuint				id;
	
	muint				state;

	public:
	
	MajaMenuItem( mvoid): MajaLabel(){ 
		state= MAJA_MENU_INACTIVE;
	}
	
	virtual ~MajaMenuItem( mvoid){
	}
	
	muint TestMouse( mint x, mint y, mint button_val)
	{
		if( pos.x<= x && x<=pos.x+size.x 
				&& 
				pos.y<=y && y<=pos.y+size.y){
			if( button_val)
				state= MAJA_MENU_ACTIVE;
			else
				state= MAJA_MENU_FOCUS;
		}
		else 
			state= MAJA_MENU_INACTIVE;

		return state;
	}
	
	mvoid SetState( mint s){ 
		state= s;
	}
	
	mint GetState( mvoid){ 
		return state;
	}
	
	GLuint GetId( mvoid){ 
		return id;
	}
	
	mvoid SetId( GLuint idt){ 
		id=idt;
	}

	virtual mcom Action( mvoid){
		return mcom_nothing;
	}
};

template
<
	class		COMOBJECT
>
class MajaMenuItemObject:	public MajaMenuItem, 
				public COMOBJECT
{
	public:
	
	MajaMenuItemObject( mvoid):	MajaMenuItem(),
					COMOBJECT()
	{
		state= MAJA_MENU_INACTIVE;
		label= COMOBJECT::GetLabel();
	}
	
	~MajaMenuItemObject( mvoid){
	}
	
	mcom Action( mvoid){
		return COMOBJECT::operator()( 0);
	}
};

namespace inner
{
	template
	<
		typename R= mcom,
		class ARG1= mint
	>
	struct MajaExit_
	{
		std::string label_;
		
		MajaExit_(){ label_="Exit";}
		
		R operator()( ARG1 i){
			return mcom_exit_program;
		}

		std::string& GetLabel(){
			return label_;
		}
	};

	template
	<
		typename R= mcom,
		class ARG1= mint
	>
	struct MajaStart_
	{
		std::string label_;
		
		MajaStart_(){label_="Start";}
		
		R operator()( ARG1 i){
			return mcom_goto_next;
		}

		std::string& GetLabel(){
			return label_;
		}
	};

	template
	<
		typename R= mcom,
		class ARG1= mint
	>
	struct MajaOption_
	{
		std::string label_;
		
		MajaOption_(){label_="Option";}
		
		R operator()( ARG1 i){
			std::cerr << std::endl << "maja_option_func: starting sub-menuing.";
			return mcom_exit_program;
		}

		std::string& GetLabel(){
			return label_;
		}
	};
}

//! a default policy
template 
<
	class ITEM
>
struct MajaItemPolicy
{
	mcom Focus(	ITEM& item){
		static MAJAMATHS::MajaQuaternion c( 1,0,0,.5);
		item.SetColor( c);

		return mcom_nothing;
	}

	mcom Active(	ITEM& item){
		return item.Action();
	}

	mcom Sleep(	ITEM& item){
		static MAJAMATHS::MajaQuaternion c( .0,.0,.0,.0);
		item.SetColor( c);

		return mcom_nothing;
	}
	
	mcom Inactive(	ITEM& item){
		static MAJAMATHS::MajaQuaternion c( .5,.25,.5,.5);
		item.SetColor( c);

		return mcom_nothing;
	}
};

}
#endif

