/*
	majamesh.h
	this is a part of Maja
	hold loaded models in mesh

	$Id: $

	$Log: $

	begin:			Jun 2002
	copyright:		(C) 2002 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/********************************************************************************
 *										*
 *	This program is free software; you can redistribute it and/or modify	*
 *	it under the terms of the GNU General Public License as published by	*
 *	the Free Software Foundation; either version 2 of the License, or	*
 *	(at your option) any later version.					*
 *	Please read the license (COPYING)					*
 *										*
 ********************************************************************************/

#ifndef __MAJA_MESH_H
#define __MAJA_MESH_H

#include <GL/gl.h>
#include <GL/glu.h>
#include "../../common/majatypes.h"
#include <string>

namespace maja
{
/*!
	A structure for all taken datas from a model
	may contain sub structures for separate:
	struct _point{ vertices, normals, colors, number, indicies};
	struct _material{ materials, ind};
	struct _texture{ tex_name, tex_id, tex_index};... and ofset and scale
	must be long indicies for huge models
	these are no more needed as it's now MajaGModel functionnality
	added: naming of the parts, rotation of the textures,
	adding: 2 textures per object as 3DS permits it
*/
class MajaMesh
{
	public:
	mchar			*name;					//! name of sub parts
	GLfloat	 		*vertices;				//! vertices
	GLfloat 		*normals;				//! normals
	GLfloat			*colors;				//! colors r,g,b,a
	GLfloat			*texels;				//! tex coords
	GLfloat			*texels2;				//! tex coords for the second texturing
	GLfloat			*texels_refl;				//! tex coords for the reflected texture
	GLfloat			*materials;				//! materials in order: ambiant, diffuse, specule, shine
	GLuint			*tex_id;				//! opengl texture identificators
	GLuint			*tex_id2;				//! 2nd texture of the object
	GLfloat			*tex_rot;				//! rotation of the texture
	GLfloat			*tex_rot2;				//! rotation of the texture
	GLuint			*tex_refl;				//! reflection textures
	GLuint	 		*in;					//! indices of the face vertices of the model	
	GLuint			count, last_count;			//! count the vertices, normals and tex coords
	GLuint			m_count;				//! count the materials
	
	MajaMesh( mvoid);
	~MajaMesh( mvoid);
};

// experimental

class MajaVert_
{
	public:
		
	GLfloat		v[4];	// x,y,z,w

	MajaVert_(){
		v[0]=v[1]=v[2]=0;
		v[3]=1;
	}
};

class MajaNorm_
{
	public:


	GLfloat		n[3];	// nx,ny,nz

	MajaNorm_(){
		n[0]=n[1]=n[2]=0;
	}
};

class MajaCol_
{
	public:

	GLfloat		c[4];	// r,g,b,a

	MajaCol_(){
		c[0]=c[1]=c[2]=0;
		c[3]=1;
	}
};

class MajaTex_
{
	public:

	GLfloat		t[3];	// u,v,w
	GLfloat		a;	// alpha

	MajaTex_(){
		a= 0;
		t[0]=t[1]=t[2]=0;
	}
};

class MajaTriTex_
{
	public:

	GLfloat		d[3];	// u=u+d
	GLfloat		s[3];	// u*=s
	GLfloat		r[3];	// u=r(u, O,i/j/k)
	GLfloat		a;	// alpha
	GLuint		i;	// tex id
	
	MajaTriTex_(){
		i= 0;
		a= 0;
		d[0]=d[1]=d[2]=0;
		s[0]=s[1]=s[2]=0;
		r[0]=r[1]=r[2]=0;
	}
};

class MajaMat_
{
	public:

	GLfloat		a[3];	// ambiance
	GLfloat		d[3];	// diffuse
	GLfloat		s[3];	// specule
	GLfloat		h;	// shininess

	MajaMat_(){
		h= 0;
		a[0]=a[1]=a[2]=0;
		d[0]=d[1]=d[2]=0;
		s[0]=s[1]=s[2]=0;
	}
};

class MajaTri_
{
	public:

	MajaVert_	V[3];	// 3 vertices
	MajaNorm_	N[3];	// one normal per v
	MajaCol_	C[3];	// one color per v
	MajaTex_	T[3];	// 3 tex coord (one for each v)
	MajaTex_	T2[3];	// 3 others tex coords (for sec texture)
	MajaTex_	T3[3];	// 3 others (for 3rd texture)
	MajaTriTex_	TT;	// a texture per tri
	MajaTriTex_	TT2;	// a texture per tri
	MajaTriTex_	TT3;	// a texture per tri
	MajaMat_	M;	// one material per triangle

	MajaTri_(){
		;
	}
};

class MajaMesh_
{
	public:

	MajaMesh_	*_n;	// any next to the scene
	MajaMesh_	*_c;	// any child to this one
	std::string	name;	// name of this mesh
	MajaTri_	*tri;	// a pointer to tris
	mlong		tri_nb;	// the nb of tris
	GLenum		type;	// the type of tri organization
	
	MajaMesh_(){
		_n= mnull;
		_c= mnull;
		name= "default mesh";
		tri= mnull;
		tri_nb= 0;
		//type= GL_VERTICES;
	}

	~MajaMesh_(){
		if( tri)
			delete[] tri;
	}

	mbool New( mlong n){
		static mbool first= mtrue;
		
		tri_nb= n;
		tri= new MajaTri_[n];
		if( !tri)
			return mfalse;
		
		first= mfalse;

		return mtrue;
	}
};


}
#endif
