/*
	module.h
	ce fichier est une partie de Maja - this is a part of Maja
	charge des modles 3D - base
	load 3D models - base
	
	$Id: $

	$Log: $

	begin:			
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef __MAJA_MODULE
#define __MAJA_MODULE

#ifdef WITH_DMALLOC
#include <dmalloc.h>
#endif
#include <stdlib.h>
#include "../../tools/maths/majamaths.h"
#include "../../tools/containers/majalist.h"
#include "../graphic/majatextures.h"
#include "../graphic/graphext.h"
#include "../modeler/majamesh.h"

#include "../graphic/majagintegrator.h"

namespace maja
{
using MAJAMATHS::MajaVector;

/*!
	Cette classe ne doit servir qu' charger des modles 3D un par un
	et crer un MajaGMesh du fichier charg.
	Tout ce qui est question de rendu doit donc tre oubli
*/
class MajaModule
{
	protected:
	//! 3ds model file that is loaded
	mchar 			*file_name;
	
	//! potential external texture of the model
	GLuint			external_tex;
	
	//! all datas for the display model
	MajaMesh 		meshes;

	MajaMesh_		*new_mesh;
	MajaMesh_		*cur_mesh;;
	mlong			nb_mesh;
	
	//! important for multi format loading
	MajaModule		*module;	
	
	//! size
	MajaVector		size_min;
	MajaVector		size_max;
	MajaVector		size;
	
	//! pos
	MajaVector		pos;							
	
	//! scale the object before loading
	MajaVector		prescale;
	
	//! new size scale to the object
	MajaVector		resize;
	
	//! some textures
	MajaTextures		maja_textures;
	
	//! count the use of meshes	
//	static GLuint		use_counter;
	
	//! that takes MajaMesh and create MajaGMesh
	MAJAGRAPHIC::MajaGIntegrator		*integrator;
//	INTEGRATOR		*integrator;
	
	//! could be used inside MajaGraphicEngine
	MAJAGRAPHIC::MajaGMesh		*model_mesh;

	public:
	MajaModule( void);
	
	virtual ~MajaModule( void);

	//!
	//!	Pre configuration:
	//!

	//! Load the 3DS model file, give the file location
	virtual bool Load( char *name);
	
	//! Use a starting specific external texture ?
//	void UseExternalTexture( GLuint tex);
	
	//! This save a MajaMesh to a simple poor information file
	bool SaveToFile( char *file);
	
	//! resize related the last size (how to size really?)
	void AutoSizing( MajaVector& size);	
	
	//! set the size (not implemented !!)
	inline void SetSize( MajaVector& size){;}
	
	//! get the size
	inline MajaVector& GetSize( void){ return module->size;}
	
	//! change position
	void AutoPositioning( MajaVector& pos);	//! deprecated
	
	//! get the position
	inline MajaVector& GetPos( void){ return module->pos;}
	
	//! set the position (not implemented!!)
	inline void SetPosition( MajaVector& pos){;}
	
	//! calculate the position
	void CalculatePos( void);
	
	//! useful for setting the object to a unit scale before
	inline void Prescale( MajaVector new_scale){ prescale= new_scale;}
	
	//! Allocate the memory for the mesh
	bool AllocateMesh( void);
	
	//! first step of getting the sizes
	void CalculateExtremums( unsigned int index);
	
	//! calculate the 3D sizes of the object
	void CalculateSize( void);

	//!
	//!	Render mode: display lists
	//!

	protected:

	//! opengl display list of the model,
	//! this is not used anymore (but later yes)
//	GLuint 		dlist;
	
	public:
	//! Create the mesh
	virtual bool Create( void){
		if( !module->Create()) 
			return false;
		return true;
	}
	
	protected:
	//! create new_mesh
	virtual void ReadInformation( void){ 
		module->ReadInformation();
	}
	
	public:
	//! Get the display list id
	//! You could not use it anymore (for now)
/*	inline virtual GLuint GetId( void){ 
		return dlist;
	}
*/
	//!
	//!	Render mode: arrays elements:
	//!

	public:
	
	//! setting a new element in the array drawing method
//	inline void UseNewElement( void){ use_counter++;}
	
//	inline void UnuseElement( void){ use_counter--;}
	
	//! make a mesh understandable by maja
	bool MakeMesh( void);
	
	//! get it: must check if one specified module exists
	MAJAGRAPHIC::MajaGMesh* GetMesh( void);

	MajaMesh& GetBlockMesh( mvoid){
		return module->meshes;
	}

	MajaMesh_* GetMesh( muint nb){
		nb= nb_mesh;
		return new_mesh;
	}
	
	//!
	//!	Tools:
	//!
	
	//! take back values from vertices, need by MajaPath
	//! return number of vertices and vertices in param v
	//! allouer v ? meilleure solution vu que MajaModule doit tre dtruit une fois plus utilis...
//	inline unsigned int GetVertices( float *v){ v= (&meshes.vertices[0]); return meshes.count;}
};
}
#endif
