/* $Id: $
 * load 3ds models
 * Copyright (C) september 2000,2001,2002 by Jean-Dominique Frattini
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Cette license est valable en terme de droit franais. Cela signifie que c'est la
 * lgislation franaise qui est en vigueur pour l'ensemble des sources contenues
 * dans la librairie. Sauf image.cpp et image.h, se reposant sur aucune license explicite,
 * mais qui sont fournies aimablement dans les librairies Mesa sous license LGPL.
 *
 * Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou
 * le modifier sous les termes de la license publique gnrale
 * publie par la Free Software Foundation; soit la version 2 de la
 * license, ou (si vous le voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir d'tre utile, mais
 * SANS AUCUNE GARANTIE; et mme une garantie implicite,
 * y compris les garanties de COMMERCIALISATION ou D'ADAPTATION
 * DANS UN BUT SPCIFIQUE. Prenez connaissance de la License Publique
 * Gnrale GNU pour plus de dtail.
 *
 * Vous devez avoir reu une copie de la License Publique Gnrale
 * GNU avec ce programme; si ce n'est pas le cas, vrivez  la
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: $
 */

#ifndef __MAJA_MODULETDS
#define __MAJA_MODULETDS

#include <lib3ds/file.h>
#include <lib3ds/camera.h>
#include <lib3ds/mesh.h>
#include <lib3ds/node.h>
#include <lib3ds/material.h>
#include <lib3ds/matrix.h>
#include <lib3ds/vector.h>
#include "module.h"

namespace maja
{
/*!
	This class will load standard 3d studio and export 3d studio max 3ds models
*/
class Maja3dsModule: public MajaModule
{
	static float 	MAJA_3DS_RESCALE;		//! global rescale to match with the system coordinate
	
	Lib3dsFile 	*file;				//! 3ds model file that is loaded
	
	Lib3dsNode 	*c,*t, *node;			//! nodes that do the model
	
	char 		*camera;			//! name of the camera found //! does i need a 3DS camera ?
	
	public:
	
	Maja3dsModule( void):MajaModule(){
		file= null;
		c= null;
		t= null;
		node= null;
		camera= null;
	}
	
	~Maja3dsModule( void){ 
		lib3ds_file_free( file);
	}
	
	private:
	
	//! Load the 3DS model file, give the file location
	virtual bool Load( char *name);
	
	//! Create the display list of the the whole model
	virtual bool Create( void);
	
	protected:
	//! create the display list without saving datas
	virtual void ReadInformation( void);
};
}
#endif
