#ifndef __MAJA_SOCKET_H
#define __MAJA_SOCKET_H

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <stdio.h>
#include <sys/un.h>
#include <unistd.h>
#include <iostream>

using namespace std;

#define MAJA_SERVER_NAME "MAJA_GAME_SERVER"
#define MAJA_SERVER_WAITING_TEXT "Server waiting"

enum MajaSocketType
{
	MST_sock_none=	0,
	MST_sock_unix=		AF_UNIX,
	MST_sock_tcp=		AF_INET,
	MST_sock_udp=		32000
};

enum MajaSocketState
{
	MSS_state_close=	0,
	MSS_state_open=	1,
	MSS_state_wait=	2,
	MSS_state_hang=	4
};

class MajaMainNetwork
{
	protected:
	
	const static char	open_msg[ ]= "_MAJA_NET_OPEN";
	const static char	exit_msg[ ]= "_MAJA_NET_EXIT";
	
	MajaSocketType	socket_type;
	MajaSocketState socket_state;
	
	struct _MajaSocket
	{
		union _MajaSockInfo
		{
			struct __MajaSockInet
			{
				struct sockaddr_in	__add_server;
				struct sockaddr_in	__add_client;

			} __sock_inet;
			
			struct __MajaSockUnix
			{
				struct sockaddr_un	__add_server;
				struct sockaddr_un	__add_client;

			} __sock_unix;
			
			_MajaSockInfo( MajaSocketType type= MST_sock_unix)
			{
				;
			}
			
			~_MajaSockInfo( void)
			{
				;
			}
		} _sock_info;
		
		int _socket_server;
		int _socket_client;
		
		long _sizeof_server;
		long _sizeof_client;
		
	} socket;
	
	char			*network_name;
	
	MajaMainNetwork( void)
	{
		socket_type= MST_sock_unix;
		SetNetworkName( MAJA_SERVER_NAME);
	}
	
	virtual ~MajaMainNetwork( void)
	{
		if( socket_state>= MSS_state_open)
			;//Close();
	}
	
	inline void SetSocketType( MajaSocketType type){ socket_type= type;}
	
	inline MajaSocketType GetSocketType( void){ return socket_type;}
	
	inline void SetNetworkName( char *n){
		if( network_name)
			delete [] network_name;
		network_name= new char[ strlen( n)+1];
		if( !network_name)
			cerr << endl << "memory full";
		strcpy( network_name, n);
	}
	
	inline char* GetNetworkName( void){ return network_name;}
	
	virtual bool Open( void)= 0;
	
	virtual bool Close( void)= 0;
};
#endif

