/*
	majaactor.h
	ce fichier est une partie de Maja - this is a part of Maja
	acteurs de jeux
	game actors
	
	$Id: $

	$Log: $

	begin:			Sep 10 2000
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef MAJA_ACTOR_H
#define MAJA_ACTOR_H

#include "../../tools/maths/majapoly.h"
#include "majaobject.h"
#include "majascene.h"
#include "majapath.h"

namespace maja
{

//from majadef.h:
//! acceleration increment,must disappear for a real time functionnality
#define DELTA_SPEED	0.125
//endfrom

enum _MajaViewMode
{
	view_mode_full_dependant,
	view_mode_half_dependant,
	view_mode_not_dependant
};

/*!
	MajaActor.
	Any acting objects as player/cpu managed.
	properties: moves, managed by something else, collisions, lying on MajaPath
	in adding: knows on which path it is, so it could react (in paused)
	adding: independant view system from the direction systme; allows to move
	to one direction and to look at in another one.
*/
class MajaActor: public MajaObject
{
	protected:
	
	//! angle when hurting any obstacle (potential MajaScene)
	MajaVector	hit_angle;
	
	//! weight of it: gravity*mass in newton (may disappear)
	MajaVector 	weight;
	
	//! life_state (not to be used here anymore)
	GLuint 		*life_state;
	
	//! true if hitting an obstacle, else false
	GLuint 		hit;
	
	//! direction vector (seems like speed vector, but is normalized)
	MajaVector 	dir;
	
	//! last saved pos (at previous time)
	MajaVector 	last_pos;
	
	//! before the last saved pos (at before the previous time)
	MajaVector 	bf_l_pos;
	
	//! last saved angles
	MajaVector 	last_angle;
	
	//! first time beeing update
	mbool 		first_update;

	MajaVector	view_angle;

	_MajaViewMode	view_mode;

	public:
	
	MajaActor();
	
	MajaActor( const MajaActor& m);
	
	~MajaActor( mvoid){
		delete life_state;
	}

	//!
	//! collision detection
	//!
	virtual mbool TestHit( MajaObject& loc);
	
	//! collision with a list of objects
	virtual mbool TestHit( MAJACONTAINER::MajaList<MajaScene>& loc);
	
	//!
	//! landing on a floor, ground (with relief)
	//!
	virtual mint ObjectOnFloor( MajaPath *flr);
	
	virtual mint ObjectOnFloor( MAJACONTAINER::MajaList<MajaPath>& list);
	
	//! call it while the actor is alive
	virtual mvoid IsAlive( mvoid);
	
	protected:
	
	//! update position, orientation...	
	virtual mbool Update( mvoid);
	
	public:

	//! check related to arguments (collision detection, ...)
	virtual mbool Realize(	const MAJACONTAINER::MajaList<MajaScene>& s, 
				const MAJACONTAINER::MajaList<MajaPath>& floor, 
				const MAJACONTAINER::MajaList<MajaActor>& plyr);
	
	MajaVector& GetSpeed( mvoid){ 
		return speed;
	}
	
	mvoid DefineLifeState( muint l);
	
	//! Save the actual position, may be protected to avoid user changes that can break the play
	protected:
	mvoid SavePos        ( mvoid);
	
	public:

	//! Set the mass of the object
	inline mvoid DefineMass( mfloat m){ mass= m; weight.x=0; weight.y= -m*GRAVITY; weight.z=0;}

	//! Calculate the new normalized direction vector
	mvoid CalculatePos( mvoid);
	
	//! Anhilihate any moves that hasn't been displayed
	mbool UndoMovement( mvoid);
	
	//! Init: specify the direction, the angle, the speed and the state of life
	virtual mbool Init( MajaVector *d, MajaVector *a, mfloat sp, muint l);
	
	//! calcul de la position: le param est l'intervalle de temps entre deux appels
	virtual mvoid CalculateDir( mvoid);
	
	protected:
	//! test the MajaPath and reacts in function
	virtual mint ReactOnPath( MajaPath& path);
	
	//! user reaction on an abnormal stated MajaPath
	virtual mbool UserReactOnPath( MajaPath& path);
	
	mvoid SetViewMode( _MajaViewMode v){ view_mode= v;}

	_MajaViewMode& GetViewMode( mvoid){ return view_mode;}

	mvoid SetViewAngle( MajaVector a){ view_angle= a;}

	MajaVector& GetViewAngle( mvoid){ return view_angle;}
};
}
#endif
