/*
	majacamera.h
	ce fichier est une partie de Maja - this is a part of Maja
	une simple camra
	a simple camera

	$Id: $

	$Log: $

	begin:			2000
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef MAJA_CAMERA_H
#define MAJA_CAMERA_H

#include "../../common/majacammodes.h"
#include <stdlib.h>
#ifdef WIN32
#include <windows.h>
#endif
#include <GL/gl.h>
#include <GL/glu.h>
#include "majaplayer.h"

namespace maja
{

//! this is the pixel percent of rounding around for players;
//! this is done by cursor moves when playing.
//! sensitivity to 1 is making one round around yourself 
//! each pixel move from the cursor (so quiete fast !)
//! default rating is: 1/33 (quiete useable)
//! another rule for making this could be to compare to
//! the window size (1 round any quarter width...), but
//! this implies we know the window sizes here. quite not
//! thinkable here.
#define MAJA_MOUSE_SENSITIVITY	1/33

#define MAJA_CAMERA_DISTANCE	4


/*!
 * strafing mode: for person viewing
 * this is to be in a player section
 * strafe_mode_null is when not strafing,
 * otherwise one of the others. Their could
 * not have any other values for this type.
 * 
 * status: useable
 */
enum _MajaStrafeMode
{
	strafe_mode_null=	0,
	strafe_mode_right=	1,
	strafe_mode_left=	2
};

/*!
 * A camera shows a portion of the world game and could be
 * manipulating by the application or the user (if the 
 * player avatar is not seenable).
 * 
 * status: useable
 *
 * divers: plus de Maja*Player dans la hirarchie des MO.
 *  la place, crer une classe template dont on pourra
 * hriter ainsi que d'un MO spcifique. L'utilisateur
 * n'aura plus qu' crer son propre type avec les primitives
 * proposes.
*/
class MajaCamera: public MajaSimplePlayer
{
	protected:
	
	//! target : where it's looking at
	MajaVector 	target;
	
	//! orientation of the objective
	MajaVector 	up;
	
	//! distance to the target
	mreal 		distance;

	//! strafing mode (a moving way)
	_MajaStrafeMode	strafe_mode;
	
	//! camera mode
	_MajaCameraMode	mode;

	MajaVector	*live_target;

	public:

	//! It's advised to create a camera by giving the 3 parameters needed for
	//! managing this type
	MajaCamera( mvoid): MajaSimplePlayer(){
		mode=		camera_mode_revolution|camera_mode_follow;
		distance=	MAJA_CAMERA_DISTANCE;
		strafe_mode=	strafe_mode_null;
		live_target=	mnull;
	}
	
	MajaCamera( const MajaCamera& cam): MajaSimplePlayer( (const MajaSimplePlayer&)cam){
		target=		cam.target;
		up=		cam.up;
		distance=	cam.distance;
		mode=		cam.mode;
		strafe_mode=	cam.strafe_mode;
		live_target=	cam.live_target;
	}
	
	~MajaCamera(){
		;
	}

	mvoid IsAlive( mvoid){
		angle= view_angle;
		Update();
		SavePos();
	}

	mvoid Free( mvoid){
		mdouble alpha= view_angle.x;
		mdouble beta=  view_angle.y;
		target= pos;
		target.y+= sin( beta);
		target.z+= cos( beta);
		RotatePoint( pos, alpha, target);
		CalculateUp();
	}

	mvoid Revolution( mvoid){
		mdouble alpha= view_angle.x;
		mdouble beta=  view_angle.y;		
		view_angle.x+= inner_clk.Step();
		view_angle.y+= inner_clk.Step();
		pos= target;
		pos.y+= sin( beta);
		pos.z+= cos( beta);
		RotatePoint( target, alpha, pos);
/*		pos.x+= distance*sin( view_angle.x)*(1-sin( view_angle.y));
		pos.y+= distance*cos( view_angle.y);
		pos.z+= distance*cos( view_angle.x)*(1-sin( view_angle.y));
*/		CalculateUp();
	}

	mvoid Follow( mvoid){
		target= *live_target;
	}

	mvoid CalculateUp( mvoid){
		mdouble alpha= view_angle.x;
		mdouble beta = view_angle.y;
		MAJAMATHS::MajaVector O;

		DefaultUp();
		up.y= sin( beta+pi_2);
		up.z= cos( beta+pi_2);		
		RotatePoint( O, alpha, up);
	}
	
	mvoid DefaultUp( mvoid){
		up.x= 0;
		up.y= 1;
		up.z= 0;
	}
	
	mvoid Refresh( mvoid){
		if( mode & camera_mode_user)
			;
					
		if( mode & camera_mode_own)
			;

		if( mode & camera_mode_follow)
			Follow();
		
		if( mode & camera_mode_revolution)
			Revolution();

		if( mode & camera_mode_pilot)
			;
			
		if( mode & camera_mode_static)			
			;

		if( mode & camera_mode_free)
			Free();
		
		View();
	}
	
	mvoid View( mvoid){
		gluLookAt( pos.x, pos.y, pos.z, target.x, target.y, target.z, up.x, up.y, up.z);
	}
	
	//! set the vision mode, too many parameters must be given, that's a bad way
	//! md is the mode of camera to be used (can changed as desired)
	virtual mvoid SetMode( _MajaCameraMode md){ 
		mode= md;
	}
	
	_MajaCameraMode& GetMode( mvoid){
		return mode;
	}
	
	//! move its target
	mvoid SetTarget( MajaVector t){ 
		target= t;
	}
	
	mvoid SetLiveTarget( MajaVector& t){
		if( !mode&camera_mode_follow)
			mode|=camera_mode_follow;
		live_target= &t;
	}
	
	//! move its orientation
	mvoid MoveUp( MajaVector u){ 
		up=u;
	}
	
	//! get some info about it:
	inline MajaVector& GetTargetPos( mvoid){ 
		return target;
	}
	
	inline MajaVector& GetUpPos( mvoid){ 
		return up;
	}
	
	inline mvoid SetDistance( real d){
		distance= d;
	}
	
	//! submit what happened with user controls
	//! all parameters are pointers to void
	//! k is the keyboard key pressed (MAJA_KEY_NULL if not)
	mvoid Submit( mvoid *k, mvoid *m)
	{
		MajaVector *tmp= (MajaVector*)m;
		static MajaVector last_mouse= *tmp;		
		MAJA_KEY *key= (mint*)k;

		view_angle.x+=((tmp->x-last_mouse.x)*MAJA_MOUSE_SENSITIVITY);
		view_angle.y+=((tmp->y-last_mouse.y)*MAJA_MOUSE_SENSITIVITY);
	
		if( view_angle.x<-pi)
			view_angle.x+=two_pi;
		if( view_angle.x>pi)
			view_angle.x-=two_pi;
		if( view_angle.y<-pi)
			view_angle.y+=two_pi;
		if( view_angle.y>pi)
			view_angle.y-=two_pi;

		last_mouse=*tmp;

		if( *key== forward){
			Accelerate( 1);
			strafe_mode= strafe_mode_null;
		}
		else if( *key== backward){
			Decelerate( 1);
			strafe_mode= strafe_mode_null;
		}
	//	else 
	//		Decelerate( .5);
		
		if( *key== step_right){
			strafe_mode= strafe_mode_right;
			angle.x=view_angle.x-pi/2;
		}
		else if( *key== step_left){
			strafe_mode= strafe_mode_left;
			angle.x=view_angle.x+pi/2;
		}
	}
	
	std::string& PrintInfo( void){
		static std::string t="";
		mchar s[48]="";
		
		sprintf( s, "Position: %f;%f;%f", pos.x, pos.y, pos.z);
		t=s;
	
		return (std::string&)t;
	}
};
}
#endif
