/* $Id: $
 * Classes declarations
 * Copyright (C) september 2000,2001,2002 Jean-Dominique Frattini
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Cette license est valable en terme de droit franais. Cela signifie que c'est la
 * lgislation franaise qui est en vigueur pour l'ensemble des sources contenues
 * dans la librairie. Sauf image.cpp et image.h, se reposant sur aucune license explicite,
 * mais qui sont fournies aimablement dans les librairies Mesa sous license LGPL.
 *
 * Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou
 * le modifier sous les termes de la license publique gnrale
 * publie par la Free Software Foundation; soit la version 2 de la
 * license, ou (si vous le voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir d'tre utile, mais
 * SANS AUCUNE GARANTIE; et mme une garantie implicite,
 * y compris les garanties de COMMERCIALISATION ou D'ADAPTATION
 * DANS UN BUT SPCIFIQUE. Prenez connaissance de la License Publique
 * Gnrale GNU pour plus de dtail.
 *
 * Vous devez avoir reu une copie de la License Publique Gnrale
 * GNU avec ce programme; si ce n'est pas le cas, vrivez  la
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: $
 */

#ifndef __MAJACAR_H
#define __MAJACAR_H

#include "majamultimotion.h"

namespace maja
{
	
/*!
	MajaCar: a class for driving a car
	New way: using now lists for sub-objects
	You could use it if you want a car with 4 wheels,
	if you want more, just derive from it !!
*/
class MajaCar: public MajaMultiMotion
{
	protected:
	//! last time angles of the tires
	MajaVector 		last_wheel_angle;
	
	//! car center rotation coordinates when turning
	MajaVector 		centre;
	
	//! angles of the tires
	MajaVector 		wheel_angle;
	
	//! rayon de la roue
	float				wheel_radius;
	
	//! roar per sec
	float				wheel_freq;
	
	private:
	//! the wheels that the user should use:
	MajaMultiMotion	*front_left_wheel;
	MajaMultiMotion	*front_right_wheel;
	MajaMultiMotion	*back_left_wheel;
	MajaMultiMotion	*back_right_wheel;
	protected:
	
	//! don't use it as it provokes memory leaks...
	struct _headlight
	{
		unsigned int		number;
//		MajaLight *headlight;
		bool		*state;
		
		_headlight( unsigned int nb);
		~_headlight( void);
//		inline bool SetLight( unsigned int nb, MajaLight& light){ this->headlight[nb]= light; return false;}
//		inline MajaLight* GetLight( unsigned int nb){ if( nb>number) return null; return headlight+=nb;}
	} *headlight;
	
	private:
	//! demo_mode: just for test, won't keep it
	bool		demo_mode;	
	public:

	MajaCar( void);
	
	MajaCar( MajaCar& s);
	
	~MajaCar( void);

	//! this not only calculate the direction of the car in relation with
	//! it wheels, but manage a bit these lasts too
	void CalculateDir( void);
	
	//! call it very often to keep the instance up to date
	void IsAlive( void);
	
	protected:
	//! Turn the wheels in according to the speed, this is not turning due to the steerings
	inline virtual void WheelsTurn( void);
	
	public:
	//! this is just existing for developping
	inline void SetDemoMode( bool set){ demo_mode= set;}
	
	//! create headlights for the car, must a day show the light cone
	//! this is not correctly implemented
	inline bool UseHeadlights( GLuint nb);
	
	//! set/get the wheel radius
	inline void SetWheelRadius( float radius){ wheel_radius= radius;}
	
	inline float GetWheelRadius( void){ return wheel_radius;}
	
	protected:
	//! default headlight, this is not correctly implemented
	inline bool AutoHeadlights( bool check);
	
	public:
	//! Set the angles of the wheels (only y for back wheels)
	inline void SetWheelAngle( MajaVector w_angle){ wheel_angle= w_angle;}
	
	//! init the wheels (some freq calculations...)
	void InitWheels( void);
	
	//! get the angles of the wheels
	inline MajaVector GetWheelAngle( void){ return wheel_angle;}
	
	//! set the wheels, one per one (after setting all the sub objects), you MUST
	//! LINK WITH THE CORRECT POINTERS TO YOUR WHEELS, and they
	//! MUST BE THE SAME !
	//! these accept per default some values
	virtual void SetFrontLeftWheel( MajaMultiMotion *flw= NULL);
	virtual void SetFrontRightWheel( MajaMultiMotion *frw= NULL);
	virtual void SetBackLeftWheel( MajaMultiMotion *blw= NULL);
	virtual void SetBackRightWheel( MajaMultiMotion *brw= NULL);
	
	//! now get them
	virtual inline MajaMultiMotion* GetFrontLeftWheel( void){ return front_left_wheel;}
	virtual inline MajaMultiMotion* GetFrontRightWheel( void){ return front_right_wheel;}
	virtual inline MajaMultiMotion* GetBackLeftWheel( void){ return back_left_wheel;}
	virtual inline MajaMultiMotion* GetBackRightWheel( void){ return back_right_wheel;}
};

/*
	A car drive by a user
*/
/*
class MajaCarPlayer: public MajaCar//, public MPlayer
{

};
*/
}
#endif
