/* $Id: $
 * majamultimotion.h, this is a part of Maja
 * Class MajaMultiMotion
 * Copyright (C) september 2000, 2001, 2002 Jean-Dominique Frattini
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Cette license est valable en terme de droit franais. Cela signifie que c'est la
 * lgislation franaise qui est en vigueur pour l'ensemble des sources contenues
 * dans la librairie. Sauf image.cpp et image.h, se reposant sur aucune license explicite,
 * mais qui sont fournies aimablement dans les librairies Mesa sous license LGPL.
 *
 * Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou
 * le modifier sous les termes de la license publique gnrale
 * publie par la Free Software Foundation; soit la version 2 de la
 * license, ou (si vous le voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir d'tre utile, mais
 * SANS AUCUNE GARANTIE; et mme une garantie implicite,
 * y compris les garanties de COMMERCIALISATION ou D'ADAPTATION
 * DANS UN BUT SPCIFIQUE. Prenez connaissance de la License Publique
 * Gnrale GNU pour plus de dtail.
 *
 * Vous devez avoir reu une copie de la License Publique Gnrale
 * GNU avec ce programme; si ce n'est pas le cas, vrivez  la
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: $
 */

#ifndef __MAJAMULTIMOTION_H
#define __MAJAMULTIMOTION_H

#include "majaactor.h"
#include "../../tools/containers/majalist.h"

namespace maja
{
/*!
	In development class for Maja's hierarchy and for special objects
	as a car. so articulation is here
	even if it was not tested enough, you could use this type, or better,
	its derive types.
*/
class MajaMultiMotion: public MajaActor
{
	protected:
  	//! decale from the main object to the more down one
  	mfloat				decale;
  	
  	//! number of sub objects
	GLuint				joints_nb;
	
	//! new way to articulation, is useable
	MAJACONTAINER::MajaPList<MajaMultiMotion, MajaVector>	sub_objects;
	
	public:	
	MajaMultiMotion( mvoid);
	
	MajaMultiMotion( const MajaMultiMotion& s);
	
	virtual ~MajaMultiMotion( mvoid);

	//! The object is putted on the floor in relation with its contact points, think maybe abstract
	mint ObjectOnFloor( MajaPath *flr);
	
	//! Test hit may be the same as MajaSimpleMotion
	mbool TestHit( MajaObject& loc);
	
	mvoid IsAlive( mvoid);
	
	//! New way to add sub objects to articulation
	GLuint AddSubObject( MajaMultiMotion& sub, MajaVector& contact);
	
	//! check if all the states are correctly defined, useful for test
	mbool Check( mvoid);
	
	protected:
	
	//! Calculate distance between minus height of a vertex and the ground
	mvoid CalculateDecale( mvoid);
};

/*!
	Will contain datas, methods for making a man moving in the world
	i only have begin to display a humanoid running in other program
	so maybe a day i could put that in it
*/
class MajaHumanoid: public MajaMultiMotion
{
	public:
	MajaHumanoid( mvoid): MajaMultiMotion(){}
	MajaHumanoid( MajaHumanoid& s){ MajaMultiMotion::MajaMultiMotion( (MajaMultiMotion&)s);}
};
}
#endif
