/* $Id: $
 * Class MajaPath, this is a part of Maja
 * Copyright (C) september 2000, 2001, 2002 Jean-Dominique Frattini
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Cette license est valable en terme de droit franais. Cela signifie que c'est la
 * lgislation franaise qui est en vigueur pour l'ensemble des sources contenues
 * dans la librairie. Sauf image.cpp et image.h, se reposant sur aucune license explicite,
 * mais qui sont fournies aimablement dans les librairies Mesa sous license LGPL.
 *
 * Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou
 * le modifier sous les termes de la license publique gnrale
 * publie par la Free Software Foundation; soit la version 2 de la
 * license, ou (si vous le voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir d'tre utile, mais
 * SANS AUCUNE GARANTIE; et mme une garantie implicite,
 * y compris les garanties de COMMERCIALISATION ou D'ADAPTATION
 * DANS UN BUT SPCIFIQUE. Prenez connaissance de la License Publique
 * Gnrale GNU pour plus de dtail.
 *
 * Vous devez avoir reu une copie de la License Publique Gnrale
 * GNU avec ce programme; si ce n'est pas le cas, vrivez  la
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: $
 */

#ifndef __MAJAPATH_H
#define __MAJAPATH_H

#include "majascene.h"

namespace maja
{
	
/*!
	section: MajaPath behaviors
*/
#define MAJA_PATH_OFF		0		//! inactive MajaPath
#define MAJA_PATH_ON		1		//! active

//! under the MajaPath: how to do for more than 1 object with different behavior (1 under & 1 over ?)
#define MAJA_PATH_UNDER		2				

#define MAJA_PATH_OVER		4		//! over (on the path)
#define MAJA_PATH_NORMAL	8		//! normal behavior: objects are on the path if 4 & 1 are validated
#define MAJA_PATH_ABNORMAL	16		//! user behavior if ~

/*!
	Terrain on which most MajaObject couldn't transgress its laws.
	First of all, all the objects lies on it as due to a gravity that we
	could applicate.
	ex: A road, a ground with relief.
*/
class MajaPath: public MajaScene
{
	protected:
		
	//! all the vertices of the display gllist
	mfloat 	*vertices;
	
	//! state: behavior of this MajaPath
	GLuint	state;
	
	//! number of vertices
	mlong   vertices_nb;

	// new way (last way!)
	struct info_{
		mfloat *v;
		mfloat n;
		mchar  *name;

		info_(){
			v= mnull;
			n= 0;
			name= mnull;
		}
		mchar* GetName(){return name;}
	};

	MAJACONTAINER::MajaList<info_>		info;
	
	public:	
	
	//! prefer this one
	MajaPath( mvoid):MajaScene(){
		vertices= mnull;
		state= MAJA_PATH_OFF;
		vertices_nb= 0;
	}
	
	//! just for polymorphism full compliance
	MajaPath( const MajaPath& m): MajaScene( (const MajaScene&)m){
		;
	}
	
	~MajaPath( mvoid){ 
		;
	}

	//!TODO: need to overload Load() in order to get all the data...

	//! Get the vertices, faces
	inline mfloat* GetVertices( mvoid){ return vertices;}

	//! Set
	//! avoid using these functions
	inline mvoid 	SetVertices( mfloat *t_param, mlong nb){ 
		vertices= t_param; 
		vertices_nb= nb;
	}
	
	//! define the behavior of this path: could be one or any addition of the define section: MajaPath behaviors at the top of the file
	inline mvoid 	SetState( GLuint s){ state= s;}
	inline GLuint 	GetState( mvoid){ return state;}
	
	inline mlong	GetVerticesNb( mvoid){ return vertices_nb;}

	mbool Load( mvoid)
	{	
		if( mod)
			delete mod;
		mod= mnull;
		mod= new MajaModule();	
		if( !mod)
			return mfalse;

		if( !mod->Load( (mchar*)model_file.c_str())){
			std::cerr << std::endl << "error while loading model.";
			return mfalse;
		}
		size= mod->GetSize();
		pos= mod->GetPos();

		vertices_nb= mod->GetBlockMesh().count;
		vertices= mod->GetBlockMesh().vertices;

		std::cerr << std::endl << "MajaPath: " << vertices_nb << " for &v= " << vertices;
		
		return mtrue;
	}
};

}

#endif
