/*
	majasengine.h
	ce fichier est une partie de Maja - this is a part of Maja
	un monde et son moteur
	a world and its engine

	$Id: $

	$Log: $

	begin:			Nov 2002
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef MAJA_SCENERY_ENGINE_H
#define MAJA_SCENERY_ENGINE_H

#include "majaobject.h"
#include "majascene.h"
#include "majapath.h"
#include "majaactor.h"
#include "majaplayer.h"		// doit tre une base et non un type dfini pour mplayer et non majaplayer...
#include "../../tools/containers/majalist.h"
#include "../../tools/maths/majavector.h"

namespace maja
{
namespace MAJASCENERY
{
	class MajaWorld: public MajaObject
	{
		protected:
		
		MAJACONTAINER::MajaList<MajaObject>	object_list;
		
		MAJACONTAINER::MajaList<MajaScene>	scene_list;
		
		MAJACONTAINER::MajaList<MajaPath>	path_list;
		
		MAJACONTAINER::MajaList<MajaActor>	actor_list;

		MAJACONTAINER::MajaList<MajaActor>	player_list;

		public:

		MajaWorld( mvoid){
			;
		}

		mvoid AddToObjectList( MajaObject& o){ 
			object_list.AddObject( &o);
		}

		mvoid AddToSceneList( MajaObject& o){ 
			scene_list.AddObject( (MajaScene*)&o);
		}

		mvoid AddToPathList( MajaObject& o){ 
			path_list.AddObject( (MajaPath*)&o);
		}

		mvoid AddToActorList( MajaObject& o, mbool is_player= mfalse){ 
			actor_list.AddObject( (MajaActor*)&o);
			if( is_player)
				player_list.AddObject( (MajaActor*)&o);
		}
	
		//! configure the world before could game on it.
		//! This set all objects to fit the terrains relief.
		mbool Configure( mvoid){
			MAJACONTAINER::MajaWidget<MajaObject>	*tmp_obj= mnull;
			MAJACONTAINER::MajaWidget<MajaScene>	*tmp_scn= mnull;
			MajaActor				tmp_act;

			if( object_list.GetNumber() && (tmp_obj= object_list.GetFirst()))
				do{
					if( tmp_obj->GetObject()->GetGravitySubmission()){
						tmp_act.SetPos( tmp_obj->GetObject()->GetPos());
						tmp_act.GetPos().Print();
						tmp_act.SetSize( tmp_obj->GetObject()->GetSize());
						tmp_act.SetOrientation( tmp_obj->GetObject()->GetOrientation());
						tmp_act.ObjectOnFloor( path_list);
						tmp_act.GetPos().Print();
						tmp_obj->GetObject()->SetPos( tmp_act.GetPos());
					}
					tmp_obj= tmp_obj->GetNext();
				}while( tmp_obj);

			if( scene_list.GetNumber() && (tmp_scn= scene_list.GetFirst()))
				do{
					if( tmp_scn->GetObject()->GetGravitySubmission()){
						tmp_act.SetPos( tmp_scn->GetObject()->GetPos());
						tmp_act.GetPos().Print();
						tmp_act.SetSize( tmp_scn->GetObject()->GetSize());
						tmp_act.SetOrientation( tmp_scn->GetObject()->GetOrientation());
						tmp_act.ObjectOnFloor( path_list);
						tmp_act.GetPos().Print();
						tmp_scn->GetObject()->SetPos( tmp_act.GetPos());
					}
					tmp_scn= tmp_scn->GetNext();
				}while( tmp_scn);

			return mtrue;
		}
	};

	class MajaWorldEngine: public MajaWorld
	{
		protected:

		mint			_control;
		MAJAMATHS::MajaVector 	*_mouse;
			
		public:

		MajaWorldEngine( mvoid): MajaWorld(){
			_control= MAJA_KEY_NULL;
			_mouse= mnull;
		}

		mvoid Run( mvoid){
			MAJACONTAINER::MajaWidget<MajaObject>	*tmp_obj= mnull;
			MAJACONTAINER::MajaWidget<MajaActor>	*tmp_act= mnull;
			MajaSimplePlayer 			*tmp_pla= mnull;

			if( object_list.GetNumber() && (tmp_obj= object_list.GetFirst()))
				do{
					tmp_obj->GetObject()->Presence();
					tmp_obj= tmp_obj->GetNext();
				}while( tmp_obj);

			if( scene_list.GetNumber() && (tmp_obj= (MAJACONTAINER::MajaWidget<MajaObject>*)scene_list.GetFirst()))
				do{
					tmp_obj->GetObject()->Presence();
					tmp_obj= (MAJACONTAINER::MajaWidget<MajaObject>*)tmp_obj->GetNext();
				}while( tmp_obj);

			if( path_list.GetNumber() && (tmp_obj= (MAJACONTAINER::MajaWidget<MajaObject>*)path_list.GetFirst()))
				do{
					tmp_obj->GetObject()->Presence();
					tmp_obj= (MAJACONTAINER::MajaWidget<MajaObject>*)tmp_obj->GetNext();
				}while( tmp_obj);

			if( actor_list.GetNumber() && (tmp_act= (MAJACONTAINER::MajaWidget<MajaActor>*)actor_list.GetFirst()))
				do{
					tmp_act->GetObject()->IsAlive();
					tmp_act->GetObject()->Realize( scene_list, path_list, actor_list);
					tmp_act= (MAJACONTAINER::MajaWidget<MajaActor>*)tmp_act->GetNext();
				}while( tmp_act);

			if( player_list.GetNumber() && (tmp_act= (MAJACONTAINER::MajaWidget<MajaActor>*)player_list.GetFirst()))
				do{
					tmp_pla= (MajaSimplePlayer*)tmp_act->GetObject();
					tmp_pla->Submit( (mvoid*)&_control, (mvoid*)_mouse);
					tmp_act= (MAJACONTAINER::MajaWidget<MajaActor>*)tmp_act->GetNext();
				}while( tmp_act);
		}

		//! setting up the keyboard and the mouse
		mvoid Set( mint *i, MAJAMATHS::MajaVector *m){
			
			if( !i){
				std::cerr << std::endl << "MajaWorldEngine: ERROR: i is a null pointer.";
				return;
			}
			if( !m){
				std::cerr << std::endl << "MajaWorldEngine: ERROR: m is a null pointer.";
				return;
			}
			_control= *i;
			_mouse= m;
			
		}
	};
/*
	template 
	<
		class C, 
		class I
	>
	class MajaScenery
	{
		MajaList<C>	camera_list;

		MajaList<I>	inter_list;

		public:

		MajaScenery( void){
			;
		}
	};

	class MajaSceneryEngine*/
}
}
#endif
