/*
	majathread.h
	ce fichier est une partie de Maja - this is a part of Maja
	gestion des threads
	threads management

	$Id: $

	$Log: $

	begin:			Mon Jul 8 2002 (from majascheduler.h)
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

/*!
	basic thread objects needed for scheduling
*/

#ifndef __MAJA_THREAD_H
#define __MAJA_THREAD_H

#include <string>
#include <iostream>
#include <errno.h>
#include "../../common/majatypes.h"

namespace maja
{
/*!
	namespace for managing threads
*/
namespace MAJATHREAD
{
	#ifndef WIN32
		#include <unistd.h>
		#ifndef _POSIX_PRIORITY_SCHEDULING
			#warning "Maja: No real-time is supported"
		#else
			#include <sched.h>
			#ifdef _POSIX_THREAD_PRIORITY_SCHEDULING
				#include <pthread.h>
				#ifndef _MAJA_THREADED
					#define _MAJA_THREADED 1
				#endif
			#else
				#warning "Maja: No real-time thread supported"
			#endif
		#endif
	#else
		#include <windows.h>
	#endif

	/*!
	 * it should have many thread components of MajaSModule define in majascheduler.h
	 */
	class MajaThread
	{
		protected:

		//! internal counter of threads
		static muint		count;
		
		protected:
		//! the pthread type
		pthread_t		thread;
		
		//! attributes for this thread
		pthread_attr_t		attributes;
		
		//! any parameters
		mvoid			*param;
		
		//! any sched_param, avoid using it
		struct sched_param	sched_params;
		
		//! priority must be btw min & max
		mint			priority;
		
		//! actual min priority for the scheduling algo
		mint			priority_min;
		
		//! idem for max
		mint			priority_max;
		
		//! schedule algo actually used
		mint			maja_schedule_algo;

		public:
		
		MajaThread( mvoid);

		virtual ~MajaThread( mvoid);

		virtual mvoid* Go( mvoid* arg){ return mnull;}

		//! Creating the thread, you could change the parameters here
		mint Create( mvoid *p=mnull);

		//! Set the parameters of this thread
		inline mvoid SetParam( mvoid *p){ param=p;}

		//! Get the parameters of this thread
		inline mvoid* GetParam( mvoid){ return param;}

		//! Joining this thread
		inline mvoid Join( mvoid){ pthread_join( thread, mnull);}

		//! Get a pointer to the pthread
		inline pthread_t* GetThreadId( mvoid){ return &thread;}
		
		protected:
		
		//! Set the pthread here (avoid use it)
		inline mvoid SetThreadId( pthread_t& t){ thread= t;}

		public:
		
		//! Get the thread attributes
		inline pthread_attr_t& GetAttributes( mvoid){ return attributes;}

		//! Set the thread attributes
		inline mvoid SetAttributes( pthread_attr_t& attr){ attributes= attr;}

		//! Get the scheduling parameters
		//! This works only if you are in real-time scheduling
		inline  sched_param& GetSchedParams( mvoid){ return sched_params;}

		//! Set the scheduling parameters
		inline  mvoid SetSchedParams( struct sched_param& p){ sched_params= p;}

		//! Setting the min & max priority
		//! generally, this is done internally by Maja, but you may need to match your
		//! system min & max real-time priority
		inline mvoid SetPriority( mint min, mint max){ priority_min=min; priority_max= max;}

		//! Setting the priority of this thread (if in real-time only)
		inline mvoid SetPriority( mint prior){ priority= prior;}

		//! Getting the min & max priorities relative to this thread context
		inline mvoid GetPriority( mint *min, mint *max){ min=&priority_min; max=&priority_max;}

		//! Getting this thread's priority
		inline mvoid GetPriority( mvoid){
			priority_min= sched_get_priority_min( maja_schedule_algo);
			priority_max= sched_get_priority_max( maja_schedule_algo);
		}

		//! the actual way of scheduling
		inline mint GetScheduleAlgo( mvoid){ 
			return maja_schedule_algo;
		}

		//! how we schedule this module ? 3 ways are offered
		inline mvoid SetScheduleAlgo( mint algo){ maja_schedule_algo= algo;}
	};
}
}//namespace maja
#endif
