/*
	mutex.h
	ce fichier est une partie de Maja - this is a part of Maja
	mutex et conditions
	mutex and conditions
	
	$Id: $

	$Log: $

	begin:			Fri Jul 5 2002
	copyright:		(C) 2000, 2001, 2002 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	Contactez-nous pour toute tentative d'utilisation commerciale, svp.			*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *	Contact us for any commercial use attempt, please.					*
 *												*
 ************************************************************************************************/

#ifndef __MAJA_MUTEX_H
#define __MAJA_MUTEX_H

#ifndef WIN32
#ifndef GNU_SOURCE
#define GNU_SOURCE
#endif
#include <unistd.h>
#endif
#include <pthread.h>

namespace maja
{
	
namespace MAJATHREAD
{
//! this is not 'til used !!
enum __MAJA_MUTEX_TYPE{
	MMT_mutex_fast=		PTHREAD_MUTEX_FAST_NP,			// 2 locks in the same threads provoke a system hang
	MMT_mutex_recursive=	PTHREAD_MUTEX_RECURSIVE_NP,		// locks are countered and, so one locked is waiting
	MMT_mutex_secure=	PTHREAD_MUTEX_ERRORCHECK_NP		// if could not lock, fail
};

class MajaMutex
{
	//! the mutex identifior
	pthread_mutex_t		mutex;

	//! attributes of this mutex
	pthread_mutexattr_t	mut_attr;

	// structure managing conditions
	struct _MajaCondition
	{
		//! matching condition waiting
		pthread_cond_t		_cond;

		//! attributes for the conditions
		pthread_condattr_t	_cond_attr;

		timespec		*_int_date;

		_MajaCondition( void){ 
			_int_date= NULL;
		}
		
		~_MajaCondition( void){
			;
		}
		
	}			*cond;

	unsigned int		cond_counter;

	public:

// TODO: array of conditions, if needed (cond_nb>1)
	MajaMutex( unsigned int cond_nb= 0): cond_counter( cond_nb){
		pthread_mutexattr_init( &mut_attr);
		pthread_mutex_init( &mutex, &mut_attr);
		if( !cond_nb){
			cond= NULL;
			return;
		}
		if( cond_nb==1)
			cond= new _MajaCondition();
		else
			return;
		pthread_condattr_init( &cond->_cond_attr);
		pthread_cond_init( &cond->_cond, &cond->_cond_attr);
	}

	~MajaMutex( void){
		if( cond){
			pthread_cond_destroy( &cond->_cond);
			pthread_condattr_destroy( &cond->_cond_attr);
			delete cond;
		}
		pthread_mutex_destroy( &mutex);
		pthread_mutexattr_destroy( &mut_attr);
	}

	//
	// MUTEX ONLY AREA
	//
	
	inline int Lock( void){ return pthread_mutex_lock( &mutex);}

	#ifdef __USE_XOPEN2K
	inline int TimedLock( int t= 10){ 
		return pthread_mutex_timedlock( &mutex, (const timespec*)t);
	}
	#endif

	inline int Unlock( void){ return pthread_mutex_unlock( &mutex);}

	inline int TryLock( void){ return pthread_mutex_trylock( &mutex);}

	inline void SetType( __MAJA_MUTEX_TYPE type){
		pthread_mutexattr_settype( &mut_attr, type);
	}

	inline int GetType( __MAJA_MUTEX_TYPE *type){
		return 0;//pthread_mutexattr_gettyoe( &attributes, type);
	}

	//
	// COND ONLY AREA
	//
	
// TODO: if more than one cond existing
	inline int CondSignal( unsigned int cond_nb= 1){
		if( !cond || !cond_nb) return -2;
		return pthread_cond_signal( &cond->_cond);
	}

	inline int CondWait( void){ return pthread_cond_wait( &cond->_cond, &mutex);}

	inline int CondTimedWait( const timespec *date= NULL){
		return pthread_cond_timedwait( &cond->_cond, &mutex, date ? date: cond->_int_date);
	}

	//
	// CLASS MANAGING AREA
	//
	pthread_mutex_t* GetMutex( void){ return &mutex;}
};
}//namespace MAJATHREAD
}//namespace maja
#endif

