/*
	majamainwindow.h
	this is a part of Maja
	window for gl
	
	$Id: $

	$Log: $

	begin:			June 2002
	copyright:		(C) 2000, 2001, 2002 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef _MAJA_MAINWINDOW__H
#define _MAJA_MAINWINDOW__H

#include <iostream>
#include "winattr.h"
#include "../../core/majaclock.h"
#include "../../core/majastates.h"
#include "../../tools/containers/majastack.h"
#include "../th/mutex.h"

namespace maja
{
	
#ifdef SHOW_WIN_FPS
#undef SHOW_WIN_FPS
#endif

#ifdef MAJA_WIN_VERSION_0_1
#undef MAJA_WIN_VERSION_0_1
#endif

/*!
 * this is in work in progress.
 * You may could use this quiete safely now.
 * TODO: 
 * These lines below list all the task to get the 0.1 version of
 * the window part. Winodws support could be report to 0.2 for conveniency.
 * glx 1.3 support (in order to expect a better multi-thread support)
 * Microsoft Windows OpenGL window support
 */

#ifndef WIN32
	//! the X11 window type
	typedef Window			_MajaWindow;

	//! the GLX context
	typedef GLXContext		_MajaContext;

	//! the GLX frame buffer config
	typedef GLXFBConfig		_MajaConfig;

	typedef GLXDrawable		_MajaDrawable;

	//! best X resolution mode (may change)
	typedef mint			_MajaBestMode;

	//! event relative to the window
	typedef XEvent			_MajaWinEvent;
#else
	//! the windows window type
	typedef HWND 			_MajaWindow;

	//! handle window context
	typedef HWC			_MajaContext;

	//! these types may not exist under windows
	typedef mint			_MajaConfig;
	typedef mint			_MajaBestMode;
	typedef mint			_MajaWinEvent;
#endif

//! window widget event that could stand in a stack
struct _MajaWidgetEvent
{
	_MajaWinEvent	_e;
	
	mchar		*_n;

	_MajaWidgetEvent( mvoid): _n( mnull){
		;
	}

	_MajaWidgetEvent( _MajaWinEvent e): _e( e), _n( mnull){
		;
	}

	_MajaWidgetEvent( const _MajaWinEvent e, const mchar *n): _e( e), _n( new mchar[strlen( n)+1]){
		if( _n)
			strcpy( _n, n);
	}

	_MajaWidgetEvent( const _MajaWidgetEvent& w){
		_e= w._e;
		_n= new mchar[strlen( w._n)+1];
		if( _n)
			strcpy( _n, w._n);
	}

	~_MajaWidgetEvent( mvoid){
		if( _n)
			delete _n;
	}

	mchar* GetName( mvoid){
		return _n;
	}

	mvoid SetName( mchar *n){
		_n= n;
	}
};

//! the mutex must be managed from the exterior;
//! this is currently the case of the graphic module.
template 
<
	class MDISPLAY=Display,
	typename MCONTEXT=_MajaContext,
	class MWINDOW= _MajaWindow
> // would have the way :X11 or W32
class MajaDrawable_: public MAJATHREAD::MajaMutex
{
	public:
		
	MDISPLAY	*_d;
	
	protected:
	
	MDISPLAY	*_r;
	
	public:

	//! the opengl context
	MCONTEXT	_c;

	private:
	
	MCONTEXT	_t;

	// crer le contexte avant de le rendre actif.
	GLXContext 	shared_context;
	
	public:

	//! info about visual
	XVisualInfo	*_v;

	MWINDOW 	_w;

	_MajaDrawable	dr;
	_MajaDrawable	re;

	//! window configuration
	_MajaConfig	*config;

	MajaDrawable_(){
		_d=mnull;
		_r=mnull;
		_v=mnull;
		_c=mnull;
		_t=mnull;
		config= mnull;
		shared_context= mnull;
	}
	
	inline mvoid CreateDisplay( mvoid){
		if( _d){
			_r= _d;
			CloseDisplay( _d);
		}
		_d= XOpenDisplay( mnull);	
		if( !_d){
			std::cerr << std::endl << "No display for X";
			throw std::exception();
		}
	}

	inline mvoid CloseDisplay( MDISPLAY *d= mnull){
		if( _c)
			glXDestroyContext( _d, _c);
		if( (_d && !d) || (d && !_d))
			XCloseDisplay( d?d:_d);
	}

	public:

	// this need an over locking if it's often used... but only in creation
	inline MCONTEXT CreateContext( mvoid){
		
		if( _c)
			_t=_c;

		if( !_d)
			throw std::exception();
		
		//#ifdef GLX_VERSION_1_3
		//_c= glXCreateNewContext( _d, *config, GLX_RGBA_TYPE, shared_context, GL_TRUE);
		//#else
		_c= glXCreateContext( _d, _v, mnull, GL_TRUE);
		//#endif

		if( !_c){
			std::cerr << " context failed to create." << std::flush;
			throw std::exception();
		}

		return _c;
	}

	MCONTEXT GetMainContext(){
		return _c;
	}

	MCONTEXT GetForeignContext(){
		return shared_context;
	}

	mvoid Enter( MCONTEXT c= mnull){
		
		/*#ifdef GLX_VERSION_1_3
		if( !glXGetCurrentContext())
			glXMakeContextCurrent( _d, dr, dr, c?c:_c);
		#else
		*/glXMakeCurrent(	_d, _w, c?c:_c);
		//#endif
	}

	mvoid Leave( mvoid){
		glXWaitGL();
	}

	mbool CreateWindow( int *glx_attr){

		//dr= glXCreateWindow( _d, *config, _w, glx_attr);
		//re= glXCreatePbuffer( _d, *config, glx_attr);

		return mtrue;
		//return dr&&re;
	}

	mvoid SwapBuffers(){
		glXSwapBuffers( _d, _w);
	}
};

#ifndef WIN32
typedef MajaDrawable_<> MajaDrawable;
#else
typedef MajaDrawable_<MajaEmptyType,HWC> MajaDrawable;
#endif

/*
template<
	class A,
	class CONTEXT,
	class C
>
class Drawer_
{
	;
};

template<
	class ContextManager
>
class MajaDrawable: public ContextManager
{
	Enter();
	Leave();
	New();
}*/

/*!
 * simple window viewed by Maja
 * Their should only have one instance of a own derived class in a normal application.
 * should do polymorphic btw X and Windows
 * it looks like it's some about huge
 * TODO: disperse into some other classes. Their should also have a MajaWindowManager
 * that will be the 2nd layer of the core, directly linked to, and who will manage the
 * connection with the appropriate modules: MajaSRender & MajaSInteractivity which are
 * using window.
 * TODO: must be threadful compliant.
 */
//template <class DRAWABLE=Display>
class MajaWindow: public MAJATHREAD::MajaMutex
{
	protected:

	_MajaBestMode		best_mode;

	//! maja global attributes for a gl mode window
	MajaWinAttr		attributes;

	//! if true, means that the window was created succesfully
	mbool			window_created;

	//! state of the window
	__MajaWindowState	state;

	//! if direct rendering is enabled
	mbool			direct_render;

	//! internal window clocking
	MajaClock		c_clock;

	muint 			fps;

	muint 			fpsave;

	public:
	
	MAJATHREAD::MajaMutex	event_mutex;

	protected:

	MajaDrawable		drawable;

	//! window-relative event
	_MajaWinEvent		event;

	//! window management related events
	MAJACONTAINER::MajaStack<_MajaWidgetEvent> *event_stack;

	//! key/mouse related events
	MAJACONTAINER::MajaStack<_MajaWidgetEvent> *key_event_stack;

	public:
	
	MajaWindow( mvoid){
		window_created= mfalse;
		state= MWS_state_dead;
		direct_render= mfalse;
		best_mode= 0;
		fps=0;
		fpsave=0;
		event_stack= mnull;
		key_event_stack= mnull;
	}

	protected:

	MajaWindow( MajaWindow& win){
		event_stack= win.event_stack;
		key_event_stack= win.key_event_stack;
	}

	virtual ~MajaWindow( mvoid)
	{
		;
	}

	public:

	//! Create the window defined with MajaWinAttr
	//! it's transparent from Linux or Window
	virtual mbool CreateWindow( mvoid)=0;

	//! Destroy the window
	virtual mvoid DestroyWindow( mvoid)= 0;

	protected:

	//! take care about thread and windowing
	//! you must call it before any other call to functions
	//! that make use of X functions !! So, you may declare
	//! any instance of MajaWindow as global and almost as
	//! the first one.
	virtual mbool CheckThread( mvoid)= 0;

	//! take care about what is supported (versions, ...)
	virtual mbool CheckSupport( mvoid)= 0;

	public:
	
	//! check if direct rendering is enabled
	virtual mbool CheckDirectRender( Display *d, GLXContext c)= 0;

	//! defines with the attributes you give
	virtual mvoid SetAttributes( MajaWinAttr &attr)= 0;
	
	//! some things to do before each drawing at screen
	virtual mvoid PreDraw()=0;
	
	//! some things to do after each drawing
	virtual mvoid PostDraw()= 0;
	
	//! redraw: all what's inside will be seen at screen
	virtual mvoid Redraw( mvoid)=0;

	virtual mvoid HudDraw( mvoid)=0;

	virtual mvoid PreHudDraw( mvoid)=0;

	virtual mvoid PostHudDraw( mvoid)=0;

	//! global drawing method
	virtual mvoid Draw()=0;

	//! resize the window
	virtual mvoid Resize( mvoid)= 0;
	
	//! change to fullscreen (not working)
	virtual mvoid SubFullscreenSwitch()= 0;

	//! return if the window was created
	virtual mbool GetCreated( mvoid)= 0;

	//! return the actual state of the window
	virtual __MajaWindowState GetState( mvoid)= 0;

	//! set the state
	virtual mvoid SetState( __MajaWindowState s)= 0;
	

	virtual MajaDrawable& GetDrawable( mvoid)= 0;
	
	
	//! get a ref to the window
	virtual _MajaWindow& GetWindow( mvoid)= 0;

	//! get them
	virtual MajaWinAttr& GetAttributes( mvoid)= 0;

	//! one event loop
	//! so call it whenever you want events about your window
	virtual _MajaWinEvent& LoopEvent( mvoid)= 0;

	virtual _MajaWinEvent& GetEvent( mvoid)= 0;

	virtual mvoid SetEvent( _MajaWinEvent& ev)= 0;

	virtual mvoid SetEventStack( MAJACONTAINER::MajaStack<_MajaWidgetEvent> *e)= 0;

	virtual MAJACONTAINER::MajaStack<_MajaWidgetEvent>* GetEventStack( mvoid)= 0;

	virtual mvoid SetKeyEventStack( MAJACONTAINER::MajaStack<_MajaWidgetEvent> *e)= 0;

	virtual MAJACONTAINER::MajaStack<_MajaWidgetEvent>* GetKeyEventStack( mvoid)= 0;

	//! create a context for gl
	virtual volatile _MajaContext CreateContext( mvoid)= 0;

	virtual volatile _MajaContext GetContext( mvoid)= 0;

	virtual std::string& GetFramerate( mvoid)= 0;
};

} //namespace maja
#endif
