/*
	majaxwindow.h
	ce fichier est une partie de Maja - this is a part of Maja
	window for glx
	
	$Id: $

	$Log: $

	begin:			June 2002
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef _MAJA_XWINDOW__H
#define _MAJA_XWINDOW__H

#include "majamainwindow.h"

namespace maja
{
	
/*!
 * simple window viewed by Maja
 * Their should only have one instance of a own derived class in a normal application.
 * should do polymorphic btw X and Windows
 * it looks like it's some about huge
 * TODO: disperse into some other classes. Their should also have a MajaWindowManager
 * that will be the 2nd layer of the core, directly linked to, and who will manage the
 * connection with the appropriate modules: MajaSRender & MajaSInteractivity which are
 * using window.
 * TODO: must be threadful compliant.
 */
#ifndef WIN32
class MajaXWindow: public MajaWindow
{
	protected:
	
	//! a screen
	int 			scr;

	private:
	
	//! video mode info
	XF86VidModeModeInfo	mode_info;
	
	XF86VidModeModeInfo 	**modes;
	protected:

	//! title (deprecated, it's now in MajaWinAttr)
	char 			*title;

	//! if true, means that we stop drawing like this
	bool 			end_loop;

	public:
	
	MajaXWindow( Display *d): MajaWindow(){
		drawable._d= d;
		end_loop= mfalse;
		CheckThread();
		title= mnull;
		scr= 0;
	}
	
	protected:
	
	MajaXWindow( mvoid);

	protected:

	MajaXWindow( const MajaXWindow& win)
	{
		drawable._v= new XVisualInfo( (const XVisualInfo&)*win.drawable._v);
		scr= win.scr;
		mode_info= win.mode_info;
		modes= win.modes;
		title= new char[strlen( win.title)+1];
		if( !title)
			throw std::exception();
		strcpy( title, win.title);
		end_loop= win.end_loop;
	}

	virtual ~MajaXWindow( mvoid)
	{
		if( title)
			delete title;
	}

	public:

	//! Create the window defined with MajaWinAttr
	//! it's transparent from Linux or Window
	bool CreateWindow( mvoid);

	protected:

	//! take care about thread and windowing
	//! you must call it before any other call to functions
	//! that make use of X functions !! So, you may declare
	//! any instance of MajaWindow as global and almost as
	//! the first one.
	bool CheckThread( mvoid);

	//! take care about what is supported (versions, ...)
	bool CheckSupport( mvoid);

	//! create the display
	bool CreateDisplay( mvoid);

	//! create a screen
	bool CreateScreen( mvoid);

	//! create the configuration
	//! TODO:
	//! if glx 1.3 is supported, then attempt to use the new way
	//! of handing a window. so use GLXFBConfig...
	bool CreateConfig( mvoid);

	//! create a visual
	bool CreateVisual( mvoid);

	public:
	
	//! create a context for gl
	volatile _MajaContext CreateContext( mvoid);

	protected:

	//! create the window on the GUI (not til a glwindow !)
	bool CreateMotherWindow( mvoid);

	//! check if direct rendering is enabled
	mbool CheckDirectRender( Display *d= mnull, GLXContext c= mnull);

	public:
	
	//! Destroy the window
	mvoid DestroyWindow( mvoid);
	
	//! defines with the attributes you give
	mvoid SetAttributes( MajaWinAttr &attr);
	
	//! get them
	MajaWinAttr& GetAttributes( mvoid);
	
	//! get a pointer to the display, must be different for Windows
	Display* GetDisplay( mvoid);
	
	//! get a ref to the window
	_MajaWindow& GetWindow( mvoid);

	protected:

	//! get the context
	volatile _MajaContext GetContext( mvoid){ return drawable._c;}

	XVisualInfo* GetVisual( mvoid){ return drawable._v;}

	//! set the screen (never use this)
	mvoid SetScreen( mint s){ scr= s;}

	public:

	//! one event loop
	//! so call it whenever you want events about your window
	_MajaWinEvent& LoopEvent( mvoid);
	
	//! some things to do before each drawing at screen
	mvoid PreDraw(){
		static mbool first= mtrue;

		if( first){
			c_clock.Start();
			first= mfalse;
		}
		static mdouble t_clk= c_clock.Clock();

		fps++;
		if( c_clock.Clock()>=(t_clk+1)){
			#ifdef SHOW_WIN_FPS
			cerr << endl << "framerate: " << fps << " images rendered last second";
			#endif
			fpsave= fps;
			fps=0;
			t_clk= c_clock.Clock();
		}
		
		glXWaitX();
		glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
	}
	
	//! some things to do after each drawing
	mvoid PostDraw(){
		glGetError();
		glXWaitGL();
		drawable.SwapBuffers();
	}
	
	//! redraw: all what are inside will be seen at screen
	mvoid Redraw( mvoid);

	mvoid HudDraw( mvoid){
		;
	}

	mvoid PreHudDraw( mvoid){
		glMatrixMode( GL_PROJECTION);
		glPushMatrix();
		glLoadIdentity();
		glOrtho( 0, attributes.width, 0, attributes.height, -1, 1);
		glMatrixMode( GL_MODELVIEW);
		glLoadIdentity();
	}

	mvoid PostHudDraw( mvoid){
		glMatrixMode( GL_PROJECTION);
		glPopMatrix();
	}

	//! global drawing method
	minline mvoid Draw(){
		PreDraw();
		Redraw();
		PreHudDraw();
		HudDraw();
		PostHudDraw();
		PostDraw();
	}

	//! resize the window
	mvoid Resize( mvoid);
	
	//! change to fullscreen (not working)
	mvoid SubFullscreenSwitch();

	//! return if the window was created
	inline mbool GetCreated( mvoid){ return window_created;}

	//! return the actual state of the window
	inline __MajaWindowState GetState( mvoid){ return state;}

	//! get the win event
	inline _MajaWinEvent& GetEvent( mvoid){ return event;}

	public:
	
	//! set the state
	inline mvoid SetState( __MajaWindowState s){ state= s;}

	MajaDrawable& GetDrawable( mvoid){ return drawable;}

	protected:

	//! set the event
	inline mvoid SetEvent( _MajaWinEvent& ev){
		event= ev;
	}

	public:
	
	inline mvoid SetEventStack( MAJACONTAINER::MajaStack<_MajaWidgetEvent> *e){
		event_stack= e;
	}
	
	inline MAJACONTAINER::MajaStack<_MajaWidgetEvent>* GetEventStack( mvoid){ return event_stack;}
	
	inline mvoid SetKeyEventStack( MAJACONTAINER::MajaStack<_MajaWidgetEvent> *e){ 
		key_event_stack= e;
	}

	inline MAJACONTAINER::MajaStack<_MajaWidgetEvent>* GetKeyEventStack( mvoid){ return key_event_stack;}

	mvoid PrintInfo( mvoid);

	std::string& GetFramerate( mvoid){ 
		static std::string s;
		char tmp[9];

		sprintf( tmp, "%d", fpsave);
		s= tmp;
		
		return s;
	}

	muint LoadLogo( mvoid);
};
#endif
}
#endif
