#ifndef MAJA_SDL_WINDOW__H
#define MAJA_SDL_WINDOW__H

#include <SDL/SDL.h>
#include "majamainwindow.h"

namespace maja
{
#ifdef MAJA_SDL_SUPPORT
class MajaSDLWindow: public MajaWindow
{	
	public:

	MajaSDLWindow( mvoid): MajaWindow(){
		SDLInit();
	}

	~MajaSDLWindow( mvoid){
		SDLQuit();
	}

	protected:
	
	mbool SDLInit( mvoid){
		if( !CheckThread())
			return mfalse;
		
		if( SDL_Init( SDL_INIT_VIDEO | SDL_INIT_EVENTTHREAD)!=0)
			cerr << endl << "MajaSDLWindow:: error at initialization.";

		return mtrue;
	}

	mvoid SDLQuit( mvoid){
		SDL_Quit();
	}

	public:

	mbool CreateWindow( mvoid){
		Uint8  video_bpp;
		Uint32 video_flags;

		video_bpp = 16;
		video_flags = SDL_SWSURFACE;
		if( attributes.fullscreen)
			video_flags |= SDL_FULLSCREEN;

		SDL_WM_SetCaption( attributes.name, attributes.name);

		drawable._d = SDL_SetVideoMode( attributes.width, attributes.height, video_bpp, video_flags);
		if( !drawable._d){
			cerr << endl << "Couldn't set SDL video mode" << SDL_GetError();
			return mfalse;
		}

//		drawable._d= SDL_LoadBMP( "images/maja.rgb");

		//cerr endl << "Running in %s mode\n", screen->flags & SDL_FULLSCREEN ?
		//	"fullscreen" : "windowed";
		
		return mtrue;
	}

	SDL_Surface* GetDrawable(){
		return drawable._d;
	}

	protected:

	mbool CheckThread( mvoid){
		#ifndef _MAJA_WINDOW_THREADED
		#define _MAJA_WINDOW_THREADED 1
		#endif
		return mtrue;
	}
	
	mbool CheckSupport( mvoid){
		return mfalse;
	}
	
	mbool CheckDirectRender( mvoid){
		return mfalse;
	}

	public:
	
	//! Destroy the window
	mvoid DestroyWindow( mvoid){
		;
	}
	
	//! defines with the attributes you give
	mvoid SetAttributes( MajaWinAttr &attr){
		attributes= attr;
	}
	
	//! get them
	MajaWinAttr& GetAttributes( mvoid){
		return attributes;
	}
	
	//! get a pointer to the display, must be different for Windows
	//! TODO: SDL doesn't let us know the display, so nor manage it...
	Display* GetDisplay( mvoid){
		return mnull; //drawable._d;
	}

	XVisualInfo* GetVisual( mvoid){ 
		return mnull;
	}
	
	//! get a ref to the window
	_MajaWindow& GetWindow( mvoid){
		return window;
	}

	//! get the context
	_MajaContext& GetContext( mvoid){ return context;}

	//! one event loop
	//! so call it whenever you want events about your window
	_MajaWinEvent& LoopEvent( mvoid){
		return event;
	}
	
	//! some things to do before each drawing at screen
	mvoid PreDraw(){
		static mbool first= mtrue;

		if( first){
			c_clock.Start();
			first= mfalse;
		}
		static mdouble t_clk= c_clock.Clock();

		fps++;
		if( c_clock.Clock()>=(t_clk+1)){
			#ifdef SHOW_WIN_FPS
			cerr << endl << "framerate: " << fps << " images rendered last second";
			#endif
			fpsave= fps;
			fps=0;
			t_clk= c_clock.Clock();
		}
		
		SDL_LockSurface( drawable._d);
		glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
	}
	
	//! some things to do after each drawing
	mvoid PostDraw(){
		glGetError();
		SDL_UnlockSurface( drawable._d);
		SDL_GL_SwapBuffers();
	}
	
	//! redraw: all what's inside will be seen at screen
	mvoid Redraw( mvoid){
		;
	}

	mvoid HudDraw( mvoid){
		;
	}

	mvoid PreHudDraw( mvoid){
		glMatrixMode( GL_PROJECTION);
		glPushMatrix();
		glLoadIdentity();
		glOrtho( 0, attributes.width, 0, attributes.height, -1, 1);
		glMatrixMode( GL_MODELVIEW);
		glLoadIdentity();
	}

	mvoid PostHudDraw( mvoid){
		glMatrixMode( GL_PROJECTION);
		glPopMatrix();
	}

	//! global drawing method
	minline mvoid Draw(){
		PreDraw();
		Redraw();
		PreHudDraw();
		HudDraw();
		PostHudDraw();
		PostDraw();
	}

	//! resize the window
	mvoid Resize( mvoid){
		;
	}
	
	//! change to fullscreen (not working)
	mvoid SubFullscreenSwitch(){
		SDL_WM_ToggleFullScreen( drawable._d);
	}

	//! return if the window was created
	minline mbool GetCreated( mvoid){ return window_created;}

	//! return the actual state of the window
	minline __MajaWindowState GetState( mvoid){ return state;}

	//! get the win event
	minline _MajaWinEvent& GetEvent( mvoid){ return event;}

	protected:

	//! set the event
	minline mvoid SetEvent( _MajaWinEvent& ev){ event= ev;}

	public:
	
	//! set the state
	minline mvoid SetState( __MajaWindowState s){ state= s;}

	minline mvoid SetEventStack( MAJACONTAINER::MajaStack<_MajaWidgetEvent> *e){ event_stack= e;}
	
	minline MAJACONTAINER::MajaStack<_MajaWidgetEvent>* GetEventStack( mvoid){ return event_stack;}
	
	minline mvoid SetKeyEventStack( MAJACONTAINER::MajaStack<_MajaWidgetEvent> *e){ key_event_stack= e;}

	minline MAJACONTAINER::MajaStack<_MajaWidgetEvent>* GetKeyEventStack( mvoid){ return key_event_stack;}
};
#endif
}
#endif
