/*
	gbase.h
	ce fichier est une partie de Maja - this is a part of Maja
	des containers sous formes de listes
	lists containers

	$Id: $

	$Log: $

	begin:			2000
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef __MAJALIST_H
#define __MAJALIST_H

#include "majawidget.h"

namespace MAJACONTAINER{

/*!
	This is a generic list that uses MajaWidget as elements
	evolution: should any lists have a bool member for knowing if
	we destroy the elements with the list when deleting the list.
	
	status: useable
*/
template <class _MajaData> class MajaList: public MajaWidget<_MajaData>
{
	protected:
	//! first MajaWidget of the list
	MajaWidget<_MajaData> *start;
	
	//! last one
	MajaWidget<_MajaData> *end;
	
	//! number of elements in this list
	unsigned int				nb_elements;
	
	//! know if need to destroy elements with the list (not implemented)
	//bool					destroy_element_at_destruct;
	
	public:
	//! The only way to create the list at first time
	MajaList<_MajaData>( void): MajaWidget<_MajaData>(){
		start= NULL;
		end= NULL;
		nb_elements=0;
	}
	
	~MajaList<_MajaData>( void){
		MajaWidget<_MajaData> *tmp;
		tmp= start;
		while( tmp){
			tmp=tmp->GetNext();
			if( !tmp) break;
			delete tmp->GetPrevious();
		}
		delete tmp;
	}

	//! This copy only the first & last and link all the list
	//! as all what's inside are pointers.
	MajaList<_MajaData>( MajaList<_MajaData>& list){
		MajaWidget<_MajaData> *tmp= start;
		*start= *(list.start);
		*end= *(list.end);
		while( tmp){
			AddObject( tmp->GetObject());
			tmp=tmp->GetNext();
		}
	};
	
	//! Add an object at the end of the list
	inline virtual bool AddObject( _MajaData *src, unsigned int val= 0, bool alloc= false)
	{
		MajaWidget<_MajaData> *tmp;
		tmp= new MajaWidget<_MajaData>( src, val, alloc);
		if( !tmp) 
			return false;
		
		if( !start){
			start= end= tmp;
		}
		else{
			tmp->previous= end;
			end->next= tmp;
			end= tmp;
		}
		tmp->next= NULL;
		nb_elements++;
		
		return true;
	}
	
	//! Remove the MajaWidget pointer given in parameter
	//! TODO: should be renamed RemoveWidget, and make RemoveObject( _MajaData *o)
	inline virtual void RemoveObject( MajaWidget<_MajaData> *sc){
		//cerr << endl << "Remove object...";
		if( !sc) 
			std::cerr << std::endl << "Nothing to delete ! unknown behavior may result.";
		
		if( sc->previous){
		      sc->previous->next= sc->next;
		      if( sc->next) sc->next->previous= sc->previous;
		      else end= sc->GetPrevious();
		}
		else{
			if( sc->next){
				sc->next->previous= NULL;
				start= sc->GetNext();
			}
			else
				start= end= NULL;
		}
		
		if( sc->GetAllocateObject()){
			_MajaData *d= sc->GetObject();
			delete d;
			d= NULL;
		}
		delete sc;
		sc= NULL;
		
		nb_elements--;

		//cerr << " done.";
	}

	inline virtual void RemoveWidget( MajaWidget<_MajaData> *w){
		RemoveObject( w);
	}
	
	//! Seek a _MajaData object in the list
	inline virtual MajaWidget<_MajaData> *SeekObject( _MajaData& obj)
	{
		MajaWidget<_MajaData> *tmp;
		
		tmp= GetFirst();		
		while( tmp){
			if( tmp->GetObject()==&obj)
				return tmp;
			tmp= tmp->GetNext();
		}
		return NULL;
	}
	
	//! Get the first object of the list
	inline virtual MajaWidget<_MajaData>* GetFirst( void){ return start;}
	
	inline virtual _MajaData* GetFirstObject( void){ return start->GetObject();}
	
	//! get the value of the first object of the list
	//!inline unsigned int GetFirstValue( void){ return start->GetValue();}
	
	//! Get the last one
	inline virtual MajaWidget<_MajaData>* GetLast( void){ return end;}
	
	inline virtual _MajaData* GetLastObject( void){ return end->GetObject();}
	
	//! get the last value of the last object
	//!inline unsigned int GetLastValue( void){ return end->GetValue();}
	
	//! Draw all the list (in Maja3DList)
	inline bool Draw( void){
		/*MajaWidget<MajaObject> *tmp;
		
		tmp= GetFirst();
		if( !tmp) return false;
		while( tmp++)
			if( !tmp->Draw()) return false;
		*/return true;
	}
	
	//! Seek an object by giving its name
	inline virtual _MajaData* Seek( char *name){
		MajaWidget<_MajaData> *tmp;
		_MajaData *obj;
		
		tmp= GetFirst();		
		while( tmp){
			obj= tmp->GetObject();
			if( !strcmp( obj->GetName(), name)) return obj;
			tmp= tmp->GetNext();
		}
		return NULL;
	}
	
	//! Seek an object value by giving its name
	inline unsigned int SeekValue( char *name){
		MajaWidget<_MajaData> *tmp;
		_MajaData *obj;
		char objname[__NAME_SIZE];
		
		tmp= GetFirst();		
		while( tmp){
			obj= tmp->GetObject();
			
			if( !strcmp( obj->GetName(), name)){
				return tmp->GetValue();
			}
			tmp= tmp->GetNext();
		}
	}
	
	//! return the number of elements in the list
	inline unsigned int GetNumber( void){ return nb_elements;}
	
	//! return return_list which is a list made of all the elements which have
	//! the names given in first, for a total of nb max elements in the return list.
	//! return_list is allocated into this method, so you'll need to destroy it yourself
	inline bool MakeListFrom( char **names, unsigned int nb, MajaList<_MajaData>* return_list){
		return_list= new MajaList<_MajaData>;
		if( !return_list)
			return false;
			
		for( unsigned int i=0; i<nb; i++){
			MajaWidget<_MajaData> *tmp;
			_MajaData *obj;
			char objname[__NAME_SIZE];
		
			tmp= GetFirst();		
			while( tmp){
				obj= tmp->GetObject();
			
				if( !strcmp( obj->GetName(), name)){
					if( !return_list->AddObject( obj)){
						delete return_list;
						return_list= NULL;
						return false;
					}
				}
				tmp= tmp->GetNext();
			}
		}		
		
		return true;
	}
};

/*!
	This is a generic list that uses MajaPWidget as elements
	should find a way to inherit with MajaList
	
	status: useable
*/
template <class _MajaData, class _MajaParam> class MajaPList: public MajaPWidget<_MajaData, _MajaParam>
{
	protected:
	//! first MajaWidget of the list
	MajaPWidget<_MajaData, _MajaParam> *start;
	
	//! last one
	MajaPWidget<_MajaData, _MajaParam> *end;
	
	//! number of elements in this list
	unsigned int				nb_elements;
	
	public:
	//! The only way to create the list at first time
	MajaPList<_MajaData, _MajaParam>( void): MajaPWidget<_MajaData, _MajaParam>(){
		start= NULL;
		end= NULL;
		nb_elements=0;
	}
	
	~MajaPList<_MajaData, _MajaParam>( void){
		MajaPWidget<_MajaData, _MajaParam> *tmp;
		tmp= start;
		while( tmp){
			tmp=(MajaPWidget<_MajaData, _MajaParam>*)tmp->GetNext();
			if( !tmp) break;
			delete tmp->GetPrevious();
		}
		delete tmp;
	}

	//! This copy only the first & last and link all the list
	//! as all what's inside are pointers.
	MajaPList<_MajaData, _MajaParam>( MajaPList<_MajaData, _MajaParam>& list){
		MajaPWidget<_MajaData, _MajaParam> *tmp= start;
		*start= *(list.start);
		*end= *(list.end);
		while( tmp){
			AddObject( tmp->GetObject(), tmp->GetParam());
			tmp=tmp->GetNext();
		}
	};
	
	//! Add an object at the end of the list
	inline virtual bool AddObject( _MajaData *src, _MajaParam *p)
	{
		MajaPWidget<_MajaData, _MajaParam> *tmp;
		tmp= new MajaPWidget<_MajaData, _MajaParam>( src, p);
		if( !tmp) return false;
		if( !start){
			start= end= tmp;
		}
		else{
			tmp->previous= end;
			end->next= tmp;
			end= tmp;
		}
		tmp->next= NULL;
		nb_elements++;
		
		return true;
	}
	
	//! Remove the MajaWidget pointer given in parameter
	inline virtual void RemoveObject( MajaPWidget<_MajaData, _MajaParam> *sc){
		if( sc->previous){
		      sc->previous->next= sc->next;
		      if( sc->next) sc->next->previous= sc->previous;
		      else end= (MajaPWidget<_MajaData, _MajaParam>*)sc->GetPrevious();
		}
		else{
			if( sc->next){
				sc->next->previous= NULL;
				start= (MajaPWidget<_MajaData, _MajaParam>*)sc->GetNext();
			}
			else
				start= end= NULL;
		}
		
		delete sc;
		
		nb_elements--;
	}
	
	//! Seek a _MajaData object in the list
// TODO: assume parameters to
	inline virtual MajaPWidget<_MajaData, _MajaParam> *SeekObject( _MajaData& obj, _MajaParam *p= NULL)
	{
		MajaPWidget<_MajaData, _MajaParam> *tmp;
		
		tmp= GetFirst();		
		while( tmp){
			if( tmp->GetObject()==&obj)
				return tmp;
			tmp= (MajaPWidget<_MajaData, _MajaParam>*)tmp->GetNext();
		}
		return NULL;
	}
	
	//! Get the first widget of the list
	inline virtual MajaPWidget<_MajaData, _MajaParam>* GetFirst( void){ return start;}
	
	//! Get the first object of the list
	inline virtual _MajaData* GetFirstObject( void){ return start->GetObject();}
	
	//! get the first parameter of the list
	inline virtual _MajaParam* GetFirstParam( void){ return start->GetParam();}
	
	//! Get the last one
	inline virtual MajaPWidget<_MajaData, _MajaParam>* GetLast( void){ return end;}
	
	inline virtual _MajaData* GetLastObject( void){ return end->GetObject();}
	
	inline virtual _MajaParam* GetLastParam( void){ return end->GetParam();}
	
	//! Seek an object by giving its name
	inline virtual _MajaData* Seek( char *name, _MajaParam *return_param){
		MajaPWidget<_MajaData, _MajaParam> *tmp;
		_MajaData *obj;
		
		tmp= GetFirst();		
		while( tmp){
			obj= tmp->GetObject();
			if( !strcmp( obj->GetName(), name)){
				return_param= tmp->GetParam();
				return obj;
			}
			tmp= (MajaPWidget<_MajaData, _MajaParam>*)tmp->GetNext();
		}
		return_param= NULL;
		return NULL;
	}
	
	//! get the number of elements in the list
	inline unsigned int GetNumber( void){ return nb_elements;}
};

/*!
	A 3D list. only experimental way
	
	notes: il y a aussi le moyen d'utiliser un type de liste dont chaque membre
	contiendrait soit une valeur ou rcursivement une autre liste, en fonction
	de la dimension choisie.
	
	status: unuseable
*/
template <class _MajaData> class Maja3DList: public MajaList<_MajaData>
{
	public:

	Maja3DList<_MajaData>( void): MajaList<_MajaData>(){
		;
	}
	
	//! Add an object at the top of a polymorphic MajaWidget
	// NO
	inline virtual bool AddUpObject( MajaWidget<_MajaData> *base, _MajaData *src)
	{
		MajaWidget<_MajaData> *tmp;
		tmp= new MajaWidget<_MajaData>( src, val);
		if( !tmp) return false;
		
		if( !base->GetLower()){
			base->GetLower()= base->GetUpper()= tmp;
		}
		else{
			tmp->SetDown( base->GetUpper());
			base->SetUp( tmp);
			base->SetUpper( tmp);
		}
		
		tmp->SetUp( NULL);
		nb_elements++;
		
		return true;
	}
	// etc...
	
	//! better to add
/*	inline virtual bool AddObject( MajaWidget<_MajaData> *base, _MajaData *src, _mwtype dsflkjsdf)
	{
		MajaWidget<_MajaData> *tmp;
		tmp= new MajaWidget<_MajaData>( src, val);
		if( !tmp) return false;
		
		if( !base->GetLower()){
			base->GetLower()= base->GetUpper()= tmp;
		}
		else{
			tmp->SetDown( base->GetUpper());
			base->SetUp( tmp);
			base->SetUpper( tmp);
		}
		
		tmp->SetUp( NULL);
		nb_elements++;
		
		return true;
	}*/
};
}// namespace MAJACONTAINER
#endif
