/*
	gbase.h
	ce fichier est une partie de Maja - this is a part of Maja
	des containers sous formes de tas (empilements  la FIFO)
	stack containers

	$Id: $

	$Log: $

	begin:			Tue Jul 16 2002
	copyright:		(C) 2000, 2001, 2002 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	Contactez-nous pour toute tentative d'utilisation commerciale, svp.			*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *	Contact us for any commercial use attempt, please.					*
 *												*
 ************************************************************************************************/

#ifndef __MAJASTACK_H
#define __MAJASTACK_H

#include "majalist.h"

//! \file
//! this is experimental but it may be quite safe to use it
//! please beware of how to instanciate these objects !!
//! see all comments below for that

namespace MAJACONTAINER{

/*!class
	This manage a stack of elements
	This is not sufficient as pushing/poping recquieres some tests,
	memory management,... We'll need later to let specify what sort
	of stack we wan't to use here: quite secure ones, or fastest ones.
	may policies could help here...
	
status: some tests passed:
		*	pushing by value
		*	peeking
*/
template <class _MajaData> class MajaStack: public MajaList<_MajaData>
{
	public:
	
	//! The only way to create the list at first time
	MajaStack<_MajaData>( void): MajaList<_MajaData>(){
		;
	}
	
	~MajaStack<_MajaData>( void){
		;
	}
	
	inline virtual void Push( _MajaData *d){
		AddObject( d);
	}
	
	//! call this member function in order to store a new data by value
	//! note that the data pushed are automatically allocated here, and
	//! will be automatically destructed when the data will be poped.
	inline virtual void Push( _MajaData d){
		AddObject( &d, 0, true);
	}
	
	//! call this member function in order to store a new data by pointer
	//! note that the data pushed will not be allocated inside the stack,
	//! and, therefore, won't be destructed when poped. SO, IF YOU ALLOCATED
	//! THE POINTER, YOU'LL NEED TO REMOVE IT WHEN POPED !
	inline virtual void Pop( _MajaData *d){
		RemoveObject( d);
	}
	
	//! call this to get a pointer to the last enter data
	//! this returns null if no data in the stack
	inline virtual _MajaData* Peek( void){
		return end? end->GetObject(): NULL;
	}
	
	//! return the number (you must check if >0) of
	//! data in the stack
	inline int GetSize( void){
		return end? GetNumber(): 0;
	}
	
	protected:
	
	//! remove an object
	//! DON'T USE IT !
	void RemoveObject( _MajaData *d){
		MajaWidget<_MajaData> *t= SeekObject( *d);
		if( t)
			MajaList<_MajaData>::RemoveObject( t);
	}
	
	//+ void SetStackMode() pour savoir si on alloue  chaque fois ou pas (important pour le core)
};

/*!
	This manage a queue of elements
	
	status: untested
*/
template <class _MajaData> class MajaQueue: public MajaList<_MajaData>
{
	public:
	
	// The only way to create the list at first time
	MajaQueue<_MajaData>( void): MajaList<_MajaData>(){
		;
	}
	
	~MajaQueue<_MajaData>( void){
		;
	}
	
	inline virtual void Enqueue( _MajaData *d){
		AddObject( d);
	}
	
	inline virtual void Dequeue( _MajaData *d){
		RemoveObject( d);
	}
	
	inline virtual _MajaData* Peek( void){
		return start->GetObject();
	}
	
	inline int GetSize( void){
		return GetNumber();
	}
};
}// namespace MAJACONTAINER
#endif
