/* $Id: $
 * majatree.h, this is a part of Maja
 * template for tree support
 * Copyright (C) september 2000,2001,2002 Jean-Dominique Frattini
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Cette license est valable en terme de droit franais. Cela signifie que c'est la
 * lgislation franaise qui est en vigueur pour l'ensemble des sources contenues
 * dans la librairie. Sauf image.cpp et image.h, se reposant sur aucune license explicite,
 * mais qui sont fournies aimablement dans les librairies Mesa sous license LGPL.
 *
 * Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou
 * le modifier sous les termes de la license publique gnrale
 * publie par la Free Software Foundation; soit la version 2 de la
 * license, ou (si vous le voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir d'tre utile, mais
 * SANS AUCUNE GARANTIE; et mme une garantie implicite,
 * y compris les garanties de COMMERCIALISATION ou D'ADAPTATION
 * DANS UN BUT SPCIFIQUE. Prenez connaissance de la License Publique
 * Gnrale GNU pour plus de dtail.
 *
 * Vous devez avoir reu une copie de la License Publique Gnrale
 * GNU avec ce programme; si ce n'est pas le cas, vrivez  la
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: $
 */

#ifndef __MAJA_TREE_H
#define __MAJA_TREE_H

#include "majalist.h"

namespace MAJACONTAINER{

template <class _MajaData> class MajaTreeBranch;

/*!
	This class will contain data for a render
	this is template but all _MajaData must be from a MajaData
	
	status: avoid using it
*/
template <class _MajaData> class MajaTreeLeaf: protected MajaWidget<_MajaData>
{
	protected:
	MajaTreeBranch<_MajaData> *dad;
	
	public:
	MajaTreeLeaf<_MajaData>( void): MajaWidget<_MajaData>(){
		dad= NULL;
	}
	
	MajaTreeLeaf<_MajaData>( _MajaData& data): MajaWidget<_MajaData>( data){
		SetWidget( &data);
	}
	
	virtual ~MajaTreeLeaf<_MajaData>( void){;}
	
	// Set the current widget of the list
	inline void SetWidget( _MajaData *obj){
		object= obj;		
		
		return;
	}

	inline MajaTreeBranch<_MajaData>* GetDad( void){ return dad;}
	
	inline void SetDad( MajaTreeBranch<_MajaData> *daddy){ dad= daddy;}
};

/*
	This class will contain parameters for datas
	It must be linked with son, dad and any brothers
	
	status: avoid using it
*/
template <class _MajaData> class MajaTreeBranch: public MajaTreeLeaf<_MajaData>
{
	protected:
	MajaTreeBranch<_MajaData> *son;
	
	public:
	MajaTreeBranch<_MajaData>( void): MajaTreeLeaf<_MajaData>(){
		son= NULL;
	}
	
	MajaTreeBranch<_MajaData>( _MajaData *obj){
		SetWidget( obj);
	}
	
	virtual ~MajaTreeBranch( void){;}
	
	// Set the current widget of the list
	inline void SetWidget( _MajaData *obj){
		object= obj;
		return;
	}

	inline MajaTreeBranch<_MajaData>* GetSon( void){ return son;}
	
	inline void SetSon( MajaTreeBranch<_MajaData> *sonny){ son= sonny;}
	
	// make them public again
	// Get the next widget of the list
	inline MajaWidget<_MajaData>* GetNext( void){ return next;}
	
	// Get the previous widget of the list
	inline MajaWidget<_MajaData>* GetPrevious( void){ return previous;}
};

/*!
	this is a strange tree but which could hands many things
	
	status: avoid using it
*/
template <class _MajaData> class MajaTree: public MajaTreeBranch<_MajaData>
{
	protected:
	MajaTreeBranch<_MajaData> *root;
	
	public:
	MajaTree<_MajaData>( void): MajaTreeBranch<_MajaData>(){ root= NULL;}
	
	MajaTree<_MajaData>( MajaTreeBranch<_MajaData> root){ this->root= root;}
	
	virtual ~MajaTree<_MajaData>( void){;}
	
	inline void SetRoot( MajaTreeBranch<_MajaData> *r){ root= r;}
	
	MajaTreeBranch<_MajaData>* GetRoot( void){ return root;}
	
	// add a branch in the tree, if father is null then it's the root branch, son could be null too
	bool AddBranch( MajaTreeBranch<_MajaData> *branch, MajaTreeBranch<_MajaData> *father= NULL, MajaTreeBranch<_MajaData> *child= NULL);
	
	bool RemoveBranch( MajaTreeBranch<_MajaData>& branch);
	
	// add a leaf to a specified father branch, or directly to the root branch if specified to null
	bool AddLeaf( MajaTreeLeaf<_MajaData> *leaf, MajaTreeBranch<_MajaData> *father= NULL);	
	
	bool RemoveLeaf( MajaTreeLeaf<_MajaData> *leaf);
};


/*!
	A basic binary tree that uses MajaTreeWidgets for nodes and leaves
	
	status: avoid using it
*/
template <class _MajaData> class MajaBinaryTree: public MajaTreeWidget<_MajaData>
{
	protected:
	MajaTreeWidget<_MajaData>	*root;
	
	public:
	MajaBinaryTree<_MajaData>( void): MajaTreeWidget<_MajaData>(){
		root= NULL;
	}
	
	inline void SetRoot( MajaTreeWidget<_MajaData> *r){ root= r;}
	
	inline MajaTreeWidget<_MajaData>* GetRoot( void){ return root;}
	
	inline void SetRootLeftChild( MajaTreeWidget<_MajaData> *lc){ lc->SetParent( root); root->SetLeftChild( lc);}
	
	inline MajaTreeWidget<_MajaData>* GetRootLeftChild( void){ return root->GetLeftChild();}
	
	inline void SetRootRightChild( MajaTreeWidget<_MajaData>* rc){ lc->SetParent( root); root->SetRightChild( rc);}
	
	inline MajaTreeWidget<_MajaData>* GetRootRightChild( void){ return root->GetRightChild();}
};
}//namespace MAJACONTAINER
#endif
