/* $Id: $
 * all widgets hierarchy, this is a part of Maja
 * Copyright (C) september 2000,2001,2002 Jean-Dominique Frattini
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Cette license est valable en terme de droit franais. Cela signifie que c'est la
 * lgislation franaise qui est en vigueur pour l'ensemble des sources contenues
 * dans la librairie. Sauf image.cpp et image.h, se reposant sur aucune license explicite,
 * mais qui sont fournies aimablement dans les librairies Mesa sous license LGPL.
 *
 * Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou
 * le modifier sous les termes de la license publique gnrale
 * publie par la Free Software Foundation; soit la version 2 de la
 * license, ou (si vous le voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir d'tre utile, mais
 * SANS AUCUNE GARANTIE; et mme une garantie implicite,
 * y compris les garanties de COMMERCIALISATION ou D'ADAPTATION
 * DANS UN BUT SPCIFIQUE. Prenez connaissance de la License Publique
 * Gnrale GNU pour plus de dtail.
 *
 * Vous devez avoir reu une copie de la License Publique Gnrale
 * GNU avec ce programme; si ce n'est pas le cas, vrivez  la
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: $
 */

#ifndef __MAJAWIDGET_H
#define __MAJAWIDGET_H

#include <stdlib.h>
#include <typeinfo>
#include <iostream>

//! MAJACONTAINER is the namespace containing
//! all the containers that maja provides. they
//! are not derived, and don't use the STL.
//! I think these containers will change some quiete
//! in order to be more competitive and stable.
namespace MAJACONTAINER{

/*!class
	a new implementation revision for the widgets
	You could tell the widget to allocate/deallocate the data
	you put into this widget.
	You could use this type safely.
	
	status: useable
*/
template <class _MajaData> class MajaWidget
{
	protected:
	//! an object
	_MajaData		*object;

	bool			with_alloc;

	public:

	//! the next MajaListeObject
	MajaWidget<_MajaData> *next;
	
	//! the previous one
	MajaWidget<_MajaData> *previous;

	MajaWidget<_MajaData>( void){
		next= NULL;
		previous= NULL;
		object= NULL;
		with_alloc= false;
	}

	//! val, deprecated, just for compatiblity
	MajaWidget<_MajaData>( _MajaData *src, int val=0, bool alloc= false){
		if( !alloc)
			object= src;
		else{
			std::cerr << std::endl << "allocating contained object";
			object= new _MajaData( *src);
			if( !object)
				std::exception();
		}
		next= NULL;
		previous= NULL;
		with_alloc= alloc;
	}

	MajaWidget<_MajaData>( MajaWidget<_MajaData>& src){
		if( !src.with_alloc)
			object= src.object;
		else{
			object= new _MajaData();
			*object= (_MajaData)*(src.GetObject());
			if( !object)
				std::exception();
		}
		next= src.next;
		previous= src.previous;
		with_alloc= src.with_alloc;
	}

	virtual ~MajaWidget<_MajaData>( void){
		if( with_alloc){
			std::cerr << std::endl << "destroying contained object";
			delete object;
		}
	}

	//! Get the next widget of the list
	inline virtual MajaWidget<_MajaData>* GetNext( void){ return next;}
	
	//! set the next object
	inline virtual void SetNext( MajaWidget<_MajaData> *n){ next= n;}

	//! Get the previous widget of the list
	inline virtual MajaWidget<_MajaData>* GetPrevious( void){ return previous;}
	
	//! set the prev
	inline virtual void SetPrevious( MajaWidget<_MajaData>* p){ previous= p;}
	
	//! get it
	inline virtual MajaWidget<_MajaData>* Get( void){ return this;}
	
	//! Get the current widget of the list
	inline virtual _MajaData* GetObject( void){ return object;}
	inline _MajaData* GetWidget( void){ return object;}	//! deprecated
	
	//! Set the current widget of the list
	inline virtual void SetObject( _MajaData *obj){ 
		if( !with_alloc)
			object= obj;
		else{
			object= new _MajaData( *obj);
			if( !object)
				return;		// warn it's not good
		}
	}
	
	//! get next object
	inline virtual _MajaData* GetNextObject( void){ return next->GetObject();}
	
	//! set the next obje
	inline virtual void SetNextObject( _MajaData *n){ next->SetObject( n);}
	
	//! get previous one
	inline virtual _MajaData* GetPreviousObject( void){ return previous->GetObject();}
	
	//! set the previous
	inline virtual void SetPreviousObject( _MajaData *p){ previous->SetObject( p);}

	//! Get the name of the object (so need any templated object got a name and this method !
//	inline virtual char* GetName( void){ return object->GetName();}
	
	//! set the name
//	inline virtual void SetName( char *n){ object->SetName( n);}

	//! set if the object is allocated or just linked by pointers or referencies
	inline virtual void SetAllocateObject( bool alloc){
		with_alloc= alloc;
	}

	//! get if it's allocated
	inline virtual bool GetAllocateObject( void){ return with_alloc;}

	protected:

	inline virtual MajaWidget<_MajaData>* operator++( void){ return next;}
	inline virtual MajaWidget<_MajaData>* operator++( int){ return next;}
	inline virtual MajaWidget<_MajaData>* operator--( void){ return previous;}
	inline virtual MajaWidget<_MajaData>* operator--( int){ return previous;}
};

/*!
	a new widget in maja: MajaPWidget (P is for Parameter)
	this type will allow you to use a widget made of 2 generic types.
	You could not use this type safely.
	this will be used internally by MajaMultiMotion for its sub-objects
	this should be associated with an appropriate list (cf MajaPList)
	
	status: useable
*/
template <class _MajaData, class _MajaParam> class MajaPWidget: public MajaWidget<_MajaData>
{
	protected:
	//! any second generic type for this widget
	_MajaParam		*param;

	public:

	MajaPWidget<_MajaData, _MajaParam>( void): MajaWidget<_MajaData>(){
		param= NULL;
	}

	//! CALL IT IF YOU USE UINT FOR PARAMETER !!!
	MajaPWidget<_MajaData, _MajaParam>(	_MajaData *src, 
						int val=0, 
						bool alloc= false):	MajaWidget<_MajaData>( src, 0, alloc){
		param= new int( val);
		if( !param) std::cerr << "MajaPWidget: memory full" << std::endl;
	}
	
	//! prefers this constructor than the others for this type
	MajaPWidget<_MajaData, _MajaParam>( _MajaData *src, _MajaParam *par){
		object= src;
		next= NULL;
		previous= NULL;
		param= par;
	}

	MajaPWidget<_MajaData, _MajaParam>( MajaPWidget<_MajaData, _MajaParam>& src){
		object= src.object;
		next= src.next;
		previous= src.previous;
		param= src.param;
	}

	virtual ~MajaPWidget( void){
		if( typeid( *param)== typeid( int))
			delete param;
	}

	//! Get the current parameter of this widget
	inline virtual _MajaParam* GetParam( void){ return param;}
	
	//! Set the current parameter
	inline virtual void SetParam( _MajaParam *p){ param= p;}
};

/*!
	a widget associated with an uint value
	you could use this type
	
	status: useable
*/
template <class _MajaData> class MajaValueWidget: public MajaWidget<_MajaData>
{
	protected:
	int value;
	
	public:
	MajaValueWidget<_MajaData>( void): MajaWidget<_MajaData>(){}
	
	MajaValueWidget<_MajaData>( _MajaData *src, int val=0): MajaWidget<_MajaData>( src){
		value= val;
	}
	
	//! get the next widget value
	inline virtual int GetNextValue( void){ return next->GetValue();}
	
	//! set next value
	inline virtual void SetNextValue( int v){ next->SetValue( v);}
	
	//! get the value of the previous
	inline virtual int GetPreviousValue( void){ return previous->GetValue();}
	
	//! set the previous value
	inline virtual void SetPreviousValue( int v){ previous->SetValue( v);}
	
	//! get the value of the current object
	inline virtual int GetValue( void){ return value;}
	
	//! set the value of the current object
	inline virtual void SetValue( int val){ value= val;}
};

/*!
	A tree widget, specially diserved for binary trees
	this is a normal widget, with one parent, which is a widget too
	this is more adequat than the unary tree with list childs
	this is experimental way, so avoid use it
	
	status: not useable
*/
template <class _MajaData> class MajaTreeWidget: public MajaWidget<_MajaData>
{
	protected:
	MajaWidget<_MajaData> *parent;			//! the parent of this tree widget
	
	public:
	MajaTreeWidget<_MajaData>( void): MajaWidget<_MajaData>(){
		parent= NULL;
	}
	
	//! get its parent
	inline MajaWidget<_MajaData>* GetParent( void){ return parent;}
	
	//! set its parent
	inline void SetParent( MajaWidget<_MajaData> *p){ parent= p;}
	
	//! set its left child (this is same as setting the previous)
	inline void SetLeftChild( MajaWidget<_MajaData> *lc){ previous= lc;}
	
	//! get the left child
	inline MajaWidget<_MajaData>* GetLeftChild( void){ return previous;}
	
	//! set its right child (this is same as setting the next)
	inline void SetRightChild( MajaWidget<_MajaData> *lc){ next= lc;}
	
	//! get the left child
	inline MajaWidget<_MajaData>* GetRightChild( void){ return next;}
};

/*!
	Maja3DWidget: a widget to be placed in a Maja3DContainer
	could have 4 more links to other MajaWidget
	This could also serve to make a representative container btw the render
	This is pure experimental & theoric, so DON'T USE IT
	the value could serve for masses or for ponderatations.
	
	status: not useable
*/
template <class _MajaData> class Maja3DWidget: public MajaValueWidget<_MajaData>
{
	protected:
	//! linked widget to up
	Maja3DWidget<_MajaData>		*up;
	
	//! linked widget to down
	Maja3DWidget<_MajaData>		*down;
	
	//! linked widget to left
	Maja3DWidget<_MajaData>		*left;
	
	//! linked widget to right
	Maja3DWidget<_MajaData>		*right;
	
	//! most higher object relative to this widget
	Maja3DWidget<_MajaData> 		*upper;
	
	//! most lower object relative to this widget
	Maja3DWidget<_MajaData> 		*lower;
	
	//! most lefter object relative to this widget
	Maja3DWidget<_MajaData> 		*lefter;
	
	//! most righter object relative to this widget
	Maja3DWidget<_MajaData> 		*righter;
	
	public:
	Maja3DWidget<_MajaData>( void): MajaValueWidget<_MajaData>(){
		AssignNull();
	}
	
	Maja3DWidget<_MajaData>( _MajaData *src, unsigned int val=0): MajaValueWidget<_MajaData>( src, val){
		AssignNull();
	}
	
	protected:
	void AssignNull( void){
		up= NULL;
		down= NULL;
		left= NULL;
		right= NULL;
		upper= NULL;
		lower= NULL;
		lefter= NULL;
		righter= NULL;
	}
	
	//! Draw the widget
	inline virtual void Draw( void){
		object->Draw();
	}
	
	//! Get the up widget of the list
	inline virtual MajaWidget<_MajaData>* GetUp( void){ return up;}
	
	//! set the up
	inline virtual void SetUp( MajaWidget<_MajaData>* d){ up= d;}
	
	//! Get the down widget of the list
	inline virtual MajaWidget<_MajaData>* GetDown( void){ return down;}
	
	//! set the up
	inline virtual void SetDown( MajaWidget<_MajaData>* d){ down= d;}
	
	//! Get the left widget of the list
	inline virtual MajaWidget<_MajaData>* GetLeft( void){ return left;}
	
	//! set the up
	inline virtual void SetLeft( MajaWidget<_MajaData>* d){ left= d;}
	
	//! Get the right widget of the list
	inline virtual MajaWidget<_MajaData>* GetRight( void){ return right;}
	
	//! set the up
	inline virtual void SetRight( MajaWidget<_MajaData>* d){ right= d;}
	
	//! get the most higher object
	inline virtual MajaWidget<_MajaData>* GetUpper( void){ return upper;}
	
	//! set it
	inline virtual void SetUpper( MajaWidget<_MajaData>* u){ upper= u;}
	
	//! get the lower one	
	inline virtual MajaWidget<_MajaData>* GetLower( void){ return lower;}
	
	//! set it
	inline virtual void SetLower( MajaWidget<_MajaData>* o){ lower= o;}
	
	//! get the object on the far left
	inline virtual MajaWidget<_MajaData>* GetLefter( void){ return lefter;}
	
	//! set it
	inline virtual void SetLefter( MajaWidget<_MajaData> *o){ lefter= o;}
	
	//! get the object on the far right
	inline virtual MajaWidget<_MajaData>* GetRighter( void){ return righter;}
	
	//! set it
	inline virtual void SetRighter( MajaWidget<_MajaData> *o){ righter= o;}
};
}// namespace MAJACONTAINER
#endif
