/* geometry.h , this is a part of Maja
 * geometry
 * Copyright (C) 2000-2002 Jean-Dominique Frattini
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Cette license est valable en terme de droit franais. Cela signifie que c'est la
 * lgislation franaise qui est en vigueur pour l'ensemble des sources contenues
 * dans la librairie. Sauf image.cpp et image.h, se reposant sur aucune license explicite,
 * mais qui sont fournies aimablement dans les librairies Mesa sous license LGPL.
 *
 * Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou
 * le modifier sous les termes de la license publique gnrale
 * publie par la Free Software Foundation; soit la version 2 de la
 * license, ou (si vous le voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir d'tre utile, mais
 * SANS AUCUNE GARANTIE; et mme une garantie implicite,
 * y compris les garanties de COMMERCIALISATION ou D'ADAPTATION
 * DANS UN BUT SPCIFIQUE. Prenez connaissance de la License Publique
 * Gnrale GNU pour plus de dtail.
 *
 * Vous devez avoir reu une copie de la License Publique Gnrale
 * GNU avec ce programme; si ce n'est pas le cas, vrivez  la
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __GEOMETRY_H
#define __GEOMETRY_H

#include "majavector.h"

namespace maja
{
	
namespace MAJAMATHS{

#ifndef M_PI
   #define M_PI		3.14159265358979323846
#endif
#ifndef pi
   #define pi 		M_PI
   #define two_pi	6.28318530717958647682
   #define pi_2 	1.57079632679489661923
   #define pi_3		1.047197551
   #define pi_4		0.78539816339744830962
#endif

#define _WITHIN(x,a,b) ( ((x)<=a) && ((x)>=b) || ((x)<=b) && ((x)>=a) )
#define _MMIN(  a, b)    	( ((a<=b) ? a: b))
#define _MMAX(  a, b)    	( ((a>=b) ? a: b))

#define _RAD_TO_DEG( a)		(a*180/pi)
#define _DEG_TO_RAD( a)  	(a*pi/180)

struct straight_2D
{
	mreal a;
	mreal b;
};

// Rotation of a point around the opengl y axis
// give center of rotation, angle in radians, the point to be transformed and the
// point transformed (that will be setted)
mvoid RotatePoint( MajaVector& centre, mreal alpha, MajaVector& point, MajaVector& p);

// the given point will automatically be transformed, so it will changed
mvoid RotatePoint( MajaVector& centre, mreal alpha, MajaVector& point);

// Rotate the point around the x axis
mvoid RotatePoint_x( MajaVector& centre, mreal alpha, MajaVector& point);

// normalize a vector to a size (default =1)
mbool NormalizeVector( MajaVector& m, mreal i=1);

// equ y=mx+p de (AB) ou z=mx+p si ogl= true
// give the equation in plan that go threw 2 points
// if ogl is true, x and z coordinates will be used, else x and y
mbool StraightEqu( MajaVector& A, MajaVector& B, mreal& a, mreal& b, mbool ogl= mtrue);
mbool StraightEqu( MajaVector& A, MajaVector& B, straight_2D equation, mbool ogl= mtrue);

// r est un point dont on cherches des triangles o il serait  l'intrieur d'eux.
// retourne les indexes des 3 sommets (vertex) du triangles dans int*[3]
//int** SeekTriangles( PathScene& flr, MajaVector r);
mreal StraightLenght( MajaVector& a, MajaVector& b);
mreal StraightLenght3D( MajaVector& a, MajaVector& b);

// found the middle point of 2 points
MajaVector MiddlePoint( MajaVector& a, MajaVector& b);

}//namespace MAJAMATHS
}
#endif
