/*
	majamatrix.h
	ce fichier est une partie de Maja - this is a part of Maja
	class reprsentant une matrice 44
	class that represents a 44 matrix

	$Id: $

	$Log: $

	begin:			Sun Aug 11 2002
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef MAJA_MATRIX_H
#define MAJA_MATRIX_H

#include "majascalar.h"
#include "majavector.h"

namespace maja
{
namespace MAJAMATHS{
	
/*!
	a simple 4*4 matrix
	
	status: useable, some tests passed (+,-,*)
*/
class 	MajaMatrix:	virtual public MajaScalar
{
	protected:

	//! number of rows (here is 4)
	muint	row_nb;
	
	//! number of columns (here is 4)
	muint	column_nb;
	
	//! value of the determinator of the matrix
	_scalar	det;

	public:

	//! default constructor
	MajaMatrix( mvoid):	MajaScalar( 16)
	{
		row_nb= column_nb= 4;
		det=0;
	}
	
	//! create a matrix with all values equal to v
	MajaMatrix( _scalar v):	MajaScalar( 16)
	{
		muint i;
		
		row_nb= column_nb= 4;
		det=0;
		
		for( i=0; i<entity_nb; i++)
			scalar_ptr[i]= v;
	}
	
	//! create a 44 matrix with a pointer to the first element
	//! of an array of 16 scalar values. the arrays is ordered by
	//! filling lines before columns
	MajaMatrix( _scalar *v):MajaScalar( 16)
	{
		muint i;
		
		row_nb= column_nb= 4;
		det=0;
		
		for( i=0; i<entity_nb; i++)
			scalar_ptr[i]= *(v+i);
	}
	
	//! create a 44 matrix with 3 vectors
	//! note that a vector is defined as a column-matrix
	MajaMatrix( MajaVector& v1, MajaVector& v2, MajaVector& v3):	MajaScalar( 16)
	{
		row_nb= column_nb= 4;
		det=0;
		
		// any better way to do this ?
		scalar_ptr[ 0 ]= v1.x;	scalar_ptr[ 1 ]= v2.x;	scalar_ptr[ 2 ]= v3.x;	scalar_ptr[ 3 ]= 0;
		scalar_ptr[ 4 ]= v1.y;	scalar_ptr[ 5 ]= v2.y;	scalar_ptr[ 6 ]= v3.y;	scalar_ptr[ 7 ]= 0;
		scalar_ptr[ 8 ]= v1.z;	scalar_ptr[ 9 ]= v2.z;	scalar_ptr[ 10 ]= v3.z;	scalar_ptr[ 11 ]= 0;
		scalar_ptr[ 12 ]= v1.z;	scalar_ptr[ 13 ]= v2.z;	scalar_ptr[ 14 ]= v3.z;	scalar_ptr[ 15 ]= 0;
	}
	
	//! copy constructor
	MajaMatrix( const MajaMatrix& m):	MajaScalar( 16)
	{
		muint i;
		
		for( i=0; i<entity_nb; i++)
			scalar_ptr[i]= m.scalar_ptr[i];
			
		row_nb= column_nb= 4;
		det=0;
	}

	~MajaMatrix( mvoid)
	{
		Free();
	}
	
	//! simple stdout print of the matrix
	//! shown on multiline, test passed
	mvoid Print( mvoid)
	{
		muint i;
		
		for( i=0; i<entity_nb; i++){
			if( !i || !(i%column_nb))
				std::cout << std::endl;
			std::cout << "\t" << scalar_ptr[i];
		}
		std::cout << std::flush;
	}

	//! set the matrix as the zero matrix, test passed
	virtual inline mvoid Zero( mvoid)
	{
		muint i=0;

		for( i=0; i<entity_nb; i++)
			scalar_ptr[i]= 0;
	}

	//! return true if it's a square matrix
	//! (column_nb= row_nb) it's the case of this class, test passed
	virtual inline mbool IsSquare( mvoid)
	{
		if( column_nb==row_nb)
			return true;

		return false;
	}

	//! set this matrix to the entity matrix
	//! be aware that it applies the identity matrix only if it's a square matrix, test passed
	virtual inline mbool Identity( mvoid)
	{
		muint i, j;
		
		if( !IsSquare())
			return false;
			
		Zero();

		for( i=0; i< column_nb; i++)
			for( j=0; j< column_nb; j++)
				if( i==j)
					scalar_ptr[ 4*i + j ]= 1;
					
		return true;
	}

	protected:
	
	//! will calculate the element co-factor of a particular place in the matrix
	//! this is useful to determinate the determinator of the matrix
	virtual inline _scalar ElemCoFactor( muint row, muint column)
	{
		
		return 0;
	}

	//! will calculate the determinator of the matrix
	//! this is useful to know if the matrix could be inverted
	virtual inline _scalar Det( mvoid)
	{
		
		return 0;
	}

	public:
	
	//! will inverse the matrix & return true if all was OK
	//! return false otherwise
	virtual inline mbool Inverse( mvoid)
	{
		if( !IsSquare())
			return false;
			
		if( !Det())
			return false;
			
		;

		return false;
	}
	
	//! transposition matrix, test passed
	virtual inline mbool Transpose( mvoid)
	{
		muint c= column_nb;
		muint i, j;
		MajaMatrix tot;
		
		for( i=0; i<column_nb; i++)
			for( j=0; j<row_nb; j++)
				tot.scalar_ptr[ row_nb*i+j ]= scalar_ptr[ column_nb*j+i ];
				
		for( i=0; i<entity_nb; i++)
			scalar_ptr[i]= tot.scalar_ptr[i];
		
		column_nb= row_nb;
		row_nb= c;

		return true;
	}
	
	//! adds two matricies, test passed
	virtual inline MajaMatrix operator+		( MajaMatrix& m)
	{
		MajaMatrix p;
		muint i;
		
		for( i=0; i<entity_nb; i++)
			p.scalar_ptr[i]= scalar_ptr[i] + m.scalar_ptr[i];

		return p;
	}
	
	//! substract the 2 matricies, test passed
	virtual inline MajaMatrix operator-		( MajaMatrix& m)
	{
		MajaMatrix p;
		muint i;

		for( i=0; i<entity_nb; i++)
			p.scalar_ptr[i]= scalar_ptr[i] - m.scalar_ptr[i];
		
		return p;
	}

	//! multiply the 2 matricies, test passed
	virtual inline MajaMatrix operator*		( MajaMatrix& m)
	{
		MajaMatrix p;
		muint i, j, k;
		
		p.Zero();
		
		for( i=0; i<4; i++)
			for( j=0; j<4; j++)
				for( k=0; k<4; k++)
					p.scalar_ptr[ 4*j + i ] += scalar_ptr[ k + 4*j ] * m.scalar_ptr[ 4*k + i ];

		return p;
	}
	
	virtual inline MajaMatrix operator/	( MajaMatrix& m)
	{
		return *this;
	}
	
	virtual inline MajaMatrix operator+=	( MajaMatrix& m)
	{
		return *this;
	}

	virtual inline MajaMatrix operator-=	( MajaMatrix& m)
	{
		return *this;
	}

	//! multiply the matrix by the matrix m & assign to the original (this)	
	virtual inline MajaMatrix operator*=	( MajaMatrix& m)
	{
		MajaMatrix p;		
		muint i, j, k;

		p.Zero();

		for( i=0; i<4; i++)
			for( j=0; j<4; j++)
				for( k=0; k<4; k++)
					p.scalar_ptr[ 4*j + i ] += scalar_ptr[ k + 4*j ] * m.scalar_ptr[ 4*k + i ];
					
		*this=p;
		
		return *this;
	}

	virtual inline MajaMatrix operator/=	( MajaMatrix& m)
	{
		return *this;
	}
	
	virtual inline MajaMatrix operator--	( mvoid)
	{
		return *this;
	}
	
	virtual inline MajaMatrix operator++	( mvoid)
	{
		return *this;
	}
	
	virtual inline MajaMatrix operator--	( mint)
	{
		return *this;
	}
	
	virtual inline MajaMatrix operator++	( mint)
	{
		return *this;
	}
	
	//! equal operator, test passed
	virtual inline MajaMatrix& operator=	( MajaMatrix m)
	{
		muint i;
		
		for( i=0; i<16; i++)
			scalar_ptr[i]= m.scalar_ptr[i];
			
		entity_nb=16;
		column_nb=row_nb=4;
		det=0;
			
		return *this;
	}
	
	//
	//! comparison operators
	//
	virtual inline mbool     operator==		( MajaMatrix& m)
	{
		muint i;
		
		for( i=0; i<16; i++)
			if( scalar_ptr[i]!= m.scalar_ptr[i])
				return mfalse;
		
		return mtrue;
	}

	virtual inline mbool     operator!=		( MajaMatrix& m)
	{
		muint i;
		
		for( i=0; i<16; i++)
			if( scalar_ptr[i]!= m.scalar_ptr[i])
				return mtrue;
		
		return mfalse;
	}
	
	virtual inline mbool     operator< 		( MajaMatrix& m)
	{
		return mfalse;
	}

	virtual inline mbool     operator<=		( MajaMatrix& m)
	{
		return mfalse;
	}

	virtual inline mbool     operator> 		( MajaMatrix& m)
	{
		return mfalse;
	}

	virtual inline mbool     operator>=		( MajaMatrix& m)
	{
		return mfalse;
	}
};

}//namespace MAJAMATHS
}
#endif
