/*
	majapoly.h
	ce fichier est une partie de Maja - this is a part of Maja
	quelques polygones simples
	some simple polygons

	$Id: $

	$Log: $

	begin:			Sun Aug 18 2002
	copyright:		(C) 2002 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef MAJA_POLY_H
#define MAJA_POLY_H

#include "majascalar.h"
#include "majavector.h"
#include "geometry.h"
#include <iostream>
#include <stdlib.h>

namespace maja
{
namespace MAJAMATHS{

static const _scalar MAJA_TRIANGLE_MARGIN =		0.01;

struct MajaPlaneEquation
{
	_scalar	a;
	_scalar	b;
	_scalar	c;
	_scalar	d;
};

class MajaTriangle
{
	mbool			allocated_here;
	
	_scalar			error_margin;
	
	MajaVector		center;
	
	MajaPlaneEquation	plane_equation;
	
	protected:
	
	MajaVector&		A;
	
	MajaVector&		B;
	
	MajaVector&		C;
	
	public:
	
	MajaTriangle( mvoid):	A( *new MajaVector), 
				B( *new MajaVector), 
				C( *new MajaVector)
	{
		if( !&A || !&B || !&C){
			std::cerr << std::endl << "memory full";
			return;
		}
		allocated_here= mtrue;
		error_margin= MAJA_TRIANGLE_MARGIN;
	}
	
	MajaTriangle( MajaVector& a, MajaVector& b, MajaVector& c):	A(a), B(b), C(c)
	{
		if( !Check()){
			std::cerr << std::endl << "not a triangle !";
			throw std::exception();
		}
		allocated_here= mfalse;
		error_margin= MAJA_TRIANGLE_MARGIN;
	}
	
	~MajaTriangle( mvoid)
	{
		if( !allocated_here)
			return;
		
		delete &A;
		delete &B;
		delete &C;
	}
	
	mbool Check( mvoid)
	{
		if( A==B || A==C || B==C)
			return mfalse;
			
		return mtrue;
	}
	
	MajaVector CalculateCenter( mvoid)
	{
		static MajaVector ab= 0;
		ab= B-A;
		static MajaVector ac= 0;
		ac= C-A;
		
		center= A+ (ab+ac)/3;
		
		return center;
	}
	
	mvoid Center( mvoid)
	{
		A-=center;
		B-=center;
		C-=center;
		
		center= 0;
	}
	
	mvoid SetTriangle( const MajaVector& a, const MajaVector& b, const MajaVector& c)
	{
		A=a;
		B=b;
		C=c;
		
		if( !Check()){
			std::cerr << std::endl << "not a triangle !";
			return;
		}
		
		Orientate();
	}
	
	mvoid GetVertex( MajaVector& a, MajaVector& b, MajaVector& c)
	{
		a=A;
		b=B;
		c=C;
	}
	
	mbool CalculatePlaneEquation( mvoid)
	{
		static MajaVector ab= 0;
		ab= B-A;
		static MajaVector ac= 0;
		ac= C-A;
		static MajaVector n= 0;
		n= ab.CrossProduct( ac);
		
		if( n.IsNull())
			return mfalse;
		
		plane_equation.a= n.x;
		plane_equation.b= n.y;
		plane_equation.c= n.z;		
		plane_equation.d= -( plane_equation.a*A.x + plane_equation.b*A.y + plane_equation.c*A.z);
		
		return mtrue;
	}
	
	mvoid GetPlaneEquation( _scalar& a, _scalar& b, _scalar& c, _scalar& d)
	{
		a= plane_equation.a;
		b= plane_equation.b;
		c= plane_equation.c;
		d= plane_equation.d;
	}
	
	mbool Orientate( mvoid)
	{
		static MajaVector b;
		b= B;		
		static MajaVector o;
		o= center;
		static MajaVector oa; 
		oa= A;
		oa-=o;
		static MajaVector ob; 
		ob= b;
		ob-=o;
		static MajaVector oc;
		oc= C;
		oc-=o;
		
		if( oa.GetYAngle( oc)>0 && oa.GetYAngle( ob)<0){
			B=C;
			C=b;
			
			return mtrue;
		}
		else if( oa.GetYAngle( ob)>0 && oa.GetYAngle( oc)<0)
			return mtrue;
		
		#ifdef DEBUG
		std::cerr << std::endl << "could not orientate !";
		#endif
		
		return mfalse;
	}

	//mbool CalculateOrientation() return true if all angles where well calculated
	
	mbool IsInnerPoint( const MajaVector& point)
	{
		_scalar	total_angle=0;

		MajaVector a;		// PA
		a= A-point;
		MajaVector b;		// PB
		b =B-point;
		MajaVector c;		// PC
		c =C-point;
		
		//std::cerr << std::endl << a.GetYAngle( b);
		//std::cerr << std::endl << b.GetYAngle( c);
		//std::cerr << std::endl << c.GetYAngle( a);

		total_angle+= fabs( a.GetYAngle( b));
		total_angle+= fabs( b.GetYAngle( c));
		total_angle+= fabs( c.GetYAngle( a));
		total_angle-= two_pi;

		if( (fabs( total_angle))<error_margin){
			std::cerr << std::endl << "diff= " << fabs( total_angle) << " (should be " << error_margin << ").";
			return mtrue;
		}
		
		return mfalse;
	}
	
	_scalar	DistanceFromPoint( const MajaVector& point)
	{
		_scalar s=0;
		
		return s;
	}
};

}//namepace MAJAMATHS
}
#endif
