/*
	majaquaternion.h
	ce fichier est une partie de Maja - this is a part of Maja
	classe quaternion
	quaternion class

	$Id: $

	$Log: $

	begin:			Sun Aug 11 2002
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#ifndef MAJA_QUATERNION_H
#define MAJA_QUATERNION_H

#include "majascalar.h"
#include "majavector.h"

namespace maja
{
namespace MAJAMATHS{

/*!
	A quaternion, or a vector made with 4 dimensions...
	
	status: useable, some tests passed
*/
class MajaQuaternion: virtual public MajaScalar
{
	protected:

	_scalar magnitude;

	public:
	
	//! just some attr changed.
	_scalar& x;
	_scalar& y;
	_scalar& z;
	_scalar& w;
	
	MajaQuaternion( mvoid):	MajaScalar( 4),
				x( *scalar_ptr),
				y( *(scalar_ptr+1)),
				z( *(scalar_ptr+2)),
				w( *(scalar_ptr+3))
	{
		muint i=0;

		for( ;i<entity_nb;i++)
			scalar_ptr[i]= 0;
		magnitude= 0;
	}
	
	MajaQuaternion( _scalar v):	MajaScalar( 4),
					x( *scalar_ptr),
					y( *(scalar_ptr+1)),
					z( *(scalar_ptr+2)),
					w( *(scalar_ptr+3))
	{
		muint i=0;

		for( ;i<entity_nb;i++)
			scalar_ptr[i]= v;

		CalculateMagnitude();
	}
	
	MajaQuaternion( _scalar v1, _scalar v2, _scalar v3, _scalar v4):MajaScalar( 4),
									x( *scalar_ptr),
									y( *(scalar_ptr+1)),
									z( *(scalar_ptr+2)),
									w( *(scalar_ptr+3))
	{
		scalar_ptr[0]=v1;
		scalar_ptr[1]=v2;
		scalar_ptr[2]=v3;
		scalar_ptr[3]=v4;
		
		CalculateMagnitude();
	}
	
	MajaQuaternion( MajaVector& v):	MajaScalar( 4),
					x( *scalar_ptr),
					y( *(scalar_ptr+1)),
					z( *(scalar_ptr+2)),
					w( *(scalar_ptr+3))
	{
		scalar_ptr[0]=v.x;
		scalar_ptr[1]=v.y;
		scalar_ptr[2]=v.z;
		scalar_ptr[3]=0;
		
		CalculateMagnitude();
	}
	
	MajaQuaternion( const MajaQuaternion& q):	MajaScalar( 4),
							x( *scalar_ptr),
							y( *(scalar_ptr+1)),
							z( *(scalar_ptr+2)),
							w( *(scalar_ptr+3))
	{
		scalar_ptr[0]= q.x;
		scalar_ptr[1]= q.y;
		scalar_ptr[2]= q.z;
		scalar_ptr[3]= q.w;
		
		CalculateMagnitude();
	}

	~MajaQuaternion( mvoid)
	{
		Free();
	}
	
	//! set the last value
	inline mvoid SetWValue( _scalar val){ w= val;}
	
	//! get the last value
	inline _scalar	GetWValue( mvoid){ return w;}
	
	//! calculate the magnitude
	_scalar CalculateMagnitude( mvoid)
	{
		return magnitude= sqrt( x*x+y*y+z*z+w*w);
	}
	
	//! print the quaternion
	mvoid Print( mvoid)
	{
		std::cout << std::endl << x << " ; " << y << " ; " << z << " ; " << w;
	}
	
	//! return the dot product of 2 quaternions
	_scalar DotProduct( const MajaQuaternion& v)
	{
		_scalar p;
		
		p= x*v.x+y*v.y+z*v.z+w*v.w;
		
		return p;
	}
	
	//! return the cross product of 2 quaternions
	MajaQuaternion CrossProduct( const MajaQuaternion& v)
	{
		MajaQuaternion	p;
		/*
		p.x= y*v.z - z*v.y -;
		p.y= z*v.x - x*v.z;
		p.z= x*v.y - y*v.x;
		p.w= w...
		*/
		return p;
	}
	
	//
	//! calculation and affectation operators:
	//
	inline MajaQuaternion operator+ ( MajaQuaternion& m)
	{
		MajaQuaternion p;
		
		p.x= x+ m.x;
		p.y= y+ m.y;
		p.z= z+ m.z;
		p.w= w+ m.w;
		
		return p;
	}
	
	//! difference btw 2 quaternions
	inline MajaQuaternion operator-		( MajaQuaternion& m)
	{
		MajaQuaternion p;
		
		p.x= x- m.x;
		p.y= y- m.y;
		p.z= z- m.z;
		p.w= w- m.w;
		
		return p;
	}
	
	//! quaternion multiplication (inherited from matrix multiplication)
	inline MajaQuaternion operator*		( MajaQuaternion& m)
	{
		MajaQuaternion p;
		
		_scalar v= m.x+m.y+m.z+m.w;
		
		p.x= x*v;
		p.y= y*v;
		p.z= z*v;
		p.w= w*v;
		
		return p;
	}
	
	//! quaternion division
	//! this is a supposedly multiplication by the inverse of each coeff to *
	inline MajaQuaternion operator/		( MajaQuaternion& m)
	{
		MajaQuaternion p;
		
		_scalar v= 1/(m.x+m.y+m.z+m.w);
		
		p.x= x*v;
		p.y= y*v;
		p.z= z*v;
		p.w= w*v;
		
		return p;
	}
	
	/*
	inline MajaQuaternion operator+=	( MajaQuaternion& m);
	inline MajaQuaternion operator-=	( MajaQuaternion& m);
	inline MajaQuaternion operator*=	( MajaQuaternion& m);
	inline MajaQuaternion operator/=	( MajaQuaternion& m);
	inline MajaQuaternion operator--		( void);
	inline MajaQuaternion operator++	( void);
	inline MajaQuaternion operator--		( int);
	inline MajaQuaternion operator++	( int);
	*/
	
	inline MajaQuaternion& operator=	( const MajaQuaternion m)
	{
		unsigned int i;
		
		for( i=0; i<4; i++)
			scalar_ptr[i]= m.scalar_ptr[i];
			
		return *this;
	}
	
	/*
	//
	//! comparison operators
	//
	inline bool     operator==		( MajaQuaternion& m);
	inline bool     operator< 		( MajaQuaternion& m);
	inline bool     operator<=		( MajaQuaternion& m);
	inline bool     operator> 		( MajaQuaternion& m);
	inline bool     operator>=		( MajaQuaternion& m);
	*/
	
	//! this is only for eventually compatibility with light, but will
	//! surely be removed as soon as possible.
	const _scalar* CastToPointer( mvoid){ return &scalar_ptr[0];}
};

}//namespace MAJAMATHS

}
#endif
