/*
	majacore.cpp
	ce fichier est une partie de Maja - this is a part of Maja
	le coeur de la gestion de Maja.
	The Maja core management.

	$Id: $

	$Log: $

	begin:		Thu Sep 05 2002
	copyright:	(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:		jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#include "core/majacore.h"

namespace maja
{
	namespace MAJACORE
	{
		const mchar MajaCore::internal_name[];

		MajaCore::MajaCore( mvoid):	MajaScheduler(),
						main_clock( *new MajaClock)
		{
			main_event_stack.Push( MET_event_starting);
			std::cerr << std::endl << internal_name << ": starting.";
			gui_is_present= mfalse;
			if( !TestGUI()){
				std::cerr << std::endl << "no compatible GUI !";
				throw std::exception();
			}
		}

		MajaCore::~MajaCore( mvoid)
		{
			std::cerr << std::endl << internal_name << ": exiting.";
			
			if( main_event_stack.GetSize()== 0)
				return;
		
			if( main_event_stack.Peek()->GetEvent()== MET_event_busy)
				; // il faut attendre que le core ne soit plus occup,
				// mais a ne devrait jamais arriver...
			else if( main_event_stack.Peek()->GetEvent()== MET_event_running)
				; // terminer ce qui doit tre fait avant de continuer
		
//			main_event_stack.Push( MET_event_stopping);
			
			//while( main_event_stack.GetSize())
				;//main_event_stack.Pop( main_event_stack.Peek());

			delete &main_clock;
		
			// normally, should exit here, but must care of all before
			//exit( 0);
		}

		mbool MajaCore::Query( mvoid)
		{
			std::cerr << std::endl << "Querying...";
			
			if( main_event_stack.Peek()->GetEvent()== MET_event_starting){
				if( main_service_stack.GetSize()>0){
					main_event_stack.Push( MET_event_on);
					if( MMU_use_graphics && !gui_is_present)
						if( !TestGUI()){
							std::cerr << " failed";
							
							return mfalse;
						}
					std::cerr << " done.";
					
					return mtrue;
				}
				else std::cerr << " failed";
			}
		
			std::cerr << " failed: not vaild event in stack";
			
			return mfalse;
		}
	
		mbool MajaCore::TestGUI( mvoid)
		{
			if( gui_is_present)
				return mtrue;
			#ifndef WIN32
				// should have X started in order to use graphic
				// it happens that it's looding for X some lately...
				std::cerr << std::endl << "MajaCore: checking for X...";
				if( system( "export | grep XAUTHORITY")== 0){
					std::cerr << " success.";
					
					return gui_is_present= mtrue;
				}
				std::cerr << " failed: not present.";
				return gui_is_present= mfalse;
			#else
				return gui_is_present= mtrue;
			#endif
		}

		mvoid* MajaCore::Go( mvoid *arg)
		{
			main_clock.Start();		
			//SetSchedulingState( MSS_state_inherited);
		
			if( general_state!= MGS_state_start){
				general_state= MGS_state_start;
				Start( mnull);
			}
		
			Schedule( arg);
		
			return mnull;
		}
	
		mvoid* MajaCore::Pause( mvoid *arg)
		{
			main_clock.Step();
		
			return mnull;
		}
	
		mvoid* MajaCore::Stop( mvoid *arg)
		{
			main_clock.Stop();
		
			return mnull;
		}
	
		mvoid* MajaCore::Reset( mvoid *arg)
		{
			return mnull;
		}
	
		mvoid* MajaCore::Exit( mvoid *arg)
		{
			return mnull;
		}
	
		mbool MajaCore::AddNewService( MajaService<MajaSModule>& service, __MajaModuleType type)
		{
			main_service_list.AddObject( &service);
			main_service_stack.AddObject( &service);
		
			return Care( service, type);
		}

		mbool MajaCore::Care( MajaService<MajaSModule>& service, __MajaModuleType type)
		{
			switch( type){
				case MMU_use_graphics:
					if( th_graphic)
						return mfalse;
					SetRenderModule( service.GetModule());
					break;
				
				case MMU_use_interactivity:
					if( th_interact)
						return mfalse;
					SetInteractivityModule( service.GetModule());
					break;

				case MMU_use_game:
					if( th_game)
						return mfalse;
					SetGameModule( service.GetModule());
					break;
				
/*				case MMU_use_user:
					if( th_user)
						return mfalse;
					SetUserModule( service.GetModule());
					break;
*/				
				default:
					std::cerr << std::endl << internal_name << ": Unrecognized module type.";
					return mfalse;
			}
		
			return mtrue;
		}
	}
}
