/*
	majamemmgr.h
	ce fichier est une partie de Maja - this is a part of Maja
	gestion principale de la mmoire
	main memory management

	$Id: $

	$Log: $

	begin:		Jan 2003
	copyright:	(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:		jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *												*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *	Contact us for any commercial use attempt, please.					*
 *												*
 ************************************************************************************************/

#include "core/majamemmgr.h"

namespace maja
{
	MajaMemoryManager<mfloat>::MajaMemoryManager( mvoid)
	{
		Init();
	}

	MajaMemoryManager<mfloat>::~MajaMemoryManager( mvoid)
	{
		if( p_main_memory_start)
			free( p_main_memory_start);
		if( p_agp_memory_start){
			if( g_ext.GetNV())
				g_ext.AgpFreeMemory( p_agp_memory_start);
			else
				free( p_agp_memory_start);
		}
	}

	export template <class T>
	mvoid MajaMemoryManager<T>::InitMemory( mulong main_size, mulong agp_size)
	{
		if( mem_init)
			return;
			
		main_memory_used= 0;
		agp_memory_used= 0;

		main_memory_size= main_size;
		agp_memory_size= agp_size;

		if( !g_ext.GetDetected())
			g_ext.Detect();

		num_agp_blocks= agp_memory_size/sizeof( T);
		num_main_blocks= main_memory_size/sizeof( T);

		if( agp_size==0 && main_size>0)
			SetMemoryType( main_t);
		else if( agp_size>0 && main_size==0)
			SetMemoryType( agp_t);
		else if( agp_size>0 && main_size>0)
			SetMemoryType( both_t);
		SetMemory();
		mem_init= mtrue;
	}

	mvoid MajaMemoryManager<mfloat>::InitMemory( mulong main_size, mulong agp_size)
	{
		if( mem_init)
			return;
			
		main_memory_used= 0;
		agp_memory_used= 0;

		main_memory_size= main_size;
		agp_memory_size= agp_size;

		if( !g_ext.GetDetected())
			g_ext.Detect();

		num_agp_blocks= agp_memory_size/sizeof( mfloat);
		num_main_blocks= main_memory_size/sizeof( mfloat);

		if( agp_size==0 && main_size>0)
			SetMemoryType( main_t);
		else if( agp_size>0 && main_size==0)
			SetMemoryType( agp_t);
		else if( agp_size>0 && main_size>0)
			SetMemoryType( both_t);
		SetMemory();
		mem_init= mtrue;
	}

	MajaMemoryManager<>::MajaMemoryManager( const MajaMemoryManager<>& mmm)
	{
		InitMemory();
	}

	export template <class T>
	mvoid MajaMemoryManager<T>::Init( mvoid)
	{
		num_main_blocks= 0;
		num_agp_blocks= 0;
		p_main_memory_start= mnull;
		p_agp_memory_start= mnull;
		p_main_memory_free= mnull;
		p_agp_memory_free= mnull;
		agp_is_not_agp= mtrue;
		rf= 0.95;
		wf= 0.05;
		pv= 0.90;
		type= none_t;
		mem_init= mfalse;
	}

	mvoid MajaMemoryManager<mfloat>::Init( mvoid)
	{
		num_main_blocks= 0;
		num_agp_blocks= 0;
		p_main_memory_start= mnull;
		p_agp_memory_start= mnull;
		p_main_memory_free= mnull;
		p_agp_memory_free= mnull;
		agp_is_not_agp= mtrue;
		rf= 0.95;
		wf= 0.05;
		pv= 0.90;
		type= none_t;
		mem_init= mfalse;
	}

	export template <class T>
	mvoid MajaMemoryManager<T>::SetMemory( mvoid)
	{
		if( g_ext.GetNV()){
			p_agp_memory_start= (T*)g_ext.AgpAllocateMemory( agp_memory_size, rf, wf, pv);
			if( !p_agp_memory_start){
				std::cerr << std::endl << "MajaMemoryManager: ERROR : bad agp memory pointer !";
				throw std::exception();
			}
			agp_is_not_agp= mfalse;
			memset( p_agp_memory_start, 0, agp_memory_size);
			p_agp_memory_free= p_agp_memory_start;
		}
		else{
			agp_is_not_agp= mtrue;
			p_agp_memory_start= (T*)malloc( agp_memory_size);
			if( !p_agp_memory_start){
				std::cerr << std::endl << "MajaMemoryManager: ERROR : bad main/agp memory pointer !";
				throw std::exception();
			}
			memset( p_agp_memory_start, 0, agp_memory_size);
			p_agp_memory_free= p_agp_memory_start;
		}
		p_main_memory_start= (T*)malloc( main_memory_size);
		if( !p_main_memory_start){
			std::cerr << std::endl << "MajaMemoryManager: ERROR : bad main memory pointer !";
			throw std::exception();
		}
		memset( p_main_memory_start, 0, main_memory_size);
		p_main_memory_free= p_main_memory_start;
	}

	mvoid MajaMemoryManager<mfloat>::SetMemory( mvoid)
	{
		if( g_ext.GetNV()){
			p_agp_memory_start= (mfloat*)g_ext.AgpAllocateMemory( agp_memory_size, rf, wf, pv);
			if( !p_agp_memory_start){
				std::cerr << std::endl << "MajaMemoryManager: ERROR : bad agp memory pointer !";
				throw std::exception();
			}
			agp_is_not_agp= mfalse;
			memset( p_agp_memory_start, 0, agp_memory_size);
			p_agp_memory_free= p_agp_memory_start;
		}
		else{
			agp_is_not_agp= mtrue;
			p_agp_memory_start= (mfloat*)malloc( agp_memory_size);
			if( !p_agp_memory_start){
				std::cerr << std::endl << "MajaMemoryManager: ERROR : bad main/agp memory pointer !";
				throw std::exception();
			}
			memset( p_agp_memory_start, 0, agp_memory_size);
			p_agp_memory_free= p_agp_memory_start;
		}
		p_main_memory_start= (mfloat*)malloc( main_memory_size);
		if( !p_main_memory_start){
			std::cerr << std::endl << "MajaMemoryManager: ERROR : bad main memory pointer !";
			throw std::exception();
		}
		memset( p_main_memory_start, 0, main_memory_size);
		p_main_memory_free= p_main_memory_start;
	}

	mtype MajaMemoryManager<>::SetMemoryType( mtype t)
	{
		return type=t;
	}

	mtype MajaMemoryManager<>::GetMemoryType( mvoid)
	{
		return type;
	}

	export template <class T>
	mvoid* MajaMemoryManager<T>::Allocate( mlong size, mtype mt){
		T* ptr= mnull;
		mtype ttype= none_t;

		if( type== both_t){
			if( mt== none_t || mt== both_t)
				return mnull;
				
			ttype= mt;
		}
		else if( type==main_t)
			ttype= main_t;
		else if( type==agp_t)
			ttype= agp_t;

		if( ttype== agp_t){
			if( agp_memory_size-agp_memory_used-size>0){
				ptr= p_agp_memory_start+agp_memory_used;		//
				p_agp_memory_free= ptr+size;
				agp_memory_used+=size;

				return ptr;
			}
			else{
				cerr << "MajaMemoryManager: try to request an out of bound agp memory";

				return mnull;
			}
		}

		if( ttype== main_t){
			if( main_memory_size-main_memory_used-size>0){
				ptr= p_main_memory_start+main_memory_used;		//
				p_main_memory_free= ptr+size;
				main_memory_used+=size;

				return ptr;
			}
			else{
				cerr << "MajaMemoryManager: try to request an out of bound memory";
				
				return mnull;
			}
		}
			
		return mnull;
	}

	mvoid* MajaMemoryManager<mfloat>::Allocate( mlong size, mtype mt){
		mfloat* ptr= mnull;
		mtype ttype= none_t;

		if( type== both_t){
			if( mt== none_t || mt== both_t)
				return mnull;
				
			ttype= mt;
		}
		else if( type==main_t)
			ttype= main_t;
		else if( type==agp_t)
			ttype= agp_t;

		if( ttype== agp_t){
			if( agp_memory_size-agp_memory_used-size>0){
				ptr= p_agp_memory_start+agp_memory_used;		//
				p_agp_memory_free= ptr+size;
				agp_memory_used+=size;

				return ptr;
			}
			else{
				std::cerr << std::endl << "MajaMemoryManager: try to request an out of bound agp memory";

				return mnull;
			}
		}

		if( ttype== main_t){
			if( main_memory_size-main_memory_used-size>0){
				ptr= p_main_memory_start+main_memory_used;		//
				p_main_memory_free= ptr+size;
				main_memory_used+=size;

				return ptr;
			}
			else{
				std::cerr << std::endl << "MajaMemoryManager: try to request an out of bound memory";
				
				return mnull;
			}
		}
			
		return mnull;
	}

	export template <class T>
	mvoid MajaMemoryManager<T>::Free( mvoid* p, mtype mt)
	{
		mulong size= 0;
		T* ptr= (T*)p;
			
		if( mt== main_t){
			size=p_main_memory_free-ptr;
			p_main_memory_free= ptr;
			main_memory_used-= size;
		}
		else if( mt== agp_t){
			size=p_agp_memory_free-ptr;
			p_agp_memory_free= ptr;
			agp_memory_used-= size;
		}
		else
			cerr << endl << "MajaMemoryManager: ERROR while freeing blocks.";				
	}

	mvoid MajaMemoryManager<mfloat>::Free( mvoid* p, mtype mt)
	{
		mulong size= 0;
		mfloat* ptr= (mfloat*)p;
			
		if( mt== main_t){
			size=p_main_memory_free-ptr;
			p_main_memory_free= ptr;
			main_memory_used-= size;
		}
		else if( mt== agp_t){
			size=p_agp_memory_free-ptr;
			p_agp_memory_free= ptr;
			agp_memory_used-= size;
		}
		else
			std::cerr << std::endl << "MajaMemoryManager: ERROR while freeing blocks.";				
	}

	export template <class T>
	mvoid MajaMemoryManager<T>::PrintInfo( mvoid)
	{
		using std::cerr;
		using std::endl;

		cerr << endl << "MajaMemoryManager instance info:";
		cerr << endl << "type size= " << sizeof( T);
		cerr << endl << "main memory size= " << main_memory_size;
		cerr << endl << "main memory blocks= " << num_main_blocks << " (related to type size)";
		cerr << endl << "main memory used= " << main_memory_used;
		cerr << endl << "main blocks free= " << (main_memory_size-main_memory_used)/sizeof(T);
		cerr << endl;
		cerr << endl << "agp memory type= " << (agp_is_not_agp?"RAM":"AGP Memory");
		cerr << endl << "agp memory size= " << agp_memory_size;
		cerr << endl << "agp memory blocks= " << num_agp_blocks;
		cerr << endl << "agp memory used= " << agp_memory_used;
		cerr << endl << "agp blocks free= " << (agp_memory_size-agp_memory_used)/sizeof(T);
		cerr << endl << "agp read frequency= " << rf << "/1";
		cerr << endl << "agp write frequencey= " << wf << "/1";
		cerr << endl << "agp priority memory= " << pv << "/1";
		cerr << endl;
		cerr << endl << "main memory address= " << p_main_memory_start;
		cerr << endl << "agp memory address= " << p_agp_memory_start;
		cerr << endl;
	}

	mvoid MajaMemoryManager<mfloat>::PrintInfo( mvoid)
	{
		using std::cerr;
		using std::endl;

		cerr << endl << "MajaMemoryManager instance info:";
		cerr << endl << "type size= " << sizeof( mfloat);
		cerr << endl << "main memory size= " << (mfloat)(main_memory_size/1024) << " Kb";
		cerr << endl << "main memory blocks= " << num_main_blocks << " (related to type size)";
		cerr << endl << "main memory used= " << main_memory_used << " bits";
		cerr << endl << "main blocks free= " << (main_memory_size-main_memory_used)/sizeof(mfloat);
		cerr << endl << "main memory address= " << p_main_memory_start;
		cerr << endl << "main free address= " << p_main_memory_free;
		cerr << endl;
		cerr << endl << "agp memory type= " << (agp_is_not_agp?"RAM":"AGP Memory");
		cerr << endl << "agp memory size= " << (mfloat)(agp_memory_size/1024) << " Kb";
		cerr << endl << "agp memory blocks= " << num_agp_blocks;
		cerr << endl << "agp memory used= " << agp_memory_used << " bits";
		cerr << endl << "agp blocks free= " << (agp_memory_size-agp_memory_used)/sizeof(mfloat);
		cerr << endl << "agp read frequency= " << rf << "/1";
		cerr << endl << "agp write frequencey= " << wf << "/1";
		cerr << endl << "agp priority memory= " << pv << "/1";
		cerr << endl << "agp memory address= " << p_agp_memory_start;
		cerr << endl << "agp free address= " << p_agp_memory_free;
		cerr << endl;
	}
}//namespace maja
