/*
	majascheduler.cpp
	ce fichier est une partie de Maja - this is a part of Maja
	scheduleur temps-rel
	real-time scheduler
	
	$Id: $

	$Log: $

	begin:			Fri Jul 5 2002
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#include "core/majascheduler.h"

namespace maja
{
namespace MAJACORE
{
MajaScheduler::MajaScheduler( mvoid): MajaSModule()
{
	maja_internal_name= new mchar[14];
	strcpy( maja_internal_name, "MajaScheduler");
	std::cerr << std::endl << maja_internal_name << ": creating...";
	th_graphic= 			mnull;
	th_interact= 			mnull;
	th_game=			mnull;
	th_user= 			mnull;
	th_winmgr=			mnull;
	use_module= 			MMU_use_none;
	general_state= 			MGS_state_off;
	scheduling_state=		MSS_state_inherited;
	scheduling_type= 		MST_type_maja;
	modules_looping=		0;
	maja_end_loop=			mfalse;
	__MAJA_SCHEDULER_PRIORITY=	0;
	pid= 				getpid();
	window=				mnull;
	std::cerr << " done.";
}

MajaScheduler::~MajaScheduler( mvoid)
{
	delete maja_internal_name;
	if( th_winmgr)
		delete th_winmgr;
	std::cerr << std::endl << typeid( this).name() << ": ended" << std::endl;
}

mvoid MajaScheduler::SetRenderModule( MajaSModule *module)
{
	th_graphic= module;
	if( !(use_module & MMU_use_graphics))
		use_module|= MMU_use_graphics;
}

mvoid MajaScheduler::SetInteractivityModule( MajaSModule *module)
{
	th_interact= module;
	if( !(use_module & MMU_use_interactivity))
		use_module|= MMU_use_interactivity;
}

mvoid MajaScheduler::SetGameModule( MajaSModule *module)
{
	th_game= module;
	if( !(use_module & MMU_use_game))
		use_module|= MMU_use_game;
}

mvoid MajaScheduler::SetUserModule( MajaSModule *module)
{
	th_user= module;
	if( !(use_module & MMU_use_user))
		use_module|= MMU_use_user;
}

mint MajaScheduler::Start( mvoid *arg)
{
	std::cerr << std::endl << maja_internal_name << ": starting...";
	
	// only from maja's own type for now
	if( scheduling_type== MST_type_maja){
		maja_schedule_algo= SCHED_OTHER;
		GetPriority();
			
		if( priority<priority_max/4){
//TODO: adjust the priority dynamically and for the best result.
			priority= 10;//					priority_max/4;
			mint sched= 0;
			sched_param t_sp;
			
			if( (sched= sched_getscheduler( pid))<0){
				perror( "sched_getscheduler");
				exit( -1);
			}
			if( sched_getparam( pid, &t_sp)<0){
				perror( "sched_getparam");
				exit( -1);
			}
			if( !&t_sp){
				std::cerr << std::endl << maja_internal_name << "no scheduling parameter could be found";
				exit( -1);
			}
			
			sched_params= t_sp;
			sched_params.sched_priority= priority;
			
			if( sched!= maja_schedule_algo){
				std::cerr << std::endl << maja_internal_name << ": switching to SCHED_RR";

				if( sched_setscheduler( pid, maja_schedule_algo, &sched_params)<0){
					perror( "\nsched_setscheduler");
					exit( -1);
				}
				__MAJA_SCHEDULER_PRIORITY= priority;
			}
			std::cerr << std::endl << "MajaScheduler: priority= " << priority;
		}
	}
	else{
		std::cerr << std::endl << maja_internal_name << ": error: only maja scheduling politic is recognized.";
		std::cerr << std::endl << maja_internal_name << ": exiting now...";
		return 0;
	}

	std::cerr << std::endl << maja_internal_name << ": starting modules...";
	
	mint nb_started= StartModules();
	if( nb_started<=0)
		std::cerr << std::endl << "MajaScheduler: started failed.";

//	if( scheduling_state== MSS_state_inherited)
//		EnterDefaultMode();

	return nb_started;
}

mint MajaScheduler::StartModules( mvoid)
{
	static mint modules_started=0;
	
	if( modules_started>0)
		return 0;
	
	StartWinMgrModule();

	if( use_module & MMU_use_graphics)
		if( StartGraphicModule()>0)
			modules_started++;
	
	if( use_module & MMU_use_interactivity)
		if( StartInteractivityModule()>0)
			modules_started++;

	if( use_module & MMU_use_game)
		if( StartGameModule()>0)
			modules_started++;

	sleep( 1);

	return modules_looping= modules_started;
}

// should all the modules have MMS_state_pause, so
// the scheduler could better start them when all
// are effectively ready (or create a state MMS_state_ready)
mint MajaScheduler::StartGraphicModule( mvoid)
{
	Lock();
	std::cerr << std::endl << "MajaScheduler: starting graphic module...";
	th_graphic->SetScheduleAlgo( SCHED_OTHER);
//	th_graphic->SetPriority( 0);//__MAJA_SCHEDULER_PRIORITY-1);	
	th_graphic->SetState( MMS_state_start);

	if( th_graphic->EnterDefaultMode()<=0){
		std::cerr << std::endl << "can't enter render module";
		Unlock();
		
		return -MMU_use_graphics;
	}
	Unlock();
	
	return 1;
}
		
mint MajaScheduler::StartInteractivityModule( mvoid)
{
	Lock();
	std::cerr << std::endl << "MajaScheduler: starting interactivity module...";
	th_interact->SetScheduleAlgo( SCHED_OTHER);
//	th_interact->SetPriority( __MAJA_SCHEDULER_PRIORITY/2);
	th_interact->SetState( MMS_state_start);
		
	if( th_interact->EnterDefaultMode()<=0){
		std::cerr << std::endl << "can't enter interactivity module";
		Unlock();
		
		return -MMU_use_interactivity;
	}
	Unlock();
	return 1;
}

mint MajaScheduler::StartGameModule( mvoid)
{
	Lock();
	std::cerr << std::endl << "MajaScheduler: starting game module...";
	th_game->SetScheduleAlgo( SCHED_OTHER);
//	th_game->SetPriority( __MAJA_SCHEDULER_PRIORITY/2);
	th_game->SetState( MMS_state_start);

	if( th_game->EnterDefaultMode()<=0){
		std::cerr << std::endl << "can't enter game module";
		Unlock();
		
		return -MMU_use_game;
	}
	Unlock();
	
	return 1;
}

mint MajaScheduler::StartWinMgrModule( mvoid)
{
	Lock();
	std::cerr << std::endl << "MajaScheduler: starting window manager module...";
	if( !window){
		std::cerr << std::endl << "MajaScheduler: can't start window services since window is not created !";
		Unlock();
		
		return -1;
	}

	if( th_winmgr){
		std::cerr << std::endl << "MajaScheduler: window manager already exists ! deleting.";
		delete th_winmgr;
	}
	th_winmgr= new MajaWinMgr();
	if( !th_winmgr){
		Unlock();
		return -10;
	}

	th_winmgr->SetWindow( window);
	th_winmgr->SetScheduleAlgo( SCHED_OTHER);	
//	th_winmgr->SetPriority( 0);//__MAJA_SCHEDULER_PRIORITY-1);
	th_winmgr->SetGraphicModule( th_graphic);
	th_winmgr->SetInteractivityModule( th_interact);
	th_winmgr->SetState( MMS_state_start);
	
	if( th_winmgr->EnterDefaultMode()<=0){
		std::cerr << std::endl << "can't enter window manager module.";
		Unlock();
		
		return -11;
	}
	Unlock();
	
	return 1;
}

mint MajaScheduler::Schedule( mvoid *arg)
{
	static mdouble	s_clk= 0;

	int_clock.Start();
	s_clk= int_clock.Clock();
	
	maja_core_mutex.stderr_mutex.Lock();
	std::cerr << std::endl << "MajaScheduler: setting all modules to on";
	maja_core_mutex.stderr_mutex.Unlock();

	
	// tudier l'influence du set de l'tat avant ou aprs le lock.

	th_winmgr->Lock();
	th_winmgr->SetState( MMS_state_on);
	th_winmgr->CondSignal();
	th_winmgr->Unlock();

	if( use_module & MMU_use_graphics){
		th_graphic->Lock();
		th_graphic->SetState( MMS_state_on);
		th_graphic->CondSignal();
		th_graphic->Unlock();
	}

	if( use_module & MMU_use_interactivity){
		th_interact->Lock();
		th_interact->SetState( MMS_state_on);
		th_interact->CondSignal();
		th_interact->Unlock();
	}

	if( use_module & MMU_use_game){
		th_game->Lock();
		th_game->SetState( MMS_state_on);
		th_game->CondSignal();
		th_game->Unlock();
	}
	
	sleep( 1);
	
	while( !maja_end_loop){		
		th_winmgr->Lock();
		if( th_winmgr->GetState()==MMS_state_off){
			maja_end_loop= mtrue;
			th_game->Lock();
			th_game->SetState( MMS_state_off);
			th_game->Unlock();
		}
		th_winmgr->Unlock();
		th_game->Lock();
		if( th_game->GetState()==MMS_state_off){
			maja_end_loop= mtrue;
			th_winmgr->Lock();
			th_winmgr->SetState( MMS_state_off);		
			th_winmgr->Unlock();
		}
		th_game->Unlock();
		
		usleep( 10000);
	}

	general_state= MGS_state_off;	
	std::cerr << std::endl << "MajaScheduler: received maja_end_loop." << std::endl;
	int_clock.Stop();
	
	return 1;
}

mint MajaScheduler::EnterDefaultMode( mvoid *arg)
{
	sched_param sp;

	if( priority== priority_min== priority_max== 0){
		GetPriority();
		std::cerr << std::endl << priority << " " << priority_min << " " << priority_max;
	}
		
	pthread_attr_init( &attributes);
	pthread_attr_setschedpolicy( &attributes, maja_schedule_algo);
	pthread_attr_getschedparam( &attributes, &sp);		
	sp.sched_priority= priority;
	pthread_attr_setschedparam( &attributes, &sp);
	std::cerr << std::endl << "MajaScheduler: priority= " << priority;

	if( !Create( mnull)){
		std::cerr << std::endl << "could not create MajaScheduler module thread";
		if( errno==EPERM)
			std::cerr << std::endl << "you may not be seteuid=0";
		return 0;
	}

	return 1;

}

}//namespace MAJACORE
}//namespace maja
