/*
	majasys.cpp
	ce fichier est une partie de Maja - this is a part of Maja
	gestion principale du systme
	main system management

	$Id: $

	$Log: $

	begin:		Sep 2000
	copyright:	(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:		jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#include "core/majasys.h"

#ifdef _GNU_SOURCE
#include <sys/sysinfo.h>
#include <sys/utsname.h>
#endif

namespace maja
{
	#ifdef _GNU_SOURCE
	extern "C"{
		struct utsname uts_name;
		struct sysinfo     info;
	}
	#endif
	
	namespace MAJACORE
	{
	
	//! one mega-bytes
	static const mlong M_ONE_MO= 1048576;

	MajaSystem<>::MajaSystem( mvoid): sys_core( *new MAJACORE::MajaCore)
	{
		SysInit();
	}

	MajaSystem<>::MajaSystem( mint argc, mchar **argv): sys_core( *new MAJACORE::MajaCore)
	{
		SysInit();
		if( !ProcessInit( argc, argv))
			throw std::exception();
	}

	export template <class GE>
	MajaSystem<GE>::MajaSystem( const MajaSystem& sys): sys_core( *new MAJACORE::MajaCore)
	{
		;
	}

	MajaSystem<>::~MajaSystem( mvoid)
	{
		using namespace std;
		cerr << endl << "MajaSystem: ending...";
		if( !ProcessExit())
			throw std::exception();
		SysExit();
		delete &sys_core;
		cerr << endl << "MajaSystem: ended." << endl;
	}

	export template <class GE>
	mvoid MajaSystem<GE>::SysInit( mvoid)
	{
		std::cout << std::endl << "Maja " << __MAJA_VERSION;
		std::cout << std::endl << __MAJA_COPYRIGHT;
		std::cout << std::endl << __MAJA_WARNINFO_1;
		std::cout << std::endl << __MAJA_WARNINFO_2 << std::flush;
		
		maja_s_render=	mnull;
		maja_s_inter= 	mnull;
		maja_s_game=  	mnull;
		maja_disp=    	mnull;
		gl_ext=		mnull;
			
		#ifndef WIN32
		sys_display= XOpenDisplay( mnull);
		window= new MajaXWindow( sys_display);
		#else
		std::cerr << std::endl << "OpenGL window is only useable threw glut";
		throw std::exception();
		#endif
		if( !window || !sys_display){
			std::cerr << std::endl << "MajaSystem: ERROR: Not compatible environment";
			throw std::exception();
		}

		maja_copyright= "Maja, version ";
		maja_copyright+= __MAJA_VERSION;
		maja_copyright+=". ";
		maja_copyright+=__MAJA_COPYRIGHT;
	}

	export template <class GE>
	mvoid MajaSystem<GE>::SysExit( mvoid)
	{
		;
	}

	export template <class GE>
	mbool MajaSystem<GE>::ProcessInit( mint argc, mchar **argv)
	{
		process_info._argv= argv;
		process_info._argc= argc;

		std::cerr << std::endl << "MajaSystem: ProcessInit()...";
		maja_disp= new MAJAGRAPHIC::MajaDisplay();
		//usleep( 100000);
		maja_s_render= new MAJAGRAPHIC::MajaRendererModule();
		maja_s_inter= new MajaInteractive();
		maja_s_game= new MajaSGamer<MajaDefaultGameEngine,MajaIEngine,MAJAGRAPHIC::MajaDisplay>();
		if( !maja_disp || !maja_s_render || !maja_s_inter || !maja_s_game){
			std::cerr << std::endl << "MajaSystem: out of memory";
			throw std::exception();
		}

		CheckArguments( argc, argv);
		window->CreateWindow();
		sys_core.SetWindow( window);
		maja_s_inter->SetWindow( window);
		maja_disp->SetMajaCopyright( maja_copyright);
		maja_disp->SetWindow( window);
		maja_disp->SetMemoryManager( mem_mgr);
		maja_disp->Init();
		maja_s_render->_d= maja_disp;
		maja_s_game->SetIEngine( maja_s_inter->GetEngine());
		maja_s_game->SetREngine( maja_disp);

		gl_ext= new MajaGExt();
		if( !gl_ext)
			return mfalse;
		gl_ext->Detect();

		// init for around 25000 faces (agp memory), the main
		// memory has the double of the agp seize.
		mem_mgr.InitMemory( 7*M_ONE_MO, 3*M_ONE_MO);
		mem_mgr.PrintInfo();

		std::cerr << std::endl << "MajaSystem: process init done.";
		
		return mtrue;
	}

	export template <class GE>
	mvoid MajaSystem<GE>::CheckArguments( mint argc, mchar **argv)
	{
		using namespace std;
		
		mint i= 1;

		if( argc<=1)
			return;

		for( ;i<argc;i++){
			if( strstr( argv[i], "-")){
				if( strstr( argv[i], "f"))
					window->GetAttributes().SetFullScreen( mtrue);
				if( strstr( argv[i], "r")){
					mint w,h;
					sscanf( argv[i+1], "%dx%d", &w, &h);
					window->GetAttributes().width= w;
					window->GetAttributes().height= h;
				}
				if( strstr( argv[i], "c"))
					use_console= mtrue;
				else
					use_console= mfalse;
				if( strstr( argv[i], "-help")){
					cout << endl << "Maja system version " << __MAJA_VERSION << endl;
					cout << endl << "Usage: EXECUTABLE_NAME [OPTIONS]...";
					cout << endl << " --help\t\t\tshow this help";
					cout << endl << " -f\t\t\tfullscreen mode";
					cout << endl << " --r WIDTHxHEIGHT\twindow size is WIDHT * HEIGHT";
					cout << endl << " -c\t\t\tuse console: DON'T USE IT !" << endl << flush;
					exit( 0);
				}
			}
		}
	}

	export template <class GE>
	mbool MajaSystem<GE>::ProcessExit( mvoid)
	{
		//TODO: destroy here, so the scheduler (and its modules) get free from
		//	this task in multi-thread.
		//window->DestroyWindow();
		
		delete maja_s_game;
		delete maja_s_inter;
		delete maja_s_render;
		delete maja_disp;
	//	delete gl_ext;

		maja_s_game= mnull;
		maja_s_inter= mnull;
		maja_s_render= mnull;
		maja_disp= mnull;
	//	gl_ext= mnull;
		
		return mtrue;
	}

	mbool MajaSystem<>::StartSubSystems( mvoid)
	{
		std::cerr << std::endl << "MajaSystem: starting sub-systems...";
		MajaService<MajaSModule> my_graph_service( maja_s_render);
		if( !sys_core.AddNewService( my_graph_service, MMU_use_graphics)){
			std::cout << std::endl << "the graphic service could not be added.";
			return mfalse;
		}

		MajaService<MajaSModule> my_inter_service( maja_s_inter);
		if( !sys_core.AddNewService( my_inter_service, MMU_use_interactivity)){
			std::cout << std::endl << "the interactivity service could not be added.";
			return mfalse;
		}

		MajaService<MajaSModule> my_game_service( maja_s_game);
		if( !sys_core.AddNewService( my_game_service, MMU_use_game)){
			std::cout << std::endl << "the game service could not be added.";
			return mfalse;
		}
	
		//! we test that we have all done correctly (a graphic module min)
		if( !sys_core.Query()){
			std::cout << std::endl << "the core is not ready.";
			return mfalse;
		}
		std::cerr << std::endl << "MajaSystem: sub-sytems started.";

		return mtrue;
	}

	mbool MajaSystem<>::StopSubSystems( mvoid)
	{
		std::cerr << std::endl << "MajaSystem: stopping sub-systems...";

		return mtrue;
	}

	mbool MajaSystem<>::Check( mvoid)
	{
		if( game_engine->GetGameState()< game_state_loading){
			std::cerr << std::endl << "MajaSystem: check failed due to game state < loading";
			return mfalse;
		}

		return mtrue;
	}

	mvoid MajaSystem<>::Go( mvoid)
	{
		if( !use_console)
			sys_core.Go();
		else
			sys_core.EnterDefaultMode();
	}

	mbool MajaSystem<>::ComputerInfoInit( mvoid)
	{
		return mfalse;
	}

	mbool MajaSystem<>::StartGameEngine( mvoid)
	{	
		std::cerr << std::endl << "MajaSystem: starting game engine...";
		game_engine= &maja_s_game->GetGameEngine();
		if( !&game_engine){
			std::cerr << " failed.";
			return mfalse;
		}
		
		game_engine->SetRenderingEngine( *maja_disp);
		game_engine->SetInteractiveEngine( *maja_s_inter->GetEngine());
		game_engine->SetMenuEngine( maja_disp->GetMenu());

		std::cerr << " done.";
		
		return mtrue;
	}

	mbool MajaSystem<>::StopGameEngine( mvoid)
	{
		;
		return mtrue;
	}

	
	}// namespace MAJACORE

	
	mbool MajaSystemInterface<>::Start( mvoid)
	{
		if(! StartSubSystems())
			return mfalse;

		Go();

		return mtrue;
	}

	mbool MajaSystemInterface<>::Stop( mvoid)
	{
		StopGameEngine();
		return StopSubSystems();
	}
}
