/*
	majawingr.cpp
	ce fichier est une partie de Maja - this is a part of Maja
	gestionnaire de fentre temps-rel
	real-time window manager
	
	$Id: $

	$Log: $

	begin:			Fri Jul 5 2002
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#include "core/majawinmgr.h"

namespace maja
{

namespace MAJACORE
{
	
mint MajaWinMgr::EnterDefaultMode( mvoid *params)
{
	Lock();
	sched_param sp;

	pthread_attr_init( &attributes);
	pthread_attr_setschedpolicy( &attributes, maja_schedule_algo);
	pthread_attr_getschedparam( &attributes, &sp);		
	sp.sched_priority= priority;
	pthread_attr_setschedparam( &attributes, &sp);
			
	if( !Create( mnull)){
		std::cerr << std::endl << "could not create MajaWinMgr thread";
		if( errno==EPERM)
			std::cerr << std::endl << "you may not be seteuid=0";
		Unlock();

		return 0;
	}
	Unlock();
	
	return 1;
}

mvoid* MajaWinMgr::Go( mvoid* arg)
{
	_MajaWinEvent	event;
	__MajaWindowState wstate;
	GLXContext c;
	
	Lock();
	if( GetState()!=MMS_state_start){				
		maja_core_mutex.stderr_mutex.Lock();
		std::cerr << std::endl << "MajaWinMgr: wanted to go, but not started ! exiting...";
		maja_core_mutex.stderr_mutex.Unlock();				
		SetState( MMS_state_off);
		Unlock();

		return mnull;
	}
	Unlock();

	maja_core_mutex.stderr_mutex.Lock();
	std::cerr << std::endl << "MajaWinMgr: waiting for scheduler order...";
	maja_core_mutex.stderr_mutex.Unlock();
	Lock();
	CondWait();
	Unlock();
	maja_core_mutex.stderr_mutex.Lock();
	std::cerr << std::endl << "MajaWinMgr: rounding...";
	maja_core_mutex.stderr_mutex.Unlock();

	window->GetDrawable().Lock();
	c= window->GetDrawable().GetMainContext();
	window->GetDrawable().Unlock();

	while( !close_thread){
		Lock();
		if( GetState()== MMS_state_off){
			maja_core_mutex.stderr_mutex.Lock();
			std::cerr << std::endl << "MajaWinMgr: state is off !!!";
			maja_core_mutex.stderr_mutex.Unlock();
			Unlock();
			
			return mnull;
		}
		
		SetState( MMS_state_busy);
		Unlock();
	
		maja_core_mutex.stderr_mutex.Lock();
		window->event_mutex.Lock();
/*		window->GetDrawable().Lock();
		if( !glXGetCurrentContext())
			window->GetDrawable().Enter( c);
*/		event= window->LoopEvent();
		window->event_mutex.Unlock();
//		window->GetDrawable().Unlock();
		maja_core_mutex.stderr_mutex.Unlock();

		events_nb++;
		
		switch( event.type){
			case Expose:
/*				window->Lock();
				window->SetState( MWS_state_need_redraw);
				window->Unlock()
				th_graphic->Lock();
				if( th_graphic->GetState()!= MMS_state_busy){
//					window->GetDrawable().Enter();
//					glXMakeCurrent(	window->GetDisplay(), window->GetWindow(), window->GetContext());
//					window->Draw();
					window->SetState( MWS_state_alive);
					events_nb--;
				}
				th_graphic->Unlock();
				window->Unlock();
*/				break;
			case ConfigureNotify:
				window->Lock();
				window->SetState( MWS_state_need_resize);
				#ifdef SHOW_WINMGR_INFO
				maja_core_mutex.stderr_mutex.Lock();
				std::cerr << std::endl << "MajaWinMgr: window need resize...";
				maja_core_mutex.stderr_mutex.Unlock();
				#endif
				window->GetDrawable().Lock();
				window->GetDrawable().Enter();
				window->Resize();
				window->GetDrawable().Leave();
				window->GetDrawable().Unlock();
				window->SetState( MWS_state_alive);
				window->Unlock();
				#ifdef SHOW_WINMGR_INFO
				maja_core_mutex.stderr_mutex.Lock();
				std::cerr << " done.";
				maja_core_mutex.stderr_mutex.Unlock();
				#endif
				break;
			case ClientMessage:
				window->Lock();
				window->SetState( MWS_state_need_close);				
				window->event_mutex.Lock();
				PushCurrentEventInStack();
				PushCurrentEventInKeyStack();
				window->event_mutex.Unlock();
				window->Unlock();
				SetEventTreated( mfalse);
				break;
			case KeyPress:
			case KeyRelease:
			case ButtonPress:
			case ButtonRelease:
			case MotionNotify:
				window->event_mutex.Lock();
				PushCurrentEventInKeyStack();
				window->event_mutex.Unlock();
				break;
			default:
				std::cerr << std::endl << "default window's event";
				window->event_mutex.Lock();
				PushCurrentEventInStack();
				window->event_mutex.Unlock();
				Lock();
				SetState( MMS_state_on);
				Unlock();
				continue;
		}

		window->Lock();
		wstate= window->GetState();
		window->Unlock();
		
		switch( wstate){
			case MWS_state_need_close:

				th_graphic->Lock();
				std::cerr << std::endl << "exiting graphic...";
				if( th_graphic->GetState()!=MMS_state_off)
					th_graphic->SetState( MMS_state_off);
				th_graphic->Unlock();
				
				th_interact->Lock();
				std::cerr << std::endl << "exiting interactivity...";
				if( th_interact->GetState()!=MMS_state_off);
					th_interact->SetState( MMS_state_off);
				th_interact->Unlock();
				
				std::cerr << std::endl << "destroying window...";
				window->Lock();
				window->GetDrawable().Lock();
				window->DestroyWindow();
				window->SetState( MWS_state_dead);
				window->GetDrawable().Unlock();
				window->Unlock();
				
				Lock();
				SetState( MMS_state_off);
				std::cerr << std::endl << "exiting window manager...";
				Unlock();

				pthread_exit( mnull);
				break;
			// it is very dangerous to try to enter fullscreen mode !
			case MWS_state_need_switch:/*
				th_graphic->SetState( MMS_state_pause);
				maja_core_mutex.window_mutex.Unlock();
				while( 1){
					th_graphic->Lock();
					if( th_graphic->GetState()==MMS_state_pause){
						maja_core_mutex.window_mutex.Lock();
						break;
					}
					th_graphic->Unlock();
					usleep( 10000);
				}
				window->DestroyWindow();
				MajaWinAttr& attr= window->GetAttributes();
				attr.fullscreen= !attr.fullscreen;
				window->SetAttributes( attr);
				window->CreateWindow()*/;
				break;
			default:
				;
		}

		th_graphic->Lock();
		if( th_graphic->GetState()==MMS_state_off){
			th_interact->Lock();
			th_interact->SetState( MMS_state_off);
			th_interact->Unlock();
			window->Lock();
			window->SetState( MWS_state_need_redraw);
			window->Unlock();
			SetState( MMS_state_off);
		}
		th_graphic->Unlock();
		Lock();
		SetState( MMS_state_on);
		Unlock();
	}// while( !close_thread){

	return mnull;
}


}
}
