/*
	graphext.cpp
	ce fichier est une partie de Maja - this is a part of Maja
	support des extensions OpenGL
	OpenGL extensions support

	$Id: $

	$Log: $

	begin:			2001
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#include "engines/graphic/graphext.h"
#include <stdlib.h>

namespace maja
{
	
mbool 				MajaGExt::use_nv= 					mfalse;
mbool 				MajaGExt::use_nv_array=		 			mfalse;
mbool 				MajaGExt::use_nv_alloc= 				mfalse;
mbool 				MajaGExt::use_nv_draw= 					mfalse;
mbool 				MajaGExt::use_gl_ext= 					mfalse;
mbool 				MajaGExt::ctx_available= 				mfalse;
mbool 				MajaGExt::detected= 					mfalse;
mbool 				MajaGExt::use_cva_array=				mfalse;
mbool				MajaGExt::use_ext_draw=					mfalse;
mlong 				MajaGExt::heap=						0;
muint				MajaGExt::count=					0;	
mlong	*			MajaGExt::memory_counter=				mnull;
GLfloat *			MajaGExt::memory_pointer=				mnull;
GLubyte *			MajaGExt::ext_string= 					mnull;
GLubyte *			MajaGExt::render_string= 				mnull;
mbool				MajaGExt::multitexture_support=				mfalse;

#ifndef WIN32
PFNGLXALLOCATEMEMORYNVPROC 			MajaGExt::AgpAllocateMemory=		0;
PFNGLXFREEMEMORYNVPROC 				MajaGExt::AgpFreeMemory=		0;
#else
PFNWGLALLOCATEMEMORYNVPROC			MajaGExt::AgpAllocateMemory=		0;
PFNWGLFREEMEMORYNVPROC 				MajaGExt::AgpFreeMemory=		0;
#endif

PFNGLVERTEXARRAYRANGENV 			MajaGExt::VertexArrayRange=		0;
PFNGLFLUSHVERTEXARRAYRANGENVPROC		MajaGExt::FlushVertexArrayRange=	0;
PFNGLVERTEXPOINTEREXTPROC 			MajaGExt::VertexPointer=		0;
PFNGLNORMALPOINTEREXTPROC			MajaGExt::NormalPointer=		0;
PFNGLCOLORPOINTEREXTPROC			MajaGExt::ColorPointer=			0;
PFNGLTEXCOORDPOINTEREXTPROC			MajaGExt::TexelPointer=			0;
PFNGLDRAWRANGEELEMENTSEXTPROC 			MajaGExt::DrawRangeElements=		0;
PFNGLINDEXPOINTEREXTPROC			MajaGExt::IndexPointer=			0;
PFNGLDRAWARRAYSEXTPROC				MajaGExt::DrawArrays= 			0;
PFNGLLOCKARRAYSEXTPROC				MajaGExt::LockArrays=			0;
PFNGLUNLOCKARRAYSEXTPROC			MajaGExt::UnlockArrays=			0;
PFNGLBINDTEXTUREEXTPROC				MajaGExt::BindTexture=			0;
PFNGLARETEXTURESRESIDENTEXTPROC			MajaGExt::AreTexturesResident=		0;
PFNGLDELETETEXTURESEXTPROC			MajaGExt::DeleteTextures=		0;
PFNGLGENTEXTURESEXTPROC				MajaGExt::GenTextures=			0;
PFNGLISTEXTUREEXTPROC				MajaGExt::IsTexture=			0;
PFNGLPRIORITIZETEXTURESEXTPROC			MajaGExt::PrioritizeTextures=		0;
PFNGLMULTITEXCOORD2FVARBPROC			MajaGExt::MultiTexCoord=		0;
PFNGLCLIENTACTIVETEXTUREARBPROC			MajaGExt::ClientActiveTexture=		0;

MajaGExt::MajaGExt( mvoid)
{
	#ifndef WIN32
		GetFunctionPointer= (__get_gl_ext_proc_ptr) glXGetProcAddressARB;
	#else
		GetFunctionPointer= (__get_gl_ext_proc_ptr) wglGetProcAddress;
	#endif
	detected= mfalse;
}

MajaGExt::~MajaGExt( mvoid)
{
	// only for count equal to 1 because it frees all the memory allocated !!
	if( &this->memory_pointer[0]!= null && count==1)
		this->FreeMemory();
}
		
mbool MajaGExt::Detect( mvoid)
{	
	if( detected){
		std::cerr << std::endl << "MajaGExt: already detected";
		return mtrue;
	}
	
	render_string= (GLubyte*)glGetString( GL_RENDERER);	
	// no valid context, so exit:
	if( !render_string){
		ctx_available= mfalse;
		return mfalse;
	}
	ctx_available= mtrue;
	
	if( strstr( (const mchar*)render_string, "GeForce") || strstr( (const mchar*)render_string, "TNT")){
		#ifndef __MAJA_GL_NV
		#define __MAJA_GL_NV
		#endif
		std::cerr << std::endl << "MajaGExt: Supported graphic card: " << render_string;
		use_nv= mtrue;
		
		#ifndef WIN32
		AgpAllocateMemory=
		(PFNGLXALLOCATEMEMORYNVPROC)GetFunctionPointer((const GLubyte *)"glXAllocateMemoryNV");
		
		AgpFreeMemory=
		(PFNGLXFREEMEMORYNVPROC)GetFunctionPointer((const GLubyte*)"glXFreeMemoryNV");
		#endif
		
		if( !AgpAllocateMemory || !AgpFreeMemory){
			use_nv_alloc= mfalse;
			#ifdef __MAJA_GL_NV
			#undef __MAJA_GL_NV
			#endif
		}
		use_nv_alloc= mtrue;
	}
	else{
		#ifdef __MAJA_GL_NV
		#undef __MAJA_GL_NV
		#endif
		use_nv= mfalse;
		std::cerr << std::endl << "MajaGExt: Unknown graphic card: " << render_string;
	}
	
	// get the extensions supported
	ext_string= (GLubyte*)glGetString( GL_EXTENSIONS);

	// no extension, so no need to continue
	if( !ext_string){
		std::cerr << std::endl << "MajaGExt: no extension found !";
		use_gl_ext= mfalse;
		return mfalse;
	}
	use_gl_ext= mtrue;

	if( strstr( (const mchar*)ext_string, "GL_ARB_multitexture")){
		std::cerr << std::endl << "MajGExt: multitexturing is supported";
		multitexture_support= mtrue;
		MultiTexCoord=
		(PFNGLMULTITEXCOORD2FVARBPROC)GetFunctionPointer((const GLubyte*)"glMultiTexCoord2fvARB");
		if( !MultiTexCoord)
			MultiTexCoord=
			(mvoid(*)(GLenum,GLfloat*))GetFunctionPointer( (const GLubyte*)"glTexCoord2fv");
	
		ClientActiveTexture=
		(PFNGLCLIENTACTIVETEXTUREARBPROC)GetFunctionPointer((const GLubyte*)"glClientActiveTextureARB");
	}
	else{
		std::cerr << std::endl << "MajaGext: multitexturing is not supported";
		multitexture_support= mfalse;
	}	
	
	VertexArrayRange=
	(PFNGLVERTEXARRAYRANGENV)GetFunctionPointer((const GLubyte*)"glVertexArrayRangeNV");
	
	FlushVertexArrayRange=
	(PFNGLFLUSHVERTEXARRAYRANGENVPROC)GetFunctionPointer((const GLubyte*)"glFlushVertexArrayRangeNV");
	
	if( !VertexArrayRange || !FlushVertexArrayRange){
		std::cerr << std::endl << "MajaGExt: NVidia card without VAR.";
		use_nv_array= mfalse;
	}
	else
		use_nv_array= mtrue;
	
	
	
	VertexPointer=
	(PFNGLVERTEXPOINTEREXTPROC)GetFunctionPointer( ( const GLubyte*)"glVertexPointerEXT");
	
	NormalPointer=
	(PFNGLNORMALPOINTEREXTPROC)GetFunctionPointer( ( const GLubyte*)"glNormalPointerEXT");
	
	ColorPointer=
	(PFNGLCOLORPOINTEREXTPROC)GetFunctionPointer( ( const GLubyte*)"glColorPointerEXT");
	
	TexelPointer =
	(PFNGLTEXCOORDPOINTEREXTPROC)GetFunctionPointer( ( const GLubyte*)"glTexCoordPointerEXT");
	
	IndexPointer=
	(PFNGLINDEXPOINTEREXTPROC)GetFunctionPointer( ( const GLubyte*)"glIndexPointerEXT");

	
	
	DrawRangeElements=
	(PFNGLDRAWRANGEELEMENTSEXTPROC)GetFunctionPointer( (const GLubyte*)"glDrawRangeElementsEXT");
	
	DrawArrays=
	(PFNGLDRAWARRAYSEXTPROC)GetFunctionPointer( (const GLubyte*)"glDrawArraysEXT");

	
	
	LockArrays=
	(PFNGLLOCKARRAYSEXTPROC)GetFunctionPointer( (const GLubyte*)"glLockArraysEXT");
	
	UnlockArrays=
	(PFNGLUNLOCKARRAYSEXTPROC)GetFunctionPointer( (const GLubyte*)"glUnlockArraysEXT");

	
	
	BindTexture=
	(PFNGLBINDTEXTUREEXTPROC)GetFunctionPointer( (const GLubyte*)"glBindTextureEXT");
	if( !BindTexture)
		(mvoid(*)(muint, muint))BindTexture= GetFunctionPointer( (const GLubyte*)"glBindTexture");
	
	AreTexturesResident=
	(PFNGLARETEXTURESRESIDENTEXTPROC)GetFunctionPointer( (const GLubyte*)"glAreTexturesResidentEXT");
	//TODO: if don't exist try the normal opengl routine... if same args !
	
	DeleteTextures=
	(PFNGLDELETETEXTURESEXTPROC)GetFunctionPointer( (const GLubyte*)"glDeleteTexturesEXT");
	
	GenTextures=
	(PFNGLGENTEXTURESEXTPROC)GetFunctionPointer( (const GLubyte*)"glGenTexturesEXT");
	if( !GenTextures)
		(mvoid(*)(int,GLuint*))GenTextures= GetFunctionPointer( (const GLubyte*)"glGenTextures");
	
	IsTexture=
	(PFNGLISTEXTUREEXTPROC)GetFunctionPointer( (const GLubyte*)"glIsTextureEXT");
	
	PrioritizeTextures=
	(PFNGLPRIORITIZETEXTURESEXTPROC)GetFunctionPointer( (const GLubyte*)"glPrioritizeTexturesEXT");
	if( !PrioritizeTextures)
		(mvoid(*)(GLsizei, GLuint*, GLclampf*))PrioritizeTextures= GetFunctionPointer( (const GLubyte*)"glPrioritizeTextures");

	if( DrawRangeElements)
		if( !use_nv_array){
			use_ext_draw= mtrue;
			use_nv_draw= mfalse;
		}
		else{
			use_ext_draw= mtrue;
			use_nv_draw= mtrue;
		}
	else{
		use_ext_draw= mfalse;
		if( use_nv_array && DrawArrays)
			use_nv_draw= mtrue;
		else
			use_nv_draw= mfalse;
	}

	if( LockArrays && UnlockArrays)
		use_cva_array= mtrue;
	
	// nombre d'entits max par appels de fonctions de rendu:
/*	GLint *max_v;
	glGetIntegerv( GL_MAX_ELEMENTS_VERTICES, max_v);
	GLint *max_i;
	glGetIntegerv( GL_MAX_ELEMENTS_INDICES, max_i);
	std::cerr << std::endl << "max vertices in a render call: " << *max_v;
	std::cerr << std::endl << "max indices in a render call: " << *max_i;
*/		
	detected= mtrue;
	return mtrue;
}

mvoid MajaGExt::ShowInfo( mvoid)
{
	if( !detected){
		std::cerr << std::endl << "Detect the info before !";
		return;
	}
	std::cerr << std::endl << "Infos";
	std::cerr << std::endl << ext_string << std::endl << render_string;
}

mlong MajaGExt::AllocateMemory( mlong bytes, mbool really_allocate)
{	
	heap+=bytes;
	
	if( really_allocate){
		if( use_nv_alloc)
			memory_pointer= (GLfloat*)AgpAllocateMemory( heap, 0.95, 0.05, 0.9);
		else
			memory_pointer= (GLfloat*)malloc( heap);
	}
	
	count++;
		
	return heap-bytes;
}

mvoid MajaGExt::FreeMemory( mvoid)
{
	if( use_nv_alloc)
		AgpFreeMemory( memory_pointer);
	else
		free( memory_pointer);
		
	count=0;
	heap=0;
}
}
