/* imagetga.cpp , this is a part of Maja
 * load TGA image files
 * now add tga loading provided by Nate Miller Date	: 05/05/1999, Contact	: vandals1@home.com
 * Copyright (C) 2000-2002 Jean-Dominique Frattini
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Cette license est valable en terme de droit franais. Cela signifie que c'est la
 * lgislation franaise qui est en vigueur pour l'ensemble des sources contenues
 * dans la librairie. Sauf image.cpp et image.h, se reposant sur aucune license explicite,
 * mais qui sont fournies aimablement dans les librairies Mesa sous license LGPL.
 *
 * Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou
 * le modifier sous les termes de la license publique gnrale
 * publie par la Free Software Foundation; soit la version 2 de la
 * license, ou (si vous le voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir d'tre utile, mais
 * SANS AUCUNE GARANTIE; et mme une garantie implicite,
 * y compris les garanties de COMMERCIALISATION ou D'ADAPTATION
 * DANS UN BUT SPCIFIQUE. Prenez connaissance de la License Publique
 * Gnrale GNU pour plus de dtail.
 *
 * Vous devez avoir reu une copie de la License Publique Gnrale
 * GNU avec ce programme; si ce n'est pas le cas, vrivez  la
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "engines/graphic/imagetga.h"

/*
 * TGA loading, by Nate Miller
 */

GLenum texFormat;

// make sure the image is ok without scaling
int checkSize (int x)
{
	if (x == 2	 || x == 4 ||
		x == 8	 || x == 16 ||
		x == 32  || x == 64 ||
		x == 128 || x == 256 || x == 512)
		return 1;
	else return 0;
}

// reads in RGBA data and returns the same, 32 bit image
unsigned char *getRGBA (FILE *s, int size)
{
	unsigned char *rgba;
	unsigned char temp;
	int bread;
	int i;

	rgba = (unsigned char*)malloc (size * 4); // allocate memory for image

	//  no memory allocated?
	if (rgba == NULL)
		return 0;

	bread = fread (rgba, sizeof (unsigned char), size * 4, s);
	
	// make sure that all of the data is where it should be
	if (bread != size * 4)
	{
		free (rgba);
		return 0;
	}
	for (i = 0; i < size * 4; i += 4 )
	{
		temp = rgba[i];
		rgba[i] = rgba[i + 2];
		rgba[i + 2] = temp;
	}

	texFormat = GL_RGBA;
	return rgba;
}

// reads in RGB data and returns RGB, 24bit image
unsigned char *getRGB (FILE *s, int size)
{
	unsigned char *rgb;
	unsigned char temp;
	int bread;
	int i;

	rgb = (unsigned char*)malloc (size * 3); // allocate memory for image
	
	//  no memory allocated?
	if (rgb == NULL)
		return 0;

	bread = fread (rgb, sizeof (unsigned char), size * 3, s); // read in the image data

	// make sure that all of the data is where it should be
	if (bread != size * 3)
	{
		// Bytes read != Bytes requested
		free (rgb);
		return 0;
	}

	for (i = 0; i < size * 3; i += 3)
	{
		temp = rgb[i];
		rgb[i] = rgb[i + 2];
		rgb[i + 2] = temp;
	}
	
	texFormat = GL_RGB;

	return rgb;
}

// get the image data
char *getData (FILE *s, int sz, int iBits)
{
	if (iBits == 32)
		return (char*)getRGBA (s, sz);
	else if (iBits == 24)
		return (char*)getRGB (s, sz);	
	else return NULL;
}

// loads a 24bit or 32 bit targa file and uploads it to memory
// id is the texture id to bind too
int loadTGA (char *name, int id)
{
	unsigned char type[4];
	unsigned char info[7];
	unsigned char *imageData = NULL;
	int imageWidth, imageHeight;
	int imageBits, size;
	FILE *s;
	
	// Could not open the file!
	if (!(s = fopen (name, "r+bt")))
		return TGA_FILE_NOT_FOUND;

	fread (&type, sizeof (char), 3, s); // read in colormap info and image type, byte 0 ignored
	fseek (s, 12, SEEK_SET);			// seek past the header and useless info
	fread (&info, sizeof (char), 6, s);

	// make sure that the colormap is 0 and type is 2, unmapped RGB
	if (type[1] != 0 || type[2] != 2)
		return TGA_BAD_IMAGE_TYPE;

	imageWidth = info[0] + info[1] * 256; // image width
	imageHeight = info[2] + info[3] * 256; // image height
	imageBits =	info[4]; // pixel depth

	size = imageWidth * imageHeight; // image size

	// Dimension imageWidth x imageHeight is not a valid dimension!!
	if (!checkSize (imageWidth) || !checkSize (imageHeight))
		return TGA_BAD_DIMENSION;


	// Image bits != Supported bits
	if (imageBits != 32 && imageBits != 24)
		return TGA_BAD_BITS;

	// get the image data
	imageData = (unsigned char*)getData (s, size, imageBits);
	
	fclose (s);

	// something went wrong
	if (imageData == NULL)
		return TGA_BAD_DATA;

	// OpenGL texture variables, change for various apps
	glBindTexture (GL_TEXTURE_2D, id);
	glPixelStorei (GL_UNPACK_ALIGNMENT, 1);
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	//glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	//glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
//	glTexImage2D (GL_TEXTURE_2D, 0, texFormat, imageWidth, imageHeight, 0, texFormat, GL_UNSIGNED_BYTE, imageData);
	gluBuild2DMipmaps( GL_TEXTURE_2D, texFormat, imageWidth, imageHeight, texFormat, GL_UNSIGNED_BYTE, imageData);

	// release our image data since it has already been uploaded
	free (imageData);

	return 1;
}
