/*
	majadisplay.cpp
	ce fichier est une partie de Maja - this is a part of Maja
	classe d'affichage
	display class
	
	$Id: $

	$Log: $

	begin:			September 2000
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#include "engines/graphic/majadisplay.h"

namespace maja
{
namespace MAJAGRAPHIC{
	
MajaDisplay::MajaDisplay( mvoid): MAJATHREAD::MajaMutex()
{
	maja_internal_name= new mchar[strlen( "MajaDisplay")+1];
	if( !maja_internal_name) 
		throw std::exception();
	strcpy( maja_internal_name, "MajaDisplay");
	camera= mnull;
	g_engine= new MajaGraphicEngine();
	if( !g_engine) 
		throw std::exception();
	g_computed= mfalse;
	use_g_engine= mtrue;
	window= mnull;
	mem_mgr= mnull;
	use_menu= mtrue;
	use_hud= mfalse;
}

MajaDisplay::~MajaDisplay( mvoid)
{
//	gl_conf.Reset();
	delete maja_internal_name;
	delete g_engine;
}

mint MajaDisplay::Init( mvoid)
{
	gl_conf.Init();

	return 1;
}

//!todo: Render won't know what is each, but it needs,
// 	so MajaDisplay may finally need to know path, scene, actors...
// 	in order to make proper shadows.
mbool MajaDisplay::DoShadows( mvoid)
{
	return mtrue;
	
	MAJACONTAINER::MajaWidget<MajaObject> *tmp;
	MajaObject *sc;
	
	tmp= object_list.GetFirst();
	if( !tmp) 
		return mfalse;
	
	//while( tmp){
		sc= tmp->GetWidget();
		glPushMatrix();
		glDisable( GL_DEPTH_TEST);
		glDisable( GL_TEXTURE_2D);
		glEnable( GL_STENCIL_TEST);    		
   		
		glStencilFunc( GL_ALWAYS, 1, 1);
		glStencilOp( GL_REPLACE, GL_REPLACE, GL_REPLACE);
	
		glColorMask( GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
		glDepthMask( GL_TRUE);
		glStencilFunc( GL_EQUAL, 1, 1);
		glStencilOp( GL_KEEP, GL_KEEP, GL_INCR);
	
		glPushMatrix();
		glPushClientAttrib( GL_COLOR_BUFFER_BIT);
		glDisable( GL_LIGHT0);
		glColor4f( 0,0,0,0.6);
	
		shadow_matrix.SetLightPosition( light->GetPosition());
		MajaQuaternion equ( 0,1,0,0);
		shadow_matrix.SetPlanEquation( equ);		
		if( !shadow_matrix.Generate()) 
			return mfalse;
		
		glMultMatrixf( (const GLfloat*)shadow_matrix.GetMatrixPointer());		
		
		if( use_g_engine)
			Render();
			
		glEnable( GL_LIGHT0);
		glColor4f( 1,1,1,0);
		glPopClientAttrib();
		glPopMatrix();
	
		glDisable( GL_STENCIL_TEST);
		glEnable( GL_TEXTURE_2D);
		glEnable( GL_DEPTH_TEST);
		glPopMatrix();
		
		tmp= (MAJACONTAINER::MajaWidget<MajaObject>*)tmp->GetNext();
	//}
	
	return mtrue;
}

mint MajaDisplay::Compute( mvoid)
{
	g_engine->SetMemoryManager( *mem_mgr);

	std::cerr << std::endl << "MajaDisplay: computing...";
	use_g_engine= mtrue;
		
	MajaGMesh *tmp;
	GLuint gmesh_count=0;

	if( object_list.GetNumber()){
		MAJACONTAINER::MajaWidget<MajaObject> *tmpact= object_list.GetFirst();
		while( tmpact){
			if( (tmp= tmpact->GetObject()->GetModel().GetMesh())!=mnull){
				tmp->pos= &tmpact->GetObject()->GetPos();
				tmp->rot= &tmpact->GetObject()->GetAngles();
				gmesh_count++;
				g_engine->AddMesh( tmp, 1);
			}
			tmpact=tmpact->GetNext();
		}
	}
	
	g_computed= mtrue;
	std::cerr << " done.";
	
	return 1;
}

mbool MajaDisplay::Check( mvoid)
{
	if( object_list.GetNumber()<1){
		std::cerr << std::endl << "MajaDisplay: there is no objects at all !";
		return mfalse;
	}
	
	if( !camera){
		std::cerr << std::endl << "MajaDisplay: there is no camera";
		return mfalse;
	}

	return mtrue;
}

}//namespace MAJAGRAPHIC
} //namespace maja
