/***************************************************************************
                          majagdata.cpp  -  description
                             -------------------
    begin                : Fri Jun 28 2002
    copyright            : (C) 2002 by Jean-Dominique Frattini
    email                : jd.frattini@tuxfamily.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "engines/graphic/majagdata.h"

namespace maja
{
namespace MAJAGRAPHIC{

MajaGData::MajaGData( void): MajaGEntity()
{
	vertices= NULL;
	colors= NULL;
	normals= NULL;
	texels= NULL;
	face_normals= NULL;
	faces_nb=0;
	param_id=0;
	index= NULL;
	to_be_destroyed= false;
	drawable= true;
}

MajaGData::MajaGData( MajaGData& data): MajaGEntity()
{
	to_be_destroyed= false;//data.to_be_destroyed;
	vertices= data.vertices;
	colors= data.colors;
	normals= data.normals;
	texels= data.texels;
	face_normals= data.face_normals;
	faces_nb= data.faces_nb;
	param_id= data.param_id;
	index= data.index;
	drawable= data.drawable;
}

MajaGData::~MajaGData( void)
{
	if( to_be_destroyed){
		delete vertices;
		delete colors;
		delete normals;
		delete texels;
		delete face_normals;
		//if( index)
			delete index;
	}
}

bool MajaGData::SetNewFace( GLfloat *ve, GLfloat *no, GLfloat *co, GLfloat *te, GLfloat *fa, GLuint nb)
{
	int i;
	if( !to_be_destroyed) to_be_destroyed= true;
	faces_nb+=nb;
	if( faces_nb==1){
		vertices= (GLfloat*)malloc( (9*faces_nb)*sizeof( GLfloat));
		colors= (GLfloat*)malloc( (12*faces_nb)*sizeof( GLfloat));
		normals= (GLfloat*)malloc( 9*faces_nb*sizeof( GLfloat));
		texels= (GLfloat*)malloc( (6*faces_nb)*sizeof( GLfloat));
//		face_normals= (GLfloat*)malloc( (3*faces_nb)*sizeof( GLfloat));
		face_normals=fa;
			
		if( !vertices || !colors || !normals || !texels /*|| !f*/){
			std::cerr << std::endl << "Plus de mmoire disponible";
			faces_nb--;
			return false;
		}
	}
	else{
		GLfloat *v, *c, *n, *t, *f;
		v= (GLfloat*)realloc( &vertices[0], (9*faces_nb)*sizeof( GLfloat));
		c= (GLfloat*)realloc( &colors[0], (12*faces_nb)*sizeof( GLfloat));
		n= (GLfloat*)realloc( &normals[0], 9*faces_nb*sizeof( GLfloat));
		t= (GLfloat*)realloc( &texels[0], (6*faces_nb)*sizeof( GLfloat));
//		f= (GLfloat*)realloc( &face_normals[0], (3*faces_nb)*sizeof( GLfloat));
		f= fa;
		
		if( !v || !c || !n || !t /*|| !f*/){
			std::cerr << std::endl << "Plus de mmoire disponible";
			faces_nb--;
			return false;
		}
		
		vertices= v;
		colors= c;
		normals= n;
		texels= t;
//		face_normals= f;
	}
		
	for( i=0; i<9; i++){
		vertices[9*(faces_nb-1)+i]= *(ve+i);
		normals[9*(faces_nb-1)+i]= *(no+i);
	}
	for( i=0; i<12; i++)
		colors[12*(faces_nb-1)+i]= *(co+i);
		
	for( i=0; i<6; i++)
		texels[6*(faces_nb-1)+i]= *(te+i);
	
//	for( int i=0; i<3; i++)
//		face_normals[3*(faces_nb-1)+i]= *(fa+i);
		
	return true;
}
}
}
