/*
	majagengine.cpp
	ce fichier est une partie de Maja - this is a part of Maja
	moteur graphique de rendu "optimis" pour geforces.
	graphic engine of "optimized" rendering for geforces.

	$Id: $

	$Log: $

	begin:			Wed Jun 19 2002
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#include "engines/graphic/majagengine.h"
#include <unistd.h>

namespace maja
{
namespace MAJAGRAPHIC{

MajaGraphicEngine::MajaGraphicEngine( mvoid): MajaGEntity()
{
	this->maja_internal_name= new mchar[strlen( "MajaGraphicEngine")+1];
	if( !this->maja_internal_name) 
		return;
	
	strcpy( this->maja_internal_name, "MajaGraphicEngine");
	std::cerr << std::endl << this->maja_internal_name << ": starting...";
	
	leaves= mnull;
	branches= mnull;
	g_technic= MGT_dlist;
	total_leaves_count=0;
	mem_mgr= mnull;
	agp_memory_pointer= mnull;
	
	data_locked= mfalse;
}

MajaGraphicEngine::~MajaGraphicEngine( mvoid)
{
	std::cerr << std::endl << this->maja_internal_name << ": exiting...";
	
	if( data_locked)
		gl_ext->UnlockArrays();
		
	if( g_technic>= MGT_cva_array)
		DisableExtArrays();
	else if( g_technic== MGT_array)
		DisableNormalArrays();
	
	if( gl_ext){
		std::cerr << std::endl << maja_internal_name << ": freeing graphic memory...";
		gl_ext->FreeMemory();
		std::cerr << " done";
	}
	
	delete[] leaves;
	free( branches);
	delete this->maja_internal_name;
	if( gl_ext) 
		delete gl_ext;
	
	std::cerr << std::endl << typeid( this).name() << ": ended";
}

GLuint MajaGraphicEngine::Compute( mvoid)
{
	std::cerr << std::endl << this->maja_internal_name << ": computing...";
	
	// here start to compute
	if( ComputeStarts()==0)
		return 0;
	// here create the leaves:
	if( ComputeLeaves()==0)
		return 0;
	// here create the branches
	if( ComputeBranches()==0)
		return 0;
	// here link the branches with the leaves (just one leave by branch for now)
	if( ComputeLinks()==0)
		return 0;
	// creating the tree
	if( ComputeTree()==0)
		return 0;
	// now do all needed for a valid fast renderer	
	if( ComputeEnds()==0)
		return 0;
	// check/change some OpenGL states to be conform
	if( !CheckStates())
		return 0;
	std::cerr << std::endl << this->maja_internal_name << ": computing done successfully";
			
	return 1;
}

GLuint MajaGraphicEngine::ComputeStarts( mvoid)
{
	if( !gl_ext->GetNV() || ! gl_ext->GetNvDraw()){	
		if( !gl_ext->GetExtRenderer()){
			std::cerr << std::endl << this->maja_internal_name << ": using normal rendering, you may use display list !";
			EnableNormalArrays();
			g_technic= MGT_array;
		}
		else{
			std::cerr << std::endl << this->maja_internal_name << ": using extension rendering.";
			EnableExtArrays();
			g_technic= MGT_cva_array;
		}
	}
	else{
		std::cerr << std::endl << this->maja_internal_name << ": using nvidia rendering.";
		EnableExtArrays();
		g_technic= MGT_nv_var_array;
	}
	
	return 1;
}

GLuint MajaGraphicEngine::ComputeLeaves( mvoid)
{
	GLuint total_faces_count=0, total_data_count=0;
	GLuint i,j;
	
	MAJACONTAINER::MajaWidget<MajaGMesh> *tmp= meshes_list.GetFirst();
	
	if( !tmp){
		std::cerr << std::endl << maja_internal_name << ": empty meshes list";
		return 0;
	}
	
	MajaGMesh *mesh= tmp->GetWidget();
	
	if( !mesh){
		std::cerr << std::endl << maja_internal_name << ": empty mesh in list";
		return 0;
	}
	
	do{
		mesh=tmp->GetWidget();
		for( i=0; i<mesh->data_nb; i++){
			if( (j=mesh->data[i].faces_nb)>0){
				total_faces_count+=j;
				total_data_count++;
			}
		}
		tmp=tmp->GetNext();
	} while( tmp);
	
	std::cerr << std::endl << maja_internal_name << ": Allocating memory...";
	agp_memory_pointer= (GLfloat*)mem_mgr->Allocate( total_faces_count*36*sizeof( GLfloat), agp_t);
	std::cerr << " done";
		
	render_info.faces_nb= total_faces_count;

	leaves= new MajaGDataTree[total_data_count];
	if( !leaves){
		std::cerr << std::endl << "memory full";
		return 0;
	}

	if( !agp_memory_pointer){
		std::cerr << std::endl << "agp memory pointer empty";
		return 0;
	}
	
	GLfloat *s= &agp_memory_pointer[0];
	total_leaves_count=0;
	GLuint nbf=0, k=0;
	GLfloat *tmp_val=mnull;
	
	#ifdef DEBUG
	std::cerr << std::endl << maja_internal_name << ": data adress is: " << s;
	std::cerr << std::endl << "total agp memory set: " << gl_ext->GetHeap() << " for " << total_faces_count << " faces";
	#endif
	
	std::cerr << std::endl << maja_internal_name << ": writing...";
	tmp= meshes_list.GetFirst();
	do{
		mesh=tmp->GetWidget();
		for( i=0; i<mesh->data_nb; i++){
			if( (nbf= mesh->data[i].faces_nb)>0){
				leaves[total_leaves_count].vertices= &s[k];
				tmp_val= &(mesh->data[i].vertices[0]);
				for( j=0; j<9*nbf; j++, k++)
					s[k]=tmp_val[j];
				
				leaves[total_leaves_count].faces_nb=	nbf;
				leaves[total_leaves_count].face_normals= mnull;
				// use it if using array dlist:
				//leaves[total_leaves_count].CreateIndex();
				
				leaves[total_leaves_count].position= mesh->pos;
				leaves[total_leaves_count].orientation= mesh->rot;
				
				total_leaves_count++;
			}
			
		}
		tmp=tmp->GetNext();
	} while( tmp);
	
	tmp= meshes_list.GetFirst();
	do{
		mesh=tmp->GetWidget();
		for( i=0; i<mesh->data_nb; i++){
			if( (nbf= mesh->data[i].faces_nb)>0){
//! TODO:
//! as for vertices, it will be well if we know pointer to normals...
				//leaves[total_leaves_count].vertices= &s[k];
				tmp_val= &(mesh->data[i].normals[0]);
				for( j=0; j<9*nbf; j++, k++)
					s[k]=tmp_val[j];
			}
			
		}
		tmp=tmp->GetNext();
	} while( tmp);
	
	tmp= meshes_list.GetFirst();
	do{
		mesh=tmp->GetWidget();
		for( i=0; i<mesh->data_nb; i++){
			if( (nbf= mesh->data[i].faces_nb)>0){			
				tmp_val= &(mesh->data[i].colors[0]);
				for( j=0; j<12*nbf; j++, k++)
					s[k]=tmp_val[j];
			}
			
		}
		tmp=tmp->GetNext();
	} while( tmp);
	
	tmp= meshes_list.GetFirst();
	do{
		mesh=tmp->GetWidget();
		for( i=0; i<mesh->data_nb; i++){
			if( (nbf= mesh->data[i].faces_nb)>0){			
				tmp_val= &(mesh->data[i].texels[0]);
				for( j=0; j<6*nbf; j++, k++)
					s[k]=tmp_val[j];
			}
			
		}
		tmp=tmp->GetNext();
	} while( tmp);
	
	std::cerr << " done";
	
	return 1;	
}

GLuint MajaGraphicEngine::ComputeBranches( mvoid)
{
	GLuint i=0, j=1; // j=1: as always as a texture anywhere
	
	total_branch_counter=0;
	MAJACONTAINER::MajaWidget<MajaGMesh> *tmp= meshes_list.GetFirst();
	MajaGMesh *mesh= tmp->GetWidget();
	branches= mnull;
	
	do{
		mesh=tmp->GetWidget();
		for( i=0; i<mesh->tex.nb/*data_nb*/; i++){
			if( mesh->data[i].faces_nb>0){
				total_branch_counter++;
				MajaGParam *ptmp= (MajaGParam*)realloc( branches, total_branch_counter*sizeof( MajaGParam));
				if( !ptmp){
					std::cerr << std::endl << "not enough memory to allocate branches";
					return 0;
				}
				branches= ptmp;
				MajaGMaterialTree *mat= new MajaGMaterialTree();
				if( mesh->tex.textures2[i])
					j++;
				MajaGTextureTree *tex= new MajaGTextureTree( j);
				if( !mat || !tex) return 0;
				mat->mat= &(mesh->mat.materials[10*i]);
//! TODO:				
// here should make the texture be resident, or attempt to...
				if( j==1){
					tex->tex_id[0]= mesh->tex.textures[i];
					tex->tex_rot[0]= mesh->tex.tex_rot[i];
				}
				else{
					tex->tex_id[0]= mesh->tex.textures[i];
					tex->tex_rot[0]= mesh->tex.tex_rot[i];
					tex->tex_id[1]= mesh->tex.textures2[i];
					tex->tex_rot[2]= mesh->tex.tex_rot[i];
				}
				branches[total_branch_counter-1].mat=mat;
				branches[total_branch_counter-1].tex=tex;
			}
			j=1;
		}
		tmp=tmp->GetNext();
	} while( tmp);
	
	#ifdef DEBUG
	std::cerr << " ; branches : " << total_branch_counter;	
	#endif

	return 1;
}

GLuint MajaGraphicEngine::ComputeLinks( mvoid)
{
	for( GLuint i=0; i<total_branch_counter; i++){
		branches[i].SetChildren( &leaves[i]);
//! TODO: ++ if more than one child:
		branches[i].child_nb=1;
	}
	
	return 1;
}
	
GLuint MajaGraphicEngine::ComputeTree( mvoid)
{
	graphic_root.SetBranch( &branches[0], total_branch_counter);
		
	return 1;
}

GLuint MajaGraphicEngine::ComputeEnds( mvoid)
{
	GLuint i=0;
	volatile GLboolean var_valid= GL_FALSE;
	
	GLfloat	*v_mem= agp_memory_pointer;
	if( !v_mem){
		std::cerr << std::endl << maja_internal_name << ": Null memory data found.";
		
		return 0;
	}

	render_info.vert_index= new GLuint[3*render_info.faces_nb];
	if( !render_info.vert_index){
		std::cerr << std::endl << "memory full";
		
		return 0;
	}
	
	for( i=0; i<3*render_info.faces_nb; i++)
		render_info.vert_index[i]=i;

	if( g_technic== MGT_array){
		glVertexPointer( 3, GL_FLOAT, 0, &v_mem[0]);
		glNormalPointer( GL_FLOAT, 0, &v_mem[render_info.faces_nb*9]);
		glColorPointer( 4, GL_FLOAT, 0, &v_mem[render_info.faces_nb*18]);
		glTexCoordPointer( 2, GL_FLOAT, 0, &v_mem[render_info.faces_nb*30]);
	}
	else if( g_technic>= MGT_cva_array){
		gl_ext->VertexPointer( 3, GL_FLOAT, 0, 9*render_info.faces_nb, &v_mem[0]);
		gl_ext->NormalPointer( GL_FLOAT, 0, 9*render_info.faces_nb, &v_mem[render_info.faces_nb*9]);
		gl_ext->ColorPointer( 4, GL_FLOAT, 0, 12*render_info.faces_nb, &v_mem[render_info.faces_nb*18]);
		gl_ext->TexelPointer( 2, GL_FLOAT, 0, 6*render_info.faces_nb, &v_mem[render_info.faces_nb*30]);
	}
	
	//!todo : program starting bug
	// here, almost all times, NV VAR is NOT VALID !
	// must check absolutely why, because it crashes maja at start repercuting on
	// the textures that could not be loaded with a segmentation fault.
	// it mays result on a context validity issue btw gl,glx,x.
	//
	// quotes: this could evoluate to be a part of some (new ?) type that will manages
	// 	the validity of OpenGL (take care about gl mutexes ?) and the system.
	if( g_technic== MGT_nv_var_array){
		gl_ext->VertexArrayRange( 36*render_info.faces_nb*sizeof( GL_FLOAT), &v_mem[0]);
	
		glGetBooleanv( GL_VERTEX_ARRAY_RANGE_VALID_NV, (GLboolean*)&var_valid);
	
		if( var_valid==GL_FALSE){
			std::cerr << std::endl << maja_internal_name << ": NVidia VAR is not valid: trying again...";
			sleep( 1);
			glGetBooleanv( GL_VERTEX_ARRAY_RANGE_VALID_NV, (GLboolean*)&var_valid);
			if( var_valid==GL_FALSE){
				std::cerr << ": NVidia VAR is not valid again !";
				g_technic= MGT_array;
			}
		}
		else if( var_valid==GL_TRUE)
			std::cerr << std::endl << maja_internal_name << ": NVidia VAR is valid";
	}

	if( g_technic== MGT_cva_array){
		gl_ext->LockArrays( 0, 36*render_info.faces_nb);
		data_locked= mtrue;
	}
	
	GLuint si=0;
	leaves[0].SetStartIndex( 0);
	leaves[0].SetIndex( &render_info.vert_index[0]);
	for( i=1; i<total_leaves_count; i++){
		si+= 3*leaves[i-1].faces_nb;
		leaves[i].SetStartIndex( si);
		leaves[i].SetIndex( &render_info.vert_index[ leaves[i-1].faces_nb ]);
		leaves[i].SetGraphicTechnic( g_technic);
	}
	
	for( i=0; i<total_branch_counter; i++)
		branches[total_branch_counter-1].SetGraphicTechnic( g_technic);
	
	return 1;
}

mvoid MajaGraphicEngine::AddMesh( MajaGMesh *m, muint nb)
{
	muint i=0;
	
	meshes_count+=nb;
	
	for( i=0; i<nb; i++)
		meshes_list.AddObject( &m[i]);
}

mbool MajaGraphicEngine::Check( mvoid)
{
	std::cerr << std::endl << this->maja_internal_name << ": checking...";
	
	if( !graphic_root.Check()){
		std::cerr << std::endl << this->maja_internal_name << ": check failed due to root";
		return mfalse;
	}
	
	std::cerr << " done";
	std::cerr << std::endl << this->maja_internal_name << ": ready to render.";
	
	return mtrue;
}

mbool MajaGraphicEngine::CheckStates( mvoid)
{
	if( !gl_conf.GetUseTexture())
		gl_conf.SetUseTexture( mtrue);

	gl_conf.SetUseHardwareDepthTest( mtrue);
	
	return mtrue;
}
}// namespace MAJAGRAPHIC
}
