/* $Id: $
 * integrate the meshes from something loaded (3ds...)
 * Copyright (C) 2002 Jean-Dominique Frattini
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Cette license est valable en terme de droit franais. Cela signifie que c'est la
 * lgislation franaise qui est en vigueur pour l'ensemble des sources contenues
 * dans la librairie. Sauf image.cpp et image.h, se reposant sur aucune license explicite,
 * mais qui sont fournies aimablement dans les librairies Mesa sous license LGPL.
 *
 * Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou
 * le modifier sous les termes de la license publique gnrale
 * publie par la Free Software Foundation; soit la version 2 de la
 * license, ou (si vous le voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir d'tre utile, mais
 * SANS AUCUNE GARANTIE; et mme une garantie implicite,
 * y compris les garanties de COMMERCIALISATION ou D'ADAPTATION
 * DANS UN BUT SPCIFIQUE. Prenez connaissance de la License Publique
 * Gnrale GNU pour plus de dtail.
 *
 * Vous devez avoir reu une copie de la License Publique Gnrale
 * GNU avec ce programme; si ce n'est pas le cas, vrivez  la
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: $
 */

#include "engines/graphic/majagintegrator.h"
#include <stdio.h>

namespace maja
{
namespace MAJAGRAPHIC{

__Maja_Integrity_Level MajaGIntegrator::integrity_level= MIL_all;

MajaGIntegrator::MajaGIntegrator( void): input_mesh( *(new MajaMesh()))
{		
	maja_internal_name= new char[strlen( "MajaGIntegrator")+1];
	if( maja_internal_name)
		strcpy( maja_internal_name, "MajGIntegrator");
	std::cerr << std::endl << maja_internal_name << ": starting...";
	mat_refering= NULL;
	tex_refering= NULL;
	param_refering= NULL;
	param_count=0;

	i_mesh= mnull;

	std::cerr << std::endl << maja_internal_name << ": OK";
}

MajaGIntegrator::MajaGIntegrator( MajaGIntegrator& i): input_mesh( i.input_mesh)
{
	maja_internal_name= new char[strlen( "MajaGIntegrator")+1];
	if( maja_internal_name)
		strcpy( maja_internal_name, "MajGIntegrator");
	std::cerr << std::endl << maja_internal_name << ": starting...";
	mat_refering= i.mat_refering;
	tex_refering= i.tex_refering;
	param_refering= i.param_refering;
	param_count= i.param_count;
	std::cerr << std::endl << maja_internal_name << ": OK";
}

MajaGIntegrator::MajaGIntegrator( MajaMesh& mesh): input_mesh( mesh)
{
	maja_internal_name= new char[strlen( "MajaGIntegrator")+1];
	if( maja_internal_name)
		strcpy( maja_internal_name, "MajaGIntegrator");
	mat_refering= NULL;
	tex_refering= NULL;
	param_refering= NULL;
	param_count=0;
	Compute();
}

MajaGIntegrator::~MajaGIntegrator( void)
{
	std::cerr << std::endl << maja_internal_name << ": exiting...";
	delete mat_refering;
	delete tex_refering;
	delete param_refering;
	delete[] maja_internal_name;
	std::cerr << std::endl << typeid( *this).name() << " : ended";
}

bool MajaGIntegrator::Compute( void)
{
	std::cerr << std::endl << maja_internal_name << ": computing...";
	if( !ComputeMaterials())
		if( integrity_level>= MIL_no_material)
			return false;
	if( !ComputeTextures())
		if( integrity_level>= MIL_no_texture)
			return false;
	if( !ComputeRefering())
		if( integrity_level>= MIL_no_texture)
			return false;
	if( !ComputeDatas())
		if( integrity_level>= MIL_no_vertice)
			return false;
	std::cerr << " done";
	
	return true;
}

bool MajaGIntegrator::ComputeMaterials( void)
{
//	std::cerr << std::endl << maja_internal_name << ": computing materials...";
	GLuint i=0, j=0, k=0;
	GLuint same_count= 0;
	bool same= false;
	mat_refering= new GLuint[input_mesh.count];
	if( !mat_refering) return false;
	
	output_mesh.mat.SetNewMaterial( input_mesh.materials);
	for( i=0; i<input_mesh.count; i++){
		for( j=1; j<output_mesh.mat.nb; j++){
			same_count=0;
			same= false;
			for( k=0; k<10; k++){
				if( input_mesh.materials[10*i+k]==output_mesh.mat.materials[10*j+k]){
					same_count++;
				}
				else break;
			} // for( k=0; k<10; k++){
			if( same_count==9){
				same= true;
				break;
			}
		} // for( j=1; j<output_mesh.mat.nb; j++){
		if( !same){
			output_mesh.mat.SetNewMaterial( &input_mesh.materials[10*i]);
			mat_refering[i]= output_mesh.mat.nb;
		}
		else
			mat_refering[i]= j;
	} // for( i=0; i<input_mesh.count; i++){
//	std::cerr << " old mat nb: " << input_mesh.count << " new: " << output_mesh.mat.nb;
	return true;
}

bool MajaGIntegrator::ComputeTextures( void)
{
//	std::cerr << std::endl << maja_internal_name << ": computing textures...";
	GLuint i=0, j=0;
	bool same= false;
	tex_refering= new GLuint[input_mesh.count];
	if( !tex_refering) return false;
	
	// c'est moyen a.
/*	if( !&input_mesh.tex_id[0]){
		output_mesh.tex.SetNewTexture( 0);
		return true;			// dangereux !!
	}
	else
*/		output_mesh.tex.SetNewTexture( input_mesh.tex_id[0]);
		
	for( i=0; i<input_mesh.count; i++){
		same= false;
		for( j=1; j<output_mesh.tex.nb; j++){
			if( input_mesh.tex_id[i]==output_mesh.tex.textures[j]){
				same= true;
				break;
			}
		} // for( j=1; j<output_mesh.mat.nb; j++){
		if( !same){
			output_mesh.tex.SetNewTexture( input_mesh.tex_id[i], input_mesh.tex_rot[i], input_mesh.tex_refl[i]);
			tex_refering[i]= output_mesh.tex.nb;
		}
		else
			tex_refering[i]= j;
	} // for( i=0; i<input_mesh.count; i++){

//	std::cerr << " old tex nb: " << input_mesh.count << " new: " << output_mesh.tex.nb;
//
	return true;
}

bool MajaGIntegrator::ComputeRefering( void)
{
//	std::cerr << std::endl << maja_internal_name << ": computing referings...";
	GLuint i=0, j=0;
	bool same= false;
	long int_count=0;	
	
	param_refering= (GLuint*)malloc( sizeof( GLuint));
	if( !param_refering) return false;
	
	param_refering[0]= tex_refering[0];
	int_count++;
	
	for( i=1; i<input_mesh.count; i++){
		for( j=0; j<i; j++){
			if( tex_refering[j]==tex_refering[i]){
				same= true;
				break;
			}
		} // for( j=1; j<output_mesh.mat.nb; j++){
		if( !same){
			GLuint *temp;
			temp= (GLuint*)realloc( param_refering, (int_count+1)*sizeof( GLuint));
			if( !temp) return false;
			param_refering= temp;
			param_refering[int_count]=tex_refering[i];
			int_count++;
		}
		same= false;
	} // for( i=0; i<input_mesh.count; i++){
	param_count= int_count;
	
//	std::cerr << " old parameters: " << input_mesh.count << " new: " << param_count;
	
	return true;
}

bool MajaGIntegrator::ComputeDatas( void)
{
	GLuint i,j;
//	std::cerr << std::endl << maja_internal_name << ": computing datas";
	
	if( output_mesh.data){
		free( output_mesh.data);
		output_mesh.data= NULL;
	}
	
	output_mesh.data= new MajaGData[param_count];
	if( !output_mesh.data) return false;
	output_mesh.data_nb= param_count;
	
	for( i=0; i<param_count; i++){
		for( j=0; j<input_mesh.count; j++){
			if( tex_refering[j]==i){
				output_mesh.data[i].SetNewFace(
					&input_mesh.vertices[9*j],
					&input_mesh.normals[9*j],
					&input_mesh.colors[12*j],
					&input_mesh.texels[6*j],
					NULL);
				output_mesh.data[i].SetParamId( param_refering[i]);	// surtout pas i ! il faut la valeur de la texture !
			}
		}
		//output_mesh.data[i].CreateIndex();
	}
	return true;
}




export template <class I>
mbool MajaGMeshMaker<I>::Compute( I& i)
{
	if( !ComputeMaterials( i))
		return mfalse;
	if( !ComputeTextures( i))
		return mfalse;
	if( !ComputeRefering( i))
		return mfalse;
	if( !ComputeDatas( i))
		return mfalse;
	
	o= new MajaGMesh;
	
	return o!=mnull;
}

export template <class I>
mbool MajaGMeshMaker<I>::ComputeMaterials( I& input)
{
	GLuint i=0, j=0, k=0;
	GLuint same_count= 0;
	mbool same= mfalse;
	
	o->mat.SetNewMaterial( input.materials);
	
	for( i=0; i<input.count; i++){
		for( j=1; j<output_mesh.mat.nb; j++){
			same_count=0;
			same= mfalse;
			for( k=0; k<10; k++)
				if( input.materials[10*i+k]==o->mat.materials[10*j+k])
					same_count++;
				else 
					break;
			if( same_count==9){
				same= mtrue;
				break;
			}
		}
		if( !same)
			o->mat.SetNewMaterial( &input.materials[10*i]);
	}
	
	return mtrue;
}

export template <class I>
mbool MajaGMeshMaker<I>::ComputeTextures( I& input)
{
	GLuint i=0, j=0;
	mbool same= mfalse;
	
	o->tex.SetNewTexture( input.tex_id[0]);
		
	for( i=0; i<input.count; i++){
		same= mfalse;
		for( j=1; j<o->tex.nb; j++){
			if( input.tex_id[i]==o->tex.textures[j]){
				same= mtrue;
				break;
			}
		}
		
		if( !same)
			output_mesh.tex.SetNewTexture( input_mesh.tex_id[i], input_mesh.tex_rot[i], input_mesh.tex_refl[i]);
	} 

	return true;
}

export template <class I>
mbool MajaGMeshMaker<I>::ComputeRefering( I& input)
{
//	std::cerr << std::endl << maja_internal_name << ": computing referings...";
	GLuint i=0, j=0;
	mbool same= false;
	mlong int_count=0;	
	
	param_refering= (GLuint*)malloc( sizeof( GLuint));
	if( !param_refering) return false;
	
	param_refering[0]= tex_refering[0];
	int_count++;
	
	for( i=1; i<input_mesh.count; i++){
		for( j=0; j<i; j++){
			if( tex_refering[j]==tex_refering[i]){
				same= true;
				break;
			}
		} // for( j=1; j<output_mesh.mat.nb; j++){
		if( !same){
			GLuint *temp;
			temp= (GLuint*)realloc( param_refering, (int_count+1)*sizeof( GLuint));
			if( !temp) return false;
			param_refering= temp;
			param_refering[int_count]=tex_refering[i];
			int_count++;
		}
		same= false;
	} // for( i=0; i<input_mesh.count; i++){
	param_count= int_count;
	
//	std::cerr << " old parameters: " << input_mesh.count << " new: " << param_count;
	
	return true;
}

export template <class I>
mbool MajaGMeshMaker<I>::ComputeDatas( I& input)
{
	GLuint i,j;
//	std::cerr << std::endl << maja_internal_name << ": computing datas";
	
	if( output_mesh.data){
		free( output_mesh.data);
		output_mesh.data= NULL;
	}
	
	output_mesh.data= new MajaGData[param_count];
	if( !output_mesh.data) return false;
	output_mesh.data_nb= param_count;
	
	for( i=0; i<param_count; i++){
		for( j=0; j<input_mesh.count; j++){
			if( tex_refering[j]==i){
				output_mesh.data[i].SetNewFace(
					&input_mesh.vertices[9*j],
					&input_mesh.normals[9*j],
					&input_mesh.colors[12*j],
					&input_mesh.texels[6*j],
					NULL);
				output_mesh.data[i].SetParamId( param_refering[i]);	// surtout pas i ! il faut la valeur de la texture !
			}
		}
		//output_mesh.data[i].CreateIndex();
	}
	return true;
}




}
}
