/*
	majalight.cpp
	ce fichier est une partie de Maja - this is a part of Maja
	gestion de la lumire basique
	basic light management

	$Id: $

	$Log: $

	begin:			Sep 2000
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#include "engines/graphic/majalight.h"

namespace maja
{
	using MAJAMATHS::MajaQuaternion;
	
	muint MajaLight::light_opened= 0;

	MajaLight::MajaLight( MajaQuaternion& pos, MajaQuaternion& diff, MajaQuaternion& spec)
	{
		light_pos= pos;
		light_diff= diff;
		light_spec= spec;
		if( light_opened<GL_LIGHT7){
			light_opened++;
			light_number= GL_LIGHT0+light_opened-1;
		}
	}

	MajaLight::MajaLight( MajaLight& l)
	{
		light_pos= l.light_pos;
		light_diff= l.light_diff;
		light_spec= l.light_spec;
		light_number= l.light_number;
	}

	mvoid MajaLight::SwitchOnLight( mvoid)
	{
		glEnable( (GLenum)light_number);
	}

	mvoid MajaLight::SwitchOffLight( mvoid)
	{
		glDisable( (GLenum)light_number);
	}

	mvoid MajaLight::SetDiffuseColor( MajaQuaternion& color)
	{
		glLightfv( (GLenum)light_number, GL_DIFFUSE, (const GLfloat*)(color.CastToPointer()));
		light_diff= color;
	}

	mvoid MajaLight::SetSpecularColor( MajaQuaternion& color)
	{
		glLightfv( (GLenum)light_number, GL_SPECULAR, (const GLfloat*)(color.CastToPointer()));
		light_spec= color;
	}

	mvoid MajaLight::SetPosition( MajaQuaternion& pos)
	{
		glLightfv( (GLenum)light_number, GL_POSITION, (const GLfloat*)(pos.CastToPointer()));
		light_pos= pos;
	}

	mvoid MajaLight::SetDirection( MajaQuaternion& dir)
	{
		glLightfv( (GLenum)light_number, GL_SPOT_DIRECTION,  (const GLfloat*)(dir.CastToPointer()));
		light_dir= dir;
	}

	mvoid MajaLight::DefaultLight( mvoid)
	{
	/*	static MajaQuaternion diff( .75,.75,.75,1);
		static MajaQuaternion spec( .75,.75,.75,1);
		static MajaQuaternion pos( -5,-5,-5,0);
		static MajaQuaternion dir( -5,-5,-5,1);

		SetDiffuseColor( diff);
		SetSpecularColor( spec);
		SetPosition( pos);
		SetDirection( dir);
		if( !glIsEnabled( GL_LIGHTING))
			glEnable( GL_LIGHTING);
		SwitchOnLight();*/
	}
}
