/* $Id: $
 * textures for Maja
 * Copyright (C) september 2000,2001,2002 Jean-Dominique Frattini
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Cette license est valable en terme de droit franais. Cela signifie que c'est la
 * lgislation franaise qui est en vigueur pour l'ensemble des sources contenues
 * dans la librairie. Sauf image.cpp et image.h, se reposant sur aucune license explicite,
 * mais qui sont fournies aimablement dans les librairies Mesa sous license LGPL.
 *
 * Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou
 * le modifier sous les termes de la license publique gnrale
 * publie par la Free Software Foundation; soit la version 2 de la
 * license, ou (si vous le voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir d'tre utile, mais
 * SANS AUCUNE GARANTIE; et mme une garantie implicite,
 * y compris les garanties de COMMERCIALISATION ou D'ADAPTATION
 * DANS UN BUT SPCIFIQUE. Prenez connaissance de la License Publique
 * Gnrale GNU pour plus de dtail.
 *
 * Vous devez avoir reu une copie de la License Publique Gnrale
 * GNU avec ce programme; si ce n'est pas le cas, vrivez  la
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: $
 */

#include "engines/graphic/majatex.h"
#include "core/routines.h"
#include <iostream>
#ifndef WIN32
#include <unistd.h>
#endif

namespace maja
{
/*!file
 * status: 	in development
 * infos:	extension support, textures residents, 
 * 		try specify all the text param in MajaTex
 */

int MajaTex::number=0;
MajaGExt MajaTex::g_ext;

void MajaTex::Init( char *file_name)
{
	this->name= new char[strlen( file_name)+1];
	if( !this->name){
		std::cerr << std::endl << "error while allocating name";
		return;
	}
	strcpy( this->name, file_name);
	if( !strcmp( file_name, MAJA_NO_TEXTURE))
		current_id= 0;
	else{
		this->number++;
		current_id= this->number;
	}
}
	
bool MajaTex::LoadImage( void)
{
	MajaRgbImage *img= NULL;
#ifndef WIN32
	MajaJpegImage *j_img= NULL;
#endif
	std::cerr << std::endl << "Loading texture: " << name << "...";

	if( !g_ext.GetDetected())
		g_ext.Detect();

	//glGenTextures( 1, &current_id);
	g_ext.GenTextures( 1, &current_id);
	//glBindTexture( GL_TEXTURE_2D, current_id);
	g_ext.BindTexture( GL_TEXTURE_2D, current_id);
	if( strstr( name, ".rgb") || strstr( name, ".RGB")){
		img= (MajaRgbImage*)malloc( sizeof( MajaRgbImage));
		if( !( img= ImageLoad( name))){
			std::cerr << "   failed.";
			return false;
		}
		glPixelStorei( GL_UNPACK_ALIGNMENT, 4);
		gluBuild2DMipmaps( GL_TEXTURE_2D, GL_RGB, img->sizeX, img->sizeY, GL_RGB, GL_UNSIGNED_BYTE, (GLvoid*)(img->data));
//		glTexImage2D (GL_TEXTURE_2D, 0, GL_RGB, img->sizeX, img->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, img->data);
		// won't be the same for each texture loaded !
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		GLclampf pf=1;
		g_ext.PrioritizeTextures( 1, &current_id, &pf);

/*		if( strstr( name, "maja")){
			int x= (int)img->sizeX;
			int y= (int)img->sizeY;
			
			FILE *fd=fopen( "images/default_logo.maja.logo", "w");
			fprintf( fd, "%d;%d\n", x, y);
			fprintf( fd, "%s", (unsigned char*)img->data);
			fprintf( fd, "\n");
			fclose( fd);
		}
*/
		free( img);
		std::cerr << " done.";
		
		return true;
	}
	else if( strstr( name, ".tga") || strstr( name, ".TGA")){
		int ret= loadTGA( name, current_id);
		if( ret== TGA_FILE_NOT_FOUND)
			std::cerr << std::endl << "texture non trouve";
		else if( ret== TGA_BAD_IMAGE_TYPE)
			std::cerr << std::endl << "ce n'est pas un format RGB non compress";
		else if( ret== TGA_BAD_DIMENSION)
			std::cerr << std::endl << "la taille doit tre un multiple de 2";
		else if( ret== TGA_BAD_BITS)
			std::cerr << std::endl << "tga est soit en 24 ou 32 bits";
		else if( ret== TGA_BAD_DATA)
			std::cerr << std::endl << "erreur de chargement";
		else{
			std::cerr << " done.";
			return true;
		}
		std::cerr << "   failed.";
		return false;
	}
#ifndef WIN32
	else if( strstr( name, ".jpeg") || strstr( name, ".JPEG") || strstr( name, ".JPG") || strstr( name, ".jpg")){
		j_img= new MajaJpegImage();
		if( !j_img)
			return false;
			
		if( !j_img->ImageLoad( name)){
			std::cerr << "   failed.";
			delete j_img;
			return false;
		}
		
		glPixelStorei( GL_UNPACK_ALIGNMENT, 4);
		gluBuild2DMipmaps( 	GL_TEXTURE_2D,
					GL_RGB,
					256, 256, //img->sizeX, img->sizeY,
					GL_RGB,
					GL_UNSIGNED_BYTE,
					(GLvoid*)   (&(*(*(*(*j_img->GetTexture())))))       //(img->data)
		);
		
		// won't be the same for each texture loaded !
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		
		//free( img);
		delete j_img;
		std::cerr << "   done.";
		
		return true;
	}
#endif
	return true;
}
}
