/* $Id: $
 * module that load 3D models, this is a part of Maja
 * Copyright (C) 2000,2001,2002 Jean-Dominique Frattini
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Cette license est valable en terme de droit franais. Cela signifie que c'est la
 * lgislation franaise qui est en vigueur pour l'ensemble des sources contenues
 * dans la librairie. Sauf image.cpp et image.h, se reposant sur aucune license explicite,
 * mais qui sont fournies aimablement dans les librairies Mesa sous license LGPL.
 *
 * Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou
 * le modifier sous les termes de la license publique gnrale
 * publie par la Free Software Foundation; soit la version 2 de la
 * license, ou (si vous le voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir d'tre utile, mais
 * SANS AUCUNE GARANTIE; et mme une garantie implicite,
 * y compris les garanties de COMMERCIALISATION ou D'ADAPTATION
 * DANS UN BUT SPCIFIQUE. Prenez connaissance de la License Publique
 * Gnrale GNU pour plus de dtail.
 *
 * Vous devez avoir reu une copie de la License Publique Gnrale
 * GNU avec ce programme; si ce n'est pas le cas, vrivez  la
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: $
 */

#include "core/majasys.h"
#include "engines/modeler/modelio.h"
#ifdef WIN32
#include <windows.h>
#endif

namespace maja
{
	
//unsigned int MajaModule::use_counter=0;

MajaModule::MajaModule( void)
{
	file_name= null;
	external_tex=0;
	module= null;
	prescale=(float)1;
	resize=(float)0;
	integrator= null;
	model_mesh= null;
	new_mesh= mnull;
	cur_mesh= mnull;
	nb_mesh=0;
}

MajaModule::~MajaModule( void)
{
	if( file_name) 
		delete file_name;
	
	if( module) 
		delete module;
	
	if( integrator) 
		delete integrator;
}

bool MajaModule::Load( char *name)
{
	file_name= new char[strlen( name)+1];
	strcpy( file_name, name);
	std::cerr << std::endl << "Loading model: " << file_name;

	// just base on what Load() would return to know if it's of this type
	if( strstr( file_name, ".3ds") || strstr( file_name, ".3DS")){
		// here call a MajaModule* polymorphic type
		module= new Maja3dsModule();
		if( !module){
			std::cerr << std::endl << " failed.";

			return false;
		}
		if( module->Load( name)){
			std::cerr << std::endl << " done.";

			return true;
		}
		std::cerr << std::endl << " failed.";
	}
	
	return false;
}

// must find a good way to store the texture
// this takes a long while searching if texture
// already exists for 3ds loading. So finding
// our own way, may be better
bool MajaModule::SaveToFile( char *file)
{
	FILE *f;
//	unsigned int i;
	std::cerr << std::endl << "saving model : " << file;
	f= fopen( file, "w");
	if( !f) return false;	
/*	fprintf( f, "%d\n", 3*meshes.count);
	for( i=0; i<3*meshes.count; i+=3){
		fprintf( f, "%f ; %f ; %f ", meshes.vertices[i], meshes.vertices[i+1], meshes.vertices[i+2]);
		fprintf( f, "%f ; %f ; %f ", meshes.normals[i], meshes.normals[i+1], meshes.normals[i+2]);
		fprintf( f, "%f ; %f ; %f ; %f ", meshes.colors[i], meshes.colors[i+1], meshes.colors[i+2], meshes.colors[i+3]);
		fprintf( f, "%f ; %f\n", meshes.texels[i], meshes.texels[i+1]);
		fprintf( f, "%f ; %f ; %f ; %f ; %f ; %f ; %f ; %f ; %f ; %f ", meshes.materials[i], meshes.materials[i+1], meshes.materials[i+2],
				meshes.materials[i+3], meshes.materials[i+4], meshes.materials[i+5], meshes.materials[i+6], meshes.materials[i+7],
				meshes.materials[i+8], meshes.materials[i+9]);
	}
*/	fclose( f);
	std::cerr << ". done.";
	return true;
}

void MajaModule::AutoSizing( MajaVector& size)
{
	size= this->size;
}

void MajaModule::AutoPositioning( MajaVector& pos)
{
	pos= this->pos;
}

bool MajaModule::AllocateMesh( void)
{
	// these 4 are data
	GLfloat 	*t_verts=null;						// temp vertices
	GLfloat 	*t_norms=null;						// temp normals
	GLfloat 	*t_colors=null;						// temp colors r,g,b,a
	GLfloat 	*t_texels=null;						// temp texels
	// and these 3 are parameters
	GLfloat 	*tmat=null;						// temp materials
	GLuint		*ttd= null;						// temp textures id
	
	char		*t_name= null;						// temp name of the part
	GLfloat		*t_texrot= null;					// temp rotations of the textures
	GLuint		*ttd2= null;						// temp 2nd textures id
	GLfloat		*t_texrot2= null;					// temp rotations of the 2nd textures
	GLuint		*ttrf=null;
	
	 // allocating to new triangles number
//!todo TODO: maybe free them when copyied finally into Gram, but assume it
	GLuint alloc_size= meshes.count*sizeof( GLfloat);
	t_verts= (GLfloat*)realloc( meshes.vertices, 9*alloc_size);
	t_norms= (GLfloat*)realloc( meshes.normals, 9*alloc_size);
	t_colors= (GLfloat*)realloc( meshes.colors, 12*alloc_size);
	t_texels= (GLfloat*)realloc( meshes.texels, 6*alloc_size);
	tmat= (GLfloat*)realloc( &meshes.materials[0], 10*alloc_size);
	ttd= (GLuint*)realloc( &meshes.tex_id[0], meshes.count*sizeof( GLuint));
	t_name= (char*)realloc( &meshes.name[0], 11*meshes.count*sizeof( char));
	t_texrot= (GLfloat*)realloc( &meshes.tex_rot[0], alloc_size);
	ttd2= (GLuint*)realloc( &meshes.tex_id2[0], meshes.count*sizeof( GLuint));
	t_texrot2= (GLfloat*)realloc( &meshes.tex_rot2[0], alloc_size);
	ttrf= (GLuint*)realloc( &meshes.tex_refl[0], meshes.count*sizeof( GLuint));
	
	if( !t_texels || !t_colors || !t_norms || !t_verts || !tmat || 
			!t_name || !t_texrot || !ttd2 || !t_texrot2 || !ttrf || !ttd){
		std::cerr << std::endl << "realloc erreur";
		return false;
	}
	
	meshes.texels= t_texels;
	meshes.colors= t_colors;
	meshes.normals= t_norms;
	meshes.vertices= t_verts;
	meshes.materials= tmat;
	meshes.tex_id= ttd;
	meshes.name= t_name;
	meshes.tex_rot= t_texrot;
	meshes.tex_id2= ttd2;
	meshes.tex_rot2= t_texrot2;
	meshes.tex_refl= ttrf;
	
	return true;
}

void MajaModule::CalculateExtremums( unsigned int k)
{
	static bool first= true;
	
	 if( first){
		size_min.x=size_max.x=size.x= meshes.vertices[k];
		size_min.y=size_max.y=size.y= meshes.vertices[k+1];
		size_min.z=size_max.z=size.z= meshes.vertices[k+2];
		first= false;
	}
	else{
		if( meshes.vertices[k]< size_min.x) size_min.x=meshes.vertices[k];
		else if( meshes.vertices[k]> size_max.x) size_max.x=meshes.vertices[k];
		if( meshes.vertices[k+1]< size_min.y) size_min.y=meshes.vertices[k+1];
		else if( meshes.vertices[k+1]> size_max.y) size_max.y=meshes.vertices[k+1];
		if( meshes.vertices[k+2]< size_min.z) size_min.z=meshes.vertices[k+2];
		else if( meshes.vertices[k+2]> size_max.z) size_max.z=meshes.vertices[k+2];
	}
}

void MajaModule::CalculateSize( void)
{
	size.x= fabs( size_max.x-size_min.x);
	size.y= fabs( size_max.y-size_min.y);
	size.z= fabs( size_max.z-size_min.z);
}

void MajaModule::CalculatePos( void)
{
	pos.x= (size_min.x+size_max.x)/2;
	pos.y= (size_min.y+size_max.y)/2;
	pos.z= (size_min.z+size_max.z)/2;
}

bool MajaModule::MakeMesh( void)
{
	if( integrator)
		delete integrator;
		
	integrator= new MAJAGRAPHIC::MajaGIntegrator( meshes);	
	if( !integrator){
		std::cerr << std::endl << "no memory";
		return false;
	}
	
	model_mesh= integrator->GetMesh();	
	if( !model_mesh){
		std::cerr << std::endl << "mesh is empty";
		return false;
	}
	
	return true;
}
/*
void MajaModule::UseExternalTexture( GLuint tex)
{
	external_tex= tex;
	if( module)
		module->UseExternalTexture( tex);
}
*/
MAJAGRAPHIC::MajaGMesh* MajaModule::GetMesh( void)
{
	if( !model_mesh)
		if( module)
			return module->GetMesh();
			
	return model_mesh;
}

}
