/* modulease.cpp , this is a part of Maja
 * load ase models
 * Copyright (C) 2000,2001,2002 Jean-Dominique Frattini
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Cette license est valable en terme de droit franais. Cela signifie que c'est la
 * lgislation franaise qui est en vigueur pour l'ensemble des sources contenues
 * dans la librairie. Sauf image.cpp et image.h, se reposant sur aucune license explicite,
 * mais qui sont fournies aimablement dans les librairies Mesa sous license LGPL.
 *
 * Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou
 * le modifier sous les termes de la license publique gnrale
 * publie par la Free Software Foundation; soit la version 2 de la
 * license, ou (si vous le voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir d'tre utile, mais
 * SANS AUCUNE GARANTIE; et mme une garantie implicite,
 * y compris les garanties de COMMERCIALISATION ou D'ADAPTATION
 * DANS UN BUT SPCIFIQUE. Prenez connaissance de la License Publique
 * Gnrale GNU pour plus de dtail.
 *
 * Vous devez avoir reu une copie de la License Publique Gnrale
 * GNU avec ce programme; si ce n'est pas le cas, vrivez  la
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "engines/modeler/modulease.h"

#include <iostream> // std::cerr for windows

// from the original author
// cringe global, here till i find a better way to handle texture ids
GLuint *texIds;

/* Function -- aseGetVertex Reads a vertex from the ase file.*/
void MajaAseModule::aseGetVertex ()
{
	int index;
	
	fscanf ( file, "%d", &index);
	
	if( index>= numVertex){
		std::cerr << std::endl << "index greater than array size";
		return;
	}
	
	index*=3;
	
	// swap y and z cause 3dsm likes too
	fscanf (file, "%f %f %f", &verts[index], &verts[index+2], &verts[index+1]);
	
	// in 3dsm negative z goes out of the screen, we want it to go in
	//verts[index+2] = -verts[index+2];
}

/* Function -- aseGetTVertex Reads a texture coord from the file.*/
void MajaAseModule::aseGetTVertex ()
{
	int index;

	// we ignore the z value
	fscanf( file, "%d", &index);
	if( index>= numTexVertex){
		std::cerr << std::endl << "index greater than array size";
		return;
	}
	
	index*=2;
	
	fscanf( file, "%f %f", &texVerts[index], &texVerts[index+1]);
	
//	texVerts[index][0] *= texture.uTile;
//	texVerts[index][1] *= texture.vTile;
}

/* Function -- aseGetNVertex Reads a vertex normal from the file*/
void MajaAseModule::aseGetNVertex ()
{
	int index;

	fscanf ( file, "%d", &index);
	if( index>= numVertex){
		std::cerr << std::endl << "index greater than array size";
		return;
	}
	index*=3;
	fscanf ( file, "%f %f %f", &vertNorms[index], &vertNorms[index+2], &vertNorms[index+1]);
	vertNorms[index+2] = -vertNorms[index+2];
}

/* Function -- aseGetFaceNormal
    Gets a face normal from the file*/
void MajaAseModule::aseGetFaceNormal( void)
{
/*	int index;

	fscanf ( file, "%d", &index);
	if( index>= numFaces){
//		std::cerr << std::endl << "index greater than array size";
		return;
	}
	fscanf ( file, "%f %f %f", &faces[index].normal[0], &faces[index].normal[2], &faces[index].normal[1]);
	faces[index].normal[2] = -faces[index].normal[2];*/
}

/* Function -- aseGetFace Reads the indicies for a face from the file*/
void MajaAseModule::aseGetFace ( void)
{

	int index;

	fscanf ( file, "%d:", &index);
	
	if( index>= numFaces){
		std::cerr << std::endl << "index greater than array size";
		return;
	}
	
	index*=3;
	
	fscanf ( file, "\tA:\t%d B:\t%d C:\t%d", &vertIndex[index], &vertIndex[index+1], &vertIndex[index+2]);
}

/* Function -- aseGetTFace
    Reads  the indicies for a texture mapped face*/
void MajaAseModule::aseGetTFace()
{
/*	int index;

	fscanf( file, "%d:", &index);
	if( index>= numTexFaces){
//		std::cerr << std::endl << "index greater than array size";

		return;


	}
	fscanf ( file, "%d %d %d", &faces[index].coordIndex[0], &faces[index].coordIndex[1], &faces[index].coordIndex[2]);*/
}

/* Function -- aseGetTextureName
    Reads in the teture name that the model will use*/
void MajaAseModule::aseGetTextureName ( void)
{
	fscanf ( file, " \"%s", texture.texfilename);
	texture.texfilename[strlen( texture.texfilename) - 1] = '\0';
}

/* Function -- aseGetGLfloatVal
    Reads a GLfloat value from the file, expects data to be like this SOMETHING 4.2323*/
vec_t MajaAseModule::aseGetFloatVal( void)
{
	vec_t v;

	fscanf( file, " %f", &v);

	return v;
}

/* Function -- aseGetData
    Fill up out model structure will the model data*/
void MajaAseModule::aseGetData( void)
{
	char data[255];

	rewind ( file);

	while( !feof( file))
	{
		fscanf( file, "%s", data);

		if (!strcmp (data, VERTEX))
			aseGetVertex();
//		else if (!strcmp (data, TVERTEX))
//			aseGetTVertex();
//		else if (!strcmp (data, NVERTEX))
//			aseGetNVertex();
//		else if (!strcmp (data, FACE_NORMAL))
//			aseGetFaceNormal();
		else if (!strcmp (data, FACE))
			aseGetFace();
/*		else if (!strcmp (data, TFACE))
			aseGetTFace();
		else if (!strcmp (data, TEXTURE))
			aseGetTextureName();
		else if (!strcmp (data, UTILE))
			texture.uTile = aseGetFloatVal();
		else if (!strcmp (data, VTILE))
			texture.vTile = aseGetFloatVal();
		else if (!strcmp (data, UOFFSET))
			texture.uOffset = aseGetFloatVal();
		else if (!strcmp (data, VOFFSET))
			texture.vOffset = aseGetFloatVal();
		else fgets (data, sizeof (data), file);
*/	}
}

/* Function -- aseLoadTexture
    Loads the texture for the model, uses my custom TGA loading code.  If you
	want to use your own code change this.  */
void MajaAseModule::aseLoadTexture( void)
{
	;
}

/*    Function -- aseDumpInfo
    Dumps info regaring the passed model.*/
void MajaAseModule::aseDumpInfo( )
{
	printf ("# Model Info\n");
	printf ("# ------------------------------\n");
	printf ("# Num Vertices %d\n", numVertex);
	printf ("# Num Tex Vertices %d\n", numTexVertex);
	printf ("# Num Faces %d\n", numFaces);
	printf ("# Num Tex Faces %d\n", numTexFaces);
	printf ("# Model has normals : %s\n", (normals) ? "Yes" : "No");
	printf ("# Texture Name : %s\n", (texture.texfilename[0] != '\0') ? texture.texfilename : "None");
}

MajaAseModule::MajaAseModule( char *file_name, GLuint& list, char *tex_file)
{
	char data[255];
	
	strcpy( this->filename, file_name);
	numVertex= numTexVertex= numFaces= numTexFaces= 0;
	

	if( !( file= fopen( file_name, "r"))){
		std::cerr << "File doesn't exist!";
		return;
	}
	
	rewind ( file);
	
	while( !feof( file)){
	
		fscanf (file, "%s", data);
		
		if (!strcmp (data, NUM_VERTEX))
			fscanf (file, "%d", &numVertex);
		
		else if (!strcmp (data, NUM_FACES))
			fscanf (file, "%d", &numFaces);
		
//		else if (!strcmp (data, NUM_TVERTEX))
//			fscanf (file, "%d", &numTexVertex);

//		else if (!strcmp (data, NUM_TFACES))
//			fscanf (file, "%d", &numTexFaces);
		
		else if( !strcmp( data, NORMALS))
			normals = 1;
		
		else fgets( data, sizeof (data), file);
		
	}
	
	if( numVertex>0){
		verts= (GLfloat*)malloc( 3*(numVertex+1)*sizeof( GLfloat));
		vertNorms= (GLfloat*)malloc( 3*(numVertex+1)*sizeof( GLfloat));
	}
	
//	if( numTexVertex>0)
//		texVerts= (GLfloat*)malloc( 2*(numTexVertex+1)*sizeof( GLfloat));
	
	if( numFaces>0)
		vertIndex= (int*) malloc( 3*(numFaces+1)*sizeof( int));
	
	if( !verts || !vertNorms || !vertIndex){
		std::cerr << std::endl << "memory full";
		return;
       }

	aseGetData();
	
//	if( tex_file[0]!='\0') strcpy( texture.texfilename, tex_file);
//	aseLoadTexture();
	#ifdef DEBUG
		aseDumpInfo();
	#endif
	fclose( file);
	
	list= glGenLists( 1);
	glNewList( list, GL_COMPILE);
	glPushMatrix();
		glScalef( 0.5,0.5,0.5);
/*		glEnable( GL_TEXTURE_2D);
		glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
		glBindTexture( GL_TEXTURE_2D, *texIds);
*/		glBegin( GL_TRIANGLES);
		
		int id;

		
		for( id=0; id<numVertex; id++){
			//glNormal3fv( vertNorms);
			glVertex3fv( verts);
			verts+=3;
		}
		
		glEnd();
//		glDisable( GL_TEXTURE_2D);
	glPopMatrix();
	glEndList();
}

MajaAseModule::~MajaAseModule( void)
{
	free( verts);
	free( vertNorms);
	free( texVerts);
	free( vertIndex);
}
