/* $Id:$
 * load 3DS models, this is a part of Maja
 * Copyright (C) 2000,2001,2002 Jean-Dominique Frattini
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Cette license est valable en terme de droit franais. Cela signifie que c'est la
 * lgislation franaise qui est en vigueur pour l'ensemble des sources contenues
 * dans la librairie. Sauf image.cpp et image.h, se reposant sur aucune license explicite,
 * mais qui sont fournies aimablement dans les librairies Mesa sous license LGPL.
 *
 * Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou
 * le modifier sous les termes de la license publique gnrale
 * publie par la Free Software Foundation; soit la version 2 de la
 * license, ou (si vous le voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir d'tre utile, mais
 * SANS AUCUNE GARANTIE; et mme une garantie implicite,
 * y compris les garanties de COMMERCIALISATION ou D'ADAPTATION
 * DANS UN BUT SPCIFIQUE. Prenez connaissance de la License Publique
 * Gnrale GNU pour plus de dtail.
 *
 * Vous devez avoir reu une copie de la License Publique Gnrale
 * GNU avec ce programme; si ce n'est pas le cas, vrivez  la
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: $
 */

#include "engines/modeler/moduletds.h"
#include "engines/modeler/modelio.h"

#ifdef WIN32
#include <iostream>
#endif

#include "engines/graphic/majagintegrator.h"

namespace maja
{
	
float Maja3dsModule::MAJA_3DS_RESCALE= 0.1;

bool Maja3dsModule::Load( char *name)
{
	file= lib3ds_file_load( name);
	
	if( !file){
		std::cerr << std::endl << "Can't load file: " << name;
		return false;
	}
	
	lib3ds_file_eval( file, 0);
	this->MAJA_3DS_RESCALE= file->master_scale;
	
	if( !Create())
		return false;

	CalculatePos();
	CalculateSize();

	return true;
}

bool Maja3dsModule::Create( void)
{
	int i= 0;
	
	if( !file)
		return false;

	// Read all nodes information
	for( node=file->nodes; node!=0; node=node->next)
		i++;

//	new_mesh= new MajaMesh_[i];
//	nb_mesh= i;
//	i=0;
	
	for( node=file->nodes; node!=0; node=node->next)
	{
//		cur_mesh= & new_mesh[i];
		ReadInformation();
//		++i;
	}
		
	MakeMesh();
	
	#ifdef DEBUG
	std::cerr << std::endl << "Maja3dsModule: info about size & pos" << std::endl;
	std::cerr << "Size: x= " << size.x << " ; y= " << size.y << " ; z= " << size.z << std::endl;
	std::cerr << "Pos: x= " << pos.x << " ; y= " << pos.y << " ; z= " << pos.z;
	#endif

	return true;
}

void Maja3dsModule::ReadInformation( void)
{
	ASSERT( file);

	if( node->type==LIB3DS_OBJECT_NODE){
		if( strcmp( node->name, "$$$DUMMY")== 0)
			return;
		// know if already done
		if( !node->user.d){
			unsigned int t, i=0, k=0;					// counters
			GLuint		mat_match_color;				// index of material for having colors
			
			// for comply with the node, do as if it has a display list
			node->user.d= 1;
			Lib3dsMesh *mesh=lib3ds_file_mesh_by_name( file, node->name);
			ASSERT( mesh);
			if( !mesh) 
				return;
			
			Lib3dsVector *normalL=(float(*)[3])malloc( 3*sizeof(Lib3dsVector)*mesh->faces);
			//Lib3dsMatrix M;
			//lib3ds_matrix_copy( M, mesh->matrix);
			//lib3ds_matrix_inv( M);
			lib3ds_mesh_calculate_normals( mesh, normalL);
			
			// how many triangles are there in all ?
			meshes.last_count= meshes.count;
			meshes.count+= mesh->faces;
			
			// allocate the memory in relation with the mesh numbers
			if( !AllocateMesh()) 
				return;
			
			// prepare for material and texture an naming:	 			
			Lib3dsMaterial *mat=0;
			Lib3dsFloat scale[2]={1,1};
			Lib3dsFloat offset[2]={0,0};
			
			
		/*	if( !cur_mesh->New( mesh->faces)){
				std::cerr << std::endl << "could not create new mesh inside...";
				exit( 1);
			}
			cur_mesh->name= mesh->name;
		*/	

			// let's get all these values: vertices, normals, colors, materials and texture coordinates	
			for( t=0; t<mesh->faces; ++t){
				
				Lib3dsFace *f=&mesh->faceL[t];
				if( f->material[0])
					mat=lib3ds_file_material_by_name( file, f->material);				
				else
					continue;

				if( strlen( mat->texture1_map.name)>0){
					meshes.tex_id[meshes.m_count]=
					maja_textures.Create( mat->texture1_map.name, MAJA_TEXTURE_DIRECTORY);
					meshes.tex_rot[meshes.m_count]= mat->texture1_map.rotation;

//					cur_mesh->tri[t].TT.i= maja_textures.Create( mat->texture1_map.name, MAJA_TEXTURE_DIRECTORY);
//					cur_mesh->tri[t].TT.r[0]= mat->texture1_map.rotation;
				}
				else{
					meshes.tex_id[meshes.m_count]= 0;
					meshes.tex_rot[meshes.m_count]= 0;
				}
				
				if( strlen( mat->texture2_map.name)>0){
					meshes.tex_id2[meshes.m_count]=
					this->maja_textures.Create( mat->texture2_map.name, MAJA_TEXTURE_DIRECTORY);
					meshes.tex_rot2[meshes.m_count]= mat->texture2_map.rotation;

//					cur_mesh->tri[t].TT2.i= maja_textures.Create( mat->texture1_map.name, MAJA_TEXTURE_DIRECTORY);
//					cur_mesh->tri[t].TT2.r[0]= mat->texture1_map.rotation;
				}
				else{
					meshes.tex_id2[meshes.m_count]= 0;
					meshes.tex_rot2[meshes.m_count]= 0;
				}
				
				if( strlen( mat->reflection_map.name)>0)
					meshes.tex_refl[meshes.m_count]=
					this->maja_textures.Create( mat->reflection_map.name, MAJA_TEXTURE_DIRECTORY);
//					cur_mesh->tri[t].TT3.i= maja_textures.Create( mat->texture1_map.name, MAJA_TEXTURE_DIRECTORY);
//					cur_mesh->tri[t].TT3.r[0]= mat->texture1_map.rotation;
				else
					meshes.tex_refl[meshes.m_count]=0;
				
				k= 10*(meshes.m_count);
				meshes.m_count++;
				
				// normally, don't need anymore to test mat, because we continue
				// if we can't link mat
				if( mat){
					meshes.materials[k]= mat->ambient[0];
					meshes.materials[k+1]= mat->ambient[1];
					meshes.materials[k+2]= mat->ambient[2];
					meshes.materials[k+3]= mat->diffuse[0];
					meshes.materials[k+4]= mat->diffuse[1];
					meshes.materials[k+5]= mat->diffuse[2];
					meshes.materials[k+6]= mat->specular[0];
					meshes.materials[k+7]= mat->specular[1];
					meshes.materials[k+8]= mat->specular[2];
					meshes.materials[k+9]= 11-0.2*mat->shininess;
				/*
					cur_mesh->tri[t].M.a[0]=mat->ambient[0];
					cur_mesh->tri[t].M.a[1]=mat->ambient[1];
					cur_mesh->tri[t].M.a[2]=mat->ambient[2];
					cur_mesh->tri[t].M.d[0]=mat->diffuse[0];
					cur_mesh->tri[t].M.d[1]=mat->diffuse[1];
					cur_mesh->tri[t].M.d[2]=mat->diffuse[2];
					cur_mesh->tri[t].M.s[0]=mat->specular[0];
					cur_mesh->tri[t].M.s[1]=mat->specular[1];
					cur_mesh->tri[t].M.s[2]=mat->specular[2];
					cur_mesh->tri[t].M.h=11-.2*mat->shininess;
			*/	}
				else{
// TODO: must assume that this material part is the same as the previous one
					Lib3dsRgba a={0.2, 0.2, 0.2, 1.0};
					Lib3dsRgba d={0.8, 0.8, 0.8, 1.0};
					Lib3dsRgba s={0.0, 0.0, 0.0, 1.0};
					meshes.materials[k]= a[0]; meshes.materials[k+1]= a[1]; meshes.materials[k+2]= a[2];
					meshes.materials[k+3]= d[0]; meshes.materials[k+4]= d[1]; meshes.materials[k+5]= d[2];
					meshes.materials[k+6]= s[0]; meshes.materials[k+7]= s[1]; meshes.materials[k+8]= s[2];

					meshes.materials[k+9]= 0;
				} // if( mat)
				
				mat_match_color= k;
			//	mat_match_color= t;
				
				// keep colors, verticies, normals and texcoords.
				for( i=0; i<3; ++i){					
					k= 12*meshes.last_count+12*t+4*i;					
					meshes.colors[k]= meshes.materials[mat_match_color+3];
					meshes.colors[k+1]= meshes.materials[mat_match_color+4];
					meshes.colors[k+2]= meshes.materials[mat_match_color+5];
					meshes.colors[k+3]= 1-mat->transparency;
/*
					cur_mesh->tri[t].C[i].c[0]=cur_mesh->tri[t].M.a[0];
					cur_mesh->tri[t].C[i].c[1]=cur_mesh->tri[t].M.a[1];
					cur_mesh->tri[t].C[i].c[2]=cur_mesh->tri[t].M.a[2];
					cur_mesh->tri[t].C[i].c[0]=1-mat->transparency;
*/
					k= 9*meshes.last_count+9*t+3*i;
					meshes.vertices[k]= mesh->pointL[f->points[i]].pos[0]*MAJA_3DS_RESCALE*prescale.x;
					meshes.vertices[k+1]= mesh->pointL[f->points[i]].pos[2]*MAJA_3DS_RESCALE*prescale.y;
					meshes.vertices[k+2]= mesh->pointL[f->points[i]].pos[1]*MAJA_3DS_RESCALE*prescale.z;
/*
					cur_mesh->tri[t].V[i].v[0]=mesh->pointL[f->points[i]].pos[1]*MAJA_3DS_RESCALE*prescale.x;
					cur_mesh->tri[t].V[i].v[1]=mesh->pointL[f->points[i]].pos[2]*MAJA_3DS_RESCALE*prescale.y;
					cur_mesh->tri[t].V[i].v[2]=mesh->pointL[f->points[i]].pos[0]*MAJA_3DS_RESCALE*prescale.z;
*/
					// here calculate the extremums:
					CalculateExtremums( k);
					
					meshes.normals[k]= normalL[3*t+i][0];
					meshes.normals[k+1]= normalL[3*t+i][2];
					meshes.normals[k+2]= normalL[3*t+i][1];
				
/*					cur_mesh->tri[t].N[i].n[0]=normalL[3*t+i][1];
					cur_mesh->tri[t].N[i].n[1]=normalL[3*t+i][2];
					cur_mesh->tri[t].N[i].n[2]=normalL[3*t+i][0];
*/					
					k= 6*meshes.last_count+6*t+2*i;
					if( mesh->texels>0){
						scale[0]=mat->texture1_map.scale[0]; scale[1]=mat->texture1_map.scale[1];
						if( scale[0]==0) scale[0]=1; if( scale[1]==0) scale[1]=1;
						offset[0]=mat->texture1_map.offset[0]; offset[1]=mat->texture1_map.offset[1];
						meshes.texels[k]= offset[0]+scale[0]*mesh->texelL[f->points[i]][0];
						meshes.texels[k+1]= offset[1]+scale[1]*mesh->texelL[f->points[i]][1];
					
/*						cur_mesh->tri[t].TT.s[0]=mat->texture1_map.scale[0];
						cur_mesh->tri[t].TT.s[1]=mat->texture1_map.scale[1];

						cur_mesh->tri[t].TT.d[0]=mat->texture1_map.offset[0];
						cur_mesh->tri[t].TT.d[1]=mat->texture1_map.offset[1];

						cur_mesh->tri[t].T[i].t[0]=mesh->texelL[f->points[i]][0];
						cur_mesh->tri[t].T[i].t[1]=mesh->texelL[f->points[i]][1];
*/					}

					// if no texture, do as if there wasn't coordinated
					else{
						meshes.texels[k]= 0;
						meshes.texels[k+1]= 0;
					}
				} // for( i=0; i<3; ++i){
				mat= 0;	
			} // for( t=0; t<mesh->faces; ++t){
			free( normalL);
		} // if( !node->user.d){
	}
	
	CalculateSize();
}

}
