/*
	majaactor.cpp
	ce fichier est une partie de Maja - this is a part of Maja
	class de base pour les acteurs
	basis class for actors

	$Id: $

	$Log: $

	begin:			Sep 2000
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#include "engines/scenery/majaactor.h"

namespace maja
{
using MAJAMATHS::MajaVector;
using MAJAMATHS::MajaTriangle;

MajaActor::MajaActor():MajaObject()
{
	first_update= mtrue;
	life_state= new GLuint( 1);
	if( !life_state)
		throw std::exception();
	hit=0;
	angle= 0;
	view_mode= view_mode_half_dependant;
	view_angle= dir;
}

MajaActor::MajaActor( const MajaActor& m): MajaObject( (const MajaObject&)m)
{
	dir= m.dir;
	life_state= new GLuint( 1);
	if( !life_state)
		throw std::exception();
	*life_state= *m.life_state;
	hit= m.hit;
	first_update= m.first_update;
	hit_angle= m.hit_angle;
	weight= m.weight;
	mass=m.mass;
	last_pos= m.last_pos;
	bf_l_pos= m.bf_l_pos;
	last_angle= m.last_angle;
	view_angle= m.view_angle;
	view_mode= m.view_mode;
}

mvoid MajaActor::DefineLifeState( GLuint l)
{
	*life_state= l;
}

mvoid MajaActor::CalculatePos( mvoid)
{
	pos.x+= dir.x*speed.x*inner_clk.step_clock;
	pos.y+= dir.y*speed.y*inner_clk.step_clock;
	pos.z+= dir.z*speed.y*inner_clk.step_clock;
}

mvoid MajaActor::CalculateDir( mvoid)
{
	mdouble alpha= angle.x, 
		beta=  angle.y;
	static mdouble old_alpha= alpha;
	static mdouble old_beta=  beta;
	MajaVector O;

	if( alpha==old_alpha && beta==old_beta)
		return;

/*	if( !gravity_submission)
		dir.y= sin( beta);
	dir.z= cos( beta);
		
	RotatePoint( O, alpha, dir);
*/
	dir.x=	sin( -alpha)*(1-sin( beta));
	if( !gravity_submission)
		dir.y= sin( beta);
	else
		dir.y= 0;
	dir.z= cos( -alpha)*(1-sin( beta));
}

// isn't implemented yet.
mbool MajaActor::UndoMovement( mvoid)
{
	pos= last_pos;
	angle= last_angle;
	
	/*
	|	un bon moyen serait de ralentir en f de l'angle
	|	de heurt. on dfinirait aussi un coefficient de
	|	rugosit au MajaObject que l'on a heurt (cf physique),
	|	que l'on dduirait. 
	|	une quation de la nouvelle vitesse serait de la forme;
	|	
	|						| cos( |  | ) |
	|	V(p(t),t,a(t))= V(p(t-1),t-1,a(t-1))* -----------------------	, avec k<>0
	|						  	k
	|	t= temps
	|	p(t)= position  instant p
	|	a(t)= orientation  instant p
	|	= angle de heurt
	|	k= coefficient de rugosit (k E IR*)
	|	
	|	Il faudrait aussi s'assurer que l'on est bien en dehors
	|	des MajaActor heurts (c pa le k)
	*/
	
	speed*=fabs( cos( hit_angle.x));
	
	return mtrue;
}

mvoid MajaActor::SavePos( mvoid)
{
	bf_l_pos= last_pos;
	last_pos= pos;
	last_angle= angle;
}

mbool MajaActor::TestHit( MajaObject& loc)
{
	MajaVector s= GetSize();
	real sizey= s.y;
	s= loc.GetSize();
	real lsizex= s.x, lsizey= s.y, lsizez= s.z;
	
//	if( ( pos.y-sizey/2> loc.GetPos().y+lsizey/2) || ( pos.y+sizey/2< loc.GetPos().y-lsizey/2))
//		return mfalse;
	
	DefineVolume();
	
	MajaVector ma[4], lma[4];
	mint i, ii;

	for( i=0; i<4; i++){
		ma[i]= volpts[i];
		lma[i]= loc.GetVolPts()[i];
	}
	
	if( loc.GetAngles().x==0 || loc.GetAngles().x==pi){
		real lx_min, lx_max, lz_min, lz_max;
		lx_min=-lsizex/2; 
		lx_max=lsizex/2;
		lz_min=-lsizez/2; 
		lz_max=lsizez/2;
		
		for( i=0; i<=3; i++)
			if( _WITHIN( ma[i].x, loc.GetPos().x+lx_min, loc.GetPos().x+lx_max))
				if( _WITHIN( ma[i].z, loc.GetPos().z+lz_min, loc.GetPos().z+lz_max)){
					hit_angle= angle-loc.GetAngles();
					return mtrue;
				}
	}
	else{
		real lx_min, lx_max, lz_min, lz_max;
      		MajaVector E( lma[0]), F( lma[1]), G( lma[2]), H( lma[3]);
	 	MajaVector V[4];

		RotatePoint( loc.GetPos(), -loc.GetAngles().x, E);
      		RotatePoint( loc.GetPos(), -loc.GetAngles().x, F);
      		RotatePoint( loc.GetPos(), -loc.GetAngles().x, G);
		RotatePoint( loc.GetPos(), -loc.GetAngles().x, H);
		for( ii=0; ii<4; ii++)
			RotatePoint( loc.GetPos(), -loc.GetAngles().x, ma[ii], V[ii]);
		// recherche des extremums de l'obstacle:
		lx_min= _MMIN( _MMIN( _MMIN( E.x, F.x), G.x), H.x);
		lx_max= _MMAX( _MMAX( _MMAX( E.x, F.x), G.x), H.x);
		lz_min= _MMIN( _MMIN( _MMIN( E.z, F.z), G.z), H.z);
		lz_max= _MMAX( _MMAX( _MMAX( E.z, F.z), G.z), H.z);
		for( i=0; i<=3; i++)
			if( _WITHIN( V[i].x, lx_min, lx_max))
				if( _WITHIN( V[i].z, lz_min, lz_max)){
					hit_angle= angle-loc.GetAngles();
					return mtrue;
				}
	}
	// Pour les objets Scene plus petits que le MajaActor
	if( s< GetSize()){          // < ne doit pas tre inclusif
		real xa, ya, xb, yb;
		real ca, cb, x, y;

		i=3;
		x=loc.GetPos().x; y=loc.GetPos().z;
		while( i-->=0){
			xa= ma[i].x; ya= ma[i].y;
 			xb= i==3 ? ma[0].x : ma[i+1].x;
 			yb= i==3 ? ma[0].y : ma[i+1].y;
 			if( _WITHIN( x, xa, xb)){
 				if( xb!= xa) 
					ca= (yb-ya)/(xb-xa); 
				else 
					return mfalse;
 				cb= (xb*ya-yb*xa)/(xb-xa);
 				if( fabs( ca*x+cb)-y<=0.01){
					hit_angle= angle-loc.GetAngles();
					return mtrue;
				}
			}
		}
	}
	return mfalse;
}

mbool MajaActor::TestHit( MAJACONTAINER::MajaList<MajaScene>& loc)
{
	MAJACONTAINER::MajaList<MajaScene> *t= &loc;
	MAJACONTAINER::MajaWidget<MajaScene> *s= t->GetFirst();
	MajaScene *p;
	
	if( !s) 
		return mfalse;
	while( s){
		p= s->GetWidget();
		if( TestHit( *p))
			return mtrue;
		s=s->GetNext();
	}
	return mfalse;
}

mint MajaActor::ObjectOnFloor( MajaPath *flr)
{
	MajaVector p( pos);
	MajaVector v1, v2, v3;
	MajaTriangle T1;
	mint i;
	mbool  tri= mfalse;
	
	// where is the object on the path (over or under)
//	mint where=0;
	
	mfloat *param= flr->GetVertices();

	// step: seek the 3 proxi-points
	//------------------------------
	// going threw the MajaPath's points while seeking if it's around the given point
	for( i=0; i< flr->GetVerticesNb()-9; i+=9){
		v1.x= param[i];
		v1.y= param[i+1];
		v1.z= param[i+2];
		v2.x= param[i+3];
		v2.y= param[i+4];
		v2.z= param[i+5];
		v3.x= param[i+6];
		v3.y= param[i+7];
		v3.z= param[i+8];
			
		T1.SetTriangle( v1, v2, v3);
			
		if( T1.IsInnerPoint( p)){
			//std::cerr << std::endl << "found a tri !";
			tri=mtrue;
			break;
		}
	}

	if( !tri){
		//std::cerr << std::endl << "not found a tri !";
		pos.y= last_pos.y;
		return 2;
	}
	std::cerr << std::endl << "tri index is " << i;

	// find where we place related to the MajaPath
	//where= ReactOnPath( *flr);
	
	MajaVector A,B,N;

	A= v2-v1;
	B= v3-v1;
	
	N= A.CrossProduct( B);
	if( (N.x==0) && (N.y==0) && (N.z==0)) 
		return -2;

	// the quation is : u(x-x1)+v(y-y1)+w(z-z1)=0, or ux+vy+wz-(ux1+vy1+vz1)=0
	// step : calculate the high of the ground when x=px and z=pz (the sudied center)
	//----------------------------------------------------------
	if( N.y!=0) 
		p.y= ((N.x*v1.x+N.y*v1.y+N.z*v1.z)-N.x*p.x-N.z*p.z)/N.y;
	else
		p.y= last_pos.y;

	pos.y= p.y+size.y;

	return 1;
}

mint MajaActor::ObjectOnFloor( MAJACONTAINER::MajaList<MajaPath>& list)
{
	if( !gravity_submission)
		return 1;

	MAJACONTAINER::MajaList<MajaPath> *t= &list;
	if( !t)
		return 0;
	MAJACONTAINER::MajaWidget<MajaPath> *s= t->GetFirst();
	if( !s) return 0;
	MajaPath *p;
	mint knowing= 0; 					// for know on which path we are
	GLuint state=0;						// the state of the path
	mbool pas_bon= mfalse;					// knwing if we're on the good path and not the terrain !
	
	while( s){
		p= s->GetWidget();
		knowing= ObjectOnFloor( p);
		state= p->GetState();
		if( state> MAJA_PATH_ABNORMAL && knowing==1){
			pas_bon= mtrue;
		}
		else if( state> MAJA_PATH_NORMAL && state< MAJA_PATH_ABNORMAL && knowing==1){
			return 1;
		}
		s=s->GetNext();
	}
	return 0;	
}

mbool MajaActor::Update( mvoid)
{
	if( first_update){
		inner_clk.Start();
		first_update=!first_update;
	}
	
	inner_clk.Step();
	
	// direction of the player
	CalculateDir();
	
	// position related from last pos
	CalculatePos();
	
	return mtrue;
}

mbool	MajaActor::Realize(	const MAJACONTAINER::MajaList<MajaScene>& s, 
				const MAJACONTAINER::MajaList<MajaPath>& floor, 
				const MAJACONTAINER::MajaList<MajaActor>& plyr)
{	
	ObjectOnFloor( (MAJACONTAINER::MajaList<MajaPath>&)floor);
	
	// Detects collisions
	if( TestHit( (MAJACONTAINER::MajaList<MajaScene>&)s) ){
		UndoMovement();
		hit++;
	}
	
/*	if( TestHit( plyr)){
		bf_l_pos= pos= last_pos;
		speed= 0;
		//UndoMovement();
		hit++;
		//if( hit>3){ hit=0; speed=-speed; cerr << endl << typeid( *this).name() << speed;	}
	}
*/	
	return mtrue;
}

mbool MajaActor::Init( MajaVector *d, MajaVector *a, mfloat sp, muint l)
{
	SetAngles( *a);
	speed= sp;
	DefineVolume();
	
	return mtrue;
}

mint MajaActor::ReactOnPath( MajaPath& path)
{
	return 0;

	GLuint state= path.GetState();				// state of the path
	GLuint state_to_state=0;							// memory state
	
	if( state>= MAJA_PATH_ABNORMAL){
		if( !UserReactOnPath( path)) 
			return 0;
		state-= MAJA_PATH_ABNORMAL;
	}
	if( state>= MAJA_PATH_NORMAL){
		state-= MAJA_PATH_NORMAL;
	}
	if( state>= MAJA_PATH_OVER){
		state_to_state= state;
		state-= MAJA_PATH_OVER;
	}
	if( state>= MAJA_PATH_UNDER){
		state_to_state= state;
		state-= MAJA_PATH_UNDER;
	}
	if( state== MAJA_PATH_ON){
		if( state_to_state>= MAJA_PATH_OVER) 
			return 1;
		if( state_to_state>= MAJA_PATH_UNDER) 
			return -1;
	}
	if( state== MAJA_PATH_OFF){
		return 0;
	}
//	__maja_exit( "Invalid MajaPath state", __MAJA_WARNING);
	
	return -2;
}

mbool MajaActor::UserReactOnPath( MajaPath& path)
{
	return mtrue;
}

mvoid MajaActor::IsAlive( mvoid)
{
	Update();
//	Realize( s, floor, plyr, see);
	SavePos();
}

}
