/* $Id: $
 * majacar.cpp , this is a part of Maja
 * Maja car
 * Copyright (C) september 2000, 2001, 2002 Jean-Dominique Frattini
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Cette license est valable en terme de droit franais. Cela signifie que c'est la
 * lgislation franaise qui est en vigueur pour l'ensemble des sources contenues
 * dans la librairie. Sauf image.cpp et image.h, se reposant sur aucune license explicite,
 * mais qui sont fournies aimablement dans les librairies Mesa sous license LGPL.
 *
 * Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou
 * le modifier sous les termes de la license publique gnrale
 * publie par la Free Software Foundation; soit la version 2 de la
 * license, ou (si vous le voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir d'tre utile, mais
 * SANS AUCUNE GARANTIE; et mme une garantie implicite,
 * y compris les garanties de COMMERCIALISATION ou D'ADAPTATION
 * DANS UN BUT SPCIFIQUE. Prenez connaissance de la License Publique
 * Gnrale GNU pour plus de dtail.
 *
 * Vous devez avoir reu une copie de la License Publique Gnrale
 * GNU avec ce programme; si ce n'est pas le cas, vrivez  la
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: $
 */

#include "engines/scenery/majacar.h"

namespace maja
{
	
MajaCar::MajaCar( void): MajaMultiMotion()
{
	headlight= null;
	wheel_radius= 0;
	wheel_freq= 0;
	front_left_wheel= NULL;
	front_right_wheel= NULL;
	back_left_wheel= NULL;
	back_right_wheel= NULL;
}

MajaCar::MajaCar( MajaCar& s): MajaMultiMotion( (MajaMultiMotion&)s)
{
	wheel_radius= s.wheel_radius;
	wheel_freq= s.wheel_freq;
	last_wheel_angle= s.last_wheel_angle;
	centre= s.centre;
	
	// for now
	headlight= NULL;
	
	front_left_wheel= s.front_left_wheel;
	front_right_wheel= s.front_right_wheel;
	back_left_wheel= s.back_left_wheel;
	back_right_wheel= s.back_right_wheel;
}

MajaCar::~MajaCar( void)
{
	//delete headlight;
}

void MajaCar::CalculateDir( void)
{
	static bool first= true;
	if( first) first= !first;
	static MajaClock dir_clk;
	if( first) dir_clk.Start();
	dir_clk.Step();

	MajaVector a( back_left_wheel->GetPos());		// back left wheel coordinates
	MajaVector b( front_left_wheel->GetPos());	// front left wheel coordinates
	MajaVector c( 0,0,0);				// center of rotation if any
	MajaVector o( c);					// origine for some rotations
	float ab;					// distance between a and b
	float bc;					// distance between b and c
	bool geom= true;				// true if turning right, else false

	// check if turning right or left:  <0 means right
	if( (wheel_angle.x<0 && speed>0) || (wheel_angle.x>0 && speed<0)){
		geom= false;
 		a.x= -a.x; b.x= -b.x;
 	}
 	
	// distance between a and b, no rotaion effectuated, so just a difference
 	ab= b.z-a.z;
 	
 	// calculate distance between b and c!
 	// create a triangle abc rectangle in b, so that ac is its hypotenuse
 	// the angle in a is pi/2-wheel_angle because, when no rotation, the tires
 	// make an angle of pi/2 in relation with ab and ac
 	bc= ab*tan( pi/2-wheel_angle.x);
 	
 	// coordinates of c relating with the turn sense
 	if( geom) 
		c.x= b.x+bc; 
	else 
		c.x= b.x-bc;
 	c.z= a.z;
 	
 	// now rotate the point, so it matches with the car's orientation
 	RotatePoint( o, angle.x, c);
 	RotatePoint( o, angle.x, a);
 	RotatePoint( o, angle.x, b);
 	
 	// real coordinates of the center: center of the car + it's position related to o:
 	centre= pos+c;
 	
	// direction of the car before the rotation applied
	dir.x= sin( wheel_angle.x+angle.x);
	dir.y= 0;
	dir.z= cos( angle.x+wheel_angle.x);
	
	// now calculate the new position of the car
	CalculatePos();
	
	// the car's orientation: i hope this is OK, but UNSURE.
	angle.x+= wheel_angle.x*dir_clk.step_clock*speed.CalculateMagnitude();
	
	// steel rotate only the front wires
	/*sub_objects.GetFirst()->GetNextObject()->angle=
			sub_objects.GetFirst()->GetNext()->GetNext()->GetNextObject()->angle=
			this->wheel_angle;*/
	front_left_wheel->GetAngles()= front_right_wheel->GetAngles()= wheel_angle;
	
	// rotate the wheels when speed!=0, and the others (before)
	/*sub_objects.GetFirstObject()->angle.y=	sub_objects.GetFirst()->GetNext()->GetNextObject()->angle.y=
			this->wheel_angle.y;*/
	back_left_wheel->GetAngles().y= back_right_wheel->GetAngles().y= wheel_angle.y;
}

void MajaCar::IsAlive( void)// const MajaSceneList& s, const MajaPathList& floor, const MajaMonitorable& plyr, bool see)
{
	Update();
//	Realize( s, floor, plyr, see);
	
	//if( angle.x!= pi) angle.x=pi;
	speed= 0.5;
	
	// must set it to 0 each time, otherwise the car flies up !
	pos.y= 0;
	
//	AutoHeadlights( true);
	
	WheelsTurn();
	
	// sauvegarde les tats:
	SavePos();
}

bool MajaCar::UseHeadlights( GLuint nb)
{
/*	headlight= new _headlight( nb);
	if( !headlight){
		__maja_exit( "it lacks memory !", __MAJA_WARNING);
		return false;
	}
*/	return true;
}

// this is the cause of the memory problems !!!!!
bool MajaCar::AutoHeadlights( bool check)
{
/*	if( check){
		UseHeadlights( 1);
		matrix4D p( 5,2,5,1);
		p+= (matrix4D&)pos;
		matrix4D c( 1,0,0,1);
		matrix4D white( 0.01,0.01,1.0,1);
		matrix4D dir( 0,-0.2333,2,1);
//		float *tmp= p.CastToPointer();
		glLightfv( GL_LIGHT1, GL_POSITION, (const GLfloat*)(p.CastToPointer()));
		glLightfv( GL_LIGHT1, GL_DIFFUSE, (const GLfloat*)(c.CastToPointer()));
		glLightfv( GL_LIGHT1, GL_SPECULAR, (const GLfloat*)(white.CastToPointer()));
		// memory is not deleted !:
		GLfloat *cut= new GLfloat( 43.333);
		glLightfv( GL_LIGHT1, GL_SPOT_CUTOFF, cut);
		glLightfv( GL_LIGHT1, GL_SPOT_DIRECTION, (const GLfloat*)(dir.CastToPointer()));
		GLfloat exp= 2.0;
		glLightfv( GL_LIGHT1, GL_SPOT_EXPONENT, &exp);
		GLfloat con=1.5;
		glLightfv( GL_LIGHT1, GL_CONSTANT_ATTENUATION, &con);
		GLfloat lin=0.5;
		glLightfv( GL_LIGHT1, GL_LINEAR_ATTENUATION, &lin);
		GLfloat qua=0.2;
		glLightfv( GL_LIGHT1, GL_QUADRATIC_ATTENUATION, &qua);
//		glDisable( GL_LIGHT0);
//		glEnable( GL_LIGHT1);
	}
*/	return check;
}

MajaCar::_headlight::_headlight( GLuint nb)
{
	number= nb;
//	headlight= new MajaLight[number];
	state= new bool[number];
//	if( !headlight || !state){
//		__maja_exit( "It lacks memory !", __MAJA_WARNING);
//		return;
//	}
	for( GLuint i=0; i<number; i++) state[i]= false;
}

MajaCar::_headlight::~_headlight( void)
{
//	delete this->headlight;
	delete this->state;
}

void MajaCar::InitWheels( void)
{
	MajaVector sc= sub_objects.GetFirstObject()->GetSize();	// joints[0].GetSize();
	
	wheel_radius= pi*sc.z;
	if( wheel_radius==0){
		// need to make a mathematic error: division by zero must return zero and not infinite !
//		__maja_exit( "MajaCar: division by zero in wheel radius calculation, 0 to avoid", __MAJA_WARNING);
		wheel_freq=0;
		
		return;
	}
	
	wheel_freq= 1/wheel_radius;
}

void MajaCar::WheelsTurn( void)
{	
	int sense=0;
	
	if( speed>0) sense=1; else if( speed<0) sense=-1;
	
	// only 'demo mode' for development
	if( demo_mode)
		wheel_angle.x= pi_4*sin( inner_clk.Clock());

	// distance done btw 2 calls
	float len= StraightLenght( pos, last_pos);
	
	// angle done
	float alpha= len/wheel_radius;
	wheel_angle.y+= alpha*sense;
}

void MajaCar::SetFrontLeftWheel( MajaMultiMotion *flw)
{
	flw ? front_left_wheel= flw : front_left_wheel= sub_objects.GetFirstObject();
}

void MajaCar::SetFrontRightWheel( MajaMultiMotion *frw)
{
	frw ? front_right_wheel= frw : front_right_wheel= sub_objects.GetFirst()->GetNextObject();
}

void MajaCar::SetBackLeftWheel( MajaMultiMotion *blw)
{
	blw ? back_left_wheel= blw: back_left_wheel= sub_objects.GetFirst()->GetNext()->GetNextObject();
}

void MajaCar::SetBackRightWheel( MajaMultiMotion *brw)
{
	brw ? back_right_wheel= brw : back_right_wheel= sub_objects.GetFirst()->GetNext()->GetNext()->GetNextObject();
}
}
