/*
	majaobject.cpp
	ce fichier est une partie de Maja - this is a part of Maja
	objet principal des MO
	MO's main object

	$Id: $

	$Log: $

	begin:			Sep 10 2000
	copyright:		(C) 2000,2001,2002,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#include "engines/scenery/majaobject.h"

namespace maja
{
	
//MajaTextures&		MajaObject::maja_textures= *(new MajaTextures());	// must be avoided

MajaObject::MajaObject( mvoid)
{
	mod= mnull;
	model_file= "";
	mass= 0;
	gravity_submission= mfalse;
}

MajaObject::MajaObject( const MajaObject& mo)
{
	pos= mo.pos; size= mo.size;
	angle= mo.angle;
	name= mo.name;
	DefineVolume();
	if( mod){
		//allocate
		//*mod= *(mo.mod);
	}
	inner_clk= mo.inner_clk;
	mass= mo.mass;
}

MajaObject::~MajaObject( mvoid)
{
	delete mod;
}

mbool MajaObject::Load( mvoid)
{
	if( mod)
		delete mod;		
	mod= new MajaModule();	
	if( !mod)
		return mfalse;

	if( !mod->Load( (mchar*)model_file.c_str())){
		std::cerr << std::endl << "error while loading model.";
		return mfalse;
	}
	size= mod->GetSize();
	pos= mod->GetPos();

	pos.Print();
	size.Print();
	
	return mtrue;
}

mvoid MajaObject::SetModelFile( std::string m_file)
{
	model_file= m_file;
}

mvoid MajaObject::Presence( mvoid)
{
	;
}

mvoid MajaObject::DefineVolume( mvoid)
{
	DefineVolumeFromCenter();
	RotateVolume();
}

mvoid MajaObject::DefineVolume( real px, real py, real pz, real qx, real qy, real qz)
{
	real x = pos.x; real y = pos.y; real z = pos.z;

	volpts[0].y= volpts[1].y= volpts[2].y= volpts[3].y= y- py;
	volpts[4].y= volpts[5].y= volpts[6].y= volpts[7].y= y+ qy;
	volpts[0].x= volpts[3].x= volpts[4].x= volpts[7].x= x- px;
	volpts[1].x= volpts[2].x= volpts[5].x= volpts[6].x= x+ qx;
	volpts[0].z= volpts[1].z= volpts[4].z= volpts[5].z= z- qz;
	volpts[2].z= volpts[3].z= volpts[6].z= volpts[7].z= z+ pz;
}

mvoid MajaObject::DefineVolumeFromCenter( mvoid)
{
	real lx= size.x/2; real ly= size.y/2; real lz= size.z/2;
	DefineVolume( lx, ly, lz, lx, ly, lz);
}

// when you can't use a graphical object that knows its geometrical center, Maja permits another way.
// But there is another restriction (that make other ways impossible): the "center" point must have the lower
// coordinates in the three dimensions: x, y and z minimum of the drawing. Later, it will be called the lower angle.
// For now, this possibility has been disable (i though it was bad to test it each time before a redrawing)
mvoid MajaObject::DefineVolumeFromAngle( mvoid)
{
	real lx= size.x; real ly= size.y; real lz= size.z;
	DefineVolume( 0,0,0, lx, ly, lz);
}

// Put a MajaObject type object at top of the given MajaObject
// This doesn't work for polymorphic MajaObjects as MajaPath
// todo: recalculate the maximum height of the object so that the other could lie on it.
mint MajaObject::ObjectOnFloor( const MajaObject& flr)
{
	mfloat decale;

	if( volpts[0].y!= flr.pos.y){
		decale= volpts[0].y- flr.pos.y+flr.size.y/2;
		pos.y-= decale;
	}
	return 1;
}

// set the angles. angle is public, so could do another way.
mvoid MajaObject::SetAngles( MajaVector a, mbool active)
{
	angle=a;
	if( !active) 
		return;
	RotateVolume();
}

// Rotate volpts[] to match with the rotation of the object. If its angles change, the vertices
// aren't the same. only angle.x (around y axis) is performed.
//
// todo: rotation around the other axis: angle.y and ~.z
mvoid MajaObject::RotateVolume( mvoid)
{
	// set the actual volpts[], avoid call to DefineVolume()
	MajaVector a( pos.x-size.x/2, pos.y, pos.z-size.z/2);
	MajaVector b( pos.x+size.x/2, a.y  , a.z);
	MajaVector c( b.x           , a.y  , pos.z+size.z/2);
	MajaVector d( a.x           , a.y  , c.z);
	mfloat ang= angle.x;

	RotatePoint( pos, ang, a); RotatePoint( pos, ang, b);
	RotatePoint( pos, ang, c); RotatePoint( pos, ang, d);
	volpts[0]= a; volpts[1]= b; volpts[2]= c; volpts[3]= d;
	volpts[4]= a; volpts[5]= b; volpts[6]= c; volpts[7]= d;
	volpts[4].y+= size.y*0.5; volpts[5].y+= size.y*0.5; volpts[6].y+= size.y*0.5; volpts[7].y+= size.y*0.5;
}

mvoid MajaObject::SetName( std::string& n)
{
	name= n;
}

mbool MajaObject::CheckName( std::string& n)
{
	if( name== n) 
		return true;
	return false;
}
/*
std::string& MajaObject::GetName( mvoid)
{
	return name;
}
*/
mchar* MajaObject::GetName( mvoid)
{
	return (mchar*)name.c_str();
}

// only compare same MajaObject type because some object may
// have the same aspect.
mbool MajaObject::operator==( MajaObject& src)
{
	if( typeid( this)!= typeid( src)) 
		return mfalse;
	if( size==src.size && angle==src.angle && name== src.name) 
		return mtrue;
	return mfalse;
}

mvoid MajaObject::FreeModel( mvoid)
{
	if( mod){ 
		delete mod; 
		mod=null;
	}
}

MajaVector& MajaObject::CalculateSpeed( mint sens)
{
	speed+= acceleration*inner_clk.Step()*sens;

	return speed;
}

MajaVector& MajaObject::Accelerate( mfloat k)
{
	speed= CalculateSpeed( -1)*k;

	return speed;
}
	
MajaVector& MajaObject::Decelerate( mfloat k)
{
	return speed= CalculateSpeed( 1)*k;
}

}
