/*
	majaxwindow.cpp
	ce fichier est une partie de Maja - this is a part of Maja
	fentre glx
	glx window
	
	$Id: $

	$Log: $

	begin:			June 2002
	copyright:		(C) 2000,2001,2003,2003 by Jean-Dominique Frattini
	email:			jd.frattini@tuxfamily.org	
*/

/************************************************************************************************
 *												*
 *	Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou le modifier	*
 *	sous les termes de la GNU General Public License comme publie par			*
 *	la Free Software Foundation; soit en utilisant la verstion 2 de cette license, ou	*
 *	(si vous le voulez) toute version ultrieure.						*
 *	-											*
 *	This program is free software; you can redistribute it and/or modify			*
 *	it under the terms of the GNU General Public License as published by			*
 *	the Free Software Foundation; either version 2 of the License, or			*
 *	(at your option) any later version.							*
 *												*
 ************************************************************************************************/

#include <iostream>
#include <GL/glu.h>
#include "layers/wm/majaxwindow.h"
#include "layers/th/majathread.h"

namespace maja
{
	
MajaXWindow::MajaXWindow( mvoid): MajaWindow()
{
	end_loop= mfalse;
	CheckThread();
	title= mnull;
}

mbool MajaXWindow::CheckThread( mvoid)
{
	#ifdef _MAJA_THREADED
	std::cerr << " enabled";
	std::cerr << std::endl << "MajaXWindow: X: init for multithreads...";
	if( (mint)XInitThreads()){
		#ifndef _MAJA_WINDOW_THREADED
		#define _MAJA_WINDOW_THREADED 1
		#endif
		std::cerr << " done.";
		return mtrue;
	}
	else{
		#ifdef _MAJA_WINDOW_THREADED
		#undef _MAJA_WINDOW_THREADED
		#endif
		std::cerr << " failed.";
		return mfalse;
	}
	#else
	std::cerr << " disabled. WARNING !!! Maja will may crash !";
	return mfalse;
	#endif
}

mbool MajaXWindow::CreateDisplay( mvoid)
{
	drawable.CreateDisplay();
	
	return mtrue;
}

//TODO: care about all: extensions, client, server...
bool MajaXWindow::CheckSupport( mvoid)
{
	int glx_rel_maj, glx_rel_min;
	
	glXQueryVersion( drawable._d, &glx_rel_maj, &glx_rel_min);

	return true;
}

bool MajaXWindow::CreateScreen( mvoid)
{
	scr= DefaultScreen( drawable._d);
	
	return true;
}

mbool MajaXWindow::CreateConfig( mvoid)
{
	#ifdef GLX_VERSION_1_3
	const mint *gattr= attributes.glx_attr;
	mint *gnb= &attributes.glx_attr_nb;
	#endif
	mint vid_rel_maj, vid_rel_min;
	mint mode_num;
	mint i;
	
	XF86VidModeQueryVersion( drawable._d, &vid_rel_maj, &vid_rel_min);
	XF86VidModeGetAllModeLines( drawable._d, scr, &mode_num, &modes);		
	mode_info= *modes[0];
	
	for( i=0; i< mode_num; i++)
		if( ( modes[i]->hdisplay==attributes.width) && ( modes[i]->vdisplay== attributes.height))
			best_mode= i;
	
	#ifdef GLX_VERSION_1_3
	drawable.config= glXChooseFBConfig( drawable._d, scr, gattr, gnb);

	if( !drawable.config){
		std::cerr << std::endl << "No config for X";
		throw std::exception();
	}
	#endif
	
	return mtrue;
}

bool MajaXWindow::CreateVisual( mvoid)
{
	#ifdef GLX_VERSION_1_3
	drawable._v= glXGetVisualFromFBConfig( drawable._d, *drawable.config);
	#else
	drawable._v= glXChooseVisual( drawable._d, scr, attributes.glx_attr); // deprecated	
	#endif
	if( !drawable._v){
		std::cerr << std::endl << "No drawable._v for X";
		return mfalse;
	}
	return mtrue;
}

volatile _MajaContext MajaXWindow::CreateContext( mvoid)
{
	return drawable.CreateContext();
}

bool MajaXWindow::CreateMotherWindow( mvoid)
{
	Colormap cmap;
	int dw, dh;
	Atom wm_delete;

	cmap= XCreateColormap( drawable._d, RootWindow( drawable._d, drawable._v->screen), drawable._v->visual, None);
	this->attributes.attributes.colormap= cmap;
	this->attributes.attributes.background_pixel= 0;
	this->attributes.attributes.border_pixel= 0;
	attributes.attributes.event_mask=	ExposureMask		| 
						KeyPressMask 		|
						KeyReleaseMask		|
						ButtonPressMask 	|
						ButtonReleaseMask 	|
						PointerMotionMask 	|
						StructureNotifyMask;
	
	if( attributes.fullscreen){
		XF86VidModeSwitchToMode( drawable._d, scr, modes[best_mode]);
		XF86VidModeSetViewPort( drawable._d, scr, 0, 0);

		dw= modes[best_mode]->hdisplay;
		dh= modes[best_mode]->vdisplay;
		XFree( modes);

		attributes.attributes.override_redirect= true;
		
		XSync( drawable._d, 0);
		drawable._w= XCreateWindow(	drawable._d, 
						RootWindow( drawable._d, drawable._v->screen), 
						0, 
						0,
						dw, 
						dh, 
						0, 
						drawable._v->depth, 
						InputOutput, 
						drawable._v->visual,
						CWBorderPixel 	| 
						CWColormap 	| 
						CWEventMask 	| 
						CWOverrideRedirect, 
						&attributes.attributes);

		XWarpPointer( drawable._d, None, drawable._w, 0,0,0,0,0,0);
		XMapRaised( drawable._d, drawable._w);
		XGrabKeyboard( drawable._d, drawable._w, true, GrabModeAsync, GrabModeAsync, CurrentTime);
		XGrabPointer( drawable._d, drawable._w, true, ButtonPressMask, GrabModeAsync, GrabModeAsync,
			drawable._w, None, CurrentTime);

	}
	else{
		XSync( drawable._d, 0);

		drawable._w= XCreateWindow(	drawable._d,					// display
						RootWindow( drawable._d, drawable._v->screen), 	// parent
						0, 						// x
						0, 						// y
						attributes.width, 
						attributes.height, 
						0, 						// border width
						drawable._v->depth, 				// depth
						InputOutput, 					// class
						drawable._v->visual, 				// visual
						CWBorderPixel 	| 
						CWColormap 	| 
						CWEventMask 	| 
						CWOverrideRedirect, 				// values masks
						&attributes.attributes); 			// attributes

		wm_delete= XInternAtom( drawable._d, "WM_DELETE_WINDOW", true);
		XSetWMProtocols( drawable._d, drawable._w, &wm_delete, 1);
		XSetStandardProperties( drawable._d, drawable._w, attributes.name, attributes.name, None, mnull, 0, mnull);
		XMapRaised( drawable._d, drawable._w);
	}

	return true;
}

mbool MajaXWindow::CheckDirectRender( Display *d, GLXContext c)
{
	if( glXIsDirect( d?d:drawable._d, c?c:drawable._c)){
		direct_render= mtrue;
		return mtrue;
	}
	direct_render= mfalse;
	std::cerr << " direct rendering is not enabled";

	return mfalse;
}

mbool MajaXWindow::CreateWindow( mvoid)
{
	uint border_dummy;
	Window win_dummy;

	if( !(&attributes)) 
		return mfalse;
	
	std::cerr << std::endl << "MajaXWindow: creating window";
	
	if( CreateDisplay())
		std::cerr << ".";
	else{
		std::cerr << " failed.";
		return mfalse;
	}
	
	CheckSupport();
	std::cerr << ".";
	CreateScreen();
	std::cerr << ".";
	CreateConfig();
	std::cerr << ".";
	CreateVisual();
	std::cerr << ".";
	CreateContext();
	std::cerr << ".";
	CreateMotherWindow();
	std::cerr << ".";
	
	// should do it in a more best way
/*	#ifdef GLX_VERSION_1_3
	mint *wattr= attributes.glx_attr;
	wattr[0]= GLX_USE_GL;
	wattr[1]= GLX_BUFFER_SIZE;
	wattr[2]= 24;
	wattr[3]= GLX_RGBA;
	wattr[4]= GLX_DOUBLEBUFFER;
	wattr[5]= GLX_RED_SIZE;
	wattr[6]= 8;
	wattr[7]= GLX_GREEN_SIZE;
	wattr[8]= 8;
	wattr[9]= GLX_BLUE_SIZE;
	wattr[10]= 4;
	wattr[11]= GLX_ALPHA_SIZE;
	wattr[12]= 4;
	wattr[13]= GLX_DEPTH_SIZE;
	wattr[14]= 24;
	wattr[15]= GLX_STENCIL_SIZE;
	wattr[16]= 16;
	for( mint ii=17; ii<attributes.glx_attr_nb; ii++)
		wattr[ii]= None;
	if( !drawable.CreateWindow( &wattr[0])){
		std::cerr << " failed to create glx drawables";
		throw std::exception();
	}
	std::cerr << ".";
	#endif
*/
	drawable.Enter();
	
	//#ifndef GLX_VERSION_1_3
	XGetGeometry(	drawable._d,		drawable._w,		&win_dummy, 
			&attributes.x, 		&attributes.y, 
			&attributes.width, 	&attributes.height,
			&border_dummy,		&attributes.depth);
	std::cerr << ".";
	//#endif
	
	CheckDirectRender();
	std::cerr << ".";
	
	//#ifndef GLX_VERSION_1_3
	XMapWindow( drawable._d, drawable._w);
	std::cerr << ".";
	//#endif
	
	glClearColor( 0,0,0,0);
	glClear( GL_COLOR_BUFFER_BIT);	
	std::cerr << ".";
	
	//#ifndef GLX_VERSION_1_3
	glXSwapBuffers( drawable._d, drawable._w);
	//#else
	//glXSwapBuffers( drawable._d, drawable.dr);
	//#endif
	
	state= MWS_state_alive;
	window_created= mtrue;

	std::cerr << " done.";

	return mtrue;
}

muint MajaXWindow::LoadLogo( mvoid)
{
	return 1;
}
	
mvoid MajaXWindow::DestroyWindow( mvoid)
{
	if( drawable._c){
		if( !glXMakeCurrent( drawable._d, None, mnull))
			std::cerr << std::endl << "can't release the drawing context";
		glXDestroyContext( drawable._d, drawable._c);
		drawable._c= mnull;
	}
	else std::cerr << std::endl << "There is no context to destroy !";
	
	XCloseDisplay( drawable._d);
	window_created= mfalse;
	state= MWS_state_dead;
	direct_render= mfalse;
}

mvoid MajaXWindow::SubFullscreenSwitch( mvoid)
{
	if( attributes.fullscreen){
		XF86VidModeSwitchToMode( drawable._d, scr, &mode_info);
		XF86VidModeSetViewPort( drawable._d, scr, 0, 0);
	}
}
	
mvoid MajaXWindow::SetAttributes( MajaWinAttr &attr)
{
	attributes= attr;
}

MajaWinAttr& MajaXWindow::GetAttributes( mvoid)
{
	return attributes;
}

Display* MajaXWindow::GetDisplay( mvoid)
{
	return drawable._d;
}

_MajaWindow& MajaXWindow::GetWindow( mvoid)
{
	return drawable._w;
}

_MajaWinEvent& MajaXWindow::LoopEvent( mvoid)
{
	// here when using glx 1.3, crash surely because there's a glx event here...
	
//	glXSelectEvent( drawable._d, drawable.dr, GLX_PBUFFER_CLOBBER_MASK);

	XNextEvent( drawable._d, &event);

	return event;
}

mvoid MajaXWindow::Redraw( mvoid)
{
	glPushMatrix();
	glBindTexture( GL_TEXTURE_2D, 1);
	glBegin( GL_QUADS);
	glTexCoord2f( 0,0); glVertex3f( 0,0,0);
	glTexCoord2f( 1,0); glVertex3f( 1,0,0);
	glTexCoord2f( 1,1); glVertex3f( 1,1,0);
	glTexCoord2f( 0,1); glVertex3f( 0,1,0);
	glEnd();
	glPopMatrix();
}

mvoid MajaXWindow::Resize( mvoid)
{
	glViewport( 0, 0, attributes.width, attributes.height);
	glMatrixMode( GL_PROJECTION);
	glLoadIdentity();
	gluPerspective( 90, attributes.width/attributes.height, 0.125, 1200);
	glMatrixMode( GL_MODELVIEW);
	glLoadIdentity();
}

mvoid MajaXWindow::PrintInfo( mvoid)
{
	std::cerr << std::endl << "window info:";
	std::cerr << std::endl << "display: " << drawable._d;
	std::cerr << std::endl << "parent window: " << RootWindow( drawable._d, drawable._v->screen);
	std::cerr << std::endl << "x= 0; y= 0";
	std::cerr << std::endl << "w= " << attributes.width << " ; h= " << attributes.height;
	std::cerr << std::endl << "depth: " << drawable._v->depth;
	std::cerr << std::endl << "visual " << drawable._v->visual;
}

}// namespace maja
