/* geometry.cpp, this is a part of Maja
 * geometry
 * Copyright (C) september 2000-2002 Jean-Dominique Frattini
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Cette license est valable en terme de droit franais. Cela signifie que c'est la
 * lgislation franaise qui est en vigueur pour l'ensemble des sources contenues
 * dans la librairie. Sauf image.cpp et image.h, se reposant sur aucune license explicite,
 * mais qui sont fournies aimablement dans les librairies Mesa sous license LGPL.
 *
 * Ce programme est un logiciel libre; vous pouvez le redistribuer et/ou
 * le modifier sous les termes de la license publique gnrale
 * publie par la Free Software Foundation; soit la version 2 de la
 * license, ou (si vous le voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir d'tre utile, mais
 * SANS AUCUNE GARANTIE; et mme une garantie implicite,
 * y compris les garanties de COMMERCIALISATION ou D'ADAPTATION
 * DANS UN BUT SPCIFIQUE. Prenez connaissance de la License Publique
 * Gnrale GNU pour plus de dtail.
 *
 * Vous devez avoir reu une copie de la License Publique Gnrale
 * GNU avec ce programme; si ce n'est pas le cas, vrivez  la
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "tools/maths/geometry.h"

namespace maja
{
namespace MAJAMATHS{

// Rotation d'un point autour d'un autre, l'angle est en radians, dans le
// sens trigonomtrique (sens inverse des aiguilles d'une montre) selon
// l'axe des y d'opengl: la hauteur
mvoid RotatePoint( MajaVector& centre, mreal alpha, MajaVector& point, MajaVector& p)
{
   mreal x = centre.x;
   mreal y = centre.z;
   mreal mx= point.x;
   mreal my= point.z;

//   alpha= -alpha;

   p.x= x*(1-cos(alpha))+(sin(alpha))*(y-my)+mx*cos(alpha);
   p.y= point.y;
   p.z= cos(alpha)*(my-y)+sin(alpha)*(mx-x)+y;
}

mvoid RotatePoint( MajaVector& centre, mreal alpha, MajaVector& point)
{
   mreal x = centre.x;
   mreal y = centre.z;
   mreal mx= point.x;
   mreal my= point.z;

//   alpha= -alpha;

   point.x= x*(1-cos(alpha))+(sin(alpha))*(y-my)+mx*cos(alpha);
   point.z= (cos(alpha))*(my-y)+(sin(alpha))*(mx-x)+y;
}

mvoid RotatePoint_x( MajaVector& centre, mreal alpha, MajaVector& point)
{
   mreal x = centre.y;
   mreal y = centre.z;
   mreal mx= point.y;
   mreal my= point.z;

//   alpha= -alpha;

   point.y= x*(1-cos(alpha))+(sin(alpha))*(y-my)+mx*cos(alpha);
   point.z= (cos(alpha))*(my-y)+(sin(alpha))*(mx-x)+y;
}

// Normalize un vecteur( norme=1) si i par dfaut (=1), sinon
// le rend  la taille voulue. C'est donc un vecteur colinaire
// au vecteur en paramtre et de norme i.
mbool NormalizeVector( MajaVector& m, mreal i)
{
	mreal k, l;

	l= sqrt( m.x*m.x+m.y*m.y+m.z*m.z);

	if( l) k= 1/l; else return mfalse;

	m.x*= i*k; m.y*= i*k; m.z*= i*k;

	return mtrue;
}

// quation cartsienne de la droite (AB) en f de x et z, a et b sont les valeurs de z=ax+b
// ogl= true <=> repre OpenGL (y en haut), sinon repre avec z en haut (en f de x et y)
mbool StraightEqu( MajaVector& A, MajaVector& B, mreal& a, mreal& b, mbool ogl)
{
	if( ogl){
		a= (B.z-A.z)/(B.x-A.x);
		b= (A.z-a*A.x);
	}
	else{
		a= (B.y-A.y)/(B.x-A.x);
		b= (A.y-a*A.x);
	}
	if( !a && !b)
		return mfalse;
	return mtrue;
}

mbool StraightEqu( MajaVector& A, MajaVector& B, straight_2D equation, mbool ogl)
{
	if( ogl){
		equation.a= (B.z-A.z)/(B.x-A.x);
		equation.b= (A.z-equation.a*A.x);
	}
	else{
		equation.a= (B.y-A.y)/(B.x-A.x);
		equation.b= (A.y-equation.a*A.x);
	}
	if( !equation.a && !equation.b)
		return mfalse;
	return mtrue;
}

mreal StraightLenght( MajaVector& a, MajaVector& b)
{
    return sqrt( ( b.x-a.x)*(b.x-a.x)+(b.z-a.z)*(b.z-a.z));
}

mreal StraightLenght3D( MajaVector& a, MajaVector& b)
{
    return sqrt( ( b.x-a.x)*(b.x-a.x)+(b.z-a.z)*(b.z-a.z)+(b.y-a.y)*(b.y-a.y));
}

MajaVector MiddlePoint( MajaVector& a, MajaVector& b)
{
    MajaVector i;
    i= a+b;
    i= i/2;

    return i;
}

}//namespace MAJAMATHS{
}
